@@ -3,7 +3,7 @@ package org.bouncycastle.asn1;
 import java.io.ByteArrayOutputStream;
 import java.io.IOException;
 import java.util.Enumeration;
-import java.util.Vector;
+import java.util.NoSuchElementException;
 
 /**
  * ASN.1 OctetStrings, with indefinite length rules, and <i>constructed form</i> support.
@@ -22,9 +22,10 @@ import java.util.Vector;
 public class BEROctetString
     extends ASN1OctetString
 {
-    private static final int MAX_LENGTH = 1000;
+    private static final int DEFAULT_CHUNK_SIZE = 1000;
 
-    private ASN1OctetString[] octs;
+    private final int chunkSize;
+    private final ASN1OctetString[] octs;
 
     /**
      * Convert a vector of octet strings into a single byte string
@@ -38,13 +39,7 @@ public class BEROctetString
         {
             try
             {
-                DEROctetString o = (DEROctetString)octs[i];
-
-                bOut.write(o.getOctets());
-            }
-            catch (ClassCastException e)
-            {
-                throw new IllegalArgumentException(octs[i].getClass().getName() + " found in input should only contain DEROctetString");
+                bOut.write(octs[i].getOctets());
             }
             catch (IOException e)
             {
@@ -62,7 +57,7 @@ public class BEROctetString
     public BEROctetString(
         byte[] string)
     {
-        super(string);
+        this(string, DEFAULT_CHUNK_SIZE);
     }
 
     /**
@@ -74,18 +69,40 @@ public class BEROctetString
     public BEROctetString(
         ASN1OctetString[] octs)
     {
-        super(toBytes(octs));
-
-        this.octs = octs;
+        this(octs, DEFAULT_CHUNK_SIZE);
     }
 
     /**
-     * Return a concatenated byte array of all the octets making up the constructed OCTET STRING
-     * @return the full OCTET STRING.
+     * Create an OCTET-STRING object from a byte[]
+     * @param string the octets making up the octet string.
+     * @param chunkSize the number of octets stored in each DER encoded component OCTET STRING.
      */
-    public byte[] getOctets()
+    public BEROctetString(
+        byte[] string,
+        int    chunkSize)
     {
-        return string;
+        this(string, null, chunkSize);
+    }
+
+    /**
+     * Multiple {@link ASN1OctetString} data blocks are input,
+     * the result is <i>constructed form</i>.
+     *
+     * @param octs an array of OCTET STRING to construct the BER OCTET STRING from.
+     * @param chunkSize the number of octets stored in each DER encoded component OCTET STRING.
+     */
+    public BEROctetString(
+        ASN1OctetString[] octs,
+        int chunkSize)
+    {
+        this(toBytes(octs), octs, chunkSize);
+    }
+
+    private BEROctetString(byte[] string, ASN1OctetString[] octs, int chunkSize)
+    {
+        super(string);
+        this.octs = octs;
+        this.chunkSize = chunkSize;
     }
 
     /**
@@ -97,7 +114,28 @@ public class BEROctetString
     {
         if (octs == null)
         {
-            return generateOcts().elements();
+            return new Enumeration()
+            {
+                int pos = 0;
+
+                public boolean hasMoreElements()
+                {
+                    return pos < string.length;
+                }
+
+                public Object nextElement()
+                {
+                    if (pos < string.length)
+                    {
+                        int length = Math.min(string.length - pos, chunkSize);
+                        byte[] chunk = new byte[length];
+                        System.arraycopy(string, pos, chunk, 0, length);
+                        pos += length;
+                        return new DEROctetString(chunk);
+                    }
+                    throw new NoSuchElementException();
+                }
+            };
         }
 
         return new Enumeration()
@@ -111,37 +149,15 @@ public class BEROctetString
 
             public Object nextElement()
             {
-                return octs[counter++];
+                if (counter < octs.length)
+                {
+                    return octs[counter++];
+                }
+                throw new NoSuchElementException();
             }
         };
     }
 
-    private Vector generateOcts()
-    { 
-        Vector vec = new Vector();
-        for (int i = 0; i < string.length; i += MAX_LENGTH) 
-        { 
-            int end; 
-
-            if (i + MAX_LENGTH > string.length) 
-            { 
-                end = string.length; 
-            } 
-            else 
-            { 
-                end = i + MAX_LENGTH; 
-            } 
-
-            byte[] nStr = new byte[end - i]; 
-
-            System.arraycopy(string, i, nStr, 0, nStr.length);
-
-            vec.addElement(new DEROctetString(nStr));
-         } 
-        
-         return vec; 
-    }
-
     boolean isConstructed()
     {
         return true;
@@ -159,37 +175,19 @@ public class BEROctetString
         return 2 + length + 2;
     }
 
-    public void encode(
-        ASN1OutputStream out)
-        throws IOException
+    void encode(ASN1OutputStream out, boolean withTag) throws IOException
     {
-        out.write(BERTags.CONSTRUCTED | BERTags.OCTET_STRING);
-
-        out.write(0x80);
-
-        //
-        // write out the octet array
-        //
-        for (Enumeration e = getObjects(); e.hasMoreElements();)
-        {
-            out.writeObject((ASN1Encodable)e.nextElement());
-        }
-
-        out.write(0x00);
-        out.write(0x00);
+        out.writeEncodedIndef(withTag, BERTags.CONSTRUCTED | BERTags.OCTET_STRING,  getObjects());
     }
 
     static BEROctetString fromSequence(ASN1Sequence seq)
     {
-        ASN1OctetString[]     v = new ASN1OctetString[seq.size()];
-        Enumeration e = seq.getObjects();
-        int                   index = 0;
-
-        while (e.hasMoreElements())
+        int count = seq.size();
+        ASN1OctetString[] v = new ASN1OctetString[count];
+        for (int i = 0; i < count; ++i)
         {
-            v[index++] = (ASN1OctetString)e.nextElement();
+            v[i] = ASN1OctetString.getInstance(seq.getObjectAt(i));
         }
-
         return new BEROctetString(v);
     }
 }
