@@ -38,12 +38,7 @@ public class PKIMessages
 
     public PKIMessages(PKIMessage[] msgs)
     {
-        ASN1EncodableVector v = new ASN1EncodableVector();
-        for (int i = 0; i < msgs.length; i++)
-        {
-            v.add(msgs[i]);
-        }
-        content = new DERSequence(v);
+        content = new DERSequence(msgs);
     }
 
     public PKIMessage[] toPKIMessageArray()
