@@ -49,10 +49,10 @@ public class CVCertificate
             ASN1Primitive tmpObj;
             while ((tmpObj = content.readObject()) != null)
             {
-                DERApplicationSpecific aSpe;
-                if (tmpObj instanceof DERApplicationSpecific)
+                ASN1ApplicationSpecific aSpe;
+                if (tmpObj instanceof ASN1ApplicationSpecific)
                 {
-                    aSpe = (DERApplicationSpecific)tmpObj;
+                    aSpe = (ASN1ApplicationSpecific)tmpObj;
                     switch (aSpe.getApplicationTag())
                     {
                     case EACTags.CERTIFICATE_CONTENT_TEMPLATE:
@@ -103,9 +103,9 @@ public class CVCertificate
         ASN1Primitive obj;
         while ((obj = aIS.readObject()) != null)
         {
-            if (obj instanceof DERApplicationSpecific)
+            if (obj instanceof ASN1ApplicationSpecific)
             {
-                setPrivateData((DERApplicationSpecific)obj);
+                setPrivateData((ASN1ApplicationSpecific)obj);
             }
             else
             {
@@ -115,10 +115,10 @@ public class CVCertificate
     }
 
     /**
-     * Create an iso7816Certificate structure from a DERApplicationSpecific.
+     * Create an iso7816Certificate structure from a ASN1ApplicationSpecific.
      *
-     * @param appSpe the DERApplicationSpecific object.
-     * @return the Iso7816CertificateStructure represented by the DERApplicationSpecific object.
+     * @param appSpe the ASN1ApplicationSpecific object.
+     * @return the Iso7816CertificateStructure represented by the ASN1ApplicationSpecific object.
      * @throws IOException if there is a problem parsing the data.
      */
     private CVCertificate(ASN1ApplicationSpecific appSpe)
@@ -160,7 +160,7 @@ public class CVCertificate
         {
             try
             {
-                return new CVCertificate(DERApplicationSpecific.getInstance(obj));
+                return new CVCertificate(ASN1ApplicationSpecific.getInstance(obj));
             }
             catch (IOException e)
             {
@@ -197,7 +197,7 @@ public class CVCertificate
      */
     public ASN1Primitive toASN1Primitive()
     {
-        ASN1EncodableVector v = new ASN1EncodableVector();
+        ASN1EncodableVector v = new ASN1EncodableVector(2);
 
         v.add(certificateBody);
 
