@@ -4,6 +4,7 @@ import java.math.BigInteger;
 import java.security.SecureRandom;
 
 import org.bouncycastle.crypto.CryptoException;
+import org.bouncycastle.crypto.CryptoServicesRegistrar;
 import org.bouncycastle.crypto.Digest;
 import org.bouncycastle.crypto.digests.SHA256Digest;
 import org.bouncycastle.util.Arrays;
@@ -48,14 +49,13 @@ import org.bouncycastle.util.Arrays;
  * These are the trivial techniques to optimize the communication.
  * <p>
  * The key confirmation process is implemented as specified in
- * <a href="http://csrc.nist.gov/publications/nistpubs/800-56A/SP800-56A_Revision1_Mar08-2007.pdf">NIST SP 800-56A Revision 1</a>,
+ * <a href="https://csrc.nist.gov/publications/nistpubs/800-56A/SP800-56A_Revision1_Mar08-2007.pdf">NIST SP 800-56A Revision 1</a>,
  * Section 8.2 Unilateral Key Confirmation for Key Agreement Schemes.
  * <p>
  * This class is stateful and NOT threadsafe.
  * Each instance should only be used for ONE complete J-PAKE exchange
  * (i.e. a new {@link JPAKEParticipant} should be constructed for each new J-PAKE exchange).
  * <p>
- * See {@link JPAKEExample} for example usage.
  */
 public class JPAKEParticipant
 {
@@ -81,9 +81,10 @@ public class JPAKEParticipant
     /**
      * Shared secret.  This only contains the secret between construction
      * and the call to {@link #calculateKeyingMaterial()}.
-     * <p/>
+     * <p>
      * i.e. When {@link #calculateKeyingMaterial()} is called, this buffer overwritten with 0's,
      * and the field is set to null.
+     * </p>
      */
     private char[] password;
 
@@ -193,7 +194,7 @@ public class JPAKEParticipant
             password,
             group,
             new SHA256Digest(),
-            new SecureRandom());
+            CryptoServicesRegistrar.getSecureRandom());
     }
 
 
@@ -502,6 +503,7 @@ public class JPAKEParticipant
      * <p>
      * After execution, the {@link #getState() state} will be {@link #STATE_ROUND_3_VALIDATED}.
      *
+     * @param round3PayloadReceived The round 3 payload received from the other participant.
      * @param keyingMaterial The keying material as returned from {@link #calculateKeyingMaterial()}.
      * @throws CryptoException if validation fails.
      * @throws IllegalStateException if called prior to {@link #calculateKeyingMaterial()}, or multiple times
