@@ -105,7 +105,7 @@ public class DSASigner
         // the randomizer is to conceal timing information related to k and x.
         BigInteger  r = params.getG().modPow(k.add(getRandomizer(q, random)), params.getP()).mod(q);
 
-        k = k.modInverse(q).multiply(m.add(x.multiply(r)));
+        k = BigIntegers.modOddInverse(q, k).multiply(m.add(x.multiply(r)));
 
         BigInteger  s = k.mod(q);
 
@@ -137,7 +137,7 @@ public class DSASigner
             return false;
         }
 
-        BigInteger  w = s.modInverse(q);
+        BigInteger w = BigIntegers.modOddInverseVar(q, s);
 
         BigInteger  u1 = m.multiply(w).mod(q);
         BigInteger  u2 = r.multiply(w).mod(q);
@@ -169,7 +169,7 @@ public class DSASigner
 
     protected SecureRandom initSecureRandom(boolean needed, SecureRandom provided)
     {
-        return !needed ? null : (provided != null) ? provided : CryptoServicesRegistrar.getSecureRandom();
+        return needed ? CryptoServicesRegistrar.getSecureRandom(provided) : null;
     }
 
     private BigInteger getRandomizer(BigInteger q, SecureRandom provided)
@@ -177,6 +177,6 @@ public class DSASigner
         // Calculate a random multiple of q to add to k. Note that g^q = 1 (mod p), so adding multiple of q to k does not change r.
         int randomBits = 7;
 
-        return BigIntegers.createRandomBigInteger(randomBits, provided != null ? provided : CryptoServicesRegistrar.getSecureRandom()).add(BigInteger.valueOf(128)).multiply(q);
+        return BigIntegers.createRandomBigInteger(randomBits, CryptoServicesRegistrar.getSecureRandom(provided)).add(BigInteger.valueOf(128)).multiply(q);
     }
 }
