@@ -4,6 +4,7 @@ import java.security.SecureRandom;
 
 import org.bouncycastle.crypto.AsymmetricCipherKeyPair;
 import org.bouncycastle.crypto.AsymmetricCipherKeyPairGenerator;
+import org.bouncycastle.crypto.CryptoServicesRegistrar;
 import org.bouncycastle.crypto.KeyGenerationParameters;
 import org.bouncycastle.pqc.crypto.rainbow.util.ComputeInField;
 import org.bouncycastle.pqc.crypto.rainbow.util.GF2Field;
@@ -15,7 +16,7 @@ import org.bouncycastle.pqc.crypto.rainbow.util.GF2Field;
  * <p>
  * Detailed information about the key generation is to be found in the paper of
  * Jintai Ding, Dieter Schmidt: Rainbow, a New Multivariable Polynomial
- * Signature Scheme. ACNS 2005: 164-175 (http://dx.doi.org/10.1007/11496137_12)
+ * Signature Scheme. ACNS 2005: 164-175 (https://dx.doi.org/10.1007/11496137_12)
  */
 public class RainbowKeyPairGenerator
     implements AsymmetricCipherKeyPairGenerator
@@ -90,7 +91,7 @@ public class RainbowKeyPairGenerator
         this.rainbowParams = (RainbowKeyGenerationParameters)param;
 
         // set source of randomness
-        this.sr = new SecureRandom();
+        this.sr = rainbowParams.getRandom();
 
         // unmarshalling:
         this.vi = this.rainbowParams.getParameters().getVi();
@@ -101,7 +102,7 @@ public class RainbowKeyPairGenerator
 
     private void initializeDefault()
     {
-        RainbowKeyGenerationParameters rbKGParams = new RainbowKeyGenerationParameters(new SecureRandom(), new RainbowParameters());
+        RainbowKeyGenerationParameters rbKGParams = new RainbowKeyGenerationParameters(CryptoServicesRegistrar.getSecureRandom(), new RainbowParameters());
         initialize(rbKGParams);
     }
 
