@@ -1,6 +1,8 @@
 package org.bouncycastle.util.test;
 
 import java.io.PrintStream;
+import java.util.Enumeration;
+import java.util.Vector;
 
 import org.bouncycastle.util.Arrays;
 
@@ -166,6 +168,11 @@ public abstract class SimpleTest
         return Arrays.areEqual(a, b);
     }
 
+    protected boolean areEqual(byte[] a, int aFromIndex, int aToIndex, byte[] b, int bFromIndex, int bToIndex)
+    {
+        return Arrays.areEqual(a, aFromIndex, aToIndex, b, bFromIndex, bToIndex);
+    }
+
     public TestResult perform()
     {
         try
@@ -183,25 +190,65 @@ public abstract class SimpleTest
         }
     }
 
-    protected static void runTest(
-        Test test)
+    public abstract void performTest()
+        throws Exception;
+
+    public static void runTest(Test test)
     {
         runTest(test, System.out);
     }
 
-    protected static void runTest(
-        Test test,
-        PrintStream out)
+    public static void runTest(Test test, PrintStream out)
     {
         TestResult result = test.perform();
 
-        out.println(result.toString());
         if (result.getException() != null)
         {
             result.getException().printStackTrace(out);
         }
+
+        out.println(result);
     }
 
-    public abstract void performTest()
-        throws Exception;
+    public static void runTests(Test[] tests)
+    {
+        runTests(tests, System.out);
+    }
+
+    public static void runTests(Test[] tests, PrintStream out)
+    {
+        Vector failures = new Vector();
+        for (int i = 0; i != tests.length; i++)
+        {
+            TestResult result = tests[i].perform();
+            if (!result.isSuccessful())
+            {
+                failures.addElement(result);
+            }
+
+            if (result.getException() != null)
+            {
+                result.getException().printStackTrace(out);
+            }
+
+            out.println(result);
+        }
+
+        out.println("-----");
+        if (failures.isEmpty())
+        {
+            out.println("All tests successful.");
+        }
+        else
+        {
+            out.println("Completed with " + failures.size() + " FAILURES:");
+
+            Enumeration e = failures.elements();
+            while (e.hasMoreElements())
+            {
+                System.out.println("=>  " + (TestResult)e.nextElement());
+            }
+        }
+
+    }
 }
