@@ -1,7 +1,7 @@
 package org.bouncycastle.asn1.test;
 
+import org.bouncycastle.util.test.SimpleTest;
 import org.bouncycastle.util.test.Test;
-import org.bouncycastle.util.test.TestResult;
 
 public class RegressionTest
 {
@@ -78,23 +78,14 @@ public class RegressionTest
         new PKIPublicationInfoTest(),
         new CertifiedKeyPairTest(),
         new PrivateKeyInfoTest(),
-        new LocaleTest()
+        new LocaleTest(),
+        new LinkedCertificateTest(),
+        new DLExternalTest(),
+        new KMACParamsTest()
     };
 
-    public static void main(
-        String[]    args)
+    public static void main(String[] args)
     {
-        for (int i = 0; i != tests.length; i++)
-        {
-            TestResult  result = tests[i].perform();
-            
-            if (result.getException() != null)
-            {
-                result.getException().printStackTrace();
-            }
-            
-            System.out.println(result);
-        }
+        SimpleTest.runTests(tests);
     }
 }
-
