@@ -1,5 +1,6 @@
 package org.bouncycastle.crypto.ec.test;
 
+import junit.extensions.TestSetup;
 import junit.framework.Test;
 import junit.framework.TestCase;
 import junit.framework.TestSuite;
@@ -34,6 +35,26 @@ public class AllTests
 
         suite.addTestSuite(AllTests.class);
 
-        return suite;
+        return new BCTestSetup(suite);
     }
+
+    static class BCTestSetup
+        extends TestSetup
+    {
+        public BCTestSetup(Test test)
+        {
+            super(test);
+        }
+
+        protected void setUp()
+        {
+
+        }
+
+        protected void tearDown()
+        {
+
+        }
+    }
+
 }
