@@ -12,11 +12,14 @@ import org.bouncycastle.util.Arrays;
 
 /**
  * Test cases for XMSSMTPublicKey class.
- * 
  */
-public class XMSSMTPublicKeyTest extends TestCase {
+public class XMSSMTPublicKeyTest
+	extends TestCase
+{
 
-	public void testPublicKeyParsingSHA256() throws IOException, ClassNotFoundException {
+	public void testPublicKeyParsingSHA256()
+		throws IOException, ClassNotFoundException
+	{
 		XMSSMTParameters params = new XMSSMTParameters(20, 10, new SHA256Digest());
 		XMSSMT mt = new XMSSMT(params, new SecureRandom());
 		mt.generateKeys();
@@ -28,14 +31,18 @@ public class XMSSMTPublicKeyTest extends TestCase {
 		assertTrue(Arrays.areEqual(publicKey, mt.exportPublicKey()));
 	}
 
-	public void testConstructor() {
-		XMSSMTParameters params = new XMSSMTParameters(20, 10, new SHA256Digest());
+	public void testConstructor()
+	{
+		XMSSMTParameters params = new XMSSMTParameters(20, 4, new SHA256Digest());
 		XMSSMTPublicKeyParameters pk = new XMSSMTPublicKeyParameters.Builder(params).build();
 
 		byte[] pkByte = pk.toByteArray();
 		/* check everything is 0 */
-		for (int i = 0; i < pkByte.length; i++) {
+		for (int i = 5; i < pkByte.length; i++)
+		{
 			assertEquals(0x00, pkByte[i]);
 		}
+
+		assertEquals(68, pkByte.length);
 	}
 }
