@@ -4,8 +4,6 @@ import java.io.IOException;
 import java.io.OutputStream;
 import java.security.AccessController;
 import java.security.PrivilegedAction;
-import java.util.ArrayList;
-import java.util.List;
 
 import javax.activation.CommandMap;
 import javax.activation.MailcapCommandMap;
@@ -68,7 +66,6 @@ public class SMIMEEnvelopedGenerator
     private static final String ENCRYPTED_CONTENT_TYPE = "application/pkcs7-mime; name=\"smime.p7m\"; smime-type=enveloped-data";
     
     private EnvelopedGenerator fact;
-    private List               recipients = new ArrayList();
 
     static
     {
@@ -80,7 +77,7 @@ public class SMIMEEnvelopedGenerator
 
                 if (commandMap instanceof MailcapCommandMap)
                 {
-                    CommandMap.setDefaultCommandMap(addCommands((MailcapCommandMap)commandMap));
+                    CommandMap.setDefaultCommandMap(MailcapUtil.addCommands((MailcapCommandMap)commandMap));
                 }
 
                 return null;
@@ -88,17 +85,6 @@ public class SMIMEEnvelopedGenerator
         });
     }
 
-    private static MailcapCommandMap addCommands(MailcapCommandMap mc)
-    {
-        mc.addMailcap("application/pkcs7-signature;; x-java-content-handler=org.bouncycastle.mail.smime.handlers.pkcs7_signature");
-        mc.addMailcap("application/pkcs7-mime;; x-java-content-handler=org.bouncycastle.mail.smime.handlers.pkcs7_mime");
-        mc.addMailcap("application/x-pkcs7-signature;; x-java-content-handler=org.bouncycastle.mail.smime.handlers.x_pkcs7_signature");
-        mc.addMailcap("application/x-pkcs7-mime;; x-java-content-handler=org.bouncycastle.mail.smime.handlers.x_pkcs7_mime");
-        mc.addMailcap("multipart/signed;; x-java-content-handler=org.bouncycastle.mail.smime.handlers.multipart_signed");
-
-        return mc;
-    }
-
     /**
      * base constructor
      */
@@ -224,7 +210,7 @@ public class SMIMEEnvelopedGenerator
 
                 if (commandMap instanceof MailcapCommandMap)
                 {
-                    _content.getDataHandler().setCommandMap(addCommands((MailcapCommandMap)commandMap));
+                    _content.getDataHandler().setCommandMap(MailcapUtil.addCommands((MailcapCommandMap)commandMap));
                 }
 
                 _content.writeTo(encrypted);
