@@ -457,10 +457,9 @@ public class SignedMailValidator
         throws IOException
     {
         ASN1InputStream aIn = new ASN1InputStream(ext);
-        ASN1OctetString octs = (ASN1OctetString)aIn.readObject();
+        ASN1OctetString octs = ASN1OctetString.getInstance(aIn.readObject());
 
-        aIn = new ASN1InputStream(octs.getOctets());
-        return aIn.readObject();
+        return ASN1Primitive.fromByteArray(octs.getOctets());
     }
 
     protected void checkSignerCert(X509Certificate cert, List errors,
@@ -468,20 +467,20 @@ public class SignedMailValidator
     {
         // get key length
         PublicKey key = cert.getPublicKey();
-        int keyLenght = -1;
+        int keyLength = -1;
         if (key instanceof RSAPublicKey)
         {
-            keyLenght = ((RSAPublicKey)key).getModulus().bitLength();
+            keyLength = ((RSAPublicKey)key).getModulus().bitLength();
         }
         else if (key instanceof DSAPublicKey)
         {
-            keyLenght = ((DSAPublicKey)key).getParams().getP().bitLength();
+            keyLength = ((DSAPublicKey)key).getParams().getP().bitLength();
         }
-        if (keyLenght != -1 && keyLenght <= shortKeyLength)
+        if (keyLength != -1 && keyLength <= shortKeyLength)
         {
             ErrorBundle msg = new ErrorBundle(RESOURCE_NAME,
                 "SignedMailValidator.shortSigningKey",
-                new Object[]{Integers.valueOf(keyLenght)});
+                new Object[]{Integers.valueOf(keyLength)});
             notifications.add(msg);
         }
 
