@@ -10,6 +10,7 @@ import java.util.Iterator;
 import java.util.List;
 
 import org.bouncycastle.asn1.x9.ECNamedCurveTable;
+import org.bouncycastle.asn1.x9.X9ECParameters;
 import org.bouncycastle.bcpg.BCPGKey;
 import org.bouncycastle.bcpg.BCPGOutputStream;
 import org.bouncycastle.bcpg.ContainedPacket;
@@ -19,6 +20,7 @@ import org.bouncycastle.bcpg.ElGamalPublicBCPGKey;
 import org.bouncycastle.bcpg.PublicKeyAlgorithmTags;
 import org.bouncycastle.bcpg.PublicKeyPacket;
 import org.bouncycastle.bcpg.RSAPublicBCPGKey;
+import org.bouncycastle.bcpg.SignatureSubpacketTags;
 import org.bouncycastle.bcpg.TrustPacket;
 import org.bouncycastle.bcpg.UserAttributePacket;
 import org.bouncycastle.bcpg.UserIDPacket;
@@ -31,7 +33,7 @@ import org.bouncycastle.util.Arrays;
 public class PGPPublicKey
     implements PublicKeyAlgorithmTags
 {
-    private static final int[] MASTER_KEY_CERTIFICATION_TYPES = new int[] { PGPSignature.POSITIVE_CERTIFICATION, PGPSignature.CASUAL_CERTIFICATION, PGPSignature.NO_CERTIFICATION, PGPSignature.DEFAULT_CERTIFICATION };
+    private static final int[] MASTER_KEY_CERTIFICATION_TYPES = new int[] { PGPSignature.POSITIVE_CERTIFICATION, PGPSignature.CASUAL_CERTIFICATION, PGPSignature.NO_CERTIFICATION, PGPSignature.DEFAULT_CERTIFICATION, PGPSignature.DIRECT_KEY };
     
     PublicKeyPacket publicPk;
     TrustPacket     trustPk;
@@ -85,7 +87,16 @@ public class PGPPublicKey
             }
             else if (key instanceof ECPublicBCPGKey)
             {
-                this.keyStrength = ECNamedCurveTable.getByOID(((ECPublicBCPGKey)key).getCurveOID()).getCurve().getFieldSize();
+                X9ECParameters ecParameters = ECNamedCurveTable.getByOID(((ECPublicBCPGKey)key).getCurveOID());
+
+                if (ecParameters != null)
+                {
+                    this.keyStrength = ecParameters.getCurve().getFieldSize();
+                }
+                else
+                {
+                    this.keyStrength = -1; // unknown
+                }
             }
         }
     }
@@ -261,7 +272,7 @@ public class PGPPublicKey
                 for (int i = 0; i != MASTER_KEY_CERTIFICATION_TYPES.length; i++)
                 {
                     long seconds = getExpirationTimeFromSig(true, MASTER_KEY_CERTIFICATION_TYPES[i]);
-                    
+
                     if (seconds >= 0)
                     {
                         return seconds;
@@ -276,6 +287,12 @@ public class PGPPublicKey
                 {
                     return seconds;
                 }
+
+                seconds = getExpirationTimeFromSig(false, PGPSignature.DIRECT_KEY);
+                if (seconds >= 0)
+                {
+                    return seconds;
+                }
             }
             
             return 0;
@@ -306,6 +323,11 @@ public class PGPPublicKey
                     continue;
                 }
 
+                if (!hashed.hasSubpacket(SignatureSubpacketTags.KEY_EXPIRE_TIME))
+                {
+                    continue;
+                }
+                
                 long current = hashed.getKeyExpirationTime();
 
                 if (sig.getKeyID() == this.getKeyID())
@@ -370,12 +392,12 @@ public class PGPPublicKey
     }
 
     /**
-     * Return true if this is a master key.
+     * Return true if this could be a master key.
      * @return true if a master key.
      */
     public boolean isMasterKey()
     {
-        return (subSigs == null);
+        return (subSigs == null) && !(this.isEncryptionKey() && publicPk.getAlgorithm() != PublicKeyAlgorithmTags.RSA_GENERAL);
     }
     
     /**
