@@ -1,6 +1,7 @@
 package org.bouncycastle.openpgp;
 
 import java.io.BufferedInputStream;
+import java.io.ByteArrayInputStream;
 import java.io.File;
 import java.io.FileInputStream;
 import java.io.IOException;
@@ -9,12 +10,13 @@ import java.io.OutputStream;
 import java.security.SecureRandom;
 import java.util.Date;
 
-import org.bouncycastle.asn1.ASN1InputStream;
 import org.bouncycastle.asn1.ASN1Integer;
 import org.bouncycastle.asn1.ASN1Sequence;
 import org.bouncycastle.bcpg.ArmoredInputStream;
+import org.bouncycastle.bcpg.BCPGInputStream;
 import org.bouncycastle.bcpg.HashAlgorithmTags;
 import org.bouncycastle.bcpg.MPInteger;
+import org.bouncycastle.bcpg.PacketTags;
 import org.bouncycastle.bcpg.PublicKeyAlgorithmTags;
 import org.bouncycastle.bcpg.SymmetricKeyAlgorithmTags;
 import org.bouncycastle.util.Arrays;
@@ -174,29 +176,62 @@ public class PGPUtil
         byte[] encoding)
         throws PGPException
     {
-        ASN1InputStream aIn = new ASN1InputStream(encoding);
-
-        ASN1Integer i1;
-        ASN1Integer i2;
+        ASN1Integer i1, i2;
 
         try
         {
-            ASN1Sequence s = (ASN1Sequence)aIn.readObject();
+            ASN1Sequence s = ASN1Sequence.getInstance(encoding);
 
-            i1 = (ASN1Integer)s.getObjectAt(0);
-            i2 = (ASN1Integer)s.getObjectAt(1);
+            i1 = ASN1Integer.getInstance(s.getObjectAt(0));
+            i2 = ASN1Integer.getInstance(s.getObjectAt(1));
         }
-        catch (IOException e)
+        catch (RuntimeException e)
         {
-            throw new PGPException("exception encoding signature", e);
+            throw new PGPException("exception decoding signature", e);
         }
 
-        MPInteger[] values = new MPInteger[2];
+        return new MPInteger[]{
+            new MPInteger(i1.getValue()),
+            new MPInteger(i2.getValue())
+        };
+    }
 
-        values[0] = new MPInteger(i1.getValue());
-        values[1] = new MPInteger(i2.getValue());
+    /**
+     * Return true if the byte[] blob probably represents key ring data.
+     * @return true if data likely represents a key ring stream.
+     */
+    public static boolean isKeyRing(byte[] blob)
+        throws IOException
+    {
+        BCPGInputStream bIn = new BCPGInputStream(new ByteArrayInputStream(blob));
 
-        return values;
+        int tag = bIn.nextPacketTag();
+
+        return tag == PacketTags.PUBLIC_KEY || tag == PacketTags.PUBLIC_SUBKEY
+            || tag == PacketTags.SECRET_KEY || tag == PacketTags.SECRET_SUBKEY;
+    }
+
+    /**
+     * Return true if the byte[] blob probably represents key box data.
+     * @return true if data likely represents a key box stream.
+     */
+    public static boolean isKeyBox(byte[] data)
+        throws IOException
+    {
+        if (data.length < 12)
+        {
+            return false;
+        }
+
+        InputStream bIn = new ByteArrayInputStream(data);
+
+        // skip size and headers
+        for (int i = 0; i != 8; i++)
+        {
+            bIn.read();
+        }
+
+        return bIn.read() == 'K' && bIn.read() == 'B' && bIn.read() == 'X' && bIn.read() == 'f';
     }
 
     /**
