@@ -1,6 +1,9 @@
 package org.bouncycastle.openpgp.operator.jcajce;
 
+import java.io.IOException;
+import java.math.BigInteger;
 import java.security.AlgorithmParameters;
+import java.security.GeneralSecurityException;
 import java.security.KeyFactory;
 import java.security.NoSuchAlgorithmException;
 import java.security.NoSuchProviderException;
@@ -21,8 +24,11 @@ import java.security.spec.ECParameterSpec;
 import java.security.spec.ECPrivateKeySpec;
 import java.security.spec.ECPublicKeySpec;
 import java.security.spec.InvalidParameterSpecException;
+import java.security.spec.KeySpec;
+import java.security.spec.PKCS8EncodedKeySpec;
 import java.security.spec.RSAPrivateCrtKeySpec;
 import java.security.spec.RSAPublicKeySpec;
+import java.security.spec.X509EncodedKeySpec;
 import java.util.Date;
 
 import javax.crypto.interfaces.DHPrivateKey;
@@ -34,6 +40,11 @@ import javax.crypto.spec.DHPublicKeySpec;
 import org.bouncycastle.asn1.ASN1ObjectIdentifier;
 import org.bouncycastle.asn1.ASN1OctetString;
 import org.bouncycastle.asn1.DEROctetString;
+import org.bouncycastle.asn1.cryptlib.CryptlibObjectIdentifiers;
+import org.bouncycastle.asn1.edec.EdECObjectIdentifiers;
+import org.bouncycastle.asn1.gnu.GNUObjectIdentifiers;
+import org.bouncycastle.asn1.pkcs.PrivateKeyInfo;
+import org.bouncycastle.asn1.x509.AlgorithmIdentifier;
 import org.bouncycastle.asn1.x509.SubjectPublicKeyInfo;
 import org.bouncycastle.asn1.x9.ECNamedCurveTable;
 import org.bouncycastle.asn1.x9.X9ECParameters;
@@ -43,7 +54,10 @@ import org.bouncycastle.bcpg.DSAPublicBCPGKey;
 import org.bouncycastle.bcpg.DSASecretBCPGKey;
 import org.bouncycastle.bcpg.ECDHPublicBCPGKey;
 import org.bouncycastle.bcpg.ECDSAPublicBCPGKey;
+import org.bouncycastle.bcpg.ECPublicBCPGKey;
 import org.bouncycastle.bcpg.ECSecretBCPGKey;
+import org.bouncycastle.bcpg.EdDSAPublicBCPGKey;
+import org.bouncycastle.bcpg.EdSecretBCPGKey;
 import org.bouncycastle.bcpg.ElGamalPublicBCPGKey;
 import org.bouncycastle.bcpg.ElGamalSecretBCPGKey;
 import org.bouncycastle.bcpg.HashAlgorithmTags;
@@ -62,9 +76,18 @@ import org.bouncycastle.openpgp.PGPKdfParameters;
 import org.bouncycastle.openpgp.PGPPrivateKey;
 import org.bouncycastle.openpgp.PGPPublicKey;
 import org.bouncycastle.openpgp.operator.KeyFingerPrintCalculator;
+import org.bouncycastle.util.Arrays;
+import org.bouncycastle.util.BigIntegers;
 
 public class JcaPGPKeyConverter
 {
+    private static final int X25519_KEY_SIZE = 32;
+    private static final int ED25519_KEY_SIZE = 32;
+
+    // We default to these as they are specified as mandatory in RFC 6631.
+    private static final PGPKdfParameters DEFAULT_KDF_PARAMETERS = new PGPKdfParameters(HashAlgorithmTags.SHA256,
+        SymmetricKeyAlgorithmTags.AES_128);
+
     private OperatorHelper helper = new OperatorHelper(new DefaultJcaJceHelper());
     private KeyFingerPrintCalculator fingerPrintCalculator = new JcaKeyFingerprintCalculator();
 
@@ -82,73 +105,20 @@ public class JcaPGPKeyConverter
         return this;
     }
 
-    public PublicKey getPublicKey(PGPPublicKey publicKey)
+    /**
+     * Convert a PrivateKey into a PGPPrivateKey.
+     *
+     * @param pub   the corresponding PGPPublicKey to privKey.
+     * @param privKey  the private key for the key in pub.
+     * @return a PGPPrivateKey
+     * @throws PGPException
+     */
+    public PGPPrivateKey getPGPPrivateKey(PGPPublicKey pub, PrivateKey privKey)
         throws PGPException
     {
-        KeyFactory fact;
+        BCPGKey privPk = getPrivateBCPGKey(pub, privKey);
 
-        PublicKeyPacket publicPk = publicKey.getPublicKeyPacket();
-
-        try
-        {
-            switch (publicPk.getAlgorithm())
-            {
-            case PublicKeyAlgorithmTags.RSA_ENCRYPT:
-            case PublicKeyAlgorithmTags.RSA_GENERAL:
-            case PublicKeyAlgorithmTags.RSA_SIGN:
-                RSAPublicBCPGKey rsaK = (RSAPublicBCPGKey)publicPk.getKey();
-                RSAPublicKeySpec rsaSpec = new RSAPublicKeySpec(rsaK.getModulus(), rsaK.getPublicExponent());
-
-                fact = helper.createKeyFactory("RSA");
-
-                return fact.generatePublic(rsaSpec);
-            case PublicKeyAlgorithmTags.DSA:
-                DSAPublicBCPGKey dsaK = (DSAPublicBCPGKey)publicPk.getKey();
-                DSAPublicKeySpec dsaSpec = new DSAPublicKeySpec(dsaK.getY(), dsaK.getP(), dsaK.getQ(), dsaK.getG());
-
-                fact = helper.createKeyFactory("DSA");
-
-                return fact.generatePublic(dsaSpec);
-            case PublicKeyAlgorithmTags.ELGAMAL_ENCRYPT:
-            case PublicKeyAlgorithmTags.ELGAMAL_GENERAL:
-                ElGamalPublicBCPGKey elK = (ElGamalPublicBCPGKey)publicPk.getKey();
-                DHPublicKeySpec elSpec = new DHPublicKeySpec(elK.getY(), elK.getP(), elK.getG());
-
-                fact = helper.createKeyFactory("ElGamal");
-
-                return fact.generatePublic(elSpec);
-            case PublicKeyAlgorithmTags.ECDH:
-                ECDHPublicBCPGKey ecdhK = (ECDHPublicBCPGKey)publicPk.getKey();
-                X9ECParameters ecdhParams = JcaJcePGPUtil.getX9Parameters(ecdhK.getCurveOID());
-                ECPoint ecdhPoint = JcaJcePGPUtil.decodePoint(ecdhK.getEncodedPoint(), ecdhParams.getCurve());
-                ECPublicKeySpec   ecDhSpec = new ECPublicKeySpec(
-                    new java.security.spec.ECPoint(ecdhPoint.getAffineXCoord().toBigInteger(), ecdhPoint.getAffineYCoord().toBigInteger()),
-                    getECParameterSpec(ecdhK.getCurveOID(), ecdhParams));
-                fact = helper.createKeyFactory("ECDH");
-
-                return fact.generatePublic(ecDhSpec);
-            case PublicKeyAlgorithmTags.ECDSA:
-                ECDSAPublicBCPGKey ecdsaK = (ECDSAPublicBCPGKey)publicPk.getKey();
-                X9ECParameters ecdsaParams = JcaJcePGPUtil.getX9Parameters(ecdsaK.getCurveOID());
-                ECPoint ecdsaPoint = JcaJcePGPUtil.decodePoint(ecdsaK.getEncodedPoint(), ecdsaParams.getCurve());
-                ECPublicKeySpec ecDsaSpec = new ECPublicKeySpec(
-                    new java.security.spec.ECPoint(ecdsaPoint.getAffineXCoord().toBigInteger(), ecdsaPoint.getAffineYCoord().toBigInteger()),
-                    getECParameterSpec(ecdsaK.getCurveOID(), ecdsaParams));
-                fact = helper.createKeyFactory("ECDSA");
-
-                return fact.generatePublic(ecDsaSpec);
-            default:
-                throw new PGPException("unknown public key algorithm encountered");
-            }
-        }
-        catch (PGPException e)
-        {
-            throw e;
-        }
-        catch (Exception e)
-        {
-            throw new PGPException("exception constructing public key", e);
-        }
+        return new PGPPrivateKey(pub.getKeyID(), pub.getPublicKeyPacket(), privPk);
     }
 
     /**
@@ -166,63 +136,7 @@ public class JcaPGPKeyConverter
     public PGPPublicKey getPGPPublicKey(int algorithm, PGPAlgorithmParameters algorithmParameters, PublicKey pubKey, Date time)
         throws PGPException
     {
-        BCPGKey bcpgKey;
-
-        if (pubKey instanceof RSAPublicKey)
-        {
-            RSAPublicKey rK = (RSAPublicKey)pubKey;
-
-            bcpgKey = new RSAPublicBCPGKey(rK.getModulus(), rK.getPublicExponent());
-        }
-        else if (pubKey instanceof DSAPublicKey)
-        {
-            DSAPublicKey dK = (DSAPublicKey)pubKey;
-            DSAParams dP = dK.getParams();
-
-            bcpgKey = new DSAPublicBCPGKey(dP.getP(), dP.getQ(), dP.getG(), dK.getY());
-        }
-        else if (pubKey instanceof DHPublicKey)
-        {
-            DHPublicKey eK = (DHPublicKey)pubKey;
-            DHParameterSpec eS = eK.getParams();
-
-            bcpgKey = new ElGamalPublicBCPGKey(eS.getP(), eS.getG(), eK.getY());
-        }
-        else if (pubKey instanceof ECPublicKey)
-        {
-            SubjectPublicKeyInfo keyInfo = SubjectPublicKeyInfo.getInstance(pubKey.getEncoded());
-
-            // TODO: should probably match curve by comparison as well
-            ASN1ObjectIdentifier  curveOid = ASN1ObjectIdentifier.getInstance(keyInfo.getAlgorithm().getParameters());
-
-            X9ECParameters params = ECNamedCurveTable.getByOID(curveOid);
-
-            ASN1OctetString key = new DEROctetString(keyInfo.getPublicKeyData().getBytes());
-            X9ECPoint derQ = new X9ECPoint(params.getCurve(), key);
-
-            if (algorithm == PGPPublicKey.ECDH)
-            {
-                PGPKdfParameters kdfParams = (PGPKdfParameters)algorithmParameters;
-                if (kdfParams == null)
-                {
-                    // We default to these as they are specified as mandatory in RFC 6631.
-                    kdfParams = new PGPKdfParameters(HashAlgorithmTags.SHA256, SymmetricKeyAlgorithmTags.AES_128);
-                }
-                bcpgKey = new ECDHPublicBCPGKey(curveOid, derQ.getPoint(), kdfParams.getHashAlgorithm(), kdfParams.getSymmetricWrapAlgorithm());
-            }
-            else if (algorithm == PGPPublicKey.ECDSA)
-            {
-                bcpgKey = new ECDSAPublicBCPGKey(curveOid, derQ.getPoint());
-            }
-            else
-            {
-                throw new PGPException("unknown EC algorithm");
-            }
-        }
-        else
-        {
-            throw new PGPException("unknown key class");
-        }
+        BCPGKey bcpgKey = getPublicBCPGKey(algorithm, algorithmParameters, pubKey, time);
 
         return new PGPPublicKey(new PublicKeyPacket(algorithm, time, bcpgKey), fingerPrintCalculator);
     }
@@ -257,64 +171,68 @@ public class JcaPGPKeyConverter
 
         try
         {
-            KeyFactory fact;
-
             switch (pubPk.getAlgorithm())
             {
-            case PGPPublicKey.RSA_ENCRYPT:
-            case PGPPublicKey.RSA_GENERAL:
-            case PGPPublicKey.RSA_SIGN:
-                RSAPublicBCPGKey rsaPub = (RSAPublicBCPGKey)pubPk.getKey();
-                RSASecretBCPGKey rsaPriv = (RSASecretBCPGKey)privPk;
-                RSAPrivateCrtKeySpec rsaPrivSpec = new RSAPrivateCrtKeySpec(
-                    rsaPriv.getModulus(),
-                    rsaPub.getPublicExponent(),
-                    rsaPriv.getPrivateExponent(),
-                    rsaPriv.getPrimeP(),
-                    rsaPriv.getPrimeQ(),
-                    rsaPriv.getPrimeExponentP(),
-                    rsaPriv.getPrimeExponentQ(),
-                    rsaPriv.getCrtCoefficient());
-
-                fact = helper.createKeyFactory("RSA");
-
-                return fact.generatePrivate(rsaPrivSpec);
-            case PGPPublicKey.DSA:
+            case PublicKeyAlgorithmTags.DSA:
+            {
                 DSAPublicBCPGKey dsaPub = (DSAPublicBCPGKey)pubPk.getKey();
                 DSASecretBCPGKey dsaPriv = (DSASecretBCPGKey)privPk;
-                DSAPrivateKeySpec dsaPrivSpec =
-                    new DSAPrivateKeySpec(dsaPriv.getX(), dsaPub.getP(), dsaPub.getQ(), dsaPub.getG());
+                DSAPrivateKeySpec dsaPrivSpec = new DSAPrivateKeySpec(dsaPriv.getX(), dsaPub.getP(), dsaPub.getQ(),
+                    dsaPub.getG());
+                return implGeneratePrivate("DSA", dsaPrivSpec);
+            }
 
-                fact = helper.createKeyFactory("DSA");
-
-                return fact.generatePrivate(dsaPrivSpec);
             case PublicKeyAlgorithmTags.ECDH:
+            {
                 ECDHPublicBCPGKey ecdhPub = (ECDHPublicBCPGKey)pubPk.getKey();
                 ECSecretBCPGKey ecdhK = (ECSecretBCPGKey)privPk;
-                ECPrivateKeySpec ecDhSpec = new ECPrivateKeySpec(
-                                                    ecdhK.getX(),
-                                                    getECParameterSpec(ecdhPub.getCurveOID()));
-                fact = helper.createKeyFactory("ECDH");
 
-                return fact.generatePrivate(ecDhSpec);
+                if (CryptlibObjectIdentifiers.curvey25519.equals(ecdhPub.getCurveOID()))
+                {
+                    // 'reverse' because the native format for X25519 private keys is little-endian
+                    return implGetPrivateKeyPKCS8("XDH", new PrivateKeyInfo(
+                        new AlgorithmIdentifier(EdECObjectIdentifiers.id_X25519),
+                        new DEROctetString(Arrays.reverse(BigIntegers.asUnsignedByteArray(ecdhK.getX())))));
+                }
+                else
+                {
+                    return implGetPrivateKeyEC("ECDH", ecdhPub, ecdhK);
+                }
+            }
+
             case PublicKeyAlgorithmTags.ECDSA:
-                ECDSAPublicBCPGKey ecdsaPub = (ECDSAPublicBCPGKey)pubPk.getKey();
-                ECSecretBCPGKey ecdsaK = (ECSecretBCPGKey)privPk;
-                ECPrivateKeySpec ecDsaSpec = new ECPrivateKeySpec(
-                                                    ecdsaK.getX(),
-                                                    getECParameterSpec(ecdsaPub.getCurveOID()));
-                fact = helper.createKeyFactory("ECDSA");
+                return implGetPrivateKeyEC("ECDSA", (ECDSAPublicBCPGKey)pubPk.getKey(), (ECSecretBCPGKey)privPk);
 
-                return fact.generatePrivate(ecDsaSpec);
-            case PGPPublicKey.ELGAMAL_ENCRYPT:
-            case PGPPublicKey.ELGAMAL_GENERAL:
+            case PublicKeyAlgorithmTags.EDDSA:
+            {
+                EdSecretBCPGKey eddsaK = (EdSecretBCPGKey)privPk;
+
+                return implGetPrivateKeyPKCS8("EdDSA", new PrivateKeyInfo(
+                    new AlgorithmIdentifier(EdECObjectIdentifiers.id_Ed25519),
+                    new DEROctetString(BigIntegers.asUnsignedByteArray(eddsaK.getX()))));
+            }
+
+            case PublicKeyAlgorithmTags.ELGAMAL_ENCRYPT:
+            case PublicKeyAlgorithmTags.ELGAMAL_GENERAL:
+            {
                 ElGamalPublicBCPGKey elPub = (ElGamalPublicBCPGKey)pubPk.getKey();
                 ElGamalSecretBCPGKey elPriv = (ElGamalSecretBCPGKey)privPk;
                 DHPrivateKeySpec elSpec = new DHPrivateKeySpec(elPriv.getX(), elPub.getP(), elPub.getG());
+                return implGeneratePrivate("ElGamal", elSpec);
+            }
 
-                fact = helper.createKeyFactory("ElGamal");
+            case PublicKeyAlgorithmTags.RSA_ENCRYPT:
+            case PublicKeyAlgorithmTags.RSA_GENERAL:
+            case PublicKeyAlgorithmTags.RSA_SIGN:
+            {
+                RSAPublicBCPGKey rsaPub = (RSAPublicBCPGKey)pubPk.getKey();
+                RSASecretBCPGKey rsaPriv = (RSASecretBCPGKey)privPk;
+                RSAPrivateCrtKeySpec rsaPrivSpec = new RSAPrivateCrtKeySpec(rsaPriv.getModulus(),
+                    rsaPub.getPublicExponent(), rsaPriv.getPrivateExponent(), rsaPriv.getPrimeP(), rsaPriv.getPrimeQ(),
+                    rsaPriv.getPrimeExponentP(), rsaPriv.getPrimeExponentQ(), rsaPriv.getCrtCoefficient());
+                return implGeneratePrivate("RSA", rsaPrivSpec);
+            }
 
-                return fact.generatePrivate(elSpec);
             default:
                 throw new PGPException("unknown public key algorithm encountered");
             }
@@ -329,50 +247,95 @@ public class JcaPGPKeyConverter
         }
     }
 
-    /**
-     * Convert a PrivateKey into a PGPPrivateKey.
-     *
-     * @param pub   the corresponding PGPPublicKey to privKey.
-     * @param privKey  the private key for the key in pub.
-     * @return a PGPPrivateKey
-     * @throws PGPException
-     */
-    public PGPPrivateKey getPGPPrivateKey(PGPPublicKey pub, PrivateKey privKey)
+    public PublicKey getPublicKey(PGPPublicKey publicKey)
         throws PGPException
     {
-        BCPGKey privPk;
+        PublicKeyPacket publicPk = publicKey.getPublicKeyPacket();
 
-        switch (pub.getAlgorithm())
+        try
         {
-        case PGPPublicKey.RSA_ENCRYPT:
-        case PGPPublicKey.RSA_SIGN:
-        case PGPPublicKey.RSA_GENERAL:
-            RSAPrivateCrtKey rsK = (RSAPrivateCrtKey)privKey;
+            switch (publicPk.getAlgorithm())
+            {
+            case PublicKeyAlgorithmTags.DSA:
+            {
+                DSAPublicBCPGKey dsaK = (DSAPublicBCPGKey)publicPk.getKey();
+                DSAPublicKeySpec dsaSpec = new DSAPublicKeySpec(dsaK.getY(), dsaK.getP(), dsaK.getQ(), dsaK.getG());
+                return implGeneratePublic("DSA", dsaSpec);
+            }
 
-            privPk = new RSASecretBCPGKey(rsK.getPrivateExponent(), rsK.getPrimeP(), rsK.getPrimeQ());
-            break;
-        case PGPPublicKey.DSA:
-            DSAPrivateKey dsK = (DSAPrivateKey)privKey;
+            case PublicKeyAlgorithmTags.ECDH:
+            {
+                ECDHPublicBCPGKey ecdhK = (ECDHPublicBCPGKey)publicPk.getKey();
 
-            privPk = new DSASecretBCPGKey(dsK.getX());
-            break;
-        case PGPPublicKey.ELGAMAL_ENCRYPT:
-        case PGPPublicKey.ELGAMAL_GENERAL:
-            DHPrivateKey esK = (DHPrivateKey)privKey;
+                if (ecdhK.getCurveOID().equals(CryptlibObjectIdentifiers.curvey25519))
+                {
+                    byte[] pEnc = BigIntegers.asUnsignedByteArray(ecdhK.getEncodedPoint());
 
-            privPk = new ElGamalSecretBCPGKey(esK.getX());
-            break;
-        case PGPPublicKey.ECDH:
-        case PGPPublicKey.ECDSA:
-            ECPrivateKey ecK = (ECPrivateKey)privKey;
+                    // skip the 0x40 header byte.
+                    if (pEnc.length < 1 || 0x40 != pEnc[0])
+                    {
+                        throw new IllegalArgumentException("Invalid Curve25519 public key");
+                    }
 
-            privPk = new ECSecretBCPGKey(ecK.getS());
-            break;
-        default:
-            throw new PGPException("unknown key class");
+                    return implGetPublicKeyX509("XDH", new SubjectPublicKeyInfo(
+                        new AlgorithmIdentifier(EdECObjectIdentifiers.id_X25519),
+                        Arrays.copyOfRange(pEnc, 1, pEnc.length)));
+                }
+                else
+                {
+                    return implGetPublicKeyEC("ECDH", ecdhK);
+                }
+            }
+
+            case PublicKeyAlgorithmTags.ECDSA:
+                return implGetPublicKeyEC("ECDSA", (ECDSAPublicBCPGKey)publicPk.getKey());
+
+            case PublicKeyAlgorithmTags.EDDSA:
+            {
+                EdDSAPublicBCPGKey eddsaK = (EdDSAPublicBCPGKey)publicPk.getKey();
+
+                byte[] pEnc = BigIntegers.asUnsignedByteArray(eddsaK.getEncodedPoint());
+
+                // skip the 0x40 header byte.
+                if (pEnc.length < 1 || 0x40 != pEnc[0])
+                {
+                    throw new IllegalArgumentException("Invalid Ed25519 public key");
+                }
+
+                return implGetPublicKeyX509("EdDSA", new SubjectPublicKeyInfo(
+                    new AlgorithmIdentifier(EdECObjectIdentifiers.id_Ed25519),
+                    Arrays.copyOfRange(pEnc, 1, pEnc.length)));
+            }
+
+            case PublicKeyAlgorithmTags.ELGAMAL_ENCRYPT:
+            case PublicKeyAlgorithmTags.ELGAMAL_GENERAL:
+            {
+                ElGamalPublicBCPGKey elK = (ElGamalPublicBCPGKey)publicPk.getKey();
+                DHPublicKeySpec elSpec = new DHPublicKeySpec(elK.getY(), elK.getP(), elK.getG());
+                return implGeneratePublic("ElGamal", elSpec);
+            }
+
+            case PublicKeyAlgorithmTags.RSA_ENCRYPT:
+            case PublicKeyAlgorithmTags.RSA_GENERAL:
+            case PublicKeyAlgorithmTags.RSA_SIGN:
+            {
+                RSAPublicBCPGKey rsaK = (RSAPublicBCPGKey)publicPk.getKey();
+                RSAPublicKeySpec rsaSpec = new RSAPublicKeySpec(rsaK.getModulus(), rsaK.getPublicExponent());
+                return implGeneratePublic("RSA", rsaSpec);
+            }
+
+            default:
+                throw new PGPException("unknown public key algorithm encountered");
+            }
+        }
+        catch (PGPException e)
+        {
+            throw e;
+        }
+        catch (Exception e)
+        {
+            throw new PGPException("exception constructing public key", e);
         }
-
-        return new PGPPrivateKey(pub.getKeyID(), pub.getPublicKeyPacket(), privPk);
     }
 
     private ECParameterSpec getECParameterSpec(ASN1ObjectIdentifier curveOid)
@@ -390,4 +353,210 @@ public class JcaPGPKeyConverter
 
         return params.getParameterSpec(ECParameterSpec.class);
     }
+
+    private BCPGKey getPrivateBCPGKey(PGPPublicKey pub, PrivateKey privKey)
+        throws PGPException
+    {
+        switch (pub.getAlgorithm())
+        {
+        case PublicKeyAlgorithmTags.DSA:
+        {
+            DSAPrivateKey dsK = (DSAPrivateKey)privKey;
+            return new DSASecretBCPGKey(dsK.getX());
+        }
+
+        case PublicKeyAlgorithmTags.ECDH:
+        {
+            if (privKey instanceof ECPrivateKey)
+            {
+                ECPrivateKey ecK = (ECPrivateKey)privKey;
+                return new ECSecretBCPGKey(ecK.getS());
+            }
+            else
+            {
+                PrivateKeyInfo pInfo = PrivateKeyInfo.getInstance(privKey.getEncoded());
+
+                try
+                {
+                    // 'reverse' because the native format for X25519 private keys is little-endian
+                    return new ECSecretBCPGKey(new BigInteger(1,
+                        Arrays.reverse(ASN1OctetString.getInstance(pInfo.parsePrivateKey()).getOctets())));
+                }
+                catch (IOException e)
+                {
+                    throw new PGPException(e.getMessage(), e);
+                }
+            }
+        }
+
+        case PublicKeyAlgorithmTags.ECDSA:
+        {
+            ECPrivateKey ecK = (ECPrivateKey)privKey;
+            return new ECSecretBCPGKey(ecK.getS());
+        }
+
+        case PublicKeyAlgorithmTags.EDDSA:
+        {
+            PrivateKeyInfo pInfo = PrivateKeyInfo.getInstance(privKey.getEncoded());
+
+            try
+            {
+                return new EdSecretBCPGKey(
+                    new BigInteger(1, ASN1OctetString.getInstance(pInfo.parsePrivateKey()).getOctets()));
+            }
+            catch (IOException e)
+            {
+                throw new PGPException(e.getMessage(), e);
+            }
+        }
+
+        case PublicKeyAlgorithmTags.ELGAMAL_ENCRYPT:
+        case PublicKeyAlgorithmTags.ELGAMAL_GENERAL:
+        {
+            DHPrivateKey esK = (DHPrivateKey)privKey;
+            return new ElGamalSecretBCPGKey(esK.getX());
+        }
+
+        case PublicKeyAlgorithmTags.RSA_ENCRYPT:
+        case PublicKeyAlgorithmTags.RSA_GENERAL:
+        case PublicKeyAlgorithmTags.RSA_SIGN:
+        {
+            RSAPrivateCrtKey rsK = (RSAPrivateCrtKey)privKey;
+            return new RSASecretBCPGKey(rsK.getPrivateExponent(), rsK.getPrimeP(), rsK.getPrimeQ());
+        }
+
+        default:
+            throw new PGPException("unknown key class");
+        }
+    }
+
+    private BCPGKey getPublicBCPGKey(int algorithm, PGPAlgorithmParameters algorithmParameters, PublicKey pubKey, Date time)
+        throws PGPException
+    {
+        if (pubKey instanceof RSAPublicKey)
+        {
+            RSAPublicKey rK = (RSAPublicKey)pubKey;
+            return new RSAPublicBCPGKey(rK.getModulus(), rK.getPublicExponent());
+        }
+        else if (pubKey instanceof DSAPublicKey)
+        {
+            DSAPublicKey dK = (DSAPublicKey)pubKey;
+            DSAParams dP = dK.getParams();
+            return new DSAPublicBCPGKey(dP.getP(), dP.getQ(), dP.getG(), dK.getY());
+        }
+        else if (pubKey instanceof DHPublicKey)
+        {
+            DHPublicKey eK = (DHPublicKey)pubKey;
+            DHParameterSpec eS = eK.getParams();
+            return new ElGamalPublicBCPGKey(eS.getP(), eS.getG(), eK.getY());
+        }
+        else if (pubKey instanceof ECPublicKey)
+        {
+            SubjectPublicKeyInfo keyInfo = SubjectPublicKeyInfo.getInstance(pubKey.getEncoded());
+
+            // TODO: should probably match curve by comparison as well
+            ASN1ObjectIdentifier  curveOid = ASN1ObjectIdentifier.getInstance(keyInfo.getAlgorithm().getParameters());
+
+            X9ECParameters params = ECNamedCurveTable.getByOID(curveOid);
+
+            ASN1OctetString key = new DEROctetString(keyInfo.getPublicKeyData().getBytes());
+            X9ECPoint derQ = new X9ECPoint(params.getCurve(), key);
+
+            if (algorithm == PGPPublicKey.ECDH)
+            {
+                PGPKdfParameters kdfParams = implGetKdfParameters(algorithmParameters);
+
+                return new ECDHPublicBCPGKey(curveOid, derQ.getPoint(), kdfParams.getHashAlgorithm(),
+                    kdfParams.getSymmetricWrapAlgorithm());
+            }
+            else if (algorithm == PGPPublicKey.ECDSA)
+            {
+                return new ECDSAPublicBCPGKey(curveOid, derQ.getPoint());
+            }
+            else
+            {
+                throw new PGPException("unknown EC algorithm");
+            }
+        }
+        else if (pubKey.getAlgorithm().regionMatches(true, 0, "ED2", 0, 3))
+        {
+            SubjectPublicKeyInfo pubInfo = SubjectPublicKeyInfo.getInstance(pubKey.getEncoded());
+            byte[] pointEnc = new byte[1 + ED25519_KEY_SIZE];
+
+            pointEnc[0] = 0x40;
+            System.arraycopy(pubInfo.getPublicKeyData().getBytes(), 0, pointEnc, 1, pointEnc.length - 1);
+
+            return new EdDSAPublicBCPGKey(GNUObjectIdentifiers.Ed25519, new BigInteger(1, pointEnc));
+        }
+        else if (pubKey.getAlgorithm().regionMatches(true, 0, "X2", 0, 2))
+        {
+            SubjectPublicKeyInfo pubInfo = SubjectPublicKeyInfo.getInstance(pubKey.getEncoded());
+            byte[] pointEnc = new byte[1 + X25519_KEY_SIZE];
+
+            pointEnc[0] = 0x40;
+            System.arraycopy(pubInfo.getPublicKeyData().getBytes(), 0, pointEnc, 1, pointEnc.length - 1);
+
+            PGPKdfParameters kdfParams = implGetKdfParameters(algorithmParameters);
+
+            return new ECDHPublicBCPGKey(CryptlibObjectIdentifiers.curvey25519, new BigInteger(1, pointEnc),
+                kdfParams.getHashAlgorithm(), kdfParams.getSymmetricWrapAlgorithm());
+        }
+        else
+        {
+            throw new PGPException("unknown key class");
+        }
+    }
+
+    private PrivateKey implGeneratePrivate(String keyAlgorithm, KeySpec keySpec)
+        throws GeneralSecurityException, PGPException
+    {
+        KeyFactory keyFactory = helper.createKeyFactory(keyAlgorithm);
+        return keyFactory.generatePrivate(keySpec);
+    }
+
+    private PublicKey implGeneratePublic(String keyAlgorithm, KeySpec keySpec)
+        throws GeneralSecurityException, PGPException
+    {
+        KeyFactory keyFactory = helper.createKeyFactory(keyAlgorithm);
+        return keyFactory.generatePublic(keySpec);
+    }
+
+    private PGPKdfParameters implGetKdfParameters(PGPAlgorithmParameters algorithmParameters)
+    {
+        return null == algorithmParameters ? DEFAULT_KDF_PARAMETERS : (PGPKdfParameters)algorithmParameters;
+    }
+
+    private PrivateKey implGetPrivateKeyEC(String keyAlgorithm, ECPublicBCPGKey ecPub, ECSecretBCPGKey ecPriv)
+        throws GeneralSecurityException, PGPException
+    {
+        ECPrivateKeySpec ecPrivSpec = new ECPrivateKeySpec(ecPriv.getX(), getECParameterSpec(ecPub.getCurveOID()));
+        return implGeneratePrivate(keyAlgorithm, ecPrivSpec);
+    }
+
+    private PrivateKey implGetPrivateKeyPKCS8(String keyAlgorithm, PrivateKeyInfo privateKeyInfo)
+        throws GeneralSecurityException, IOException, PGPException
+    {
+        PKCS8EncodedKeySpec pkcs8Spec = new PKCS8EncodedKeySpec(privateKeyInfo.getEncoded());
+        return implGeneratePrivate(keyAlgorithm, pkcs8Spec);
+    }
+
+    private PublicKey implGetPublicKeyEC(String keyAlgorithm, ECPublicBCPGKey ecPub) throws GeneralSecurityException, IOException, PGPException
+    {
+        ASN1ObjectIdentifier curveOID = ecPub.getCurveOID();
+        X9ECParameters x9Params = JcaJcePGPUtil.getX9Parameters(curveOID);
+        ECPoint ecPubPoint = JcaJcePGPUtil.decodePoint(ecPub.getEncodedPoint(), x9Params.getCurve());
+        ECPublicKeySpec ecPubSpec = new ECPublicKeySpec(
+            new java.security.spec.ECPoint(
+                ecPubPoint.getAffineXCoord().toBigInteger(),
+                ecPubPoint.getAffineYCoord().toBigInteger()),
+            getECParameterSpec(curveOID, x9Params));
+        return implGeneratePublic(keyAlgorithm, ecPubSpec);
+    }
+
+    private PublicKey implGetPublicKeyX509(String keyAlgorithm, SubjectPublicKeyInfo subjectPublicKeyInfo)
+        throws GeneralSecurityException, IOException, PGPException
+    {
+        X509EncodedKeySpec x509Spec = new X509EncodedKeySpec(subjectPublicKeyInfo.getEncoded());
+        return implGeneratePublic(keyAlgorithm, x509Spec);
+    }
 }
