@@ -89,7 +89,8 @@ public class PGPECMessageTest
     private void testMasterKey()
         throws Exception
     {
-        PGPSecretKey key = PGPSecretKey.parseSecretKeyFromSExpr(new ByteArrayInputStream(sExprKeyMaster), new JcePBEProtectionRemoverFactory("test".toCharArray()), new JcaKeyFingerprintCalculator());
+        SExprParser parser = new SExprParser(null);
+        PGPSecretKey key = parser.parseSecretKey(new ByteArrayInputStream(sExprKeyMaster), new JcePBEProtectionRemoverFactory("test".toCharArray()), new JcaKeyFingerprintCalculator());
 
         PGPSignatureGenerator signGen = new PGPSignatureGenerator(new JcaPGPContentSignerBuilder(PGPPublicKey.ECDSA, HashAlgorithmTags.SHA256).setProvider("BC"));
 
@@ -114,6 +115,7 @@ public class PGPECMessageTest
     private void testEncMessage()
         throws Exception
     {
+        SExprParser parser = new SExprParser(null);
         PGPObjectFactory pgpFact = new JcaPGPObjectFactory(encMessage);
 
         PGPEncryptedDataList encList = (PGPEncryptedDataList)pgpFact.nextObject();
@@ -122,7 +124,7 @@ public class PGPECMessageTest
 
         PGPPublicKey publicKey = new JcaPGPPublicKeyRing(testPubKey).getPublicKey(encP.getKeyID());
 
-        PGPSecretKey secretKey = PGPSecretKey.parseSecretKeyFromSExpr(new ByteArrayInputStream(sExprKeySub), new JcePBEProtectionRemoverFactory("test".toCharArray()), publicKey);
+        PGPSecretKey secretKey = parser.parseSecretKey(new ByteArrayInputStream(sExprKeySub), new JcePBEProtectionRemoverFactory("test".toCharArray()), publicKey);
 
         InputStream clear = encP.getDataStream(new JcePublicKeyDataDecryptorFactoryBuilder().setProvider("BC").build(secretKey.extractPrivateKey(null)));
 
@@ -226,6 +228,7 @@ public class PGPECMessageTest
     private void testBCSignedEncMessage()
         throws Exception
     {
+        SExprParser parser = new SExprParser(null);
         PGPObjectFactory pgpFact = new JcaPGPObjectFactory(signedEncMessage);
 
         PGPEncryptedDataList encList = (PGPEncryptedDataList)pgpFact.nextObject();
@@ -236,7 +239,7 @@ public class PGPECMessageTest
 
         PGPPublicKey publicKey = publicKeyRing.getPublicKey(encP.getKeyID());
 
-        PGPSecretKey secretKey = PGPSecretKey.parseSecretKeyFromSExpr(new ByteArrayInputStream(sExprKeySub), new JcePBEProtectionRemoverFactory("test".toCharArray()), publicKey);
+        PGPSecretKey secretKey = parser.parseSecretKey(new ByteArrayInputStream(sExprKeySub), new JcePBEProtectionRemoverFactory("test".toCharArray()), publicKey);
 
         InputStream clear = encP.getDataStream(new JcePublicKeyDataDecryptorFactoryBuilder().setProvider("BC").build(secretKey.extractPrivateKey(null)));
 
