@@ -2,8 +2,8 @@ package org.bouncycastle.openpgp.test;
 
 import java.security.Security;
 
+import org.bouncycastle.util.test.SimpleTest;
 import org.bouncycastle.util.test.Test;
-import org.bouncycastle.util.test.TestResult;
 
 public class RegressionTest
 {
@@ -29,25 +29,16 @@ public class RegressionTest
         new PGPECDHTest(),
         new PGPECMessageTest(),
         new PGPParsingTest(),
+        new PGPEdDSATest(),
         new SExprTest(),
         new ArmoredInputStreamTest(),
         new PGPUtilTest()
     };
 
-    public static void main(
-        String[] args)
+    public static void main(String[] args)
     {
         Security.addProvider(new org.bouncycastle.jce.provider.BouncyCastleProvider());
 
-        for (int i = 0; i != tests.length; i++)
-        {
-            TestResult result = tests[i].perform();
-            System.out.println(result);
-            if (result.getException() != null)
-            {
-                result.getException().printStackTrace();
-            }
-        }
+        SimpleTest.runTests(tests);
     }
 }
-
