@@ -11,11 +11,21 @@ import java.security.SignatureException;
 import java.security.spec.AlgorithmParameterSpec;
 import java.security.spec.MGF1ParameterSpec;
 import java.security.spec.PSSParameterSpec;
+import java.util.List;
 
+import org.bouncycastle.asn1.ASN1EncodableVector;
+import org.bouncycastle.asn1.ASN1Encoding;
 import org.bouncycastle.asn1.ASN1Integer;
+import org.bouncycastle.asn1.ASN1Sequence;
+import org.bouncycastle.asn1.DERBitString;
+import org.bouncycastle.asn1.DERSequence;
+import org.bouncycastle.asn1.misc.MiscObjectIdentifiers;
 import org.bouncycastle.asn1.pkcs.PKCSObjectIdentifiers;
 import org.bouncycastle.asn1.pkcs.RSASSAPSSparams;
 import org.bouncycastle.asn1.x509.AlgorithmIdentifier;
+import org.bouncycastle.jcajce.CompositePrivateKey;
+import org.bouncycastle.jcajce.io.OutputStreamFactory;
+import org.bouncycastle.jcajce.spec.CompositeAlgorithmSpec;
 import org.bouncycastle.jcajce.util.DefaultJcaJceHelper;
 import org.bouncycastle.jcajce.util.NamedJcaJceHelper;
 import org.bouncycastle.jcajce.util.ProviderJcaJceHelper;
@@ -24,8 +34,9 @@ import org.bouncycastle.operator.DefaultDigestAlgorithmIdentifierFinder;
 import org.bouncycastle.operator.DefaultSignatureAlgorithmIdentifierFinder;
 import org.bouncycastle.operator.DigestAlgorithmIdentifierFinder;
 import org.bouncycastle.operator.OperatorCreationException;
-import org.bouncycastle.operator.OperatorStreamException;
 import org.bouncycastle.operator.RuntimeOperatorException;
+import org.bouncycastle.operator.SignatureAlgorithmIdentifierFinder;
+import org.bouncycastle.util.io.TeeOutputStream;
 
 public class JcaContentSignerBuilder
 {
@@ -54,6 +65,14 @@ public class JcaContentSignerBuilder
             this.sigAlgId = new AlgorithmIdentifier(
                                     PKCSObjectIdentifiers.id_RSASSA_PSS, createPSSParams(pssSpec));
         }
+        else if (sigParamSpec instanceof CompositeAlgorithmSpec)
+        {
+            CompositeAlgorithmSpec compSpec = (CompositeAlgorithmSpec)sigParamSpec;
+
+            this.sigAlgSpec = compSpec;
+            this.sigAlgId = new AlgorithmIdentifier(
+                                    MiscObjectIdentifiers.id_alg_composite, createCompParams(compSpec));
+        }
         else
         {
             throw new IllegalArgumentException("unknown sigParamSpec: "
@@ -85,6 +104,11 @@ public class JcaContentSignerBuilder
     public ContentSigner build(PrivateKey privateKey)
         throws OperatorCreationException
     {
+        if (privateKey instanceof CompositePrivateKey)
+        {
+            return buildComposite((CompositePrivateKey)privateKey);
+        }
+        
         try
         {
             final Signature sig = helper.createSignature(sigAlgId);
@@ -101,7 +125,7 @@ public class JcaContentSignerBuilder
 
             return new ContentSigner()
             {
-                private SignatureOutputStream stream = new SignatureOutputStream(sig);
+                private OutputStream stream = OutputStreamFactory.createStream(sig);
 
                 public AlgorithmIdentifier getAlgorithmIdentifier()
                 {
@@ -117,7 +141,7 @@ public class JcaContentSignerBuilder
                 {
                     try
                     {
-                        return stream.getSignature();
+                        return sig.sign();
                     }
                     catch (SignatureException e)
                     {
@@ -132,59 +156,78 @@ public class JcaContentSignerBuilder
         }
     }
 
-    private class SignatureOutputStream
-        extends OutputStream
+    private ContentSigner buildComposite(CompositePrivateKey privateKey)
+        throws OperatorCreationException
     {
-        private Signature sig;
-
-        SignatureOutputStream(Signature sig)
+        try
         {
-            this.sig = sig;
+            List<PrivateKey> privateKeys = privateKey.getPrivateKeys();
+            final ASN1Sequence sigAlgIds = ASN1Sequence.getInstance(sigAlgId.getParameters());
+            final Signature[] sigs = new Signature[sigAlgIds.size()];
+
+            for (int i = 0; i != sigAlgIds.size(); i++)
+            {
+                sigs[i] = helper.createSignature(AlgorithmIdentifier.getInstance(sigAlgIds.getObjectAt(i)));
+
+                if (random != null)
+                {
+                    sigs[i].initSign(privateKeys.get(i), random);
+                }
+                else
+                {
+                    sigs[i].initSign(privateKeys.get(i));
+                }
+            }
+
+            OutputStream sStream = OutputStreamFactory.createStream(sigs[0]);
+            for (int i = 1; i != sigs.length; i++)
+            {
+                sStream = new TeeOutputStream(sStream, OutputStreamFactory.createStream(sigs[i]));
+            }
+
+            final OutputStream sigStream = sStream;
+
+            return new ContentSigner()
+            {
+                OutputStream stream = sigStream;
+
+                public AlgorithmIdentifier getAlgorithmIdentifier()
+                {
+                    return sigAlgId;
+                }
+
+                public OutputStream getOutputStream()
+                {
+                    return stream;
+                }
+
+                public byte[] getSignature()
+                {
+                    try
+                    {
+                        ASN1EncodableVector sigV = new ASN1EncodableVector();
+
+                        for (int i = 0; i != sigs.length; i++)
+                        {
+                            sigV.add(new DERBitString(sigs[i].sign()));
+                        }
+
+                        return new DERSequence(sigV).getEncoded(ASN1Encoding.DER);
+                    }
+                    catch (IOException e)
+                    {
+                        throw new RuntimeOperatorException("exception encoding signature: " + e.getMessage(), e);
+                    }
+                    catch (SignatureException e)
+                    {
+                        throw new RuntimeOperatorException("exception obtaining signature: " + e.getMessage(), e);
+                    }
+                }
+            };
         }
-
-        public void write(byte[] bytes, int off, int len)
-            throws IOException
+        catch (GeneralSecurityException e)
         {
-            try
-            {
-                sig.update(bytes, off, len);
-            }
-            catch (SignatureException e)
-            {
-                throw new OperatorStreamException("exception in content signer: " + e.getMessage(), e);
-            }
-        }
-
-        public void write(byte[] bytes)
-            throws IOException
-        {
-            try
-            {
-                sig.update(bytes);
-            }
-            catch (SignatureException e)
-            {
-                throw new OperatorStreamException("exception in content signer: " + e.getMessage(), e);
-            }
-        }
-
-        public void write(int b)
-            throws IOException
-        {
-            try
-            {
-                sig.update((byte)b);
-            }
-            catch (SignatureException e)
-            {
-                throw new OperatorStreamException("exception in content signer: " + e.getMessage(), e);
-            }
-        }
-
-        byte[] getSignature()
-            throws SignatureException
-        {
-            return sig.sign();
+            throw new OperatorCreationException("cannot create signer: " + e.getMessage(), e);
         }
     }
 
@@ -200,4 +243,32 @@ public class JcaContentSignerBuilder
             new ASN1Integer(pssSpec.getSaltLength()),
             new ASN1Integer(pssSpec.getTrailerField()));
     }
+
+    private static ASN1Sequence createCompParams(CompositeAlgorithmSpec compSpec)
+    {
+        SignatureAlgorithmIdentifierFinder algFinder = new DefaultSignatureAlgorithmIdentifierFinder();
+        ASN1EncodableVector v = new ASN1EncodableVector();
+
+        List<String> algorithmNames = compSpec.getAlgorithmNames();
+        List<AlgorithmParameterSpec> algorithmSpecs = compSpec.getParameterSpecs();
+
+        for (int i = 0; i != algorithmNames.size(); i++)
+        {
+            AlgorithmParameterSpec sigSpec = algorithmSpecs.get(i);
+            if (sigSpec == null)
+            {
+                v.add(algFinder.find(algorithmNames.get(i)));
+            }
+            else if (sigSpec instanceof PSSParameterSpec)
+            {
+                v.add(createPSSParams((PSSParameterSpec)sigSpec));
+            }
+            else
+            {
+                throw new IllegalArgumentException("unrecognized parameterSpec");
+            }
+        }
+
+        return new DERSequence(v);
+    }
 }
