@@ -1,6 +1,5 @@
 package org.bouncycastle.operator.jcajce;
 
-import java.io.IOException;
 import java.io.OutputStream;
 import java.security.GeneralSecurityException;
 import java.security.Provider;
@@ -10,20 +9,26 @@ import java.security.SignatureException;
 import java.security.cert.CertificateEncodingException;
 import java.security.cert.CertificateException;
 import java.security.cert.X509Certificate;
+import java.util.List;
 
+import org.bouncycastle.asn1.ASN1Sequence;
+import org.bouncycastle.asn1.DERBitString;
+import org.bouncycastle.asn1.misc.MiscObjectIdentifiers;
 import org.bouncycastle.asn1.x509.AlgorithmIdentifier;
 import org.bouncycastle.asn1.x509.SubjectPublicKeyInfo;
 import org.bouncycastle.cert.X509CertificateHolder;
 import org.bouncycastle.cert.jcajce.JcaX509CertificateHolder;
+import org.bouncycastle.jcajce.CompositePublicKey;
+import org.bouncycastle.jcajce.io.OutputStreamFactory;
 import org.bouncycastle.jcajce.util.DefaultJcaJceHelper;
 import org.bouncycastle.jcajce.util.NamedJcaJceHelper;
 import org.bouncycastle.jcajce.util.ProviderJcaJceHelper;
 import org.bouncycastle.operator.ContentVerifier;
 import org.bouncycastle.operator.ContentVerifierProvider;
 import org.bouncycastle.operator.OperatorCreationException;
-import org.bouncycastle.operator.OperatorStreamException;
 import org.bouncycastle.operator.RawContentVerifier;
 import org.bouncycastle.operator.RuntimeOperatorException;
+import org.bouncycastle.util.io.TeeOutputStream;
 
 public class JcaContentVerifierProviderBuilder
 {
@@ -69,8 +74,6 @@ public class JcaContentVerifierProviderBuilder
 
         return new ContentVerifierProvider()
         {
-            private SignatureOutputStream stream;
-
             public boolean hasAssociatedCertificate()
             {
                 return true;
@@ -84,28 +87,34 @@ public class JcaContentVerifierProviderBuilder
             public ContentVerifier get(AlgorithmIdentifier algorithm)
                 throws OperatorCreationException
             {
-                try
+                if (algorithm.getAlgorithm().equals(MiscObjectIdentifiers.id_alg_composite))
                 {
-                    Signature sig = helper.createSignature(algorithm);
-
-                    sig.initVerify(certificate.getPublicKey());
-
-                    stream = new SignatureOutputStream(sig);
-                }
-                catch (GeneralSecurityException e)
-                {
-                    throw new OperatorCreationException("exception on setup: " + e, e);
-                }
-
-                Signature rawSig = createRawSig(algorithm, certificate.getPublicKey());
-
-                if (rawSig != null)
-                {
-                    return new RawSigVerifier(algorithm, stream, rawSig);
+                    return createCompositeVerifier(algorithm, certificate.getPublicKey());
                 }
                 else
                 {
-                    return new SigVerifier(algorithm, stream);
+                    Signature sig;
+                    try
+                    {
+                        sig = helper.createSignature(algorithm);
+
+                        sig.initVerify(certificate.getPublicKey());
+                    }
+                    catch (GeneralSecurityException e)
+                    {
+                        throw new OperatorCreationException("exception on setup: " + e, e);
+                    }
+
+                    Signature rawSig = createRawSig(algorithm, certificate.getPublicKey());
+
+                    if (rawSig != null)
+                    {
+                        return new RawSigVerifier(algorithm, sig, rawSig);
+                    }
+                    else
+                    {
+                        return new SigVerifier(algorithm, sig);
+                    }
                 }
             }
         };
@@ -129,17 +138,54 @@ public class JcaContentVerifierProviderBuilder
             public ContentVerifier get(AlgorithmIdentifier algorithm)
                 throws OperatorCreationException
             {
-                SignatureOutputStream stream = createSignatureStream(algorithm, publicKey);
-
-                Signature rawSig = createRawSig(algorithm, publicKey);
-
-                if (rawSig != null)
+                if (algorithm.getAlgorithm().equals(MiscObjectIdentifiers.id_alg_composite))
                 {
-                    return new RawSigVerifier(algorithm, stream, rawSig);
+                    return createCompositeVerifier(algorithm, publicKey);
+                }
+
+                if (publicKey instanceof CompositePublicKey)
+                {
+                    List<PublicKey> keys = ((CompositePublicKey)publicKey).getPublicKeys();
+
+                    for (int i = 0; i != keys.size(); i++)
+                    {
+                        try
+                        {
+                            Signature sig = createSignature(algorithm, (PublicKey)keys.get(i));
+
+                            Signature rawSig = createRawSig(algorithm, (PublicKey)keys.get(i));
+
+                            if (rawSig != null)
+                            {
+                                return new RawSigVerifier(algorithm, sig, rawSig);
+                            }
+                            else
+                            {
+                                return new SigVerifier(algorithm, sig);
+                            }
+                        }
+                        catch (OperatorCreationException e)
+                        {
+                            // skip incorrect keys
+                        }
+                    }
+
+                    throw new OperatorCreationException("no matching algorithm found for key");
                 }
                 else
                 {
-                    return new SigVerifier(algorithm, stream);
+                    Signature sig = createSignature(algorithm, publicKey);
+
+                    Signature rawSig = createRawSig(algorithm, publicKey);
+
+                    if (rawSig != null)
+                    {
+                        return new RawSigVerifier(algorithm, sig, rawSig);
+                    }
+                    else
+                    {
+                        return new SigVerifier(algorithm, sig);
+                    }
                 }
             }
         };
@@ -151,7 +197,52 @@ public class JcaContentVerifierProviderBuilder
         return this.build(helper.convertPublicKey(publicKey));
     }
 
-    private SignatureOutputStream createSignatureStream(AlgorithmIdentifier algorithm, PublicKey publicKey)
+    private ContentVerifier createCompositeVerifier(AlgorithmIdentifier compAlgId, PublicKey publicKey)
+        throws OperatorCreationException
+    {
+        if (publicKey instanceof CompositePublicKey)
+        {
+            List<PublicKey> pubKeys = ((CompositePublicKey)publicKey).getPublicKeys();
+            ASN1Sequence keySeq = ASN1Sequence.getInstance(compAlgId.getParameters());
+            Signature[] sigs = new Signature[keySeq.size()];
+            for (int i = 0; i != keySeq.size(); i++)
+            {
+                AlgorithmIdentifier sigAlg = AlgorithmIdentifier.getInstance(keySeq.getObjectAt(i));
+                if (pubKeys.get(i) != null)
+                {
+                    sigs[i] = createSignature(sigAlg, (PublicKey)pubKeys.get(i));
+                }
+                else
+                {
+                    sigs[i] = null;
+                }
+            }
+
+            return new CompositeVerifier(sigs);
+        }
+        else
+        {
+            ASN1Sequence keySeq = ASN1Sequence.getInstance(compAlgId.getParameters());
+            Signature[] sigs = new Signature[keySeq.size()];
+            for (int i = 0; i != keySeq.size(); i++)
+            {
+                AlgorithmIdentifier sigAlg = AlgorithmIdentifier.getInstance(keySeq.getObjectAt(i));
+                try
+                {
+                    sigs[i] = createSignature(sigAlg, publicKey);
+                }
+                catch (Exception e)
+                {
+                    sigs[i] = null;
+                    // continue
+                }
+            }
+
+            return new CompositeVerifier(sigs);
+        }
+    }
+
+    private Signature createSignature(AlgorithmIdentifier algorithm, PublicKey publicKey)
         throws OperatorCreationException
     {
         try
@@ -160,7 +251,7 @@ public class JcaContentVerifierProviderBuilder
 
             sig.initVerify(publicKey);
 
-            return new SignatureOutputStream(sig);
+            return sig;
         }
         catch (GeneralSecurityException e)
         {
@@ -190,14 +281,16 @@ public class JcaContentVerifierProviderBuilder
     private class SigVerifier
         implements ContentVerifier
     {
-        private AlgorithmIdentifier algorithm;
+        private final AlgorithmIdentifier algorithm;
+        private final Signature signature;
 
-        protected SignatureOutputStream stream;
+        protected final OutputStream stream;
 
-        SigVerifier(AlgorithmIdentifier algorithm, SignatureOutputStream stream)
+        SigVerifier(AlgorithmIdentifier algorithm, Signature signature)
         {
             this.algorithm = algorithm;
-            this.stream = stream;
+            this.signature = signature;
+            this.stream = OutputStreamFactory.createStream(signature);
         }
 
         public AlgorithmIdentifier getAlgorithmIdentifier()
@@ -219,7 +312,7 @@ public class JcaContentVerifierProviderBuilder
         {
             try
             {
-                return stream.verify(expected);
+                return signature.verify(expected);
             }
             catch (SignatureException e)
             {
@@ -234,9 +327,9 @@ public class JcaContentVerifierProviderBuilder
     {
         private Signature rawSignature;
 
-        RawSigVerifier(AlgorithmIdentifier algorithm, SignatureOutputStream stream, Signature rawSignature)
+        RawSigVerifier(AlgorithmIdentifier algorithm, Signature standardSig, Signature rawSignature)
         {
-            super(algorithm, stream);
+            super(algorithm, standardSig);
             this.rawSignature = rawSignature;
         }
 
@@ -279,7 +372,7 @@ public class JcaContentVerifierProviderBuilder
                 // standard signature will not be freed if verify is not called on it.
                 try
                 {
-                    stream.verify(expected);
+                    rawSignature.verify(expected);
                 }
                 catch (Exception e)
                 {
@@ -289,59 +382,69 @@ public class JcaContentVerifierProviderBuilder
         }
     }
 
-    private class SignatureOutputStream
-        extends OutputStream
+    private class CompositeVerifier
+        implements ContentVerifier
     {
-        private Signature sig;
+        private Signature[] sigs;
+        private OutputStream stream;
 
-        SignatureOutputStream(Signature sig)
+        public CompositeVerifier(Signature[] sigs)
+            throws OperatorCreationException
         {
-            this.sig = sig;
+            this.sigs = sigs;
+
+            int start = 0;
+            while (start < sigs.length && sigs[start] == null)
+            {
+                start++;
+            }
+
+            if (start == sigs.length)
+            {
+                throw new OperatorCreationException("no matching signature found in composite");
+            }
+            this.stream = OutputStreamFactory.createStream(sigs[start]);
+            for (int i = start + 1; i != sigs.length; i++)
+            {
+                if (sigs[i] != null)
+                {
+                    this.stream = new TeeOutputStream(this.stream, OutputStreamFactory.createStream(sigs[i]));
+                }
+            }
         }
 
-        public void write(byte[] bytes, int off, int len)
-            throws IOException
+        public AlgorithmIdentifier getAlgorithmIdentifier()
+        {
+            return new AlgorithmIdentifier(MiscObjectIdentifiers.id_alg_composite);
+        }
+
+        public OutputStream getOutputStream()
+        {
+            return stream;
+        }
+
+        public boolean verify(byte[] expected)
         {
             try
             {
-                sig.update(bytes, off, len);
+                ASN1Sequence sigSeq = ASN1Sequence.getInstance(expected);
+                boolean failed = false;
+                for (int i = 0; i != sigSeq.size(); i++)
+                {
+                    if (sigs[i] != null)
+                    {
+                        if (!sigs[i].verify(DERBitString.getInstance(sigSeq.getObjectAt(i)).getBytes()))
+                        {
+                            failed = true;
+                        }
+                    }
+                }
+                return !failed;
             }
             catch (SignatureException e)
             {
-                throw new OperatorStreamException("exception in content signer: " + e.getMessage(), e);
+                throw new RuntimeOperatorException("exception obtaining signature: " + e.getMessage(), e);
             }
         }
-
-        public void write(byte[] bytes)
-            throws IOException
-        {
-            try
-            {
-                sig.update(bytes);
-            }
-            catch (SignatureException e)
-            {
-                throw new OperatorStreamException("exception in content signer: " + e.getMessage(), e);
-            }
-        }
-
-        public void write(int b)
-            throws IOException
-        {
-            try
-            {
-                sig.update((byte)b);
-            }
-            catch (SignatureException e)
-            {
-                throw new OperatorStreamException("exception in content signer: " + e.getMessage(), e);
-            }
-        }
-
-        boolean verify(byte[] expected)
-            throws SignatureException
-        {
-            return sig.verify(expected);
-        }
     }
 }
\ No newline at end of file
