@@ -88,7 +88,12 @@ public class TimeStampTokenGenerator
     public static final int R_TENTHS_OF_SECONDS = 1;
 
     /**
-     * Create time-stamps with a resolution of 1 microsecond.
+     * Create time-stamps with a resolution of 1 hundredth of a second.
+     */
+    public static final int R_HUNDREDTHS_OF_SECONDS = 2;
+
+    /**
+     * @deprecated use R_HUNDREDTHS_OF_SECONDS - this field will be deleted!!
      */
     public static final int R_MICROSECONDS = 2;
 
@@ -504,7 +509,7 @@ public class TimeStampTokenGenerator
                 sBuild.delete(dotIndex + 2, sBuild.length());
             }
             break;
-        case R_MICROSECONDS:
+        case R_HUNDREDTHS_OF_SECONDS:
             if (sBuild.length() > dotIndex + 3)
             {
                 sBuild.delete(dotIndex + 3, sBuild.length());
