@@ -1,6 +1,6 @@
 package org.bouncycastle.cert.selector.jcajce;
 
-import org.bouncycastle.jce.cert.X509CertSelector;
+import java.security.cert.X509CertSelector;
 
 import org.bouncycastle.asn1.ASN1OctetString;
 import org.bouncycastle.asn1.x500.X500Name;
