@@ -23,7 +23,9 @@ import javax.crypto.KeyGenerator;
 import javax.crypto.Mac;
 import javax.crypto.NoSuchPaddingException;
 import javax.crypto.SecretKey;
+import javax.crypto.SecretKeyFactory;
 import javax.crypto.spec.IvParameterSpec;
+import javax.crypto.spec.PBEKeySpec;
 import javax.crypto.spec.RC2ParameterSpec;
 import javax.crypto.spec.SecretKeySpec;
 
@@ -33,17 +35,20 @@ import org.bouncycastle.asn1.ASN1ObjectIdentifier;
 import org.bouncycastle.asn1.ASN1OctetString;
 import org.bouncycastle.asn1.DERNull;
 import org.bouncycastle.asn1.DEROctetString;
+import org.bouncycastle.asn1.pkcs.PBKDF2Params;
 import org.bouncycastle.asn1.pkcs.PKCSObjectIdentifiers;
 import org.bouncycastle.asn1.pkcs.RC2CBCParameter;
 import org.bouncycastle.asn1.x509.AlgorithmIdentifier;
 import org.bouncycastle.cms.CMSAlgorithm;
 import org.bouncycastle.cms.CMSEnvelopedDataGenerator;
 import org.bouncycastle.cms.CMSException;
+import org.bouncycastle.cms.PasswordRecipient;
 import org.bouncycastle.operator.DefaultSecretKeySizeProvider;
 import org.bouncycastle.operator.GenericKey;
 import org.bouncycastle.operator.SecretKeySizeProvider;
 import org.bouncycastle.operator.SymmetricKeyUnwrapper;
 import org.bouncycastle.operator.jcajce.JceAsymmetricKeyUnwrapper;
+import org.bouncycastle.operator.jcajce.JceKTSKeyUnwrapper;
 
 public class EnvelopedDataHelper
 {
@@ -302,12 +307,12 @@ public class EnvelopedDataHelper
         }
         catch (Exception e)
         {
-            throw new CMSException("cannot create key pair generator: " + e.getMessage(), e);
+            throw new CMSException("cannot create key agreement: " + e.getMessage(), e);
         }
     }
 
     AlgorithmParameterGenerator createAlgorithmParameterGenerator(ASN1ObjectIdentifier algorithm)
-        throws CMSException
+        throws GeneralSecurityException
     {
         String algorithmName = (String)BASE_CIPHER_NAMES.get(algorithm);
 
@@ -329,7 +334,7 @@ public class EnvelopedDataHelper
         }
         catch (Exception e)
         {
-            throw new CMSException("cannot create key pair generator: " + e.getMessage(), e);
+            throw new GeneralSecurityException(e.toString());
         }
     }
 
@@ -622,6 +627,11 @@ public class EnvelopedDataHelper
         return helper.createAsymmetricUnwrapper(keyEncryptionAlgorithm, keyEncryptionKey);
     }
 
+    public JceKTSKeyUnwrapper createAsymmetricUnwrapper(AlgorithmIdentifier keyEncryptionAlgorithm, PrivateKey keyEncryptionKey, byte[] partyUInfo, byte[] partyVInfo)
+    {
+        return helper.createAsymmetricUnwrapper(keyEncryptionAlgorithm, keyEncryptionKey, partyUInfo, partyVInfo);
+    }
+
     public SymmetricKeyUnwrapper createSymmetricUnwrapper(AlgorithmIdentifier keyEncryptionAlgorithm, SecretKey keyEncryptionKey)
     {
         return helper.createSymmetricUnwrapper(keyEncryptionAlgorithm, keyEncryptionKey);
@@ -662,6 +672,40 @@ public class EnvelopedDataHelper
         throw new IllegalStateException("unknown parameter spec: " + paramSpec);
     }
 
+    SecretKeyFactory createSecretKeyFactory(String keyFactoryAlgorithm)
+        throws NoSuchProviderException, NoSuchAlgorithmException
+    {
+        return helper.createSecretKeyFactory(keyFactoryAlgorithm);
+    }
+
+    byte[] calculateDerivedKey(int schemeID, char[] password, AlgorithmIdentifier derivationAlgorithm, int keySize)
+        throws CMSException
+    {
+        PBKDF2Params params = PBKDF2Params.getInstance(derivationAlgorithm.getParameters());
+
+        try
+        {
+            SecretKeyFactory keyFact;
+
+            if (schemeID == PasswordRecipient.PKCS5_SCHEME2)
+            {
+                keyFact = helper.createSecretKeyFactory("PBKDF2with8BIT");
+            }
+            else
+            {
+                keyFact = helper.createSecretKeyFactory("PBKDF2");
+            }
+
+            SecretKey key = keyFact.generateSecret(new PBEKeySpec(password, params.getSalt(), params.getIterationCount().intValue(), keySize));
+
+            return key.getEncoded();
+        }
+        catch (Exception e)
+        {
+             throw new CMSException("Unable to calculate dervied key from password: " + e.getMessage(), e);
+        }
+    }
+
     static interface JCECallback
     {
         Object doInJCE()
