@@ -90,7 +90,7 @@ public class TimeStampTokenGenerator
     /**
      * Create time-stamps with a resolution of 1 microsecond.
      */
-    public static final int R_MICROSECONDS = 2;
+    public static final int R_HUNDREDTHS_OF_SECONDS = 2;
 
     /**
      * Create time-stamps with a resolution of 1 millisecond.
@@ -505,13 +505,13 @@ public class TimeStampTokenGenerator
         case R_TENTHS_OF_SECONDS:
             if (sBuild.length() > dotIndex + 2)
             {
-                sBuild = new StringBuffer(sBuild.toString().substring(0, dotIndex + 2));
+                sBuild.delete(dotIndex + 2, sBuild.length());
             }
             break;
-        case R_MICROSECONDS:
+        case R_HUNDREDTHS_OF_SECONDS:
             if (sBuild.length() > dotIndex + 3)
             {
-                sBuild = new StringBuffer(sBuild.toString().substring(0, dotIndex + 3));
+                sBuild.delete(dotIndex + 3, sBuild.length());
             }
             break;
         case R_MILLISECONDS:
@@ -524,12 +524,12 @@ public class TimeStampTokenGenerator
         // remove trailing zeros
         while (sBuild.charAt(sBuild.length() - 1) == '0')
         {
-            sBuild = new StringBuffer(sBuild.toString().substring(0, sBuild.length() - 1));
+            sBuild.deleteCharAt(sBuild.length() - 1);
         }
 
         if (sBuild.length() - 1 == dotIndex)
         {
-            sBuild = new StringBuffer(sBuild.toString().substring(0, sBuild.length() - 1));
+            sBuild.deleteCharAt(sBuild.length() - 1);
         }
 
         sBuild.append("Z");
