@@ -17,11 +17,8 @@ import java.util.Iterator;
 
 import javax.crypto.SecretKey;
 
-import junit.framework.Test;
-import junit.framework.TestCase;
-import junit.framework.TestSuite;
+import org.bouncycastle.asn1.ASN1Encoding;
 import org.bouncycastle.asn1.ASN1InputStream;
-import org.bouncycastle.asn1.DEROutputStream;
 import org.bouncycastle.asn1.DERSet;
 import org.bouncycastle.asn1.DERUTF8String;
 import org.bouncycastle.asn1.cms.Attribute;
@@ -53,6 +50,10 @@ import org.bouncycastle.jce.provider.BouncyCastleProvider;
 import org.bouncycastle.util.encoders.Base64;
 import org.bouncycastle.util.encoders.Hex;
 
+import junit.framework.Test;
+import junit.framework.TestCase;
+import junit.framework.TestSuite;
+
 public class NewEnvelopedDataStreamTest
     extends TestCase
 {
@@ -398,9 +399,7 @@ public class NewEnvelopedDataStreamTest
 
         bOut.reset();
 
-        DEROutputStream dOut = new DEROutputStream(bOut);
-
-        dOut.writeObject(aIn.readObject());
+        aIn.readObject().encodeTo(bOut, ASN1Encoding.DER);
 
         verifyData(bOut, CMSEnvelopedDataGenerator.AES128_CBC, data);
     }
