@@ -2,11 +2,13 @@ package org.bouncycastle.mozilla.test;
 
 import java.security.Security;
 
+import junit.extensions.TestSetup;
+import junit.framework.Test;
+import junit.framework.TestCase;
+import junit.framework.TestSuite;
 import org.bouncycastle.jce.provider.BouncyCastleProvider;
 import org.bouncycastle.util.test.SimpleTestResult;
 
-import junit.framework.*;
-
 public class AllTests
     extends TestCase
 {
@@ -38,6 +40,26 @@ public class AllTests
         
         suite.addTestSuite(AllTests.class);
         
-        return suite;
+        return new BCTestSetup(suite);
     }
+
+    static class BCTestSetup
+        extends TestSetup
+    {
+        public BCTestSetup(Test test)
+        {
+            super(test);
+        }
+
+        protected void setUp()
+        {
+
+        }
+
+        protected void tearDown()
+        {
+
+        }
+    }
+
 }
