@@ -3,6 +3,8 @@ module org.bouncycastle.provider
     requires java.sql;
     requires java.naming;
 
+    provides java.security.Provider with org.bouncycastle.jce.provider.BouncyCastleProvider,org.bouncycastle.pqc.jcajce.provider.BouncyCastlePQCProvider;
+
     opens org.bouncycastle.jcajce.provider.asymmetric.edec to java.base;
     opens org.bouncycastle.pqc.jcajce.provider.qtesla to java.base;
     opens org.bouncycastle.pqc.jcajce.provider.lms to java.base;
@@ -28,6 +30,7 @@ module org.bouncycastle.provider
     exports org.bouncycastle.asn1.gnu;
     exports org.bouncycastle.asn1.iana;
     exports org.bouncycastle.asn1.icao;
+    exports org.bouncycastle.asn1.isara;
     exports org.bouncycastle.asn1.isismtt;
     exports org.bouncycastle.asn1.isismtt.ocsp;
     exports org.bouncycastle.asn1.isismtt.x509;
@@ -108,6 +111,7 @@ module org.bouncycastle.provider
     exports org.bouncycastle.jcajce.provider.symmetric;
     exports org.bouncycastle.jcajce.provider.symmetric.util;
     exports org.bouncycastle.jcajce.provider.util;
+    exports org.bouncycastle.jcajce.interfaces;
     exports org.bouncycastle.jcajce.spec;
     exports org.bouncycastle.jcajce.util;
     exports org.bouncycastle.jce;
@@ -135,6 +139,7 @@ module org.bouncycastle.provider
     exports org.bouncycastle.pqc.crypto.mceliece;
     exports org.bouncycastle.pqc.crypto.newhope;
     exports org.bouncycastle.pqc.crypto.ntru;
+    exports org.bouncycastle.pqc.crypto.qtesla;
     exports org.bouncycastle.pqc.crypto.rainbow;
     exports org.bouncycastle.pqc.crypto.rainbow.util;
     exports org.bouncycastle.pqc.crypto.sphincs;
