@@ -1,6 +1,7 @@
 package org.bouncycastle.jcajce.provider.asymmetric.ecgost12;
 
 import java.math.BigInteger;
+import java.security.AlgorithmParameters;
 import java.security.InvalidKeyException;
 import java.security.PrivateKey;
 import java.security.PublicKey;
@@ -202,6 +203,11 @@ public class ECGOST2012SignatureSpi512
         throw new UnsupportedOperationException("engineSetParameter unsupported");
     }
 
+    protected AlgorithmParameters engineGetParameters()
+    {
+        return null;
+    }
+
     /**
      * @deprecated replaced with #engineSetParameter(java.security.spec.AlgorithmParameterSpec)
      */
