@@ -5,6 +5,8 @@ import java.security.spec.AlgorithmParameterSpec;
 import java.security.spec.InvalidKeySpecException;
 import java.security.spec.InvalidParameterSpecException;
 import java.security.spec.KeySpec;
+import java.util.HashMap;
+import java.util.Map;
 
 import javax.crypto.SecretKey;
 import javax.crypto.spec.PBEKeySpec;
@@ -14,9 +16,13 @@ import org.bouncycastle.asn1.ASN1Encoding;
 import org.bouncycastle.asn1.ASN1ObjectIdentifier;
 import org.bouncycastle.asn1.ASN1Primitive;
 import org.bouncycastle.asn1.cryptopro.CryptoProObjectIdentifiers;
+import org.bouncycastle.asn1.gm.GMObjectIdentifiers;
+import org.bouncycastle.asn1.nist.NISTObjectIdentifiers;
 import org.bouncycastle.asn1.pkcs.PBKDF2Params;
 import org.bouncycastle.asn1.pkcs.PKCSObjectIdentifiers;
 import org.bouncycastle.crypto.CipherParameters;
+import org.bouncycastle.crypto.PasswordConverter;
+import org.bouncycastle.jcajce.PBKDF2Key;
 import org.bouncycastle.jcajce.provider.config.ConfigurableProvider;
 import org.bouncycastle.jcajce.provider.symmetric.util.BCPBEKey;
 import org.bouncycastle.jcajce.provider.symmetric.util.BaseAlgorithmParameters;
@@ -24,9 +30,27 @@ import org.bouncycastle.jcajce.provider.symmetric.util.BaseSecretKeyFactory;
 import org.bouncycastle.jcajce.provider.symmetric.util.PBE;
 import org.bouncycastle.jcajce.provider.util.AlgorithmProvider;
 import org.bouncycastle.jcajce.spec.PBKDF2KeySpec;
+import org.bouncycastle.util.Integers;
 
 public class PBEPBKDF2
 {
+    private static final Map prfCodes = new HashMap();
+
+    static
+    {
+        prfCodes.put(CryptoProObjectIdentifiers.gostR3411Hmac, Integers.valueOf(PBE.GOST3411));
+        prfCodes.put(PKCSObjectIdentifiers.id_hmacWithSHA1, Integers.valueOf(PBE.SHA1));
+        prfCodes.put(PKCSObjectIdentifiers.id_hmacWithSHA256, Integers.valueOf(PBE.SHA256));
+        prfCodes.put(PKCSObjectIdentifiers.id_hmacWithSHA224, Integers.valueOf(PBE.SHA224));
+        prfCodes.put(PKCSObjectIdentifiers.id_hmacWithSHA384, Integers.valueOf(PBE.SHA384));
+        prfCodes.put(PKCSObjectIdentifiers.id_hmacWithSHA512, Integers.valueOf(PBE.SHA512));
+        prfCodes.put(NISTObjectIdentifiers.id_hmacWithSHA3_256, Integers.valueOf(PBE.SHA3_256));
+        prfCodes.put(NISTObjectIdentifiers.id_hmacWithSHA3_224, Integers.valueOf(PBE.SHA3_224));
+        prfCodes.put(NISTObjectIdentifiers.id_hmacWithSHA3_384, Integers.valueOf(PBE.SHA3_384));
+        prfCodes.put(NISTObjectIdentifiers.id_hmacWithSHA3_512, Integers.valueOf(PBE.SHA3_512));
+        prfCodes.put(GMObjectIdentifiers.hmac_sm3, Integers.valueOf(PBE.SM3));
+    }
+
     private PBEPBKDF2()
     {
 
@@ -64,10 +88,10 @@ public class PBEPBKDF2
             Class paramSpec)
             throws InvalidParameterSpecException
         {
-            if (paramSpec == PBEParameterSpec.class)
+            if (paramSpec == PBEParameterSpec.class || paramSpec == AlgorithmParameterSpec.class)
             {
                 return new PBEParameterSpec(params.getSalt(),
-                                params.getIterationCount().intValue());
+                    params.getIterationCount().intValue());
             }
 
             throw new InvalidParameterSpecException("unknown parameter spec passed to PBKDF2 PBE parameters object.");
@@ -82,10 +106,10 @@ public class PBEPBKDF2
                 throw new InvalidParameterSpecException("PBEParameterSpec required to initialise a PBKDF2 PBE parameters algorithm parameters object");
             }
 
-            PBEParameterSpec    pbeSpec = (PBEParameterSpec)paramSpec;
+            PBEParameterSpec pbeSpec = (PBEParameterSpec)paramSpec;
 
             this.params = new PBKDF2Params(pbeSpec.getSalt(),
-                                pbeSpec.getIterationCount());
+                pbeSpec.getIterationCount());
         }
 
         protected void engineInit(
@@ -144,7 +168,8 @@ public class PBEPBKDF2
 
                 if (pbeSpec.getSalt() == null)
                 {
-                    throw new InvalidKeySpecException("missing required salt");
+                    return new PBKDF2Key(((PBEKeySpec)keySpec).getPassword(),
+                        scheme == PKCS5S2 ? PasswordConverter.ASCII : PasswordConverter.UTF8);
                 }
 
                 if (pbeSpec.getIterationCount() <= 0)
@@ -193,31 +218,12 @@ public class PBEPBKDF2
         private int getDigestCode(ASN1ObjectIdentifier algorithm)
             throws InvalidKeySpecException
         {
-            if (algorithm.equals(CryptoProObjectIdentifiers.gostR3411Hmac))
+            Integer code = (Integer)prfCodes.get(algorithm);
+            if (code != null)
             {
-                return GOST3411;
+                return code.intValue();
             }
-            else if (algorithm.equals(PKCSObjectIdentifiers.id_hmacWithSHA1))
-            {
-                return SHA1;
-            }
-            else if (algorithm.equals(PKCSObjectIdentifiers.id_hmacWithSHA256))
-            {
-                return SHA256;
-            }
-            else if (algorithm.equals(PKCSObjectIdentifiers.id_hmacWithSHA224))
-            {
-                return SHA224;
-            }
-            else if (algorithm.equals(PKCSObjectIdentifiers.id_hmacWithSHA384))
-            {
-                return SHA384;
-            }
-            else if (algorithm.equals(PKCSObjectIdentifiers.id_hmacWithSHA512))
-            {
-                return SHA512;
-            }
-
+            
             throw new InvalidKeySpecException("Invalid KeySpec: unknown PRF algorithm " + algorithm);
         }
     }
@@ -257,6 +263,7 @@ public class PBEPBKDF2
             super("PBKDF2", PKCS5S2_UTF8, SHA384);
         }
     }
+
     public static class PBKDF2withSHA512
         extends BasePBKDF2
     {
@@ -266,6 +273,51 @@ public class PBEPBKDF2
         }
     }
 
+    public static class PBKDF2withGOST3411
+        extends BasePBKDF2
+    {
+        public PBKDF2withGOST3411()
+        {
+            super("PBKDF2", PKCS5S2_UTF8, GOST3411);
+        }
+    }
+
+    public static class PBKDF2withSHA3_224
+        extends BasePBKDF2
+    {
+        public PBKDF2withSHA3_224()
+        {
+            super("PBKDF2", PKCS5S2_UTF8, SHA3_224);
+        }
+    }
+
+    public static class PBKDF2withSHA3_256
+        extends BasePBKDF2
+    {
+        public PBKDF2withSHA3_256()
+        {
+            super("PBKDF2", PKCS5S2_UTF8, SHA3_256);
+        }
+    }
+
+    public static class PBKDF2withSHA3_384
+        extends BasePBKDF2
+    {
+        public PBKDF2withSHA3_384()
+        {
+            super("PBKDF2", PKCS5S2_UTF8, SHA3_384);
+        }
+    }
+
+    public static class PBKDF2withSHA3_512
+        extends BasePBKDF2
+    {
+        public PBKDF2withSHA3_512()
+        {
+            super("PBKDF2", PKCS5S2_UTF8, SHA3_512);
+        }
+    }
+
     public static class PBKDF2with8BIT
         extends BasePBKDF2
     {
@@ -275,6 +327,14 @@ public class PBEPBKDF2
         }
     }
 
+    public static class PBKDF2withSM3
+        extends BasePBKDF2
+    {
+        public PBKDF2withSM3() {
+            super("PBKDF2", PKCS5S2_UTF8, SM3);
+        }
+    }
+
     public static class Mappings
         extends AlgorithmProvider
     {
@@ -299,6 +359,12 @@ public class PBEPBKDF2
             provider.addAlgorithm("SecretKeyFactory.PBKDF2WITHHMACSHA256", PREFIX + "$PBKDF2withSHA256");
             provider.addAlgorithm("SecretKeyFactory.PBKDF2WITHHMACSHA384", PREFIX + "$PBKDF2withSHA384");
             provider.addAlgorithm("SecretKeyFactory.PBKDF2WITHHMACSHA512", PREFIX + "$PBKDF2withSHA512");
+            provider.addAlgorithm("SecretKeyFactory.PBKDF2WITHHMACSHA3-224", PREFIX + "$PBKDF2withSHA3_224");
+            provider.addAlgorithm("SecretKeyFactory.PBKDF2WITHHMACSHA3-256", PREFIX + "$PBKDF2withSHA3_256");
+            provider.addAlgorithm("SecretKeyFactory.PBKDF2WITHHMACSHA3-384", PREFIX + "$PBKDF2withSHA3_384");
+            provider.addAlgorithm("SecretKeyFactory.PBKDF2WITHHMACSHA3-512", PREFIX + "$PBKDF2withSHA3_512");
+            provider.addAlgorithm("SecretKeyFactory.PBKDF2WITHHMACGOST3411", PREFIX + "$PBKDF2withGOST3411");
+            provider.addAlgorithm("SecretKeyFactory.PBKDF2WITHHMACSM3", PREFIX + "$PBKDF2withSM3");
         }
     }
 }
