@@ -1,7 +1,6 @@
 package org.bouncycastle.jcajce.provider.symmetric.util;
 
 import java.lang.reflect.Constructor;
-import java.lang.reflect.Method;
 import java.nio.ByteBuffer;
 import java.security.AlgorithmParameters;
 import java.security.InvalidAlgorithmParameterException;
@@ -24,7 +23,9 @@ import javax.crypto.spec.PBEParameterSpec;
 import javax.crypto.spec.RC2ParameterSpec;
 import javax.crypto.spec.RC5ParameterSpec;
 
+import org.bouncycastle.asn1.DEROctetString;
 import org.bouncycastle.asn1.cms.GCMParameters;
+import org.bouncycastle.asn1.pkcs.PKCSObjectIdentifiers;
 import org.bouncycastle.crypto.BlockCipher;
 import org.bouncycastle.crypto.BufferedBlockCipher;
 import org.bouncycastle.crypto.CipherParameters;
@@ -34,6 +35,7 @@ import org.bouncycastle.crypto.InvalidCipherTextException;
 import org.bouncycastle.crypto.OutputLengthException;
 import org.bouncycastle.crypto.engines.DSTU7624Engine;
 import org.bouncycastle.crypto.modes.AEADBlockCipher;
+import org.bouncycastle.crypto.modes.AEADCipher;
 import org.bouncycastle.crypto.modes.CBCBlockCipher;
 import org.bouncycastle.crypto.modes.CCMBlockCipher;
 import org.bouncycastle.crypto.modes.CFBBlockCipher;
@@ -71,45 +73,47 @@ import org.bouncycastle.jcajce.PKCS12KeyWithParameters;
 import org.bouncycastle.jcajce.spec.AEADParameterSpec;
 import org.bouncycastle.jcajce.spec.GOST28147ParameterSpec;
 import org.bouncycastle.jcajce.spec.RepeatedSecretKeySpec;
+import org.bouncycastle.util.Arrays;
 import org.bouncycastle.util.Strings;
 
 public class BaseBlockCipher
     extends BaseWrapCipher
     implements PBE
 {
+    private static final int BUF_SIZE = 512;
     private static final Class gcmSpecClass = ClassUtil.loadClass(BaseBlockCipher.class, "javax.crypto.spec.GCMParameterSpec");
 
     //
     // specs we can handle.
     //
-    private Class[]                 availableSpecs =
-                                    {
-                                        RC2ParameterSpec.class,
-                                        RC5ParameterSpec.class,
-                                        gcmSpecClass,
-                                        GOST28147ParameterSpec.class,
-                                        IvParameterSpec.class,
-                                        PBEParameterSpec.class
-                                    };
+    private Class[] availableSpecs =
+        {
+            RC2ParameterSpec.class,
+            RC5ParameterSpec.class,
+            gcmSpecClass,
+            GOST28147ParameterSpec.class,
+            IvParameterSpec.class,
+            PBEParameterSpec.class
+        };
 
-    private BlockCipher             baseEngine;
-    private BlockCipherProvider     engineProvider;
-    private GenericBlockCipher      cipher;
-    private ParametersWithIV        ivParam;
-    private AEADParameters          aeadParams;
+    private BlockCipher baseEngine;
+    private BlockCipherProvider engineProvider;
+    private GenericBlockCipher cipher;
+    private ParametersWithIV ivParam;
+    private AEADParameters aeadParams;
 
     private int keySizeInBits;
     private int scheme = -1;
     private int digest;
 
-    private int                     ivLength = 0;
+    private int ivLength = 0;
 
-    private boolean                 padded;
-    private boolean                 fixedIv = true;
-    private PBEParameterSpec        pbeSpec = null;
-    private String                  pbeAlgorithm = null;
+    private boolean padded;
+    private boolean fixedIv = true;
+    private PBEParameterSpec pbeSpec = null;
+    private String pbeAlgorithm = null;
 
-    private String                  modeName = null;
+    private String modeName = null;
 
     protected BaseBlockCipher(
         BlockCipher engine)
@@ -153,6 +157,17 @@ public class BaseBlockCipher
         this.cipher = new AEADGenericBlockCipher(engine);
     }
 
+    protected BaseBlockCipher(
+        AEADCipher engine,
+        boolean fixedIv,
+        int ivLength)
+    {
+        this.baseEngine = null;
+        this.fixedIv = fixedIv;
+        this.ivLength = ivLength;
+        this.cipher = new AEADGenericBlockCipher(engine);
+    }
+
     protected BaseBlockCipher(
         AEADBlockCipher engine,
         boolean fixedIv,
@@ -204,6 +219,10 @@ public class BaseBlockCipher
 
     protected int engineGetBlockSize()
     {
+        if (baseEngine == null)
+        {
+            return -1;
+        }
         return baseEngine.getBlockSize();
     }
 
@@ -218,13 +237,13 @@ public class BaseBlockCipher
     }
 
     protected int engineGetKeySize(
-        Key     key)
+        Key key)
     {
         return key.getEncoded().length * 8;
     }
 
     protected int engineGetOutputSize(
-        int     inputLen)
+        int inputLen)
     {
         return cipher.getOutputSize(inputLen);
     }
@@ -247,19 +266,35 @@ public class BaseBlockCipher
             }
             else if (aeadParams != null)
             {
-                try
+                // CHACHA20-Poly1305
+                if (baseEngine == null)
                 {
-                    engineParams = createParametersInstance("GCM");
-                    engineParams.init(new GCMParameters(aeadParams.getNonce(), aeadParams.getMacSize() / 8).getEncoded());
+                    try
+                    {
+                        engineParams = createParametersInstance(PKCSObjectIdentifiers.id_alg_AEADChaCha20Poly1305.getId());
+                        engineParams.init(new DEROctetString(aeadParams.getNonce()).getEncoded());
+                    }
+                    catch (Exception e)
+                    {
+                        throw new RuntimeException(e.toString());
+                    }
                 }
-                catch (Exception e)
+                else
                 {
-                    throw new RuntimeException(e.toString());
+                    try
+                    {
+                        engineParams = createParametersInstance("GCM");
+                        engineParams.init(new GCMParameters(aeadParams.getNonce(), aeadParams.getMacSize() / 8).getEncoded());
+                    }
+                    catch (Exception e)
+                    {
+                        throw new RuntimeException(e.toString());
+                    }
                 }
             }
             else if (ivParam != null)
             {
-                String  name = cipher.getUnderlyingCipher().getAlgorithmName();
+                String name = cipher.getUnderlyingCipher().getAlgorithmName();
 
                 if (name.indexOf('/') >= 0)
                 {
@@ -282,9 +317,13 @@ public class BaseBlockCipher
     }
 
     protected void engineSetMode(
-        String  mode)
+        String mode)
         throws NoSuchAlgorithmException
     {
+        if (baseEngine == null)
+        {
+            throw new NoSuchAlgorithmException("no mode supported for this algorithm");
+        }
         modeName = Strings.toUpperCase(mode);
 
         if (modeName.equals("ECB"))
@@ -296,7 +335,7 @@ public class BaseBlockCipher
         {
             ivLength = baseEngine.getBlockSize();
             cipher = new BufferedGenericBlockCipher(
-                            new CBCBlockCipher(baseEngine));
+                new CBCBlockCipher(baseEngine));
         }
         else if (modeName.startsWith("OFB"))
         {
@@ -306,12 +345,12 @@ public class BaseBlockCipher
                 int wordSize = Integer.parseInt(modeName.substring(3));
 
                 cipher = new BufferedGenericBlockCipher(
-                                new OFBBlockCipher(baseEngine, wordSize));
+                    new OFBBlockCipher(baseEngine, wordSize));
             }
             else
             {
                 cipher = new BufferedGenericBlockCipher(
-                        new OFBBlockCipher(baseEngine, 8 * baseEngine.getBlockSize()));
+                    new OFBBlockCipher(baseEngine, 8 * baseEngine.getBlockSize()));
             }
         }
         else if (modeName.startsWith("CFB"))
@@ -322,29 +361,34 @@ public class BaseBlockCipher
                 int wordSize = Integer.parseInt(modeName.substring(3));
 
                 cipher = new BufferedGenericBlockCipher(
-                                new CFBBlockCipher(baseEngine, wordSize));
+                    new CFBBlockCipher(baseEngine, wordSize));
             }
             else
             {
                 cipher = new BufferedGenericBlockCipher(
-                        new CFBBlockCipher(baseEngine, 8 * baseEngine.getBlockSize()));
+                    new CFBBlockCipher(baseEngine, 8 * baseEngine.getBlockSize()));
             }
         }
-        else if (modeName.startsWith("PGP"))
+        else if (modeName.startsWith("PGPCFB"))
         {
-            boolean inlineIV = modeName.equalsIgnoreCase("PGPCFBwithIV");
+            boolean inlineIV = modeName.equals("PGPCFBWITHIV");
 
+            if (!inlineIV && modeName.length() != 6)
+            {
+                throw new NoSuchAlgorithmException("no mode support for " + modeName);
+            }
+            
             ivLength = baseEngine.getBlockSize();
             cipher = new BufferedGenericBlockCipher(
                 new PGPCFBBlockCipher(baseEngine, inlineIV));
         }
-        else if (modeName.equalsIgnoreCase("OpenPGPCFB"))
+        else if (modeName.equals("OPENPGPCFB"))
         {
             ivLength = 0;
             cipher = new BufferedGenericBlockCipher(
                 new OpenPGPCFBBlockCipher(baseEngine));
         }
-        else if (modeName.startsWith("SIC"))
+        else if (modeName.equals("SIC"))
         {
             ivLength = baseEngine.getBlockSize();
             if (ivLength < 16)
@@ -353,16 +397,16 @@ public class BaseBlockCipher
             }
             fixedIv = false;
             cipher = new BufferedGenericBlockCipher(new BufferedBlockCipher(
-                        new SICBlockCipher(baseEngine)));
+                new SICBlockCipher(baseEngine)));
         }
-        else if (modeName.startsWith("CTR"))
+        else if (modeName.equals("CTR"))
         {
             ivLength = baseEngine.getBlockSize();
             fixedIv = false;
             if (baseEngine instanceof DSTU7624Engine)
             {
                 cipher = new BufferedGenericBlockCipher(new BufferedBlockCipher(
-                                    new KCTRBlockCipher(baseEngine)));
+                    new KCTRBlockCipher(baseEngine)));
             }
             else
             {
@@ -370,24 +414,24 @@ public class BaseBlockCipher
                     new SICBlockCipher(baseEngine)));
             }
         }
-        else if (modeName.startsWith("GOFB"))
+        else if (modeName.equals("GOFB"))
         {
             ivLength = baseEngine.getBlockSize();
             cipher = new BufferedGenericBlockCipher(new BufferedBlockCipher(
-                        new GOFBBlockCipher(baseEngine)));
+                new GOFBBlockCipher(baseEngine)));
         }
-        else if (modeName.startsWith("GCFB"))
+        else if (modeName.equals("GCFB"))
         {
             ivLength = baseEngine.getBlockSize();
             cipher = new BufferedGenericBlockCipher(new BufferedBlockCipher(
-                        new GCFBBlockCipher(baseEngine)));
+                new GCFBBlockCipher(baseEngine)));
         }
-        else if (modeName.startsWith("CTS"))
+        else if (modeName.equals("CTS"))
         {
             ivLength = baseEngine.getBlockSize();
             cipher = new BufferedGenericBlockCipher(new CTSBlockCipher(new CBCBlockCipher(baseEngine)));
         }
-        else if (modeName.startsWith("CCM"))
+        else if (modeName.equals("CCM"))
         {
             ivLength = 12; // CCM nonce 7..13 bytes
             if (baseEngine instanceof DSTU7624Engine)
@@ -399,7 +443,7 @@ public class BaseBlockCipher
                 cipher = new AEADGenericBlockCipher(new CCMBlockCipher(baseEngine));
             }
         }
-        else if (modeName.startsWith("OCB"))
+        else if (modeName.equals("OCB"))
         {
             if (engineProvider != null)
             {
@@ -414,12 +458,12 @@ public class BaseBlockCipher
                 throw new NoSuchAlgorithmException("can't support mode " + mode);
             }
         }
-        else if (modeName.startsWith("EAX"))
+        else if (modeName.equals("EAX"))
         {
             ivLength = baseEngine.getBlockSize();
             cipher = new AEADGenericBlockCipher(new EAXBlockCipher(baseEngine));
         }
-        else if (modeName.startsWith("GCM"))
+        else if (modeName.equals("GCM"))
         {
             ivLength = baseEngine.getBlockSize();
             if (baseEngine instanceof DSTU7624Engine)
@@ -438,10 +482,15 @@ public class BaseBlockCipher
     }
 
     protected void engineSetPadding(
-        String  padding)
-    throws NoSuchPaddingException
+        String padding)
+        throws NoSuchPaddingException
     {
-        String  paddingName = Strings.toUpperCase(padding);
+        if (baseEngine == null)
+        {
+            throw new NoSuchPaddingException("no padding supported for this algorithm");
+        }
+
+        String paddingName = Strings.toUpperCase(padding);
 
         if (paddingName.equals("NOPADDING"))
         {
@@ -494,13 +543,13 @@ public class BaseBlockCipher
     }
 
     protected void engineInit(
-        int                     opmode,
-        Key                     key,
-        AlgorithmParameterSpec  params,
-        SecureRandom            random)
+        int opmode,
+        Key key,
+        final AlgorithmParameterSpec params,
+        SecureRandom random)
         throws InvalidKeyException, InvalidAlgorithmParameterException
     {
-        CipherParameters        param;
+        CipherParameters param;
 
         this.pbeSpec = null;
         this.pbeAlgorithm = null;
@@ -518,7 +567,7 @@ public class BaseBlockCipher
         //
         // for RC5-64 we must have some default parameters
         //
-        if (params == null && baseEngine.getAlgorithmName().startsWith("RC5-64"))
+        if (params == null && (baseEngine != null && baseEngine.getAlgorithmName().startsWith("RC5-64")))
         {
             throw new InvalidAlgorithmParameterException("RC5 requires an RC5ParametersSpec to be passed in.");
         }
@@ -538,7 +587,7 @@ public class BaseBlockCipher
                 throw new InvalidKeyException("PKCS12 requires a SecretKey/PBEKey");
             }
 
-            if (params instanceof  PBEParameterSpec)
+            if (params instanceof PBEParameterSpec)
             {
                 pbeSpec = (PBEParameterSpec)params;
             }
@@ -715,10 +764,10 @@ public class BaseBlockCipher
         }
         else if (params instanceof GOST28147ParameterSpec)
         {
-            GOST28147ParameterSpec    gost28147Param = (GOST28147ParameterSpec)params;
+            GOST28147ParameterSpec gost28147Param = (GOST28147ParameterSpec)params;
 
             param = new ParametersWithSBox(
-                       new KeyParameter(key.getEncoded()), ((GOST28147ParameterSpec)params).getSbox());
+                new KeyParameter(key.getEncoded()), ((GOST28147ParameterSpec)params).getSbox());
 
             if (gost28147Param.getIV() != null && ivLength != 0)
             {
@@ -735,7 +784,7 @@ public class BaseBlockCipher
         }
         else if (params instanceof RC2ParameterSpec)
         {
-            RC2ParameterSpec    rc2Param = (RC2ParameterSpec)params;
+            RC2ParameterSpec rc2Param = (RC2ParameterSpec)params;
 
             param = new RC2Parameters(key.getEncoded(), ((RC2ParameterSpec)params).getEffectiveKeyBits());
 
@@ -754,7 +803,7 @@ public class BaseBlockCipher
         }
         else if (params instanceof RC5ParameterSpec)
         {
-            RC5ParameterSpec    rc5Param = (RC5ParameterSpec)params;
+            RC5ParameterSpec rc5Param = (RC5ParameterSpec)params;
 
             param = new RC5Parameters(key.getEncoded(), ((RC5ParameterSpec)params).getRounds());
             if (baseEngine.getAlgorithmName().startsWith("RC5"))
@@ -798,26 +847,17 @@ public class BaseBlockCipher
                 throw new InvalidAlgorithmParameterException("GCMParameterSpec can only be used with AEAD modes.");
             }
 
-            try
+            final KeyParameter keyParam;
+            if (param instanceof ParametersWithIV)
             {
-                Method tLen = gcmSpecClass.getDeclaredMethod("getTLen", new Class[0]);
-                Method iv= gcmSpecClass.getDeclaredMethod("getIV", new Class[0]);
+                keyParam = (KeyParameter)((ParametersWithIV)param).getParameters();
+            }
+            else
+            {
+                keyParam = (KeyParameter)param;
+            }
 
-                KeyParameter keyParam;
-                if (param instanceof ParametersWithIV)
-                {
-                    keyParam = (KeyParameter)((ParametersWithIV)param).getParameters();
-                }
-                else
-                {
-                    keyParam = (KeyParameter)param;
-                }
-                param = aeadParams = new AEADParameters(keyParam, ((Integer)tLen.invoke(params, new Object[0])).intValue(), (byte[])iv.invoke(params, new Object[0]));
-            }
-            catch (Exception e)
-            {
-                throw new InvalidAlgorithmParameterException("Cannot process GCMParameterSpec.");
-            }
+            param = aeadParams = GcmSpecUtil.extractAeadParameters(keyParam, params);
         }
         else if (params != null && !(params instanceof PBEParameterSpec))
         {
@@ -826,7 +866,7 @@ public class BaseBlockCipher
 
         if ((ivLength != 0) && !(param instanceof ParametersWithIV) && !(param instanceof AEADParameters))
         {
-            SecureRandom    ivRandom = random;
+            SecureRandom ivRandom = random;
 
             if (ivRandom == null)
             {
@@ -835,7 +875,7 @@ public class BaseBlockCipher
 
             if ((opmode == Cipher.ENCRYPT_MODE) || (opmode == Cipher.WRAP_MODE))
             {
-                byte[]  iv = new byte[ivLength];
+                byte[] iv = new byte[ivLength];
 
                 ivRandom.nextBytes(iv);
                 param = new ParametersWithIV(param, iv);
@@ -848,7 +888,6 @@ public class BaseBlockCipher
         }
 
 
-
         if (random != null && padded)
         {
             param = new ParametersWithRandom(param, random);
@@ -872,12 +911,16 @@ public class BaseBlockCipher
 
             if (cipher instanceof AEADGenericBlockCipher && aeadParams == null)
             {
-                AEADBlockCipher aeadCipher = ((AEADGenericBlockCipher)cipher).cipher;
+                AEADCipher aeadCipher = ((AEADGenericBlockCipher)cipher).cipher;
 
                 aeadParams = new AEADParameters((KeyParameter)ivParam.getParameters(), aeadCipher.getMac().length * 8, ivParam.getIV());
             }
         }
-        catch (final Exception e)
+        catch (IllegalArgumentException e)
+        {
+            throw new InvalidAlgorithmParameterException(e.getMessage(), e);
+        }
+        catch (Exception e)
         {
             throw new InvalidKeyOrParametersException(e.getMessage(), e);
         }
@@ -937,33 +980,17 @@ public class BaseBlockCipher
     }
 
     protected void engineInit(
-        int                 opmode,
-        Key                 key,
+        int opmode,
+        Key key,
         AlgorithmParameters params,
-        SecureRandom        random) 
-    throws InvalidKeyException, InvalidAlgorithmParameterException
+        SecureRandom random)
+        throws InvalidKeyException, InvalidAlgorithmParameterException
     {
-        AlgorithmParameterSpec  paramSpec = null;
+        AlgorithmParameterSpec paramSpec = null;
 
         if (params != null)
         {
-            for (int i = 0; i != availableSpecs.length; i++)
-            {
-                if (availableSpecs[i] == null)
-                {
-                    continue;
-                }
-
-                try
-                {
-                    paramSpec = params.getParameterSpec(availableSpecs[i]);
-                    break;
-                }
-                catch (Exception e)
-                {
-                    // try again if possible
-                }
-            }
+            paramSpec = SpecUtil.extractSpec(params, availableSpecs);
 
             if (paramSpec == null)
             {
@@ -972,14 +999,14 @@ public class BaseBlockCipher
         }
 
         engineInit(opmode, key, paramSpec, random);
-        
+
         engineParams = params;
     }
 
     protected void engineInit(
-        int                 opmode,
-        Key                 key,
-        SecureRandom        random) 
+        int opmode,
+        Key key,
+        SecureRandom random)
         throws InvalidKeyException
     {
         try
@@ -997,40 +1024,67 @@ public class BaseBlockCipher
         cipher.updateAAD(input, offset, length);
     }
 
-    protected void engineUpdateAAD(ByteBuffer bytebuffer)
+    protected void engineUpdateAAD(ByteBuffer src)
     {
-        int offset = bytebuffer.arrayOffset() + bytebuffer.position();
-        int length = bytebuffer.limit() - bytebuffer.position();
-        engineUpdateAAD(bytebuffer.array(), offset, length);
+        int remaining = src.remaining();
+        if (remaining < 1)
+        {
+            // No data to update
+        }
+        else if (src.hasArray())
+        {
+            engineUpdateAAD(src.array(), src.arrayOffset() + src.position(), remaining);
+            src.position(src.limit());
+        }
+        else if (remaining <= BUF_SIZE)
+        {
+            byte[] data = new byte[remaining];
+            src.get(data);
+            engineUpdateAAD(data, 0, data.length);
+            Arrays.fill(data, (byte)0);
+        }
+        else
+        {
+            byte[] data = new byte[BUF_SIZE];
+            do
+            {
+                int length = Math.min(data.length, remaining);
+                src.get(data, 0, length);
+                engineUpdateAAD(data, 0, length);
+                remaining -= length;
+            }
+            while (remaining > 0);
+            Arrays.fill(data, (byte)0);
+        }
     }
 
     protected byte[] engineUpdate(
-        byte[]  input,
-        int     inputOffset,
-        int     inputLen) 
+        byte[] input,
+        int inputOffset,
+        int inputLen)
     {
-        int     length = cipher.getUpdateOutputSize(inputLen);
+        int length = cipher.getUpdateOutputSize(inputLen);
 
         if (length > 0)
         {
-                byte[]  out = new byte[length];
+            byte[] out = new byte[length];
 
-                int len = cipher.processBytes(input, inputOffset, inputLen, out, 0);
+            int len = cipher.processBytes(input, inputOffset, inputLen, out, 0);
 
-                if (len == 0)
-                {
-                    return null;
-                }
-                else if (len != out.length)
-                {
-                    byte[]  tmp = new byte[len];
+            if (len == 0)
+            {
+                return null;
+            }
+            else if (len != out.length)
+            {
+                byte[] tmp = new byte[len];
 
-                    System.arraycopy(out, 0, tmp, 0, len);
+                System.arraycopy(out, 0, tmp, 0, len);
 
-                    return tmp;
-                }
+                return tmp;
+            }
 
-                return out;
+            return out;
         }
 
         cipher.processBytes(input, inputOffset, inputLen, null, 0);
@@ -1039,11 +1093,11 @@ public class BaseBlockCipher
     }
 
     protected int engineUpdate(
-        byte[]  input,
-        int     inputOffset,
-        int     inputLen,
-        byte[]  output,
-        int     outputOffset)
+        byte[] input,
+        int inputOffset,
+        int inputLen,
+        byte[] output,
+        int outputOffset)
         throws ShortBufferException
     {
         if (outputOffset + cipher.getUpdateOutputSize(inputLen) > output.length)
@@ -1063,13 +1117,13 @@ public class BaseBlockCipher
     }
 
     protected byte[] engineDoFinal(
-        byte[]  input,
-        int     inputOffset,
-        int     inputLen) 
+        byte[] input,
+        int inputOffset,
+        int inputLen)
         throws IllegalBlockSizeException, BadPaddingException
     {
-        int     len = 0;
-        byte[]  tmp = new byte[engineGetOutputSize(inputLen)];
+        int len = 0;
+        byte[] tmp = new byte[engineGetOutputSize(inputLen)];
 
         if (inputLen != 0)
         {
@@ -1090,7 +1144,12 @@ public class BaseBlockCipher
             return tmp;
         }
 
-        byte[]  out = new byte[len];
+        if (len > tmp.length)
+        {
+            throw new IllegalBlockSizeException("internal buffer overflow");
+        }
+
+        byte[] out = new byte[len];
 
         System.arraycopy(tmp, 0, out, 0, len);
 
@@ -1098,14 +1157,14 @@ public class BaseBlockCipher
     }
 
     protected int engineDoFinal(
-        byte[]  input,
-        int     inputOffset,
-        int     inputLen,
-        byte[]  output,
-        int     outputOffset)
+        byte[] input,
+        int inputOffset,
+        int inputLen,
+        byte[] output,
+        int outputOffset)
         throws IllegalBlockSizeException, BadPaddingException, ShortBufferException
     {
-        int     len = 0;
+        int len = 0;
 
         if (outputOffset + engineGetOutputSize(inputLen) > output.length)
         {
@@ -1225,17 +1284,20 @@ public class BaseBlockCipher
             throw new UnsupportedOperationException("AAD is not supported in the current mode.");
         }
 
-        public int processByte(byte in, byte[] out, int outOff) throws DataLengthException
+        public int processByte(byte in, byte[] out, int outOff)
+            throws DataLengthException
         {
             return cipher.processByte(in, out, outOff);
         }
 
-        public int processBytes(byte[] in, int inOff, int len, byte[] out, int outOff) throws DataLengthException
+        public int processBytes(byte[] in, int inOff, int len, byte[] out, int outOff)
+            throws DataLengthException
         {
             return cipher.processBytes(in, inOff, len, out, outOff);
         }
 
-        public int doFinal(byte[] out, int outOff) throws IllegalStateException, BadPaddingException
+        public int doFinal(byte[] out, int outOff)
+            throws IllegalStateException, BadPaddingException
         {
             try
             {
@@ -1253,7 +1315,8 @@ public class BaseBlockCipher
     {
         private static final Constructor aeadBadTagConstructor;
 
-        static {
+        static
+        {
             Class aeadBadTagClass = ClassUtil.loadClass(BaseBlockCipher.class, "javax.crypto.AEADBadTagException");
             if (aeadBadTagClass != null)
             {
@@ -1277,9 +1340,9 @@ public class BaseBlockCipher
             }
         }
 
-        private AEADBlockCipher cipher;
+        private AEADCipher cipher;
 
-        AEADGenericBlockCipher(AEADBlockCipher cipher)
+        AEADGenericBlockCipher(AEADCipher cipher)
         {
             this.cipher = cipher;
         }
@@ -1292,7 +1355,12 @@ public class BaseBlockCipher
 
         public String getAlgorithmName()
         {
-            return cipher.getUnderlyingCipher().getAlgorithmName();
+            if (cipher instanceof AEADBlockCipher)
+            {
+                return ((AEADBlockCipher)cipher).getUnderlyingCipher().getAlgorithmName();
+            }
+
+            return cipher.getAlgorithmName();
         }
 
         public boolean wrapOnNoPadding()
@@ -1302,7 +1370,12 @@ public class BaseBlockCipher
 
         public org.bouncycastle.crypto.BlockCipher getUnderlyingCipher()
         {
-            return cipher.getUnderlyingCipher();
+            if (cipher instanceof AEADBlockCipher)
+            {
+                return ((AEADBlockCipher)cipher).getUnderlyingCipher();
+            }
+
+            return null;
         }
 
         public int getOutputSize(int len)
@@ -1320,17 +1393,20 @@ public class BaseBlockCipher
             cipher.processAADBytes(input, offset, length);
         }
 
-        public int processByte(byte in, byte[] out, int outOff) throws DataLengthException
+        public int processByte(byte in, byte[] out, int outOff)
+            throws DataLengthException
         {
             return cipher.processByte(in, out, outOff);
         }
 
-        public int processBytes(byte[] in, int inOff, int len, byte[] out, int outOff) throws DataLengthException
+        public int processBytes(byte[] in, int inOff, int len, byte[] out, int outOff)
+            throws DataLengthException
         {
             return cipher.processBytes(in, inOff, len, out, outOff);
         }
 
-        public int doFinal(byte[] out, int outOff) throws IllegalStateException, BadPaddingException
+        public int doFinal(byte[] out, int outOff)
+            throws IllegalStateException, BadPaddingException
         {
             try
             {
@@ -1344,7 +1420,7 @@ public class BaseBlockCipher
                     try
                     {
                         aeadBadTag = (BadPaddingException)aeadBadTagConstructor
-                                .newInstance(new Object[]{e.getMessage()});
+                            .newInstance(new Object[]{e.getMessage()});
                     }
                     catch (Exception i)
                     {
