@@ -75,6 +75,7 @@ import org.bouncycastle.jce.provider.PKIXNameConstraintValidator;
 import org.bouncycastle.jce.provider.PKIXNameConstraintValidatorException;
 import org.bouncycastle.jce.provider.PKIXPolicyNode;
 import org.bouncycastle.util.Integers;
+import org.bouncycastle.util.Objects;
 
 /**
  * PKIXCertPathReviewer<br>
@@ -95,6 +96,7 @@ public class PKIXCertPathReviewer extends CertPathValidatorUtilities
 
     protected PKIXParameters pkixParams;
 
+    protected Date currentDate;
     protected Date validDate;
 
     // state variables
@@ -152,7 +154,8 @@ public class PKIXCertPathReviewer extends CertPathValidatorUtilities
 
         // b)
 
-        validDate = getValidDate(pkixParams);
+        currentDate = new Date();
+        validDate = getValidityDate(pkixParams, currentDate);
 
         // c) part of pkixParams
 
@@ -699,7 +702,7 @@ public class PKIXCertPathReviewer extends CertPathValidatorUtilities
         // validation date
         {
             ErrorBundle msg = new ErrorBundle(RESOURCE_NAME,"CertPathReviewer.certPathValidDate",
-                    new Object[] {new TrustedInput(validDate), new TrustedInput(new Date())});
+                    new Object[] {new TrustedInput(validDate), new TrustedInput(currentDate)});
             addNotification(msg);
         }
         
@@ -791,7 +794,7 @@ public class PKIXCertPathReviewer extends CertPathValidatorUtilities
             if (sign != null)
             {
                 boolean[] ku = sign.getKeyUsage(); 
-                if (ku != null && !ku[5])
+                if (ku != null && (ku.length <= 5 || !ku[5]))
                 {
                     ErrorBundle msg = new ErrorBundle(RESOURCE_NAME, "CertPathReviewer.trustKeyUsage");
                     addNotification(msg);
@@ -1052,9 +1055,9 @@ public class PKIXCertPathReviewer extends CertPathValidatorUtilities
 
                 // n)
 
-                boolean[] _usage = cert.getKeyUsage();
+                boolean[] keyUsage = cert.getKeyUsage();
 
-                if ((_usage != null) && !_usage[KEY_CERT_SIGN])
+                if (keyUsage != null && (keyUsage.length <= KEY_CERT_SIGN || !keyUsage[KEY_CERT_SIGN]))
                 {
                     ErrorBundle msg = new ErrorBundle(RESOURCE_NAME,"CertPathReviewer.noCertSign");
                     addError(msg,index);
@@ -1545,14 +1548,14 @@ public class PKIXCertPathReviewer extends CertPathValidatorUtilities
                                 switch (constraint.getTagNo())
                                 {
                                 case 0:
-                                    tmpInt = ASN1Integer.getInstance(constraint, false).getValue().intValue();
+                                    tmpInt = ASN1Integer.getInstance(constraint, false).intValueExact();
                                     if (tmpInt < explicitPolicy)
                                     {
                                         explicitPolicy = tmpInt;
                                     }
                                     break;
                                 case 1:
-                                    tmpInt = ASN1Integer.getInstance(constraint, false).getValue().intValue();
+                                    tmpInt = ASN1Integer.getInstance(constraint, false).intValueExact();
                                     if (tmpInt < policyMapping)
                                     {
                                         policyMapping = tmpInt;
@@ -1578,7 +1581,7 @@ public class PKIXCertPathReviewer extends CertPathValidatorUtilities
                         
                         if (iap != null)
                         {
-                            int _inhibitAnyPolicy = iap.getValue().intValue();
+                            int _inhibitAnyPolicy = iap.intValueExact();
                         
                             if (_inhibitAnyPolicy < inhibitAnyPolicy)
                             {
@@ -1625,7 +1628,7 @@ public class PKIXCertPathReviewer extends CertPathValidatorUtilities
                         switch (constraint.getTagNo())
                         {
                         case 0:
-                            int tmpInt = ASN1Integer.getInstance(constraint, false).getValue().intValue();
+                            int tmpInt = ASN1Integer.getInstance(constraint, false).intValueExact();
                             if (tmpInt == 0)
                             {
                                 explicitPolicy = 0;
@@ -2044,13 +2047,13 @@ public class PKIXCertPathReviewer extends CertPathValidatorUtilities
         Iterator crl_iter;
         try 
         {
-            Collection crl_coll = CRL_UTIL.findCRLs(crlselect, paramsPKIX);
+            Collection crl_coll = PKIXCRLUtil.findCRLs(crlselect, paramsPKIX);
             crl_iter = crl_coll.iterator();
             
             if (crl_coll.isEmpty())
             {
-                // notifcation - no local crls found
-                crl_coll = CRL_UTIL.findCRLs(new X509CRLStoreSelector(),paramsPKIX);
+                // notification - no local crls found
+                crl_coll = PKIXCRLUtil.findCRLs(new X509CRLStoreSelector(),paramsPKIX);
                 Iterator it = crl_coll.iterator();
                 List nonMatchingCrlNames = new ArrayList();
                 while (it.hasNext())
@@ -2065,7 +2068,6 @@ public class PKIXCertPathReviewer extends CertPathValidatorUtilities
                             Integers.valueOf(numbOfCrls)});
                 addNotification(msg,index);
             }
-
         }
         catch (AnnotatedException ae)
         {
@@ -2074,35 +2076,35 @@ public class PKIXCertPathReviewer extends CertPathValidatorUtilities
             addError(msg,index);
             crl_iter = new ArrayList().iterator();
         }
+
         boolean validCrlFound = false;
         X509CRL crl = null;
         while (crl_iter.hasNext())
         {
             crl = (X509CRL)crl_iter.next();
-            
-            if (crl.getNextUpdate() == null
-                || paramsPKIX.getDate().before(crl.getNextUpdate()))
+
+            Date thisUpdate = crl.getThisUpdate();
+            Date nextUpdate = crl.getNextUpdate();
+            Object[] arguments = new Object[]{ new TrustedInput(thisUpdate), new TrustedInput(nextUpdate) };
+
+            if (nextUpdate == null || validDate.before(crl.getNextUpdate()))
             {
                 validCrlFound = true;
-                ErrorBundle msg = new ErrorBundle(RESOURCE_NAME,
-                        "CertPathReviewer.localValidCRL",
-                        new Object[] {new TrustedInput(crl.getThisUpdate()), new TrustedInput(crl.getNextUpdate())});
+                ErrorBundle msg = new ErrorBundle(RESOURCE_NAME, "CertPathReviewer.localValidCRL", arguments);
                 addNotification(msg,index);
                 break;
             }
-            else
-            {
-                ErrorBundle msg = new ErrorBundle(RESOURCE_NAME,
-                        "CertPathReviewer.localInvalidCRL",
-                        new Object[] {new TrustedInput(crl.getThisUpdate()), new TrustedInput(crl.getNextUpdate())});
-                addNotification(msg,index);
-            }
+
+            ErrorBundle msg = new ErrorBundle(RESOURCE_NAME, "CertPathReviewer.localInvalidCRL", arguments);
+            addNotification(msg,index);
         }
-        
+
         // if no valid crl was found in the CertStores try to get one from a
         // crl distribution point
         if (!validCrlFound)
         {
+            X500Principal certIssuer = cert.getIssuerX500Principal();
+
             X509CRL onlineCRL = null;
             Iterator urlIt = crlDistPointUrls.iterator();
             while (urlIt.hasNext())
@@ -2113,40 +2115,36 @@ public class PKIXCertPathReviewer extends CertPathValidatorUtilities
                     onlineCRL = getCRL(location);
                     if (onlineCRL != null)
                     {
+                        X500Principal crlIssuer = onlineCRL.getIssuerX500Principal();
+
                         // check if crl issuer is correct
-                        if (!cert.getIssuerX500Principal().equals(onlineCRL.getIssuerX500Principal()))
+                        if (!certIssuer.equals(crlIssuer))
                         {
-                            ErrorBundle msg = new ErrorBundle(RESOURCE_NAME,
-                                        "CertPathReviewer.onlineCRLWrongCA",
-                                        new Object[] {new UntrustedInput(onlineCRL.getIssuerX500Principal().getName()),
-                                                      new UntrustedInput(cert.getIssuerX500Principal().getName()),
-                                                      new UntrustedUrlInput(location)});
+                            ErrorBundle msg = new ErrorBundle(RESOURCE_NAME, "CertPathReviewer.onlineCRLWrongCA",
+                                new Object[]{ new UntrustedInput(crlIssuer.getName()), new UntrustedInput(certIssuer.getName()),
+                                    new UntrustedUrlInput(location) });
                             addNotification(msg,index);
                             continue;
                         }
-                        
-                        if (onlineCRL.getNextUpdate() == null
-                            || pkixParams.getDate().before(onlineCRL.getNextUpdate()))
+
+                        Date thisUpdate = onlineCRL.getThisUpdate();
+                        Date nextUpdate = onlineCRL.getNextUpdate();
+                        Object[] arguments = new Object[]{ new TrustedInput(thisUpdate), new TrustedInput(nextUpdate),
+                            new UntrustedUrlInput(location) };
+
+                        if (nextUpdate == null || validDate.before(nextUpdate))
                         {
                             validCrlFound = true;
-                            ErrorBundle msg = new ErrorBundle(RESOURCE_NAME,
-                                    "CertPathReviewer.onlineValidCRL",
-                                    new Object[] {new TrustedInput(onlineCRL.getThisUpdate()),
-                                                  new TrustedInput(onlineCRL.getNextUpdate()),
-                                                  new UntrustedUrlInput(location)});
-                            addNotification(msg,index);
+                            ErrorBundle msg = new ErrorBundle(RESOURCE_NAME, "CertPathReviewer.onlineValidCRL",
+                                arguments);
+                            addNotification(msg, index);
                             crl = onlineCRL;
                             break;
                         }
-                        else
-                        {
-                            ErrorBundle msg = new ErrorBundle(RESOURCE_NAME,
-                                    "CertPathReviewer.onlineInvalidCRL",
-                                    new Object[] {new TrustedInput(onlineCRL.getThisUpdate()),
-                                                  new TrustedInput(onlineCRL.getNextUpdate()),
-                                                  new UntrustedUrlInput(location)});
-                            addNotification(msg,index);
-                        }
+
+                        ErrorBundle msg = new ErrorBundle(RESOURCE_NAME, "CertPathReviewer.onlineInvalidCRL",
+                            arguments);
+                        addNotification(msg, index);
                     }
                 }
                 catch (CertPathReviewerException cpre)
@@ -2162,10 +2160,9 @@ public class PKIXCertPathReviewer extends CertPathValidatorUtilities
         {
             if (sign != null)
             {
-                boolean[] keyusage = sign.getKeyUsage();
+                boolean[] keyUsage = sign.getKeyUsage();
 
-                if (keyusage != null
-                    && (keyusage.length < 7 || !keyusage[CRL_SIGN]))
+                if (keyUsage != null && (keyUsage.length <= CRL_SIGN || !keyUsage[CRL_SIGN]))
                 {
                     ErrorBundle msg = new ErrorBundle(RESOURCE_NAME,"CertPathReviewer.noCrlSigningPermited");
                     throw new CertPathReviewerException(msg);
@@ -2209,7 +2206,7 @@ public class PKIXCertPathReviewer extends CertPathValidatorUtilities
                     }
                     if (reasonCode != null)
                     {
-                        reason = crlReasons[reasonCode.getValue().intValue()];
+                        reason = crlReasons[reasonCode.intValueExact()];
                     }
                 }
 
@@ -2243,11 +2240,12 @@ public class PKIXCertPathReviewer extends CertPathValidatorUtilities
             //
             // warn if a new crl is available
             //
-            if (crl.getNextUpdate() != null && crl.getNextUpdate().before(pkixParams.getDate()))
+            Date nextUpdate = crl.getNextUpdate();
+            if (!(nextUpdate == null || validDate.before(nextUpdate)))
             {
-                ErrorBundle msg = new ErrorBundle(RESOURCE_NAME,"CertPathReviewer.crlUpdateAvailable",
-                        new Object[] {new TrustedInput(crl.getNextUpdate())});
-                addNotification(msg,index);
+                ErrorBundle msg = new ErrorBundle(RESOURCE_NAME, "CertPathReviewer.crlUpdateAvailable",
+                    new Object[]{ new TrustedInput(nextUpdate) });
+                addNotification(msg, index);
             }
             
             //
@@ -2303,7 +2301,7 @@ public class PKIXCertPathReviewer extends CertPathValidatorUtilities
                 Iterator it;
                 try 
                 {
-                    it  = CRL_UTIL.findCRLs(baseSelect, paramsPKIX).iterator();
+                    it  = PKIXCRLUtil.findCRLs(baseSelect, paramsPKIX).iterator();
                 }
                 catch (AnnotatedException ae)
                 {
@@ -2324,25 +2322,14 @@ public class PKIXCertPathReviewer extends CertPathValidatorUtilities
                         ErrorBundle msg = new ErrorBundle(RESOURCE_NAME,"CertPathReviewer.distrPtExtError");
                         throw new CertPathReviewerException(msg,ae);
                     }
-                    
-                    if (idp == null)
+
+                    if (Objects.areEqual(idp, baseIdp))
                     {
-                        if (baseIdp == null)
-                        {
-                            foundBase = true;
-                            break;
-                        }
-                    }
-                    else
-                    {
-                        if (idp.equals(baseIdp))
-                        {
-                            foundBase = true;
-                            break;
-                        }
+                        foundBase = true;
+                        break;
                     }
                 }
-                
+
                 if (!foundBase)
                 {
                     ErrorBundle msg = new ErrorBundle(RESOURCE_NAME,"CertPathReviewer.noBaseCRL");
@@ -2389,7 +2376,6 @@ public class PKIXCertPathReviewer extends CertPathValidatorUtilities
             ErrorBundle msg = new ErrorBundle(RESOURCE_NAME,"CertPathReviewer.noValidCrlFound");
             throw new CertPathReviewerException(msg);
         }
-    
     }
     
     protected Vector getCRLDistUrls(CRLDistPoint crlDistPoints)
