@@ -9,7 +9,6 @@ import java.security.AlgorithmParameters;
 import java.security.InvalidKeyException;
 import java.security.Key;
 import java.security.KeyFactory;
-import java.security.KeyStore;
 import java.security.KeyStoreException;
 import java.security.KeyStoreSpi;
 import java.security.NoSuchAlgorithmException;
@@ -38,10 +37,6 @@ import javax.crypto.NoSuchPaddingException;
 import javax.crypto.SecretKey;
 import javax.crypto.SecretKeyFactory;
 import javax.crypto.spec.SecretKeySpec;
-import javax.security.auth.callback.Callback;
-import javax.security.auth.callback.CallbackHandler;
-import javax.security.auth.callback.PasswordCallback;
-import javax.security.auth.callback.UnsupportedCallbackException;
 
 import org.bouncycastle.asn1.ASN1Encodable;
 import org.bouncycastle.asn1.ASN1InputStream;
@@ -58,8 +53,6 @@ import org.bouncycastle.asn1.bc.ObjectStoreIntegrityCheck;
 import org.bouncycastle.asn1.bc.PbkdMacIntegrityCheck;
 import org.bouncycastle.asn1.bc.SecretKeyData;
 import org.bouncycastle.asn1.cms.CCMParameters;
-import org.bouncycastle.asn1.misc.MiscObjectIdentifiers;
-import org.bouncycastle.asn1.misc.ScryptParams;
 import org.bouncycastle.asn1.nist.NISTObjectIdentifiers;
 import org.bouncycastle.asn1.oiw.OIWObjectIdentifiers;
 import org.bouncycastle.asn1.pkcs.EncryptedPrivateKeyInfo;
@@ -71,15 +64,11 @@ import org.bouncycastle.asn1.pkcs.PKCSObjectIdentifiers;
 import org.bouncycastle.asn1.pkcs.PrivateKeyInfo;
 import org.bouncycastle.asn1.x509.AlgorithmIdentifier;
 import org.bouncycastle.asn1.x9.X9ObjectIdentifiers;
+import org.bouncycastle.crypto.CryptoServicesRegistrar;
 import org.bouncycastle.crypto.PBEParametersGenerator;
-import org.bouncycastle.crypto.digests.SHA3Digest;
 import org.bouncycastle.crypto.digests.SHA512Digest;
 import org.bouncycastle.crypto.generators.PKCS5S2ParametersGenerator;
-import org.bouncycastle.crypto.generators.SCrypt;
 import org.bouncycastle.crypto.params.KeyParameter;
-import org.bouncycastle.crypto.util.PBKDF2Config;
-import org.bouncycastle.crypto.util.PBKDFConfig;
-import org.bouncycastle.crypto.util.ScryptConfig;
 import org.bouncycastle.jce.provider.BouncyCastleProvider;
 import org.bouncycastle.util.Arrays;
 import org.bouncycastle.util.Strings;
@@ -87,8 +76,8 @@ import org.bouncycastle.util.Strings;
 class BcFKSKeyStoreSpi
     extends KeyStoreSpi
 {
-    private static final Map<String, ASN1ObjectIdentifier> oidMap = new HashMap<String, ASN1ObjectIdentifier>();
-    private static final Map<ASN1ObjectIdentifier, String> publicAlgMap = new HashMap<ASN1ObjectIdentifier, String>();
+    private static final Map  oidMap = new HashMap ();
+    private static final Map  publicAlgMap = new HashMap ();
 
     static
     {
@@ -128,8 +117,8 @@ class BcFKSKeyStoreSpi
     private final static BigInteger PROTECTED_SECRET_KEY = BigInteger.valueOf(4);
 
     private final BouncyCastleProvider provider;
-    private final Map<String, ObjectData> entries = new HashMap<String, ObjectData>();
-    private final Map<String, PrivateKey> privateKeyCache = new HashMap<String, PrivateKey>();
+    private final Map  entries = new HashMap ();
+    private final Map  privateKeyCache = new HashMap ();
 
     private AlgorithmIdentifier hmacAlgorithm;
     private KeyDerivationFunc hmacPkbdAlgorithm;
@@ -166,7 +155,7 @@ class BcFKSKeyStoreSpi
                     KeyFactory kFact;
                     if (provider != null)
                     {
-                        kFact = KeyFactory.getInstance(pInfo.getPrivateKeyAlgorithm().getAlgorithm().getId(), provider);
+                        kFact = KeyFactory.getInstance(pInfo.getPrivateKeyAlgorithm().getAlgorithm().getId(), provider.getName());
                     }
                     else
                     {
@@ -272,7 +261,7 @@ class BcFKSKeyStoreSpi
         {
             try
             {
-                CertificateFactory certFact = CertificateFactory.getInstance("X.509", provider);
+                CertificateFactory certFact = CertificateFactory.getInstance("X.509", provider.getName());
 
                 return certFact.generateCertificate(new ByteArrayInputStream(org.bouncycastle.asn1.x509.Certificate.getInstance(cert).getEncoded()));
             }
@@ -344,7 +333,7 @@ class BcFKSKeyStoreSpi
 
                 byte[] encodedKey = key.getEncoded();
 
-                KeyDerivationFunc pbkdAlgId = generatePkbdAlgorithmIdentifier(PKCSObjectIdentifiers.id_PBKDF2, 256 / 8);
+                KeyDerivationFunc pbkdAlgId = generatePkbdAlgorithmIdentifier(256 / 8);
                 byte[] keyBytes = generateKey(pbkdAlgId, "PRIVATE_KEY_ENCRYPTION", ((password != null) ? password : new char[0]));
 
                 Cipher c;
@@ -387,7 +376,7 @@ class BcFKSKeyStoreSpi
             {
                 byte[] encodedKey = key.getEncoded();
 
-                KeyDerivationFunc pbkdAlgId = generatePkbdAlgorithmIdentifier(PKCSObjectIdentifiers.id_PBKDF2, 256 / 8);
+                KeyDerivationFunc pbkdAlgId = generatePkbdAlgorithmIdentifier(256 / 8);
                 byte[] keyBytes = generateKey(pbkdAlgId, "SECRET_KEY_ENCRYPTION", ((password != null) ? password : new char[0]));
 
                 Cipher c;
@@ -447,7 +436,7 @@ class BcFKSKeyStoreSpi
 
     private SecureRandom getDefaultSecureRandom()
     {
-        return new SecureRandom();
+        return CryptoServicesRegistrar.getSecureRandom();
     }
 
     private EncryptedPrivateKeyData createPrivateKeySequence(EncryptedPrivateKeyInfo encryptedPrivateKeyInfo, Certificate[] chain)
@@ -570,9 +559,9 @@ class BcFKSKeyStoreSpi
         lastModifiedDate = new Date();
     }
 
-    public Enumeration<String> engineAliases()
+    public Enumeration  engineAliases()
     {
-        final Iterator<String> it = new HashSet(entries.keySet()).iterator();
+        final Iterator  it = new HashSet(entries.keySet()).iterator();
 
         return new Enumeration()
         {
@@ -631,11 +620,6 @@ class BcFKSKeyStoreSpi
 
     public String engineGetCertificateAlias(Certificate certificate)
     {
-        if (certificate == null)
-        {
-            return null;
-        }
-
         byte[] encodedCert;
         try
         {
@@ -646,7 +630,7 @@ class BcFKSKeyStoreSpi
             return null;
         }
 
-        for (Iterator<String> it = entries.keySet().iterator(); it.hasNext(); )
+        for (Iterator  it = entries.keySet().iterator(); it.hasNext(); )
         {
             String alias = (String)it.next();
             ObjectData ent = (ObjectData)entries.get(alias);
@@ -685,48 +669,30 @@ class BcFKSKeyStoreSpi
         byte[] differentiator = PBEParametersGenerator.PKCS12PasswordToBytes(purpose.toCharArray());
 
         int keySizeInBytes;
+        PKCS5S2ParametersGenerator pGen = new PKCS5S2ParametersGenerator(new SHA512Digest());
 
-        if (MiscObjectIdentifiers.id_scrypt.equals(pbkdAlgorithm.getAlgorithm()))
-        {
-            ScryptParams params = ScryptParams.getInstance(pbkdAlgorithm.getParameters());
-
-            return SCrypt.generate(Arrays.concatenate(encPassword, differentiator), params.getSalt(),
-                params.getCostParameter().intValue(), params.getBlockSize().intValue(),
-                params.getBlockSize().intValue(), params.getKeyLength().intValue());
-        }
-        else if (pbkdAlgorithm.getAlgorithm().equals(PKCSObjectIdentifiers.id_PBKDF2))
+        if (pbkdAlgorithm.getAlgorithm().equals(PKCSObjectIdentifiers.id_PBKDF2))
         {
             PBKDF2Params pbkdf2Params = PBKDF2Params.getInstance(pbkdAlgorithm.getParameters());
 
             if (pbkdf2Params.getPrf().getAlgorithm().equals(PKCSObjectIdentifiers.id_hmacWithSHA512))
             {
-                PKCS5S2ParametersGenerator pGen = new PKCS5S2ParametersGenerator(new SHA512Digest());
 
                 pGen.init(Arrays.concatenate(encPassword, differentiator), pbkdf2Params.getSalt(), pbkdf2Params.getIterationCount().intValue());
 
                 keySizeInBytes = pbkdf2Params.getKeyLength().intValue();
-
-                return ((KeyParameter)pGen.generateDerivedParameters(keySizeInBytes * 8)).getKey();
-            }
-            else if (pbkdf2Params.getPrf().getAlgorithm().equals(NISTObjectIdentifiers.id_hmacWithSHA3_512))
-            {
-                PKCS5S2ParametersGenerator pGen = new PKCS5S2ParametersGenerator(new SHA3Digest(512));
-
-                pGen.init(Arrays.concatenate(encPassword, differentiator), pbkdf2Params.getSalt(), pbkdf2Params.getIterationCount().intValue());
-
-                keySizeInBytes = pbkdf2Params.getKeyLength().intValue();
-
-                return ((KeyParameter)pGen.generateDerivedParameters(keySizeInBytes * 8)).getKey();
             }
             else
             {
-                throw new IOException("BCFKS KeyStore: unrecognized MAC PBKD PRF: " + pbkdf2Params.getPrf().getAlgorithm());
+                throw new IOException("BCFKS KeyStore: unrecognized MAC PBKD PRF.");
             }
         }
         else
         {
             throw new IOException("BCFKS KeyStore: unrecognized MAC PBKD.");
         }
+
+        return ((KeyParameter)pGen.generateDerivedParameters(keySizeInBytes * 8)).getKey();
     }
 
     private void verifyMac(byte[] content, PbkdMacIntegrityCheck integrityCheck, char[] password)
@@ -772,7 +738,7 @@ class BcFKSKeyStoreSpi
     {
         ObjectData[] dataArray = (ObjectData[])entries.values().toArray(new ObjectData[entries.size()]);
 
-        KeyDerivationFunc pbkdAlgId = generatePkbdAlgorithmIdentifier(hmacPkbdAlgorithm, 256 / 8);
+        KeyDerivationFunc pbkdAlgId = generatePkbdAlgorithmIdentifier(256 / 8);
         byte[] keyBytes = generateKey(pbkdAlgId, "STORE_ENCRYPTION", ((password != null) ? password : new char[0]));
 
         ObjectStoreData storeData = new ObjectStoreData(hmacAlgorithm, creationDate, lastModifiedDate, new ObjectDataSequence(dataArray), null);
@@ -818,18 +784,13 @@ class BcFKSKeyStoreSpi
         }
 
         // update the salt
-        if (MiscObjectIdentifiers.id_scrypt.equals(hmacPkbdAlgorithm.getAlgorithm()))
-        {
-            ScryptParams sParams = ScryptParams.getInstance(hmacPkbdAlgorithm.getParameters());
+        PBKDF2Params pbkdf2Params = PBKDF2Params.getInstance(hmacPkbdAlgorithm.getParameters());
 
-            hmacPkbdAlgorithm = generatePkbdAlgorithmIdentifier(hmacPkbdAlgorithm, sParams.getKeyLength().intValue());
-        }
-        else
-        {
-            PBKDF2Params pbkdf2Params = PBKDF2Params.getInstance(hmacPkbdAlgorithm.getParameters());
+        byte[] pbkdSalt = new byte[pbkdf2Params.getSalt().length];
+        getDefaultSecureRandom().nextBytes(pbkdSalt);
+
+        hmacPkbdAlgorithm = new KeyDerivationFunc(hmacPkbdAlgorithm.getAlgorithm(), new PBKDF2Params(pbkdSalt, pbkdf2Params.getIterationCount().intValue(), pbkdf2Params.getKeyLength().intValue(), pbkdf2Params.getPrf()));
 
-            hmacPkbdAlgorithm = generatePkbdAlgorithmIdentifier(hmacPkbdAlgorithm, pbkdf2Params.getKeyLength().intValue());
-        }
         byte[] mac = calculateMac(encStoreData.getEncoded(), hmacAlgorithm, hmacPkbdAlgorithm, password);
 
         ObjectStore store = new ObjectStore(encStoreData, new ObjectStoreIntegrityCheck(new PbkdMacIntegrityCheck(hmacAlgorithm, hmacPkbdAlgorithm, mac)));
@@ -854,25 +815,15 @@ class BcFKSKeyStoreSpi
             // initialise defaults
             lastModifiedDate = creationDate = new Date();
 
-            // basic initialisation
             hmacAlgorithm = new AlgorithmIdentifier(PKCSObjectIdentifiers.id_hmacWithSHA512, DERNull.INSTANCE);
-            hmacPkbdAlgorithm = generatePkbdAlgorithmIdentifier(PKCSObjectIdentifiers.id_PBKDF2, 512 / 8);
+            hmacPkbdAlgorithm = generatePkbdAlgorithmIdentifier(512 / 8);
 
             return;
         }
 
         ASN1InputStream aIn = new ASN1InputStream(inputStream);
 
-        ObjectStore store;
-
-        try
-        {
-            store = ObjectStore.getInstance(aIn.readObject());
-        }
-        catch (Exception e)
-        {
-            throw new IOException(e.getMessage());
-        }
+        ObjectStore store = ObjectStore.getInstance(aIn.readObject());
 
         ObjectStoreIntegrityCheck integrityCheck = store.getIntegrityCheck();
         if (integrityCheck.getType() == ObjectStoreIntegrityCheck.PBKD_MAC_CHECK)
@@ -956,7 +907,7 @@ class BcFKSKeyStoreSpi
             else
             {
                 c = Cipher.getInstance("AES/CCM/NoPadding", provider);
-                algParams = AlgorithmParameters.getInstance("CCM", provider);
+                algParams = AlgorithmParameters.getInstance("CCM", provider.getName());
             }
 
             algParams.init(ccmParameters.getEncoded());
@@ -974,73 +925,11 @@ class BcFKSKeyStoreSpi
         }
     }
 
-    private KeyDerivationFunc generatePkbdAlgorithmIdentifier(PBKDFConfig pbkdfConfig, int keySizeInBytes)
-    {
-        if (MiscObjectIdentifiers.id_scrypt.equals(pbkdfConfig.getAlgorithm()))
-        {
-            ScryptConfig scryptConfig = (ScryptConfig)pbkdfConfig;
-
-            byte[] pbkdSalt = new byte[scryptConfig.getSaltLength()];
-            getDefaultSecureRandom().nextBytes(pbkdSalt);
-
-            ScryptParams params = new ScryptParams(
-                pbkdSalt,
-                scryptConfig.getCostParameter(), scryptConfig.getBlockSize(), scryptConfig.getParallelizationParameter(), keySizeInBytes);
-
-            return new KeyDerivationFunc(MiscObjectIdentifiers.id_scrypt, params);
-        }
-        else
-        {
-            PBKDF2Config pbkdf2Config = (PBKDF2Config)pbkdfConfig;
-
-            byte[] pbkdSalt = new byte[pbkdf2Config.getSaltLength()];
-            getDefaultSecureRandom().nextBytes(pbkdSalt);
-
-            return new KeyDerivationFunc(PKCSObjectIdentifiers.id_PBKDF2, new PBKDF2Params(pbkdSalt, pbkdf2Config.getIterationCount(), keySizeInBytes, pbkdf2Config.getPRF()));
-        }
-    }
-
-    private KeyDerivationFunc generatePkbdAlgorithmIdentifier(KeyDerivationFunc baseAlg, int keySizeInBytes)
-    {
-        if (MiscObjectIdentifiers.id_scrypt.equals(baseAlg.getAlgorithm()))
-        {
-            ScryptParams oldParams = ScryptParams.getInstance(baseAlg.getParameters());
-
-            byte[] pbkdSalt = new byte[oldParams.getSalt().length];
-            getDefaultSecureRandom().nextBytes(pbkdSalt);
-
-            ScryptParams params = new ScryptParams(
-                pbkdSalt,
-                oldParams.getCostParameter(), oldParams.getBlockSize(), oldParams.getParallelizationParameter(), BigInteger.valueOf(keySizeInBytes));
-
-            return new KeyDerivationFunc(MiscObjectIdentifiers.id_scrypt, params);
-        }
-        else
-        {
-            PBKDF2Params oldParams = PBKDF2Params.getInstance(baseAlg.getParameters());
-  
-            byte[] pbkdSalt = new byte[oldParams.getSalt().length];
-            getDefaultSecureRandom().nextBytes(pbkdSalt);
-
-            PBKDF2Params params = new PBKDF2Params(pbkdSalt,
-                oldParams.getIterationCount().intValue(), keySizeInBytes, oldParams.getPrf());
-            return new KeyDerivationFunc(PKCSObjectIdentifiers.id_PBKDF2, params);
-        }
-    }
-
-    private KeyDerivationFunc generatePkbdAlgorithmIdentifier(ASN1ObjectIdentifier derivationAlgorithm, int keySizeInBytes)
+    private KeyDerivationFunc generatePkbdAlgorithmIdentifier(int keySizeInBytes)
     {
         byte[] pbkdSalt = new byte[512 / 8];
         getDefaultSecureRandom().nextBytes(pbkdSalt);
-
-        if (PKCSObjectIdentifiers.id_PBKDF2.equals(derivationAlgorithm))
-        {
-            return new KeyDerivationFunc(PKCSObjectIdentifiers.id_PBKDF2, new PBKDF2Params(pbkdSalt, 50 * 1024, keySizeInBytes, new AlgorithmIdentifier(PKCSObjectIdentifiers.id_hmacWithSHA512, DERNull.INSTANCE)));
-        }
-        else
-        {
-            throw new IllegalStateException("unknown derivation algorithm: " + derivationAlgorithm);
-        }
+        return new KeyDerivationFunc(PKCSObjectIdentifiers.id_PBKDF2, new PBKDF2Params(pbkdSalt, 1024, keySizeInBytes, new AlgorithmIdentifier(PKCSObjectIdentifiers.id_hmacWithSHA512, DERNull.INSTANCE)));
     }
 
     public static class Std
@@ -1074,7 +963,7 @@ class BcFKSKeyStoreSpi
 
         public Throwable getCause()
         {
-            return cause;
+           return cause;
         }
     }
 }
