@@ -5,8 +5,6 @@ import java.security.InvalidParameterException;
 import java.security.KeyPair;
 import java.security.SecureRandom;
 import java.security.spec.AlgorithmParameterSpec;
-import java.security.spec.ECGenParameterSpec;
-import java.security.spec.NamedParameterSpec;
 
 import org.bouncycastle.asn1.edec.EdECObjectIdentifiers;
 import org.bouncycastle.crypto.AsymmetricCipherKeyPair;
@@ -20,7 +18,6 @@ import org.bouncycastle.crypto.params.Ed25519KeyGenerationParameters;
 import org.bouncycastle.crypto.params.Ed448KeyGenerationParameters;
 import org.bouncycastle.crypto.params.X25519KeyGenerationParameters;
 import org.bouncycastle.crypto.params.X448KeyGenerationParameters;
-import org.bouncycastle.jcajce.provider.asymmetric.util.ECUtil;
 import org.bouncycastle.jcajce.spec.EdDSAParameterSpec;
 import org.bouncycastle.jcajce.spec.XDHParameterSpec;
 import org.bouncycastle.jce.spec.ECNamedCurveGenParameterSpec;
@@ -109,18 +106,10 @@ public class KeyPairGeneratorSpi
     {
         this.secureRandom = secureRandom;
 
-        if (paramSpec instanceof ECGenParameterSpec)
-        {
-            initializeGenerator(((ECGenParameterSpec)paramSpec).getName());
-        }
-        else if (paramSpec instanceof ECNamedCurveGenParameterSpec)
+        if (paramSpec instanceof ECNamedCurveGenParameterSpec)
         {
             initializeGenerator(((ECNamedCurveGenParameterSpec)paramSpec).getName());
         }
-        else if (paramSpec instanceof NamedParameterSpec)
-        {
-            initializeGenerator(((NamedParameterSpec)paramSpec).getName());
-        }
         else if (paramSpec instanceof EdDSAParameterSpec)
         {
             initializeGenerator(((EdDSAParameterSpec)paramSpec).getCurveName());
@@ -131,16 +120,7 @@ public class KeyPairGeneratorSpi
         }
         else
         {
-            String name = ECUtil.getNameFrom(paramSpec);
-
-            if (name != null)
-            {
-                initializeGenerator(name);
-            }
-            else
-            {
-                throw new InvalidAlgorithmParameterException("invalid parameterSpec: " + paramSpec);
-            }
+            throw new InvalidAlgorithmParameterException("invalid parameterSpec: " + paramSpec);
         }
     }
 
@@ -219,9 +199,9 @@ public class KeyPairGeneratorSpi
         case Ed25519:
             return new KeyPair(new BCEdDSAPublicKey(kp.getPublic()), new BCEdDSAPrivateKey(kp.getPrivate()));
         case X448:
-            return new KeyPair(new BC11XDHPublicKey(kp.getPublic()), new BC11XDHPrivateKey(kp.getPrivate()));
+            return new KeyPair(new BCXDHPublicKey(kp.getPublic()), new BCXDHPrivateKey(kp.getPrivate()));
         case X25519:
-            return new KeyPair(new BC11XDHPublicKey(kp.getPublic()), new BC11XDHPrivateKey(kp.getPrivate()));
+            return new KeyPair(new BCXDHPublicKey(kp.getPublic()), new BCXDHPrivateKey(kp.getPrivate()));
         }
 
         throw new IllegalStateException("generator not correctly initialized");
