@@ -11,6 +11,8 @@ import java.security.InvalidAlgorithmParameterException;
 import java.security.InvalidKeyException;
 import java.security.InvalidParameterException;
 import java.security.Key;
+import java.security.NoSuchAlgorithmException;
+import java.security.NoSuchProviderException;
 import java.security.PrivateKey;
 import java.security.PublicKey;
 import java.security.SecureRandom;
@@ -25,6 +27,7 @@ import javax.crypto.CipherInputStream;
 import javax.crypto.CipherOutputStream;
 import javax.crypto.IllegalBlockSizeException;
 import javax.crypto.KeyGenerator;
+import javax.crypto.NoSuchPaddingException;
 import javax.crypto.SecretKey;
 import javax.crypto.SecretKeyFactory;
 import javax.crypto.ShortBufferException;
@@ -601,6 +604,23 @@ public class BlockCipherTest
             "3d9255",
     };
 
+    static String[] cipherModes = new String[]
+    {
+        "OFB",
+        "CFB",
+        "PGP",
+        "OpenPGPCFB",
+        "SIC",
+        "CTR",
+        "GOFB",
+        "GCFB",
+        "CTS",
+        "CCM",
+        "OCB",
+        "EAX",
+        "GCM"
+    };
+
     static byte[]   input1 = Hex.decode("000102030405060708090a0b0c0d0e0fff0102030405060708090a0b0c0d0e0f");
     static byte[]   input2 = Hex.decode("000102030405060708090a0b0c0d0e0fff0102030405060708090a0b0c");
     static byte[]   inputLargeBlock = Hex.decode("000102030405060708090a0b0c0d0e0fff0102030405060708090a0b0c0d0e0f" +
@@ -1578,7 +1598,106 @@ public class BlockCipherTest
             fail("unexpected exception.", e);
         }
     }
-    
+
+    private void testIncorrectCipherModes()
+    {
+        for (int i = 0; i != cipherModes.length; i++)
+        {
+            try
+            {
+                Cipher.getInstance("AES/" + cipherModes[i] + "NOT_REAL" + "/NoPadding", "BC");
+                fail("\"AES/" + cipherModes[i] + "NOT_REAL/NoPadding\"" + "returned");
+            }
+            catch (NoSuchAlgorithmException e)
+            {
+                if (!(e.getMessage().indexOf("can't support mode ") >= 0))      // old JVM
+                {
+                    isEquals("1 got: " + e.getMessage(), "No such algorithm: AES/" + cipherModes[i] + "NOT_REAL/NoPadding", e.getMessage());
+                }
+            }
+            catch (NoSuchPaddingException e)
+            {
+                fail(e.toString());
+            }
+            catch (NoSuchProviderException e)
+            {
+                fail(e.toString());
+            }
+        }
+
+        for (int i = 0; i != cipherModes.length; i++)
+        {
+            try
+            {
+                Cipher.getInstance("AES/" + cipherModes[i] + "256" + "/NoPadding", "BC");
+                fail("\"AES/" + cipherModes[i] + "256/NoPadding\"" + "returned");
+            }
+            catch (NoSuchAlgorithmException e)
+            {
+                if (!(e.getMessage().indexOf("can't support mode ") >= 0))      // old JVM
+                {
+                    isEquals("2 got: " + e.getMessage(), "No such algorithm: AES/" + cipherModes[i] + "256/NoPadding", e.getMessage());
+                }
+            }
+            catch (NoSuchPaddingException e)
+            {
+                fail(e.toString());
+            }
+            catch (NoSuchProviderException e)
+            {
+                fail(e.toString());
+            }
+        }
+
+        for (int i = 0; i != cipherModes.length; i++)
+        {
+            try
+            {
+                Cipher.getInstance("AES/" + cipherModes[i] + "2" + "/NoPadding", "BC");
+                fail("\"AES/" + cipherModes[i] + "2/NoPadding\"" + "returned");
+            }
+            catch (NoSuchAlgorithmException e)
+            {
+                if (!(e.getMessage().indexOf("can't support mode ") >= 0))      // old JVM
+                {
+                    isEquals("3 got: " + e.getMessage(), "No such algorithm: AES/" + cipherModes[i] + "2/NoPadding", e.getMessage());
+                }
+            }
+            catch (NoSuchPaddingException e)
+            {
+                fail(e.toString());
+            }
+            catch (NoSuchProviderException e)
+            {
+                fail(e.toString());
+            }
+        }
+
+        for (int i = 0; i != cipherModes.length; i++)
+        {
+            try
+            {
+                Cipher.getInstance("AES/" + cipherModes[i] + "9" + "/NoPadding", "BC");
+                fail("\"AES/" + cipherModes[i] + "9/NoPadding\"" + "returned");
+            }
+            catch (NoSuchAlgorithmException e)
+            {
+                if (!(e.getMessage().indexOf("can't support mode ") >= 0))      // old JVM
+                {
+                    isEquals("No such algorithm: AES/" + cipherModes[i] + "9/NoPadding", e.getMessage());
+                }
+            }
+            catch (NoSuchPaddingException e)
+            {
+                fail(e.toString());
+            }
+            catch (NoSuchProviderException e)
+            {
+                fail(e.toString());
+            }
+        }
+    }
+
     public void performTest()
     {
         for (int i = 0; i != cipherTests1.length; i += 2)
@@ -1618,6 +1737,7 @@ public class BlockCipherTest
         }
         
         testExceptions();
+        testIncorrectCipherModes();
     }
 
     public static void main(
