@@ -1,13 +1,25 @@
 package org.bouncycastle.jsse;
 
+import javax.net.ssl.SSLEngine;
+
 /**
  * A BCJSSE-specific interface to expose extended functionality on {@link javax.net.ssl.SSLEngine}
  * implementations.
  */
 public interface BCSSLEngine
 {
+    String getApplicationProtocol();
+
+    BCApplicationProtocolSelector<SSLEngine> getBCHandshakeApplicationProtocolSelector();
+
+    void setBCHandshakeApplicationProtocolSelector(BCApplicationProtocolSelector<SSLEngine> selector);
+
+    void setBCSessionToResume(BCExtendedSSLSession session);
+
     BCExtendedSSLSession getBCHandshakeSession();
 
+    BCExtendedSSLSession getBCSession();
+
     /**
      * Returns an accessor for extended SSL connection data. Unlike
      * {@link BCSSLSocket#getConnection} this method does not block until handshaking is complete.
@@ -17,6 +29,8 @@ public interface BCSSLEngine
      */
     BCSSLConnection getConnection();
 
+    String getHandshakeApplicationProtocol();
+
     /**
      * Returns a {@link BCSSLParameters} with properties reflecting the current configuration.
      * @return the current {@link BCSSLParameters parameters}
