@@ -20,8 +20,8 @@ public class BouncyCastleJsseProvider
 {
     public static final String PROVIDER_NAME = "BCJSSE";
 
-    private static final double PROVIDER_VERSION = 1.0010;
-    private static final String PROVIDER_INFO = "Bouncy Castle JSSE Provider Version 1.0.10";
+    private static final double PROVIDER_VERSION = 1.0011;
+    private static final String PROVIDER_INFO = "Bouncy Castle JSSE Provider Version 1.0.11";
 
     private Map<String, BcJsseService> serviceMap = new HashMap<String, BcJsseService>();
     private Map<String, EngineCreator> creatorMap = new HashMap<String, EngineCreator>();
@@ -29,10 +29,15 @@ public class BouncyCastleJsseProvider
     private final boolean isInFipsMode;
 
     public BouncyCastleJsseProvider()
+    {
+        this(false);
+    }
+
+    public BouncyCastleJsseProvider(boolean fipsMode)
     {
         super(PROVIDER_NAME, PROVIDER_VERSION, PROVIDER_INFO);
 
-        this.isInFipsMode = configure(false, new JcaTlsCryptoProvider());
+        this.isInFipsMode = configure(fipsMode, new JcaTlsCryptoProvider());
     }
 
     public BouncyCastleJsseProvider(Provider provider)
@@ -146,7 +151,7 @@ public class BouncyCastleJsseProvider
         {
             public Object createInstance(Object constructorParameter)
             {
-                return new ProvKeyManagerFactorySpi(cryptoProvider.getHelper());
+                return new ProvKeyManagerFactorySpi(fipsMode, cryptoProvider.getHelper());
             }
         });
         addAlias("Alg.Alias.KeyManagerFactory.X509", "X.509");
@@ -156,7 +161,7 @@ public class BouncyCastleJsseProvider
         {
             public Object createInstance(Object constructorParameter)
             {
-                return new ProvTrustManagerFactorySpi(cryptoProvider.getHelper());
+                return new ProvTrustManagerFactorySpi(fipsMode, cryptoProvider.getHelper());
             }
         });
         addAlias("Alg.Alias.TrustManagerFactory.X.509", "PKIX");
@@ -191,18 +196,19 @@ public class BouncyCastleJsseProvider
             {
                 public Object createInstance(Object constructorParameter)
                 {
-                    return new ProvSSLContextSpi(fipsMode, cryptoProvider, new String[]{ "TLSv1.2", "TLSv1.1", "TLSv1" });
+                    return new ProvSSLContextSpi(fipsMode, cryptoProvider,
+                        new String[]{ "TLSv1.2", "TLSv1.1", "TLSv1" });
+                }
+            });
+        addAlgorithmImplementation("SSLContext.TLSV1.3", "org.bouncycastle.jsse.provider.SSLContext.TLSv1_3",
+            new EngineCreator()
+            {
+                public Object createInstance(Object constructorParameter)
+                {
+                    return new ProvSSLContextSpi(fipsMode, cryptoProvider,
+                        new String[]{ "TLSv1.3", "TLSv1.2", "TLSv1.1", "TLSv1" });
                 }
             });
-        // TODO[tls13]
-//        addAlgorithmImplementation("SSLContext.TLSV1.3", "org.bouncycastle.jsse.provider.SSLContext.TLSv1_3",
-//            new EngineCreator()
-//            {
-//                public Object createInstance(Object constructorParameter)
-//                {
-//                    return new ProvSSLContextSpi(fipsMode, cryptoProvider, new String[]{ "TLSv1.3", "TLSv1.2", "TLSv1.1", "TLSv1" });
-//                }
-//            });
         addAlgorithmImplementation("SSLContext.DEFAULT", "org.bouncycastle.jsse.provider.SSLContext.Default",
             new EngineCreator()
             {
