@@ -5,32 +5,46 @@ import java.security.Principal;
 import java.security.PrivateKey;
 import java.security.cert.X509Certificate;
 
+import javax.net.ssl.SSLEngine;
 import javax.net.ssl.X509ExtendedKeyManager;
 import javax.net.ssl.X509KeyManager;
 
-import org.bouncycastle.jcajce.util.JcaJceHelper;
+import org.bouncycastle.jsse.BCX509ExtendedKeyManager;
+import org.bouncycastle.jsse.BCX509Key;
 
-/*
- * Note that chooseEngineClientAlias() and chooseEngineServerAlias() are inherited (they return
- * null), for compatibility with SunJSSE provider.
- */
 final class ImportX509KeyManager_5
-    extends X509ExtendedKeyManager
+    extends BCX509ExtendedKeyManager
+    implements ImportX509KeyManager
 {
-    final JcaJceHelper helper;
-    final X509KeyManager x509KeyManager;
+    final X509ExtendedKeyManager x509KeyManager;
 
-    ImportX509KeyManager_5(JcaJceHelper helper, X509KeyManager x509KeyManager)
+    ImportX509KeyManager_5(X509ExtendedKeyManager x509KeyManager)
     {
-        this.helper = helper;
         this.x509KeyManager = x509KeyManager;
     }
 
+    public X509KeyManager unwrap()
+    {
+        return x509KeyManager;
+    }
+
     public String chooseClientAlias(String[] keyType, Principal[] issuers, Socket socket)
     {
         return x509KeyManager.chooseClientAlias(keyType, issuers, socket);
     }
 
+    @Override
+    public String chooseEngineClientAlias(String[] keyType, Principal[] issuers, SSLEngine engine)
+    {
+        return x509KeyManager.chooseEngineClientAlias(keyType, issuers, engine);
+    }
+
+    @Override
+    public String chooseEngineServerAlias(String keyType, Principal[] issuers, SSLEngine engine)
+    {
+        return x509KeyManager.chooseEngineServerAlias(keyType, issuers, engine);
+    }
+
     public String chooseServerAlias(String keyType, Principal[] issuers, Socket socket)
     {
         return x509KeyManager.chooseServerAlias(keyType, issuers, socket);
@@ -46,6 +60,12 @@ final class ImportX509KeyManager_5
         return x509KeyManager.getClientAliases(keyType, issuers);
     }
 
+    @Override
+    public BCX509Key getKeyBC(String alias)
+    {
+        return ProvX509Key.from(x509KeyManager, alias);
+    }
+
     public PrivateKey getPrivateKey(String alias)
     {
         return x509KeyManager.getPrivateKey(alias);
