@@ -44,17 +44,38 @@ class PropertyUtils
         }
     }
 
+    static boolean getBooleanSecurityProperty(String propertyName, boolean defaultValue)
+    {
+        String propertyValue = getSecurityProperty(propertyName);
+        if (null != propertyValue)
+        {
+            if ("true".equalsIgnoreCase(propertyValue))
+            {
+                LOG.log(Level.INFO, "Found boolean security property [" + propertyName + "]: " + true);
+                return true;
+            }
+            if ("false".equalsIgnoreCase(propertyValue))
+            {
+                LOG.log(Level.INFO, "Found boolean security property [" + propertyName + "]: " + false);
+                return false;
+            }
+            LOG.log(Level.WARNING, "Unrecognized value for boolean security property [" + propertyName + "]: " + propertyValue);
+        }
+        LOG.log(Level.FINE, "Boolean security property [" + propertyName + "] defaulted to: " + defaultValue);
+        return defaultValue;
+    }
+
     static boolean getBooleanSystemProperty(String propertyName, boolean defaultValue)
     {
         String propertyValue = getSystemProperty(propertyName);
         if (null != propertyValue)
         {
-            if ("true".equals(propertyValue))
+            if ("true".equalsIgnoreCase(propertyValue))
             {
                 LOG.log(Level.INFO, "Found boolean system property [" + propertyName + "]: " + true);
                 return true;
             }
-            if ("false".equals(propertyValue))
+            if ("false".equalsIgnoreCase(propertyValue))
             {
                 LOG.log(Level.INFO, "Found boolean system property [" + propertyName + "]: " + false);
                 return false;
@@ -93,17 +114,43 @@ class PropertyUtils
         return defaultValue;
     }
 
+    static String getStringSecurityProperty(String propertyName, String defaultValue)
+    {
+        String propertyValue = getSecurityProperty(propertyName);
+        if (null != propertyValue)
+        {
+            LOG.log(Level.INFO, "Found string security property [" + propertyName + "]: " + propertyValue);
+            return propertyValue;
+        }
+        LOG.log(Level.WARNING, "String security property [" + propertyName + "] defaulted to: " + defaultValue);
+        return defaultValue;
+    }
+
     static String getStringSystemProperty(String propertyName)
     {
         String propertyValue = getSystemProperty(propertyName);
         if (null != propertyValue)
         {
-            LOG.log(Level. INFO, "Found string system property [" + propertyName + "]: " + propertyValue);
+            LOG.log(Level.INFO, "Found string system property [" + propertyName + "]: " + propertyValue);
             return propertyValue;
         }
         return null;
     }
 
+    static String[] getStringArraySecurityProperty(String propertyName, String defaultValue)
+    {
+        String propertyValue = getStringSecurityProperty(propertyName, defaultValue);
+
+        return parseStringArray(propertyValue);
+    }
+
+    static String[] getStringArraySystemProperty(String propertyName)
+    {
+        String propertyValue = getStringSystemProperty(propertyName);
+
+        return parseStringArray(propertyValue);
+    }
+
     private static String getRangeString(int minimumValue, int maximumValue)
     {
         StringBuilder sb = new StringBuilder(32);
@@ -120,4 +167,27 @@ class PropertyUtils
         }
         return sb.toString();
     }
+
+    private static String[] parseStringArray(String propertyValue)
+    {
+        if (null == propertyValue)
+        {
+            return null;
+        }
+
+        String[] entries = JsseUtils.stripDoubleQuotes(propertyValue.trim()).split(",");
+        String[] result = new String[entries.length];
+        int count = 0;
+        for (String entry : entries)
+        {
+            entry = entry.trim();
+            if (entry.length() < 1)
+            {
+                continue;
+            }
+
+            result[count++] = entry;
+        }
+        return JsseUtils.resize(result, count);
+    }
 }
