@@ -16,16 +16,17 @@ import javax.net.ssl.SSLEngineResult.Status;
 import javax.net.ssl.SSLException;
 import javax.net.ssl.SSLParameters;
 import javax.net.ssl.SSLSession;
-import javax.net.ssl.X509ExtendedKeyManager;
 
 import org.bouncycastle.jsse.BCApplicationProtocolSelector;
 import org.bouncycastle.jsse.BCExtendedSSLSession;
 import org.bouncycastle.jsse.BCSSLConnection;
 import org.bouncycastle.jsse.BCSSLEngine;
 import org.bouncycastle.jsse.BCSSLParameters;
+import org.bouncycastle.jsse.BCX509Key;
 import org.bouncycastle.tls.AlertDescription;
 import org.bouncycastle.tls.RecordFormat;
 import org.bouncycastle.tls.RecordPreview;
+import org.bouncycastle.tls.SecurityParameters;
 import org.bouncycastle.tls.TlsClientProtocol;
 import org.bouncycastle.tls.TlsFatalAlert;
 import org.bouncycastle.tls.TlsProtocol;
@@ -156,18 +157,14 @@ class ProvSSLEngine
         }
     }
 
-    public ProvX509Key chooseClientKey(String[] keyTypes, Principal[] issuers)
+    public BCX509Key chooseClientKey(String[] keyTypes, Principal[] issuers)
     {
-        X509ExtendedKeyManager x509KeyManager = getContextData().getX509KeyManager();
-        String alias = x509KeyManager.chooseEngineClientAlias(keyTypes, issuers, this);
-        return ProvX509Key.from(x509KeyManager, alias);
+        return getContextData().getX509KeyManager().chooseEngineClientKeyBC(keyTypes, JsseUtils.clone(issuers), this);
     }
 
-    public ProvX509Key chooseServerKey(String keyType, Principal[] issuers)
+    public BCX509Key chooseServerKey(String keyType, Principal[] issuers)
     {
-        X509ExtendedKeyManager x509KeyManager = getContextData().getX509KeyManager();
-        String alias = x509KeyManager.chooseEngineServerAlias(keyType, issuers, this);
-        return ProvX509Key.from(x509KeyManager, alias);
+        return getContextData().getX509KeyManager().chooseEngineServerKeyBC(keyType, JsseUtils.clone(issuers), this);
     }
 
     @Override
@@ -455,7 +452,7 @@ class ProvSSLEngine
                 {
                     resultStatus = Status.BUFFER_UNDERFLOW;
                 }
-                else if (hasInsufficientSpace(dsts, offset, length, preview.getApplicationDataLimit()))
+                else if (hasInsufficientSpace(dsts, offset, length, preview.getContentLimit()))
                 {
                     resultStatus = Status.BUFFER_OVERFLOW;
                 }
@@ -600,7 +597,7 @@ class ProvSSLEngine
                     {
                         RecordPreview preview = protocol.previewOutputRecord(srcRemaining);
 
-                        int srcLimit = preview.getApplicationDataLimit();
+                        int srcLimit = preview.getContentLimit();
                         int dstLimit = preview.getRecordSize();
 
                         if (dst.remaining() < dstLimit)
@@ -609,21 +606,22 @@ class ProvSSLEngine
                         }
                         else
                         {
-                            for (int srcIndex = 0; srcIndex < length && srcLimit > 0; ++srcIndex)
+                            // TODO Support writing application data using ByteBuffer array directly
+
+                            byte[] applicationData = new byte[srcLimit];
+
+                            for (int srcIndex = 0; srcIndex < length && bytesConsumed < srcLimit; ++srcIndex)
                             {
                                 ByteBuffer src = srcs[offset + srcIndex];
-                                int count = Math.min(src.remaining(), srcLimit);
+                                int count = Math.min(src.remaining(), srcLimit - bytesConsumed);
                                 if (count > 0)
                                 {
-                                    byte[] input = new byte[count];
-                                    src.get(input);
-    
-                                    protocol.writeApplicationData(input, 0, count);
-    
+                                    src.get(applicationData, bytesConsumed, count);
                                     bytesConsumed += count;
-                                    srcLimit -= count;
                                 }
                             }
+
+                            protocol.writeApplicationData(applicationData, 0, bytesConsumed);
                         }
                     }
                 }
@@ -721,9 +719,23 @@ class ProvSSLEngine
         this.connection = connection;
     }
 
-    public synchronized void notifyHandshakeSession(ProvSSLSessionHandshake handshakeSession)
+    public synchronized void notifyHandshakeSession(ProvSSLSessionContext sslSessionContext,
+        SecurityParameters securityParameters, JsseSecurityParameters jsseSecurityParameters,
+        ProvSSLSession resumedSession)
     {
-        this.handshakeSession = handshakeSession;
+        String peerHost = getPeerHost();
+        int peerPort = getPeerPort();
+
+        if (null != resumedSession)
+        {
+            this.handshakeSession = new ProvSSLSessionResumed(sslSessionContext, peerHost, peerPort, securityParameters,
+                jsseSecurityParameters, resumedSession.getTlsSession(), resumedSession.getJsseSessionParameters());
+        }
+        else
+        {
+            this.handshakeSession = new ProvSSLSessionHandshake(sslSessionContext, peerHost, peerPort,
+                securityParameters, jsseSecurityParameters);
+        }
     }
 
     public synchronized String selectApplicationProtocol(List<String> protocols)
