@@ -11,16 +11,21 @@ import java.net.UnknownHostException;
 import java.security.Principal;
 import java.security.cert.CertificateException;
 import java.security.cert.X509Certificate;
+import java.util.List;
 import java.util.logging.Level;
 import java.util.logging.Logger;
 
 import javax.net.ssl.SSLParameters;
 import javax.net.ssl.SSLSession;
+import javax.net.ssl.SSLSocket;
 
+import org.bouncycastle.jsse.BCApplicationProtocolSelector;
 import org.bouncycastle.jsse.BCExtendedSSLSession;
 import org.bouncycastle.jsse.BCSSLConnection;
 import org.bouncycastle.jsse.BCSSLParameters;
+import org.bouncycastle.jsse.BCX509Key;
 import org.bouncycastle.tls.AlertDescription;
+import org.bouncycastle.tls.SecurityParameters;
 import org.bouncycastle.tls.TlsClientProtocol;
 import org.bouncycastle.tls.TlsFatalAlert;
 import org.bouncycastle.tls.TlsProtocol;
@@ -35,7 +40,6 @@ class ProvSSLSocketDirect
     protected final AppDataInput appDataIn = new AppDataInput();
     protected final AppDataOutput appDataOut = new AppDataOutput();
 
-    protected final ProvSSLContextSpi context;
     protected final ContextData contextData;
     protected final ProvSSLParameters sslParameters;
 
@@ -50,70 +54,57 @@ class ProvSSLSocketDirect
     protected ProvSSLSessionHandshake handshakeSession = null;
 
     /** This constructor is the one used (only) by ProvSSLServerSocket */
-    ProvSSLSocketDirect(ProvSSLContextSpi context, ContextData contextData, boolean enableSessionCreation,
+    ProvSSLSocketDirect(ContextData contextData, boolean enableSessionCreation,
         boolean useClientMode, ProvSSLParameters sslParameters)
     {
-        super();
-
-        this.context = context;
         this.contextData = contextData;
         this.enableSessionCreation = enableSessionCreation;
         this.useClientMode = useClientMode;
         this.sslParameters = sslParameters;
     }
 
-    protected ProvSSLSocketDirect(ProvSSLContextSpi context, ContextData contextData)
+    protected ProvSSLSocketDirect(ContextData contextData)
     {
-        super();
-
-        this.context = context;
         this.contextData = contextData;
-        this.sslParameters = context.getDefaultParameters(!useClientMode);
+        this.sslParameters = contextData.getContext().getDefaultSSLParameters(useClientMode);
     }
 
-    protected ProvSSLSocketDirect(ProvSSLContextSpi context, ContextData contextData, InetAddress address, int port, InetAddress clientAddress, int clientPort)
+    protected ProvSSLSocketDirect(ContextData contextData, InetAddress address, int port, InetAddress clientAddress, int clientPort)
         throws IOException
     {
-        super(address, port, clientAddress, clientPort);
-
-        this.context = context;
         this.contextData = contextData;
-        this.sslParameters = context.getDefaultParameters(!useClientMode);
+        this.sslParameters = contextData.getContext().getDefaultSSLParameters(useClientMode);
+
+        implBind(clientAddress, clientPort);
+        implConnect(address, port);
     }
 
-    protected ProvSSLSocketDirect(ProvSSLContextSpi context, ContextData contextData, InetAddress address, int port) throws IOException
+    protected ProvSSLSocketDirect(ContextData contextData, InetAddress address, int port) throws IOException
     {
-        super(address, port);
-
-        this.context = context;
         this.contextData = contextData;
-        this.sslParameters = context.getDefaultParameters(!useClientMode);
+        this.sslParameters = contextData.getContext().getDefaultSSLParameters(useClientMode);
+
+        implConnect(address, port);
     }
 
-    protected ProvSSLSocketDirect(ProvSSLContextSpi context, ContextData contextData, String host, int port, InetAddress clientAddress, int clientPort)
+    protected ProvSSLSocketDirect(ContextData contextData, String host, int port, InetAddress clientAddress, int clientPort)
         throws IOException, UnknownHostException
     {
-        super(host, port, clientAddress, clientPort);
-
-        this.context = context;
         this.contextData = contextData;
-        this.sslParameters = context.getDefaultParameters(!useClientMode);
+        this.sslParameters = contextData.getContext().getDefaultSSLParameters(useClientMode);
         this.peerHost = host;
+
+        implBind(clientAddress, clientPort);
+        implConnect(host, port);
     }
 
-    protected ProvSSLSocketDirect(ProvSSLContextSpi context, ContextData contextData, String host, int port) throws IOException, UnknownHostException
+    protected ProvSSLSocketDirect(ContextData contextData, String host, int port) throws IOException, UnknownHostException
     {
-        super(host, port);
-
-        this.context = context;
         this.contextData = contextData;
-        this.sslParameters = context.getDefaultParameters(!useClientMode);
+        this.sslParameters = contextData.getContext().getDefaultSSLParameters(useClientMode);
         this.peerHost = host;
-    }
 
-    public ProvSSLContextSpi getContext()
-    {
-        return context;
+        implConnect(host, port);
     }
 
     public ContextData getContextData()
@@ -125,7 +116,7 @@ class ProvSSLSocketDirect
     {
         try
         {
-            contextData.getX509TrustManager().checkClientTrusted(chain, authType, this);
+            contextData.getX509TrustManager().checkClientTrusted(chain.clone(), authType, this);
         }
         catch (CertificateException e)
         {
@@ -137,7 +128,7 @@ class ProvSSLSocketDirect
     {
         try
         {
-            contextData.getX509TrustManager().checkServerTrusted(chain, authType, this);
+            contextData.getX509TrustManager().checkServerTrusted(chain.clone(), authType, this);
         }
         catch (CertificateException e)
         {
@@ -145,14 +136,14 @@ class ProvSSLSocketDirect
         }
     }
 
-    public String chooseClientAlias(String[] keyType, Principal[] issuers)
+    public BCX509Key chooseClientKey(String[] keyTypes, Principal[] issuers)
     {
-        return contextData.getX509KeyManager().chooseClientAlias(keyType, issuers, this);
+        return getContextData().getX509KeyManager().chooseClientKeyBC(keyTypes, JsseUtils.clone(issuers), this);
     }
 
-    public String chooseServerAlias(String keyType, Principal[] issuers)
+    public BCX509Key chooseServerKey(String keyType, Principal[] issuers)
     {
-        return contextData.getX509KeyManager().chooseServerAlias(keyType, issuers, this);
+        return getContextData().getX509KeyManager().chooseServerKeyBC(keyType, JsseUtils.clone(issuers), this);
     }
 
     @Override
@@ -181,11 +172,51 @@ class ProvSSLSocketDirect
         notifyConnected();
     }
 
+    @Override
+    protected void finalize() throws Throwable
+    {
+        try
+        {
+            close();
+        }
+        catch (IOException e1)
+        {
+            try
+            {
+                super.close();
+            }
+            catch (IOException e2)
+            {
+                // Ignore
+            }
+        }
+        finally
+        {
+            super.finalize();
+        }
+    }
+
+    // An SSLSocket method from JDK 9, but also a BCSSLSocket method
+    public synchronized String getApplicationProtocol()
+    {
+        return null == connection ? null : connection.getApplicationProtocol();
+    }
+
+    public synchronized BCApplicationProtocolSelector<SSLSocket> getBCHandshakeApplicationProtocolSelector()
+    {
+        return sslParameters.getSocketAPSelector();
+    }
+
     public synchronized BCExtendedSSLSession getBCHandshakeSession()
     {
         return handshakeSession;
     }
 
+    public BCExtendedSSLSession getBCSession()
+    {
+        return getSessionImpl();
+    }
+
     public synchronized BCSSLConnection getConnection()
     {
         try
@@ -218,7 +249,13 @@ class ProvSSLSocketDirect
         return enableSessionCreation;
     }
 
-    @Override
+    // An SSLSocket method from JDK 9, but also a BCSSLSocket method
+    public synchronized String getHandshakeApplicationProtocol()
+    {
+        return null == handshakeSession ? null : handshakeSession.getApplicationProtocol();
+    }
+
+    // An SSLSocket method from JDK 7
     public synchronized SSLSession getHandshakeSession()
     {
         return null == handshakeSession ? null : handshakeSession.getExportSSLSession();
@@ -248,16 +285,12 @@ class ProvSSLSocketDirect
     }
 
     @Override
-    public synchronized SSLSession getSession()
+    public SSLSession getSession()
     {
-        getConnection();
-
-        ProvSSLSession sslSession = (null == connection) ? ProvSSLSession.NULL_SESSION : connection.getSession();
-
-        return sslSession.getExportSSLSession();
+        return getSessionImpl().getExportSSLSession();
     }
 
-    @Override
+    // An SSLSocket method from JDK 6
     public synchronized SSLParameters getSSLParameters()
     {
         return SSLParametersUtil.getSSLParameters(sslParameters);
@@ -266,13 +299,13 @@ class ProvSSLSocketDirect
     @Override
     public synchronized String[] getSupportedCipherSuites()
     {
-        return context.getSupportedCipherSuites();
+        return contextData.getContext().getSupportedCipherSuites();
     }
 
     @Override
     public synchronized String[] getSupportedProtocols()
     {
-        return context.getSupportedProtocols();
+        return contextData.getContext().getSupportedProtocols();
     }
 
     @Override
@@ -287,6 +320,29 @@ class ProvSSLSocketDirect
         return sslParameters.getWantClientAuth();
     }
 
+    public synchronized void setBCHandshakeApplicationProtocolSelector(BCApplicationProtocolSelector<SSLSocket> selector)
+    {
+        sslParameters.setSocketAPSelector(selector);
+    }
+
+    public synchronized void setBCSessionToResume(BCExtendedSSLSession session)
+    {
+        if (null == session)
+        {
+            throw new NullPointerException("'session' cannot be null");
+        }
+        if (!(session instanceof ProvSSLSession))
+        {
+            throw new IllegalArgumentException("Session-to-resume must be a session returned from 'getBCSession'");
+        }
+        if (null != protocol)
+        {
+            throw new IllegalArgumentException("Session-to-resume cannot be set after the handshake has begun");
+        }
+
+        sslParameters.setSessionToResume((ProvSSLSession)session);
+    }
+
     @Override
     public synchronized void setEnabledCipherSuites(String[] suites)
     {
@@ -322,7 +378,7 @@ class ProvSSLSocketDirect
         SSLParametersUtil.setParameters(this.sslParameters, parameters);
     }
 
-    @Override
+    // An SSLSocket method from JDK 6
     public synchronized void setSSLParameters(SSLParameters sslParameters)
     {
         SSLParametersUtil.setSSLParameters(this.sslParameters, sslParameters);
@@ -338,7 +394,7 @@ class ProvSSLSocketDirect
 
         if (this.useClientMode != useClientMode)
         {
-            context.updateDefaultProtocols(sslParameters, !useClientMode);
+            contextData.getContext().updateDefaultSSLParameters(sslParameters, useClientMode);
 
             this.useClientMode = useClientMode;
         }
@@ -350,6 +406,20 @@ class ProvSSLSocketDirect
         sslParameters.setWantClientAuth(want);
     }
 
+    @Override
+    public void shutdownInput() throws IOException
+    {
+        // TODO[tls13]
+        throw new UnsupportedOperationException("shutdownInput() not supported in TLS");
+    }
+
+    @Override
+    public void shutdownOutput() throws IOException
+    {
+        // TODO[tls13]
+        throw new UnsupportedOperationException("shutdownOutput() not supported in TLS");
+    }
+
     @Override
     public synchronized void startHandshake() throws IOException
     {
@@ -369,7 +439,7 @@ class ProvSSLSocketDirect
                 clientProtocol.setResumableHandshake(resumable);
                 this.protocol = clientProtocol;
 
-                ProvTlsClient client = new ProvTlsClient(this, sslParameters.copy());
+                ProvTlsClient client = new ProvTlsClient(this, sslParameters);
                 this.protocolPeer = client;
 
                 clientProtocol.connect(client);
@@ -380,7 +450,7 @@ class ProvSSLSocketDirect
                 serverProtocol.setResumableHandshake(resumable);
                 this.protocol = serverProtocol;
 
-                ProvTlsServer server = new ProvTlsServer(this, sslParameters.copy());
+                ProvTlsServer server = new ProvTlsServer(this, sslParameters);
                 this.protocolPeer = server;
 
                 serverProtocol.accept(server);
@@ -414,18 +484,51 @@ class ProvSSLSocketDirect
 
     public synchronized void notifyHandshakeComplete(ProvSSLConnection connection)
     {
-        if (null != handshakeSession && !handshakeSession.isValid())
+        if (null != handshakeSession)
         {
-            connection.getSession().invalidate();
+            if (!handshakeSession.isValid())
+            {
+                connection.getSession().invalidate();
+            }
+
+            handshakeSession.getJsseSecurityParameters().clear();
         }
 
         this.handshakeSession = null;
         this.connection = connection;
+
+        notifyHandshakeCompletedListeners(connection.getSession().exportSSLSession);
     }
 
-    public synchronized void notifyHandshakeSession(ProvSSLSessionHandshake handshakeSession)
+    public synchronized void notifyHandshakeSession(ProvSSLSessionContext sslSessionContext,
+        SecurityParameters securityParameters, JsseSecurityParameters jsseSecurityParameters,
+        ProvSSLSession resumedSession)
     {
-        this.handshakeSession = handshakeSession;
+        String peerHost = getPeerHost();
+        int peerPort = getPeerPort();
+
+        if (null != resumedSession)
+        {
+            this.handshakeSession = new ProvSSLSessionResumed(sslSessionContext, peerHost, peerPort, securityParameters,
+                jsseSecurityParameters, resumedSession.getTlsSession(), resumedSession.getJsseSessionParameters());
+        }
+        else
+        {
+            this.handshakeSession = new ProvSSLSessionHandshake(sslSessionContext, peerHost, peerPort,
+                securityParameters, jsseSecurityParameters);
+        }
+    }
+
+    public synchronized String selectApplicationProtocol(List<String> protocols)
+    {
+        return sslParameters.getSocketAPSelector().select(this, protocols);
+    }
+
+    synchronized ProvSSLSession getSessionImpl()
+    {
+        getConnection();
+
+        return null == connection ? ProvSSLSession.NULL_SESSION : connection.getSession();
     }
 
     synchronized void handshakeIfNecessary(boolean resumable) throws IOException
@@ -476,6 +579,26 @@ class ProvSSLSocketDirect
 
     class AppDataInput extends InputStream
     {
+        @Override
+        public int read() throws IOException
+        {
+            byte[] buf = new byte[1];
+            int ret = read(buf, 0, 1);
+            return ret <= 0 ? -1 : buf[0] & 0xFF;
+        }
+
+        @Override
+        public int read(byte[] b, int off, int len) throws IOException
+        {
+            if (len < 1)
+            {
+                return 0;
+            }
+
+            handshakeIfNecessary(true);
+            return protocol.readApplicationData(b, off, len);
+        }
+
         @Override
         public int available() throws IOException
         {
@@ -492,57 +615,14 @@ class ProvSSLSocketDirect
         {
             ProvSSLSocketDirect.this.close();
         }
-
-        @Override
-        public int read() throws IOException
-        {
-            handshakeIfNecessary(true);
-
-            byte[] buf = new byte[1];
-            int ret = protocol.readApplicationData(buf, 0, 1);
-            return ret < 0 ? -1 : buf[0] & 0xFF;
-        }
-
-        @Override
-        public int read(byte[] b, int off, int len) throws IOException
-        {
-            if (len < 1)
-            {
-                return 0;
-            }
-
-            handshakeIfNecessary(true);
-            return protocol.readApplicationData(b, off, len);
-        }
     }
 
     class AppDataOutput extends OutputStream
     {
-        @Override
-        public void close() throws IOException
-        {
-            ProvSSLSocketDirect.this.close();
-        }
-
-        @Override
-        public void flush() throws IOException
-        {
-            synchronized (ProvSSLSocketDirect.this)
-            {
-                if (protocol != null)
-                {
-                    protocol.flush();
-                }
-            }
-        }
-
         @Override
         public void write(int b) throws IOException
         {
-            handshakeIfNecessary(true);
-
-            byte[] buf = new byte[]{ (byte)b };
-            protocol.writeApplicationData(buf, 0, 1);
+            write(new byte[]{ (byte)b }, 0, 1);
         }
 
         @Override
@@ -554,5 +634,11 @@ class ProvSSLSocketDirect
                 protocol.writeApplicationData(b, off, len);
             }
         }
+
+        @Override
+        public void close() throws IOException
+        {
+            ProvSSLSocketDirect.this.close();
+        }
     }
 }
