@@ -12,16 +12,21 @@ import java.nio.channels.SocketChannel;
 import java.security.Principal;
 import java.security.cert.CertificateException;
 import java.security.cert.X509Certificate;
+import java.util.List;
 import java.util.logging.Level;
 import java.util.logging.Logger;
 
 import javax.net.ssl.SSLParameters;
 import javax.net.ssl.SSLSession;
+import javax.net.ssl.SSLSocket;
 
+import org.bouncycastle.jsse.BCApplicationProtocolSelector;
 import org.bouncycastle.jsse.BCExtendedSSLSession;
 import org.bouncycastle.jsse.BCSSLConnection;
 import org.bouncycastle.jsse.BCSSLParameters;
+import org.bouncycastle.jsse.BCX509Key;
 import org.bouncycastle.tls.AlertDescription;
+import org.bouncycastle.tls.SecurityParameters;
 import org.bouncycastle.tls.TlsClientProtocol;
 import org.bouncycastle.tls.TlsFatalAlert;
 import org.bouncycastle.tls.TlsProtocol;
@@ -49,7 +54,6 @@ class ProvSSLSocketWrap
     protected final AppDataInput appDataIn = new AppDataInput();
     protected final AppDataOutput appDataOut = new AppDataOutput();
 
-    protected final ProvSSLContextSpi context;
     protected final ContextData contextData;
     protected final Socket wrapSocket;
     protected final InputStream consumed;
@@ -66,44 +70,33 @@ class ProvSSLSocketWrap
     protected ProvSSLConnection connection = null;
     protected ProvSSLSessionHandshake handshakeSession = null;
 
-    protected ProvSSLSocketWrap(ProvSSLContextSpi context, ContextData contextData, Socket s, InputStream consumed, boolean autoClose)
+    protected ProvSSLSocketWrap(ContextData contextData, Socket s, InputStream consumed, boolean autoClose)
         throws IOException
     {
-        super();
-
-        this.context = context;
         this.contextData = contextData;
         this.wrapSocket = checkSocket(s);
         this.consumed = consumed;
         this.autoClose = autoClose;
         this.useClientMode = false;
-        this.sslParameters = context.getDefaultParameters(!useClientMode);
+        this.sslParameters = contextData.getContext().getDefaultSSLParameters(useClientMode);
 
         notifyConnected();
     }
 
-    protected ProvSSLSocketWrap(ProvSSLContextSpi context, ContextData contextData, Socket s, String host, int port, boolean autoClose)
+    protected ProvSSLSocketWrap(ContextData contextData, Socket s, String host, int port, boolean autoClose)
         throws IOException
     {
-        super();
-
-        this.context = context;
         this.contextData = contextData;
         this.wrapSocket = checkSocket(s);
         this.consumed = null;
         this.peerHost = host;
         this.autoClose = autoClose;
         this.useClientMode = true;
-        this.sslParameters = context.getDefaultParameters(!useClientMode);
+        this.sslParameters = contextData.getContext().getDefaultSSLParameters(useClientMode);
 
         notifyConnected();
     }
 
-    public ProvSSLContextSpi getContext()
-    {
-        return context;
-    }
-
     public ContextData getContextData()
     {
         return contextData;
@@ -119,7 +112,7 @@ class ProvSSLSocketWrap
     {
         try
         {
-            contextData.getX509TrustManager().checkClientTrusted(chain, authType, this);
+            contextData.getX509TrustManager().checkClientTrusted(chain.clone(), authType, this);
         }
         catch (CertificateException e)
         {
@@ -131,7 +124,7 @@ class ProvSSLSocketWrap
     {
         try
         {
-            contextData.getX509TrustManager().checkServerTrusted(chain, authType, this);
+            contextData.getX509TrustManager().checkServerTrusted(chain.clone(), authType, this);
         }
         catch (CertificateException e)
         {
@@ -139,14 +132,14 @@ class ProvSSLSocketWrap
         }
     }
 
-    public String chooseClientAlias(String[] keyType, Principal[] issuers)
+    public BCX509Key chooseClientKey(String[] keyTypes, Principal[] issuers)
     {
-        return contextData.getX509KeyManager().chooseClientAlias(keyType, issuers, this);
+        return getContextData().getX509KeyManager().chooseClientKeyBC(keyTypes, JsseUtils.clone(issuers), this);
     }
 
-    public String chooseServerAlias(String keyType, Principal[] issuers)
+    public BCX509Key chooseServerKey(String keyType, Principal[] issuers)
     {
-        return contextData.getX509KeyManager().chooseServerAlias(keyType, issuers, this);
+        return getContextData().getX509KeyManager().chooseServerKeyBC(keyType, JsseUtils.clone(issuers), this);
     }
 
     @Override
@@ -185,9 +178,31 @@ class ProvSSLSocketWrap
     }
 
     @Override
-    public SocketChannel getChannel()
+    protected void finalize() throws Throwable
     {
-        return wrapSocket.getChannel();
+        try
+        {
+            close();
+        }
+        catch (IOException e)
+        {
+            // Ignore
+        }
+        finally
+        {
+            super.finalize();
+        }
+    }
+
+    // An SSLSocket method from JDK 9, but also a BCSSLSocket method
+    public synchronized String getApplicationProtocol()
+    {
+        return null == connection ? null : connection.getApplicationProtocol();
+    }
+
+    public synchronized BCApplicationProtocolSelector<SSLSocket> getBCHandshakeApplicationProtocolSelector()
+    {
+        return sslParameters.getSocketAPSelector();
     }
 
     public synchronized BCExtendedSSLSession getBCHandshakeSession()
@@ -195,6 +210,17 @@ class ProvSSLSocketWrap
         return handshakeSession;
     }
 
+    public BCExtendedSSLSession getBCSession()
+    {
+        return getSessionImpl();
+    }
+
+    @Override
+    public SocketChannel getChannel()
+    {
+        return wrapSocket.getChannel();
+    }
+
     public synchronized BCSSLConnection getConnection()
     {
         try
@@ -227,7 +253,13 @@ class ProvSSLSocketWrap
         return enableSessionCreation;
     }
 
-    @Override
+    // An SSLSocket method from JDK 9, but also a BCSSLSocket method
+    public synchronized String getHandshakeApplicationProtocol()
+    {
+        return null == handshakeSession ? null : handshakeSession.getApplicationProtocol();
+    }
+
+    // An SSLSocket method from JDK 7
     public synchronized SSLSession getHandshakeSession()
     {
         return null == handshakeSession ? null : handshakeSession.getExportSSLSession();
@@ -312,13 +344,9 @@ class ProvSSLSocketWrap
     }
 
     @Override
-    public synchronized SSLSession getSession()
+    public SSLSession getSession()
     {
-        getConnection();
-
-        ProvSSLSession sslSession = (null == connection) ? ProvSSLSession.NULL_SESSION : connection.getSession();
-
-        return sslSession.getExportSSLSession();
+        return getSessionImpl().getExportSSLSession();
     }
 
     @Override
@@ -338,7 +366,7 @@ class ProvSSLSocketWrap
         return SSLParametersUtil.getParameters(sslParameters);
     }
 
-    @Override
+    // An SSLSocket method from JDK 6
     public synchronized SSLParameters getSSLParameters()
     {
         return SSLParametersUtil.getSSLParameters(sslParameters);
@@ -347,13 +375,13 @@ class ProvSSLSocketWrap
     @Override
     public synchronized String[] getSupportedCipherSuites()
     {
-        return context.getSupportedCipherSuites();
+        return contextData.getContext().getSupportedCipherSuites();
     }
 
     @Override
     public synchronized String[] getSupportedProtocols()
     {
-        return context.getSupportedProtocols();
+        return contextData.getContext().getSupportedProtocols();
     }
 
     @Override
@@ -410,6 +438,29 @@ class ProvSSLSocketWrap
         return wrapSocket.isOutputShutdown();
     }
 
+    public synchronized void setBCHandshakeApplicationProtocolSelector(BCApplicationProtocolSelector<SSLSocket> selector)
+    {
+        sslParameters.setSocketAPSelector(selector);
+    }
+
+    public synchronized void setBCSessionToResume(BCExtendedSSLSession session)
+    {
+        if (null == session)
+        {
+            throw new NullPointerException("'session' cannot be null");
+        }
+        if (!(session instanceof ProvSSLSession))
+        {
+            throw new IllegalArgumentException("Session-to-resume must be a session returned from 'getBCSession'");
+        }
+        if (null != protocol)
+        {
+            throw new IllegalArgumentException("Session-to-resume cannot be set after the handshake has begun");
+        }
+
+        sslParameters.setSessionToResume((ProvSSLSession)session);
+    }
+
     @Override
     public synchronized void setEnabledCipherSuites(String[] suites)
     {
@@ -487,7 +538,7 @@ class ProvSSLSocketWrap
         wrapSocket.setSoTimeout(timeout);
     }
 
-    @Override
+    // An SSLSocket method from JDK 6
     public synchronized void setSSLParameters(SSLParameters sslParameters)
     {
         SSLParametersUtil.setSSLParameters(this.sslParameters, sslParameters);
@@ -515,7 +566,7 @@ class ProvSSLSocketWrap
 
         if (this.useClientMode != useClientMode)
         {
-            context.updateDefaultProtocols(sslParameters, !useClientMode);
+            contextData.getContext().updateDefaultSSLParameters(sslParameters, useClientMode);
 
             this.useClientMode = useClientMode;
         }
@@ -527,6 +578,18 @@ class ProvSSLSocketWrap
         sslParameters.setWantClientAuth(want);
     }
 
+    @Override
+    public void shutdownInput() throws IOException
+    {
+        wrapSocket.shutdownInput();
+    }
+
+    @Override
+    public void shutdownOutput() throws IOException
+    {
+        wrapSocket.shutdownOutput();
+    }
+
     @Override
     public synchronized void startHandshake() throws IOException
     {
@@ -551,7 +614,7 @@ class ProvSSLSocketWrap
                 clientProtocol.setResumableHandshake(resumable);
                 this.protocol = clientProtocol;
 
-                ProvTlsClient client = new ProvTlsClient(this, sslParameters.copy());
+                ProvTlsClient client = new ProvTlsClient(this, sslParameters);
                 this.protocolPeer = client;
 
                 clientProtocol.connect(client);
@@ -562,7 +625,7 @@ class ProvSSLSocketWrap
                 serverProtocol.setResumableHandshake(resumable);
                 this.protocol = serverProtocol;
 
-                ProvTlsServer server = new ProvTlsServer(this, sslParameters.copy());
+                ProvTlsServer server = new ProvTlsServer(this, sslParameters);
                 this.protocolPeer = server;
 
                 serverProtocol.accept(server);
@@ -603,18 +666,51 @@ class ProvSSLSocketWrap
 
     public synchronized void notifyHandshakeComplete(ProvSSLConnection connection)
     {
-        if (null != handshakeSession && !handshakeSession.isValid())
+        if (null != handshakeSession)
         {
-            connection.getSession().invalidate();
+            if (!handshakeSession.isValid())
+            {
+                connection.getSession().invalidate();
+            }
+
+            handshakeSession.getJsseSecurityParameters().clear();
         }
 
         this.handshakeSession = null;
         this.connection = connection;
+
+        notifyHandshakeCompletedListeners(connection.getSession().exportSSLSession);
     }
 
-    public synchronized void notifyHandshakeSession(ProvSSLSessionHandshake handshakeSession)
+    public synchronized void notifyHandshakeSession(ProvSSLSessionContext sslSessionContext,
+        SecurityParameters securityParameters, JsseSecurityParameters jsseSecurityParameters,
+        ProvSSLSession resumedSession)
     {
-        this.handshakeSession = handshakeSession;
+        String peerHost = getPeerHost();
+        int peerPort = getPeerPort();
+
+        if (null != resumedSession)
+        {
+            this.handshakeSession = new ProvSSLSessionResumed(sslSessionContext, peerHost, peerPort, securityParameters,
+                jsseSecurityParameters, resumedSession.getTlsSession(), resumedSession.getJsseSessionParameters());
+        }
+        else
+        {
+            this.handshakeSession = new ProvSSLSessionHandshake(sslSessionContext, peerHost, peerPort,
+                securityParameters, jsseSecurityParameters);
+        }
+    }
+
+    public synchronized String selectApplicationProtocol(List<String> protocols)
+    {
+        return sslParameters.getSocketAPSelector().select(this, protocols);
+    }
+
+    synchronized ProvSSLSession getSessionImpl()
+    {
+        getConnection();
+
+        return null == connection ? ProvSSLSession.NULL_SESSION : connection.getSession();
     }
 
     synchronized void handshakeIfNecessary(boolean resumable) throws IOException
@@ -713,25 +809,10 @@ class ProvSSLSocketWrap
             ProvSSLSocketWrap.this.close();
         }
 
-        @Override
-        public void flush() throws IOException
-        {
-            synchronized (ProvSSLSocketWrap.this)
-            {
-                if (protocol != null)
-                {
-                    protocol.flush();
-                }
-            }
-        }
-
         @Override
         public void write(int b) throws IOException
         {
-            handshakeIfNecessary(true);
-
-            byte[] buf = new byte[]{ (byte)b };
-            protocol.writeApplicationData(buf, 0, 1);
+            write(new byte[]{ (byte)b }, 0, 1);
         }
 
         @Override
