@@ -3,6 +3,10 @@ package org.bouncycastle.jsse.provider;
 import java.io.IOException;
 import java.security.Principal;
 import java.security.cert.X509Certificate;
+import java.util.List;
+
+import org.bouncycastle.jsse.BCX509Key;
+import org.bouncycastle.tls.SecurityParameters;
 
 interface ProvTlsManager
 {
@@ -10,11 +14,9 @@ interface ProvTlsManager
 
     void checkServerTrusted(X509Certificate[] chain, String authType) throws IOException;
 
-    String chooseClientAlias(String[] keyType, Principal[] issuers);
+    BCX509Key chooseClientKey(String[] keyTypes, Principal[] issuers);
 
-    String chooseServerAlias(String keyType, Principal[] issuers);
-
-    ProvSSLContextSpi getContext();
+    BCX509Key chooseServerKey(String keyType, Principal[] issuers);
 
     boolean getEnableSessionCreation();
 
@@ -28,5 +30,8 @@ interface ProvTlsManager
 
     void notifyHandshakeComplete(ProvSSLConnection connection);
 
-    void notifyHandshakeSession(ProvSSLSessionHandshake handshakeSession);
+    void notifyHandshakeSession(ProvSSLSessionContext sslSessionContext, SecurityParameters securityParameters,
+        JsseSecurityParameters jsseSecurityParameters, ProvSSLSession resumedSession);
+
+    String selectApplicationProtocol(List<String> protocols);
 }
