@@ -1,6 +1,7 @@
 package org.bouncycastle.jsse.provider;
 
 import java.io.IOException;
+import java.security.Principal;
 import java.security.cert.X509Certificate;
 import java.util.Collection;
 import java.util.Collections;
@@ -12,13 +13,10 @@ import java.util.Vector;
 import java.util.logging.Level;
 import java.util.logging.Logger;
 
-import javax.net.ssl.SSLException;
-import javax.security.auth.x500.X500Principal;
-
 import org.bouncycastle.asn1.x500.X500Name;
 import org.bouncycastle.jsse.BCSNIMatcher;
 import org.bouncycastle.jsse.BCSNIServerName;
-import org.bouncycastle.jsse.BCX509ExtendedTrustManager;
+import org.bouncycastle.jsse.BCX509Key;
 import org.bouncycastle.jsse.java.security.BCAlgorithmConstraints;
 import org.bouncycastle.tls.AlertDescription;
 import org.bouncycastle.tls.AlertLevel;
@@ -32,6 +30,7 @@ import org.bouncycastle.tls.ProtocolName;
 import org.bouncycastle.tls.ProtocolVersion;
 import org.bouncycastle.tls.SecurityParameters;
 import org.bouncycastle.tls.ServerName;
+import org.bouncycastle.tls.SessionParameters;
 import org.bouncycastle.tls.SignatureAlgorithm;
 import org.bouncycastle.tls.SignatureAndHashAlgorithm;
 import org.bouncycastle.tls.TlsCredentials;
@@ -39,8 +38,11 @@ import org.bouncycastle.tls.TlsExtensionsUtils;
 import org.bouncycastle.tls.TlsFatalAlert;
 import org.bouncycastle.tls.TlsSession;
 import org.bouncycastle.tls.TlsUtils;
+import org.bouncycastle.tls.TrustedAuthority;
 import org.bouncycastle.tls.crypto.TlsCertificate;
 import org.bouncycastle.tls.crypto.impl.jcajce.JcaTlsCrypto;
+import org.bouncycastle.util.Arrays;
+import org.bouncycastle.util.encoders.Hex;
 
 class ProvTlsServer
     extends DefaultTlsServer
@@ -48,13 +50,21 @@ class ProvTlsServer
 {
     private static final Logger LOG = Logger.getLogger(ProvTlsServer.class.getName());
 
+    // TODO[jsse] Integrate this into NamedGroupInfo
     private static final int provEphemeralDHKeySize = PropertyUtils.getIntegerSystemProperty("jdk.tls.ephemeralDHKeySize", 2048, 1024, 8192);
 
-    // TODO[jsse] Support providing OCSP CertificateStatus
+    // TODO[resumption] Enable by default in due course
+    private static final boolean provServerEnableSessionResumption = PropertyUtils
+        .getBooleanSystemProperty("org.bouncycastle.jsse.server.enableSessionResumption", false);
+
+    // TODO[jsse] Support status_request and status_request_v2 extensions
 //    private static final boolean provServerEnableStatusRequest = PropertyUtils.getBooleanSystemProperty(
 //        "jdk.tls.server.enableStatusRequestExtension", false);
     private static final boolean provServerEnableStatusRequest = false;
 
+    private static final boolean provServerEnableTrustedCAKeys = PropertyUtils
+        .getBooleanSystemProperty("org.bouncycastle.jsse.server.enableTrustedCAKeysExtension", false);
+
     protected final ProvTlsManager manager;
     protected final ProvSSLParameters sslParameters;
     protected final JsseSecurityParameters jsseSecurityParameters = new JsseSecurityParameters();
@@ -65,17 +75,12 @@ class ProvTlsServer
     protected TlsCredentials credentials = null;
     protected boolean handshakeComplete = false;
 
-    ProvTlsServer(ProvTlsManager manager, ProvSSLParameters sslParameters) throws SSLException
+    ProvTlsServer(ProvTlsManager manager, ProvSSLParameters sslParameters)
     {
         super(manager.getContextData().getCrypto());
 
         this.manager = manager;
         this.sslParameters = sslParameters.copyForConnection();
-
-        if (!manager.getEnableSessionCreation())
-        {
-            throw new SSLException("Session resumption not implemented yet and session creation is disabled");
-        }
     }
 
     @Override
@@ -84,20 +89,29 @@ class ProvTlsServer
         return provServerEnableStatusRequest;
     }
 
+    @Override
+    protected boolean allowMultiCertStatus()
+    {
+        return provServerEnableStatusRequest;
+    }
+
+    @Override
+    protected boolean allowTrustedCAIndication()
+    {
+        return null != jsseSecurityParameters.trustedIssuers;
+    }
+
     @Override
     protected int getMaximumNegotiableCurveBits()
     {
-        int[] clientSupportedGroups = context.getSecurityParametersHandshake().getClientSupportedGroups();
-        return SupportedGroups.getServerMaximumNegotiableCurveBits(manager.getContextData().getContext().isFips(),
-            clientSupportedGroups);
+        return NamedGroupInfo.getMaximumBitsServerECDH(jsseSecurityParameters.namedGroups);
     }
 
     @Override
     protected int getMaximumNegotiableFiniteFieldBits()
     {
-        int[] clientSupportedGroups = context.getSecurityParametersHandshake().getClientSupportedGroups();
-        int maxBits = SupportedGroups.getServerMaximumNegotiableFiniteFieldBits(
-            manager.getContextData().getContext().isFips(), clientSupportedGroups);
+        int maxBits = NamedGroupInfo.getMaximumBitsServerFFDHE(jsseSecurityParameters.namedGroups);
+
         return maxBits >= provEphemeralDHKeySize ? maxBits : 0;
     }
 
@@ -129,7 +143,7 @@ class ProvTlsServer
     @Override
     protected boolean selectCipherSuite(int cipherSuite) throws IOException
     {
-        TlsCredentials cipherSuiteCredentials = selectCredentials(cipherSuite);
+        TlsCredentials cipherSuiteCredentials = selectCredentials(jsseSecurityParameters.trustedIssuers, cipherSuite);
 
         if (null == cipherSuiteCredentials)
         {
@@ -151,40 +165,25 @@ class ProvTlsServer
     {
         minimumFiniteFieldBits = Math.max(minimumFiniteFieldBits, provEphemeralDHKeySize);
 
-        int[] clientSupportedGroups = context.getSecurityParametersHandshake().getClientSupportedGroups();
-        if (clientSupportedGroups == null)
-        {
-            return selectDHDefault(minimumFiniteFieldBits);
-        }
-
-        return SupportedGroups.getServerSelectedFiniteField(getCrypto(), manager.getContextData().getContext().isFips(),
-            minimumFiniteFieldBits, clientSupportedGroups);
+        return NamedGroupInfo.selectServerFFDHE(jsseSecurityParameters.namedGroups, minimumFiniteFieldBits);
     }
 
     @Override
     protected int selectDHDefault(int minimumFiniteFieldBits)
     {
-        return SupportedGroups.getServerDefaultDH(manager.getContextData().getContext().isFips(),
-            minimumFiniteFieldBits);
+        throw new UnsupportedOperationException();
     }
 
     @Override
     protected int selectECDH(int minimumCurveBits)
     {
-        int[] clientSupportedGroups = context.getSecurityParametersHandshake().getClientSupportedGroups();
-        if (clientSupportedGroups == null)
-        {
-            return selectECDHDefault(minimumCurveBits);
-        }
-
-        return SupportedGroups.getServerSelectedCurve(getCrypto(), manager.getContextData().getContext().isFips(),
-            minimumCurveBits, clientSupportedGroups);
+        return NamedGroupInfo.selectServerECDH(jsseSecurityParameters.namedGroups, minimumCurveBits);
     }
 
     @Override
     protected int selectECDHDefault(int minimumCurveBits)
     {
-        return SupportedGroups.getServerDefaultECDH(manager.getContextData().getContext().isFips(), minimumCurveBits);
+        throw new UnsupportedOperationException();
     }
 
     @Override
@@ -243,70 +242,113 @@ class ProvTlsServer
     @Override
     public CertificateRequest getCertificateRequest() throws IOException
     {
-        boolean shouldRequest = sslParameters.getNeedClientAuth() || sslParameters.getWantClientAuth();
-        if (!shouldRequest)
+        if (!isClientAuthEnabled())
         {
             return null;
         }
 
         final ContextData contextData = manager.getContextData();
+        final ProtocolVersion negotiatedVersion = context.getServerVersion();
+
+        // TODO[jsse] May want this selection to depend on the peer's supported_groups (create alternate method)?
+        List<SignatureSchemeInfo> signatureSchemes = contextData.getActiveCertsSignatureSchemes(true, sslParameters,
+            new ProtocolVersion[]{ negotiatedVersion }, jsseSecurityParameters.namedGroups);
+
+        // TODO[tls13] From TLS 1.3 these are allowed to be different (no JSSE API to configure this though)
+        jsseSecurityParameters.localSigSchemes = signatureSchemes;
+        jsseSecurityParameters.localSigSchemesCert = signatureSchemes;
+
+        Vector<SignatureAndHashAlgorithm> serverSigAlgs = SignatureSchemeInfo
+            .getSignatureAndHashAlgorithms(jsseSecurityParameters.localSigSchemes);
+
+        /*
+         * TODO[tls13] It appears SunJSSE will add a system property for this (default enabled?),
+         * perhaps "jdk.tls[.client/server].enableCAExtension" or similar.
+         */
+        Vector<X500Name> certificateAuthorities = JsseUtils
+            .getCertificateAuthorities(contextData.getX509TrustManager());
+
+        if (TlsUtils.isTLSv13(negotiatedVersion))
+        {
+            /*
+             * TODO[tls13] RFC 8446 4.4.2.1. A server MAY request that a client present an OCSP response
+             * with its certificate by sending an empty "status_request" extension in its
+             * CertificateRequest message.
+             */
+
+            /*
+             * RFC 8446 4.3.2. This field SHALL be zero length unless used for the post-handshake
+             * authentication exchanges [..].
+             */
+            byte[] certificateRequestContext = TlsUtils.EMPTY_BYTES;
+
+            Vector<SignatureAndHashAlgorithm> serverSigAlgsCert = null;
+            if (jsseSecurityParameters.localSigSchemes != jsseSecurityParameters.localSigSchemesCert)
+            {
+                serverSigAlgsCert = SignatureSchemeInfo
+                    .getSignatureAndHashAlgorithms(jsseSecurityParameters.localSigSchemesCert);
+            }
+
+            return new CertificateRequest(certificateRequestContext, serverSigAlgs, serverSigAlgsCert,
+                certificateAuthorities);
+        }
 
         // TODO[jsse] These should really be based on TlsCrypto support
         short[] certificateTypes = new short[]{ ClientCertificateType.ecdsa_sign,
             ClientCertificateType.rsa_sign, ClientCertificateType.dss_sign };
 
-        Vector<SignatureAndHashAlgorithm> serverSigAlgs = null;
-
-        if (TlsUtils.isSignatureAlgorithmsExtensionAllowed(context.getServerVersion()))
-        {
-            // TODO[tls13] CertificateRequest can contain distinct serverSigAlgsCert
-
-            List<SignatureSchemeInfo> signatureSchemes = contextData.getActiveSignatureSchemes(sslParameters,
-                new ProtocolVersion[]{ context.getServerVersion() });
-
-            // TODO[tls13] Legacy schemes (cert-only for TLS 1.3) complicate this 
-            jsseSecurityParameters.localSigSchemes = signatureSchemes;
-            jsseSecurityParameters.localSigSchemesCert = signatureSchemes;
-
-            serverSigAlgs = contextData.getSignatureAndHashAlgorithms(signatureSchemes);
-        }
-
-        Vector<X500Name> certificateAuthorities = null;
-        {
-            Set<X500Principal> caSubjects = new HashSet<X500Principal>();
-
-            BCX509ExtendedTrustManager x509TrustManager = contextData.getX509TrustManager();
-            for (X509Certificate caCert : x509TrustManager.getAcceptedIssuers())
-            {
-                caSubjects.add(caCert.getSubjectX500Principal());
-            }
-
-            if (!caSubjects.isEmpty())
-            {
-                certificateAuthorities = new Vector<X500Name>(caSubjects.size());
-                for (X500Principal caSubject : caSubjects)
-                {
-                    certificateAuthorities.add(X500Name.getInstance(caSubject.getEncoded()));
-                }
-            }
-        }
-
         return new CertificateRequest(certificateTypes, serverSigAlgs, certificateAuthorities);
     }
 
     @Override
     public CertificateStatus getCertificateStatus() throws IOException
     {
-        // TODO[jsse] Support providing OCSP CertificateStatus
-//        if (CertificateStatusType.ocsp == certificateStatusRequest.getStatusType())
+        // TODO[jsse] Support status_request and status_request_v2 extensions
+//        SecurityParameters securityParameters = context.getSecurityParametersHandshake();
+//        int statusRequestVersion = securityParameters.getStatusRequestVersion();
+//
+//        if (statusRequestVersion == 2)
 //        {
-//            OCSPStatusRequest ocspStatusRequest = certificateStatusRequest.getOCSPStatusRequest();
+//            int count = statusRequestV2.size();
+//            for (int i = 0; i < count; ++i)
+//            {
+//                CertificateStatusRequestItemV2 item = (CertificateStatusRequestItemV2)statusRequestV2.get(i);
+//                short statusType = item.getStatusType();
+//                if (CertificateStatusType.ocsp_multi == statusType)
+//                {
+//                    int chainLength = credentials.getCertificate().getLength();
+//                    Vector ocspResponseList = new Vector(chainLength);
+//                    for (int j = 0; j < chainLength; ++j)
+//                    {
+//                        // TODO Actual OCSP response
+//                        ocspResponseList.add(null);
+//                    }
 //
-//            @SuppressWarnings("unchecked")
-//            Vector<ResponderID> responderIDList = ocspStatusRequest.getResponderIDList();
-//            Extensions requestExtensions = ocspStatusRequest.getRequestExtensions();
+//                    return new CertificateStatus(CertificateStatusType.ocsp_multi, ocspResponseList);
+//                }
+//                else if (CertificateStatusType.ocsp == statusType)
+//                {
+//                    // TODO Actual OCSP response
+//                    OCSPResponse ocspResponse;
 //
-//            X509Certificate eeCert = JsseUtils.getEndEntity(getCrypto(), credentials.getCertificate());
+//                    return new CertificateStatus(CertificateStatusType.ocsp, ocspResponse);
+//                }
+//            }
+//        }
+//        else if (statusRequestVersion == 1)
+//        {
+//            if (CertificateStatusType.ocsp == certificateStatusRequest.getStatusType())
+//            {
+//                OCSPStatusRequest ocspStatusRequest = certificateStatusRequest.getOCSPStatusRequest();
+//
+//                @SuppressWarnings("unchecked")
+//                Vector<ResponderID> responderIDList = ocspStatusRequest.getResponderIDList();
+//                Extensions requestExtensions = ocspStatusRequest.getRequestExtensions();
+//
+//                X509Certificate eeCert = JsseUtils.getEndEntity(getCrypto(), credentials.getCertificate());
+//
+//                // ...
+//            }
 //        }
 
         return null;
@@ -318,34 +360,30 @@ class ProvTlsServer
         return manager.getContextData().getCrypto();
     }
 
+    @Override
+    public int[] getSupportedGroups() throws IOException
+    {
+        // Setup the local supported groups
+        {
+            ProtocolVersion[] activeProtocolVersions = new ProtocolVersion[]{ context.getServerVersion() };
+
+            jsseSecurityParameters.namedGroups = manager.getContextData().getNamedGroups(sslParameters, activeProtocolVersions);
+        }
+
+        return NamedGroupInfo.getSupportedGroupsLocalServer(jsseSecurityParameters.namedGroups);
+    }
+
     @Override
     public int getSelectedCipherSuite() throws IOException
     {
         final ContextData contextData = manager.getContextData();
         final SecurityParameters securityParameters = context.getSecurityParametersHandshake();
 
-        /*
-         * TODO[jsse] Ideally, setting the handshake session would be done in getSessionToResume, but
-         * that is currently never called.
-         */
+        // Setup the peer supported groups
         {
-            ProvSSLSessionContext sslSessionContext = contextData.getServerSessionContext();
-            String peerHost = manager.getPeerHost();
-            int peerPort = manager.getPeerPort();
+            int[] clientSupportedGroups = securityParameters.getClientSupportedGroups();
 
-            ProvSSLSessionHandshake handshakeSession;
-            if (null == sslSession)
-            {
-                handshakeSession = new ProvSSLSessionHandshake(sslSessionContext, peerHost, peerPort,
-                    securityParameters, jsseSecurityParameters);
-            }
-            else
-            {
-                handshakeSession = new ProvSSLSessionResumed(sslSessionContext, peerHost, peerPort, securityParameters,
-                    jsseSecurityParameters, sslSession.getTlsSession(), sslSession.getJsseSessionParameters());
-            }
-
-            manager.notifyHandshakeSession(handshakeSession);
+            NamedGroupInfo.notifyPeer(jsseSecurityParameters.namedGroups, clientSupportedGroups);
         }
 
         // Setup the peer supported signature schemes  
@@ -357,7 +395,11 @@ class ProvTlsServer
             Vector<SignatureAndHashAlgorithm> clientSigAlgsCert = (Vector<SignatureAndHashAlgorithm>)
                 securityParameters.getClientSigAlgsCert();
 
-            // TODO[tls13] Legacy schemes (cert-only for TLS 1.3) complicate these conversions 
+            /*
+             * TODO[tls13] Legacy schemes (cert-only for TLS 1.3) complicate these conversions. Consider which
+             * (if any) of these should be constrained by locally enabled schemes (especially once
+             * jdk.tls.signatureSchemes support added).
+             */
             jsseSecurityParameters.peerSigSchemes = contextData.getSignatureSchemes(clientSigAlgs);
             jsseSecurityParameters.peerSigSchemesCert = (clientSigAlgs == clientSigAlgsCert)
                 ?   jsseSecurityParameters.peerSigSchemes
@@ -390,12 +432,13 @@ class ProvTlsServer
         super.getServerExtensions();
 
         /*
-         * TODO[jsse] RFC 6066 When resuming a session, the server MUST NOT include a server_name
-         * extension in the server hello.
+         * RFC 6066 When resuming a session, the server MUST NOT include a server_name extension
+         * in the server hello.
          */
         if (null != matchedSNIServerName)
         {
-            TlsExtensionsUtils.addServerNameExtensionServer(checkServerExtensions());
+            // TODO[tls13] Resumption/PSK. SNI is always negotiated per-connection in TLS 1.3
+            TlsExtensionsUtils.addServerNameExtensionServer(serverExtensions);
         }
 
         @SuppressWarnings("unchecked")
@@ -408,26 +451,75 @@ class ProvTlsServer
     public TlsSession getSessionToResume(byte[] sessionID)
     {
         ProvSSLSessionContext sslSessionContext = manager.getContextData().getServerSessionContext();
-        ProvSSLSession availableSSLSession = sslSessionContext.getSessionImpl(sessionID);
 
-        if (null != availableSSLSession)
+        if (provServerEnableSessionResumption)
         {
-            TlsSession sessionToResume = availableSSLSession.getTlsSession();
-            if (null != sessionToResume && isResumable(availableSSLSession))
+            ProvSSLSession availableSSLSession = sslSessionContext.getSessionImpl(sessionID);
+
+            if (null != availableSSLSession)
             {
-                this.sslSession = availableSSLSession;
-                return sessionToResume;
+                TlsSession sessionToResume = availableSSLSession.getTlsSession();
+                if (isResumable(availableSSLSession, sessionToResume))
+                {
+                    this.sslSession = availableSSLSession;
+                    return sessionToResume;
+                }
             }
         }
 
         if (!manager.getEnableSessionCreation())
         {
-            throw new IllegalStateException("No resumable sessions and session creation is disabled");
+            throw new IllegalStateException("Cannot resume session and session creation is disabled");
         }
 
         return null;
     }
 
+    @Override
+    public byte[] getNewSessionID()
+    {
+        // TODO[tls13] Resumption/PSK
+        if (!provServerEnableSessionResumption || TlsUtils.isTLSv13(context))
+        {
+            return null;
+        }
+
+        return context.getNonceGenerator().generateNonce(32);
+    }
+
+    @Override
+    public void notifySession(TlsSession session)
+    {
+        byte[] sessionID = session.getSessionID();
+
+        boolean isResumed = (null != sslSession && sslSession.getTlsSession() == session);
+        if (isResumed)
+        {
+            LOG.fine("Server resumed session: " + Hex.toHexString(sessionID));
+        }
+        else
+        {
+            this.sslSession = null;
+
+            if (sessionID == null || sessionID.length < 1)
+            {
+                LOG.fine("Server did not specify a session ID");
+            }
+            else
+            {
+                LOG.fine("Server specified new session: " + Hex.toHexString(sessionID));
+            }
+
+            if (!manager.getEnableSessionCreation())
+            {
+                throw new IllegalStateException("Cannot resume session and session creation is disabled");
+            }
+        }
+
+        manager.notifyHandshakeSession(manager.getContextData().getServerSessionContext(),
+            context.getSecurityParametersHandshake(), jsseSecurityParameters, sslSession);
+    }
+
     @Override
     public void notifyAlertRaised(short alertLevel, short alertDescription, String message, Throwable cause)
     {
@@ -480,7 +572,7 @@ class ProvTlsServer
     public void notifyClientCertificate(Certificate clientCertificate) throws IOException
     {
         // NOTE: This method isn't called unless we returned non-null from getCertificateRequest() earlier
-        if (!sslParameters.getNeedClientAuth() && !sslParameters.getWantClientAuth())
+        if (!isClientAuthEnabled())
         {
             throw new TlsFatalAlert(AlertDescription.internal_error);
         }
@@ -546,11 +638,13 @@ class ProvTlsServer
             ProvSSLSessionContext sslSessionContext = manager.getContextData().getServerSessionContext();
             String peerHost = manager.getPeerHost();
             int peerPort = manager.getPeerPort();
-            JsseSessionParameters jsseSessionParameters = new JsseSessionParameters(
-                sslParameters.getEndpointIdentificationAlgorithm());
+            JsseSessionParameters jsseSessionParameters = new JsseSessionParameters(null, matchedSNIServerName);
+            // TODO[tls13] Resumption/PSK
+            boolean addToCache = provServerEnableSessionResumption && !TlsUtils.isTLSv13(context)
+                && context.getSecurityParametersHandshake().isExtendedMasterSecret();
 
             this.sslSession = sslSessionContext.reportSession(peerHost, peerPort, connectionTlsSession,
-                jsseSessionParameters);
+                jsseSessionParameters, addToCache);
         }
 
         manager.notifyHandshakeComplete(new ProvSSLConnection(context, sslSession));
@@ -561,13 +655,11 @@ class ProvTlsServer
     {
         if (!secureRenegotiation)
         {
-            boolean allowLegacyHelloMessages = PropertyUtils.getBooleanSystemProperty("sun.security.ssl.allowLegacyHelloMessages", true);
+            boolean allowLegacyHelloMessages = PropertyUtils.getBooleanSystemProperty(
+                "sun.security.ssl.allowLegacyHelloMessages", true);
+
             if (!allowLegacyHelloMessages)
             {
-                /*
-                 * RFC 5746 3.4/3.6. In this case, some clients/servers may want to terminate the handshake instead
-                 * of continuing; see Section 4.1/4.3 for discussion.
-                 */
                 throw new TlsFatalAlert(AlertDescription.handshake_failure);
             }
         }
@@ -603,6 +695,20 @@ class ProvTlsServer
                 LOG.fine("Server accepted SNI: " + matchedSNIServerName);
             }
         }
+
+        if (provServerEnableTrustedCAKeys)
+        {
+            @SuppressWarnings("unchecked")
+            Vector<TrustedAuthority> trustedCAKeys = this.trustedCAKeys;
+
+            jsseSecurityParameters.trustedIssuers = JsseUtils.getTrustedIssuers(trustedCAKeys);
+        }
+    }
+
+    @Override
+    public boolean requiresCloseNotify()
+    {
+        return JsseUtils.requireCloseNotify();
     }
 
     @Override
@@ -617,19 +723,75 @@ class ProvTlsServer
         return JsseUtils.useExtendedMasterSecret();
     }
 
-    protected boolean isResumable(ProvSSLSession availableSSLSession)
+    protected boolean isClientAuthEnabled()
     {
-        /*
-         * TODO[jsse] - Note that session resumption is not yet implemented in the low-level TLS layer anyway.
-         * 
-         * Checks that will need to be done here before this can return true:
-         * - endpoint ID algorithm consistency
-         * - SNI consistency
-         */
-        return false;
+        return sslParameters.getNeedClientAuth() || sslParameters.getWantClientAuth();
     }
 
-    protected TlsCredentials selectCredentials(int cipherSuite) throws IOException
+    protected boolean isResumable(ProvSSLSession provSSLSession, TlsSession tlsSession)
+    {
+        if (null == tlsSession || !tlsSession.isResumable())
+        {
+            return false;
+        }
+
+        {
+            SecurityParameters securityParameters = context.getSecurityParametersHandshake();
+
+            // TODO[resumption] Avoid the copy somehow?
+            SessionParameters sessionParameters = tlsSession.exportSessionParameters();
+
+            if (null == sessionParameters ||
+                !securityParameters.getNegotiatedVersion().equals(sessionParameters.getNegotiatedVersion()) ||
+                !Arrays.contains(getCipherSuites(), sessionParameters.getCipherSuite()) ||
+                !Arrays.contains(offeredCipherSuites, sessionParameters.getCipherSuite()))
+            {
+                return false;
+            }
+
+            // TODO[resumption] Consider support for related system properties
+            if (!sessionParameters.isExtendedMasterSecret())
+            {
+                return false;
+            }
+
+            // TODO[tls13] Resumption/PSK 
+            if (TlsUtils.isTLSv13(sessionParameters.getNegotiatedVersion()))
+            {
+                return false;
+            }
+        }
+
+        {
+            /*
+             * TODO[tls13] RFC 8446 4.2.11. In TLS versions prior to TLS 1.3, the Server Name
+             * Identification (SNI) value was intended to be associated with the session (Section 3
+             * of [RFC6066]), with the server being required to enforce that the SNI value
+             * associated with the session matches the one specified in the resumption handshake.
+             * However, in reality the implementations were not consistent on which of two supplied
+             * SNI values they would use, leading to the consistency requirement being de facto
+             * enforced by the clients. In TLS 1.3, the SNI value is always explicitly specified in
+             * the resumption handshake, and there is no need for the server to associate an SNI
+             * value with the ticket. Clients, however, SHOULD store the SNI with the PSK to fulfill
+             * the requirements of Section 4.6.1.
+             */
+            JsseSessionParameters jsseSessionParameters = provSSLSession.getJsseSessionParameters();
+
+            BCSNIServerName connectionSNI = matchedSNIServerName;
+            BCSNIServerName sessionSNI = jsseSessionParameters.getMatchedSNIServerName();
+
+            if (!JsseUtils.equals(connectionSNI, sessionSNI))
+            {
+                LOG.finest(
+                    "Session not resumable - SNI mismatch; connection: " + connectionSNI + ", session: " + sessionSNI);
+                return false;
+            }
+        }
+
+        return true;
+    }
+
+    protected TlsCredentials selectCredentials(Principal[] issuers, int cipherSuite) throws IOException
     {
         int keyExchangeAlgorithm = TlsUtils.getKeyExchangeAlgorithm(cipherSuite);
         switch (keyExchangeAlgorithm)
@@ -638,16 +800,22 @@ class ProvTlsServer
         case KeyExchangeAlgorithm.DHE_RSA:
         case KeyExchangeAlgorithm.ECDHE_ECDSA:
         case KeyExchangeAlgorithm.ECDHE_RSA:
-        case KeyExchangeAlgorithm.NULL:
         case KeyExchangeAlgorithm.RSA:
         {
             if (KeyExchangeAlgorithm.RSA == keyExchangeAlgorithm
                 || !TlsUtils.isSignatureAlgorithmsExtensionAllowed(context.getServerVersion()))
             {
-                return selectServerCredentialsLegacy(keyExchangeAlgorithm);
+                return selectServerCredentialsLegacy(issuers, keyExchangeAlgorithm);
             }
 
-            return selectServerCredentials(keyExchangeAlgorithm);
+            return selectServerCredentials12(issuers, keyExchangeAlgorithm);
+        }
+
+        case KeyExchangeAlgorithm.NULL:
+        {
+            byte[] certificateRequestContext = TlsUtils.EMPTY_BYTES;
+
+            return selectServerCredentials13(issuers, certificateRequestContext);
         }
 
         default:
@@ -655,9 +823,11 @@ class ProvTlsServer
         }
     }
 
-    protected TlsCredentials selectServerCredentials(int keyExchangeAlgorithm) throws IOException
+    protected TlsCredentials selectServerCredentials12(Principal[] issuers, int keyExchangeAlgorithm) throws IOException
     {
         BCAlgorithmConstraints algorithmConstraints = sslParameters.getAlgorithmConstraints();
+        boolean post13Active = TlsUtils.isTLSv13(context);
+        boolean pre13Active = !post13Active;
 
         final short legacySignatureAlgorithm = TlsUtils.getLegacySignatureAlgorithmServer(keyExchangeAlgorithm);
 
@@ -671,11 +841,6 @@ class ProvTlsServer
 
             final short signatureAlgorithm = signatureSchemeInfo.getSignatureAlgorithm();
 
-            /*
-             * TODO[jsse] Probably insufficient for rsa_pss_* signature algorithms. In fact we'd
-             * prefer that none of the legacy names shadow the signature scheme key algorithms
-             * (maybe rename them to e.g. "KE_RSA" etc.?), to avoid this conditional.
-             */
             String keyType = (legacySignatureAlgorithm == signatureAlgorithm)
                 ?   JsseUtils.getKeyTypeLegacyServer(keyExchangeAlgorithm)
                 :   JsseUtils.getKeyType(signatureSchemeInfo);
@@ -685,27 +850,94 @@ class ProvTlsServer
                 continue;
             }
 
-            if (!signatureSchemeInfo.isActive(algorithmConstraints))
+            // TODO[tls13] Somewhat redundant if we get all active signature schemes later (for CertificateRequest)
+            if (!signatureSchemeInfo.isActive(algorithmConstraints, pre13Active, post13Active,
+                jsseSecurityParameters.namedGroups))
             {
                 continue;
             }
 
-            ProvX509Key x509Key = manager.chooseServerKey(keyType, null);
+            BCX509Key x509Key = manager.chooseServerKey(keyType, issuers);
             if (null == x509Key
                 || !JsseUtils.isUsableKeyForServer(signatureAlgorithm, x509Key.getPrivateKey()))
             {
+                if (LOG.isLoggable(Level.FINER))
+                {
+                    LOG.finer("Server (1.2) found no credentials for signature scheme '" + signatureSchemeInfo
+                        + "' (keyType '" + keyType + "')");
+                }
+
                 keyManagerMissCache.add(keyType);
                 continue;
             }
 
+            if (LOG.isLoggable(Level.FINE))
+            {
+                LOG.fine("Server (1.2) selected credentials for signature scheme '" + signatureSchemeInfo
+                    + "' (keyType '" + keyType + "'), with private key algorithm '"
+                    + JsseUtils.getPrivateKeyAlgorithm(x509Key.getPrivateKey()) + "'");
+            }
+
             return JsseUtils.createCredentialedSigner(context, getCrypto(), x509Key,
                 signatureSchemeInfo.getSignatureAndHashAlgorithm());
         }
 
+        LOG.fine("Server (1.2) did not select any credentials");
+
         return null;
     }
 
-    protected TlsCredentials selectServerCredentialsLegacy(int keyExchangeAlgorithm) throws IOException
+    protected TlsCredentials selectServerCredentials13(Principal[] issuers, byte[] certificateRequestContext)
+        throws IOException
+    {
+        BCAlgorithmConstraints algorithmConstraints = sslParameters.getAlgorithmConstraints();
+
+        for (SignatureSchemeInfo signatureSchemeInfo : jsseSecurityParameters.peerSigSchemes)
+        {
+            String keyType = JsseUtils.getKeyType(signatureSchemeInfo);
+            if (keyManagerMissCache.contains(keyType))
+            {
+                continue;
+            }
+
+            // TODO[tls13] Somewhat redundant if we get all active signature schemes later (for CertificateRequest)
+            if (!signatureSchemeInfo.isActive(algorithmConstraints, false, true, jsseSecurityParameters.namedGroups))
+            {
+                continue;
+            }
+
+            BCX509Key x509Key = manager.chooseServerKey(keyType, issuers);
+            if (null == x509Key ||
+                !JsseUtils.isUsableKeyForServer(signatureSchemeInfo.getSignatureAlgorithm(), x509Key.getPrivateKey()))
+            {
+                if (LOG.isLoggable(Level.FINER))
+                {
+                    LOG.finer("Server (1.3) found no credentials for signature scheme '" + signatureSchemeInfo
+                        + "' (keyType '" + keyType + "')");
+                }
+
+                keyManagerMissCache.add(keyType);
+                continue;
+            }
+
+            if (LOG.isLoggable(Level.FINE))
+            {
+                LOG.fine("Server (1.3) selected credentials for signature scheme '" + signatureSchemeInfo
+                    + "' (keyType '" + keyType + "'), with private key algorithm '"
+                    + JsseUtils.getPrivateKeyAlgorithm(x509Key.getPrivateKey()) + "'");
+            }
+
+            return JsseUtils.createCredentialedSigner13(context, getCrypto(), x509Key,
+                signatureSchemeInfo.getSignatureAndHashAlgorithm(), certificateRequestContext);
+        }
+
+        LOG.fine("Server (1.3) did not select any credentials");
+
+        return null;
+    }
+
+    protected TlsCredentials selectServerCredentialsLegacy(Principal[] issuers, int keyExchangeAlgorithm)
+        throws IOException
     {
         String keyType = JsseUtils.getKeyTypeLegacyServer(keyExchangeAlgorithm);
         if (keyManagerMissCache.contains(keyType))
@@ -713,7 +945,7 @@ class ProvTlsServer
             return null;
         }
 
-        ProvX509Key x509Key = manager.chooseServerKey(keyType, null);
+        BCX509Key x509Key = manager.chooseServerKey(keyType, issuers);
         if (null == x509Key
             || !JsseUtils.isUsableKeyForServerLegacy(keyExchangeAlgorithm, x509Key.getPrivateKey()))
         {
