@@ -32,10 +32,15 @@ class ProvTrustManagerFactorySpi
 {
     private static final Logger LOG = Logger.getLogger(ProvTrustManagerFactorySpi.class.getName());
 
+    private static final boolean provKeyStoreTypeCompat = PropertyUtils
+        .getBooleanSecurityProperty("keystore.type.compat", false);
+
     static KeyStore getDefaultTrustStore() throws Exception
     {
         String defaultType = KeyStore.getDefaultType();
 
+        boolean defaultCacertsToJKS = provKeyStoreTypeCompat && "pkcs12".equalsIgnoreCase(defaultType);
+
         String tsPath = null;
         char[] tsPassword = null;
 
@@ -59,7 +64,10 @@ class ProvTrustManagerFactorySpi
                 String jsseCacertsPath = javaHome + "/lib/security/jssecacerts".replace("/", File.separator);
                 if (new File(jsseCacertsPath).exists())
                 {
-                    defaultType = "jks";
+                    if (defaultCacertsToJKS)
+                    {
+                        defaultType = "jks";
+                    }
                     tsPath = jsseCacertsPath;
                 }
                 else
@@ -67,7 +75,10 @@ class ProvTrustManagerFactorySpi
                     String cacertsPath = javaHome + "/lib/security/cacerts".replace("/", File.separator);
                     if (new File(cacertsPath).exists())
                     {
-                        defaultType = "jks";
+                        if (defaultCacertsToJKS)
+                        {
+                            defaultType = "jks";
+                        }
                         tsPath = cacertsPath;
                     }
                 }
@@ -108,12 +119,14 @@ class ProvTrustManagerFactorySpi
         return ks;
     }
 
+    protected final boolean isInFipsMode;
     protected final JcaJceHelper helper;
 
     protected ProvX509TrustManager x509TrustManager;
 
-    ProvTrustManagerFactorySpi(JcaJceHelper helper)
+    ProvTrustManagerFactorySpi(boolean isInFipsMode, JcaJceHelper helper)
     {
+        this.isInFipsMode = isInFipsMode;
         this.helper = helper;
     }
 
@@ -164,7 +177,7 @@ class ProvTrustManagerFactorySpi
 
         try
         {
-            this.x509TrustManager = new ProvX509TrustManager(helper, trustAnchors);
+            this.x509TrustManager = new ProvX509TrustManager(isInFipsMode, helper, trustAnchors);
         }
         catch (InvalidAlgorithmParameterException e)
         {
@@ -184,7 +197,7 @@ class ProvTrustManagerFactorySpi
                 throw new InvalidAlgorithmParameterException("parameters must inherit from PKIXParameters");
             }
 
-            this.x509TrustManager = new ProvX509TrustManager(helper, (PKIXParameters)certPathParameters);
+            this.x509TrustManager = new ProvX509TrustManager(isInFipsMode, helper, (PKIXParameters)certPathParameters);
         }
         else if (null == spec)
         {
@@ -196,6 +209,14 @@ class ProvTrustManagerFactorySpi
         }
     }
 
+    private static void collectTrustAnchor(Set<TrustAnchor> trustAnchors, Certificate certificate)
+    {
+        if (certificate instanceof X509Certificate)
+        {
+            trustAnchors.add(new TrustAnchor((X509Certificate)certificate, null));
+        }
+    }
+
     private static KeyStore createTrustStore(String defaultType)
         throws NoSuchProviderException, KeyStoreException
     {
@@ -214,20 +235,24 @@ class ProvTrustManagerFactorySpi
             return Collections.emptySet();
         }
 
-        Set<TrustAnchor> anchors = new HashSet<TrustAnchor>(trustStore.size());
+        Set<TrustAnchor> trustAnchors = new HashSet<TrustAnchor>();
         for (Enumeration<String> en = trustStore.aliases(); en.hasMoreElements();)
         {
             String alias = (String)en.nextElement();
             if (trustStore.isCertificateEntry(alias))
             {
-                Certificate cert = trustStore.getCertificate(alias);
-                if (cert instanceof X509Certificate)
+                collectTrustAnchor(trustAnchors, trustStore.getCertificate(alias));
+            }
+            else if (trustStore.isKeyEntry(alias))
+            {
+                Certificate[] chain = trustStore.getCertificateChain(alias);
+                if (null != chain && chain.length > 0)
                 {
-                    anchors.add(new TrustAnchor((X509Certificate)cert, null));
+                    collectTrustAnchor(trustAnchors, chain[0]);
                 }
             }
         }
-        return anchors;
+        return trustAnchors;
     }
 
     private static String getTrustStoreType(String defaultType)
