@@ -7,6 +7,7 @@ import java.security.NoSuchAlgorithmException;
 import java.security.Provider;
 import java.security.cert.CertPath;
 import java.security.cert.CertPathBuilder;
+import java.security.cert.CertSelector;
 import java.security.cert.CertStore;
 import java.security.cert.CertStoreParameters;
 import java.security.cert.Certificate;
@@ -39,6 +40,7 @@ import org.bouncycastle.jsse.BCSNIHostName;
 import org.bouncycastle.jsse.BCSSLParameters;
 import org.bouncycastle.jsse.BCX509ExtendedTrustManager;
 import org.bouncycastle.jsse.java.security.BCAlgorithmConstraints;
+import org.bouncycastle.tls.KeyExchangeAlgorithm;
 
 class ProvX509TrustManager
     extends BCX509ExtendedTrustManager
@@ -47,37 +49,50 @@ class ProvX509TrustManager
 
     private static final boolean provCheckRevocation = PropertyUtils
         .getBooleanSystemProperty("com.sun.net.ssl.checkRevocation", false);
+    private static final boolean provTrustManagerCheckEKU = PropertyUtils
+        .getBooleanSystemProperty("org.bouncycastle.jsse.trustManager.checkEKU", true);
 
-    private static final Map<String, Integer> serverKeyUsageMap = createServerKeyUsageMap();
+    private static final Map<String, Integer> keyUsagesServer = createKeyUsagesServer();
 
-    private static Map<String, Integer> createServerKeyUsageMap()
+    private static void addKeyUsageServer(Map<String, Integer> keyUsages, int keyUsage, int... keyExchangeAlgorithms)
     {
-        Map<String, Integer> kus = new HashMap<String, Integer>();
+        for (int keyExchangeAlgorithm : keyExchangeAlgorithms)
+        {
+            String authType = JsseUtils.getAuthTypeServer(keyExchangeAlgorithm);
 
-        kus.put("DHE_DSS", ProvAlgorithmChecker.KU_DIGITAL_SIGNATURE);
-        kus.put("DHE_RSA", ProvAlgorithmChecker.KU_DIGITAL_SIGNATURE);
-        kus.put("ECDHE_ECDSA", ProvAlgorithmChecker.KU_DIGITAL_SIGNATURE);
-        kus.put("ECDHE_RSA", ProvAlgorithmChecker.KU_DIGITAL_SIGNATURE);
-        kus.put("UNKNOWN", ProvAlgorithmChecker.KU_DIGITAL_SIGNATURE);  // TLS 1.3
-
-        kus.put("RSA", ProvAlgorithmChecker.KU_KEY_ENCIPHERMENT);
-
-        kus.put("DH_DSS", ProvAlgorithmChecker.KU_KEY_AGREEMENT);
-        kus.put("DH_RSA", ProvAlgorithmChecker.KU_KEY_AGREEMENT);
-        kus.put("ECDH_ECDSA", ProvAlgorithmChecker.KU_KEY_AGREEMENT);
-        kus.put("ECDH_RSA", ProvAlgorithmChecker.KU_KEY_AGREEMENT);
-
-        return Collections.unmodifiableMap(kus);
+            if (null != keyUsages.put(authType, keyUsage))
+            {
+                throw new IllegalStateException("Duplicate keys in server key usages");
+            }
+        }
     }
 
+    private static Map<String, Integer> createKeyUsagesServer()
+    {
+        Map<String, Integer> keyUsages = new HashMap<String, Integer>();
+
+        addKeyUsageServer(keyUsages, ProvAlgorithmChecker.KU_DIGITAL_SIGNATURE, KeyExchangeAlgorithm.DHE_DSS,
+            KeyExchangeAlgorithm.DHE_RSA, KeyExchangeAlgorithm.ECDHE_ECDSA, KeyExchangeAlgorithm.ECDHE_RSA,
+            KeyExchangeAlgorithm.NULL);
+
+        addKeyUsageServer(keyUsages, ProvAlgorithmChecker.KU_KEY_ENCIPHERMENT, KeyExchangeAlgorithm.RSA);
+
+        addKeyUsageServer(keyUsages, ProvAlgorithmChecker.KU_KEY_AGREEMENT, KeyExchangeAlgorithm.DH_DSS,
+            KeyExchangeAlgorithm.DH_RSA, KeyExchangeAlgorithm.ECDH_ECDSA, KeyExchangeAlgorithm.ECDH_RSA);
+
+        return Collections.unmodifiableMap(keyUsages);
+    }
+
+    private final boolean isInFipsMode;
     private final JcaJceHelper helper;
     private final Set<X509Certificate> trustedCerts;
     private final PKIXBuilderParameters pkixParametersTemplate;
     private final X509TrustManager exportX509TrustManager;
 
-    ProvX509TrustManager(JcaJceHelper helper, Set<TrustAnchor> trustAnchors)
+    ProvX509TrustManager(boolean isInFipsMode, JcaJceHelper helper, Set<TrustAnchor> trustAnchors)
         throws InvalidAlgorithmParameterException
     {
+        this.isInFipsMode = isInFipsMode;
         this.helper = helper;
         this.trustedCerts = getTrustedCerts(trustAnchors);
 
@@ -95,9 +110,10 @@ class ProvX509TrustManager
         this.exportX509TrustManager = X509TrustManagerUtil.exportX509TrustManager(this);
     }
 
-    ProvX509TrustManager(JcaJceHelper helper, PKIXParameters baseParameters)
+    ProvX509TrustManager(boolean isInFipsMode, JcaJceHelper helper, PKIXParameters baseParameters)
         throws InvalidAlgorithmParameterException
     {
+        this.isInFipsMode = isInFipsMode;
         this.helper = helper;
         this.trustedCerts = getTrustedCerts(baseParameters.getTrustAnchors());
 
@@ -108,8 +124,7 @@ class ProvX509TrustManager
         }
         else if (baseParameters instanceof PKIXBuilderParameters)
         {
-            this.pkixParametersTemplate = (PKIXBuilderParameters)baseParameters.clone();
-            this.pkixParametersTemplate.setTargetCertConstraints(null);
+            this.pkixParametersTemplate = (PKIXBuilderParameters)baseParameters;
         }
         else
         {
@@ -204,9 +219,6 @@ class ProvX509TrustManager
             certStore = CertStore.getInstance("Collection", certStoreParameters);
         }
 
-        X509CertSelector certSelector = new X509CertSelector();
-        certSelector.setCertificate(eeCert);
-
         CertPathBuilder pkixBuilder;
         try
         {
@@ -218,9 +230,10 @@ class ProvX509TrustManager
         }
 
         PKIXBuilderParameters pkixParameters = (PKIXBuilderParameters)pkixParametersTemplate.clone();
-        pkixParameters.addCertPathChecker(new ProvAlgorithmChecker(helper, algorithmConstraints));
+        pkixParameters.addCertPathChecker(new ProvAlgorithmChecker(isInFipsMode, helper, algorithmConstraints));
         pkixParameters.addCertStore(certStore);
-        pkixParameters.setTargetCertConstraints(certSelector);
+        pkixParameters.setTargetCertConstraints(
+            createTargetCertConstraints(eeCert, pkixParameters.getTargetCertConstraints()));
 
         if (!statusResponses.isEmpty())
         {
@@ -280,7 +293,7 @@ class ProvX509TrustManager
         try
         {
             BCAlgorithmConstraints algorithmConstraints = TransportData.getAlgorithmConstraints(transportData, false);
-            List<byte[]> statusResponses = TransportData.getStatusResponses(transportData, checkServerTrusted);
+            List<byte[]> statusResponses = TransportData.getStatusResponses(transportData);
 
             X509Certificate[] trustedChain = buildCertPath(chain, algorithmConstraints, statusResponses);
 
@@ -342,7 +355,9 @@ class ProvX509TrustManager
 
     static KeyPurposeId getRequiredExtendedKeyUsage(boolean forServer)
     {
-        return forServer
+        return !provTrustManagerCheckEKU
+            ?   null
+            :   forServer
             ?   KeyPurposeId.id_kp_serverAuth
             :   KeyPurposeId.id_kp_clientAuth;
     }
@@ -355,7 +370,7 @@ class ProvX509TrustManager
             return ProvAlgorithmChecker.KU_DIGITAL_SIGNATURE;
         }
 
-        Integer requiredKeyUsage = serverKeyUsageMap.get(authType);
+        Integer requiredKeyUsage = keyUsagesServer.get(authType);
         if (null == requiredKeyUsage)
         {
             throw new CertificateException("Unsupported server authType: " + authType);
@@ -427,6 +442,23 @@ class ProvX509TrustManager
         checkEndpointID(peerHost, certificate, endpointIDAlg);
     }
 
+    private static X509CertSelector createTargetCertConstraints(final X509Certificate eeCert,
+        final CertSelector userConstraints)
+    {
+        return new X509CertSelector()
+        {
+            {
+                setCertificate(eeCert);
+            }
+
+            @Override
+            public boolean match(Certificate cert)
+            {
+                return super.match(cert) && (null == userConstraints || userConstraints.match(cert));
+            }
+        };
+    }
+
     private static X509Certificate getTrustedCert(TrustAnchor trustAnchor) throws CertificateException
     {
         X509Certificate trustedCert = trustAnchor.getTrustedCert();
