@@ -15,10 +15,12 @@ public abstract class AbstractTlsClient
     implements TlsClient
 {
     protected TlsClientContext context;
+    protected ProtocolVersion[] protocolVersions;
     protected int[] cipherSuites;
 
     protected Vector supportedGroups;
     protected Vector supportedSignatureAlgorithms;
+    protected Vector supportedSignatureAlgorithmsCert;
 
     public AbstractTlsClient(TlsCrypto crypto)
     {
@@ -56,9 +58,10 @@ public abstract class AbstractTlsClient
     protected Vector getNamedGroupRoles()
     {
         Vector namedGroupRoles = TlsUtils.getNamedGroupRoles(getCipherSuites());
+        Vector sigAlgs = supportedSignatureAlgorithms, sigAlgsCert = supportedSignatureAlgorithmsCert;
 
-        if (null == supportedSignatureAlgorithms
-            || TlsUtils.containsAnySignatureAlgorithm(supportedSignatureAlgorithms, SignatureAlgorithm.ecdsa))
+        if ((null == sigAlgs || TlsUtils.containsAnySignatureAlgorithm(sigAlgs, SignatureAlgorithm.ecdsa))
+            || (null != sigAlgsCert && TlsUtils.containsAnySignatureAlgorithm(sigAlgsCert, SignatureAlgorithm.ecdsa)))
         {
             TlsUtils.addToSet(namedGroupRoles, NamedGroupRole.ecdsa);
         }
@@ -96,6 +99,11 @@ public abstract class AbstractTlsClient
         return new DefaultTlsSRPConfigVerifier();
     }
 
+    protected Vector getCertificateAuthorities()
+    {
+        return null;
+    }
+
     protected Vector getProtocolNames()
     {
         return null;
@@ -106,12 +114,18 @@ public abstract class AbstractTlsClient
         return new CertificateStatusRequest(CertificateStatusType.ocsp, new OCSPStatusRequest(null, null));
     }
 
-    protected Vector getSNIServerNames()
+    /**
+     * @return a {@link Vector} of {@link CertificateStatusRequestItemV2} (or null).
+     */
+    protected Vector getMultiCertStatusRequest()
     {
         return null;
     }
 
-    protected abstract int[] getSupportedCipherSuites();
+    protected Vector getSNIServerNames()
+    {
+        return null;
+    }
 
     /**
      * The default {@link #getClientExtensions()} implementation calls this to determine which named
@@ -130,20 +144,21 @@ public abstract class AbstractTlsClient
 
         if (namedGroupRoles.contains(Integers.valueOf(NamedGroupRole.ecdh)))
         {
-            TlsUtils.addIfSupported(supportedGroups, crypto, NamedGroup.x25519);
+            TlsUtils.addIfSupported(supportedGroups, crypto,
+                new int[]{ NamedGroup.x25519, NamedGroup.x448 });
         }
 
-        if (namedGroupRoles.contains(Integers.valueOf(NamedGroupRole.ecdh))
-            || namedGroupRoles.contains(Integers.valueOf(NamedGroupRole.ecdsa)))
+        if (namedGroupRoles.contains(Integers.valueOf(NamedGroupRole.ecdh)) ||
+            namedGroupRoles.contains(Integers.valueOf(NamedGroupRole.ecdsa)))
         {
-            TlsUtils.addIfSupported(supportedGroups, crypto, new int[]{
-                NamedGroup.secp256r1, NamedGroup.secp384r1 });
+            TlsUtils.addIfSupported(supportedGroups, crypto,
+                new int[]{ NamedGroup.secp256r1, NamedGroup.secp384r1 });
         }
 
         if (namedGroupRoles.contains(Integers.valueOf(NamedGroupRole.dh)))
         {
-            TlsUtils.addIfSupported(supportedGroups, crypto, new int[]{
-                NamedGroup.ffdhe2048, NamedGroup.ffdhe3072, NamedGroup.ffdhe4096 });
+            TlsUtils.addIfSupported(supportedGroups, crypto,
+                new int[]{ NamedGroup.ffdhe2048, NamedGroup.ffdhe3072, NamedGroup.ffdhe4096 });
         }
 
         return supportedGroups;
@@ -154,19 +169,41 @@ public abstract class AbstractTlsClient
         return TlsUtils.getDefaultSupportedSignatureAlgorithms(context);
     }
 
+    protected Vector getSupportedSignatureAlgorithmsCert()
+    {
+        return null;
+    }
+
+    protected Vector getTrustedCAIndication()
+    {
+        return null;
+    }
+
     public void init(TlsClientContext context)
     {
         this.context = context;
 
+        this.protocolVersions = getSupportedVersions();
         this.cipherSuites = getSupportedCipherSuites();
     }
 
+    public ProtocolVersion[] getProtocolVersions()
+    {
+        return protocolVersions;
+    }
+
+    public int[] getCipherSuites()
+    {
+        return cipherSuites;
+    }
+
     public void notifyHandshakeBeginning() throws IOException
     {
         super.notifyHandshakeBeginning();
 
         this.supportedGroups = null;
         this.supportedSignatureAlgorithms = null;
+        this.supportedSignatureAlgorithmsCert = null;
     }
 
     public TlsSession getSessionToResume()
@@ -184,17 +221,27 @@ public abstract class AbstractTlsClient
         return false;
     }
 
-    public int[] getCipherSuites()
-    {
-        return cipherSuites;
-    }
-
     public Hashtable getClientExtensions()
         throws IOException
     {
         Hashtable clientExtensions = new Hashtable();
 
-        TlsExtensionsUtils.addEncryptThenMACExtension(clientExtensions);
+        boolean offeringTLSv13Plus = false;
+        boolean offeringPreTLSv13 = false;
+        {
+            ProtocolVersion[] supportedVersions = getProtocolVersions();
+            for (int i = 0; i < supportedVersions.length; ++i)
+            {
+                if (TlsUtils.isTLSv13(supportedVersions[i]))
+                {
+                    offeringTLSv13Plus = true;
+                }
+                else
+                {
+                    offeringPreTLSv13 = true;
+                }
+            }
+        }
 
         Vector protocolNames = getProtocolNames();
         if (protocolNames != null)
@@ -214,6 +261,33 @@ public abstract class AbstractTlsClient
             TlsExtensionsUtils.addStatusRequestExtension(clientExtensions, statusRequest);
         }
 
+        if (offeringTLSv13Plus)
+        {
+            Vector certificateAuthorities = getCertificateAuthorities();
+            if (certificateAuthorities != null)
+            {
+                TlsExtensionsUtils.addCertificateAuthoritiesExtension(clientExtensions, certificateAuthorities);
+            }
+        }
+
+        if (offeringPreTLSv13)
+        {
+            // TODO Shouldn't add if no offered cipher suite uses a block cipher?
+            TlsExtensionsUtils.addEncryptThenMACExtension(clientExtensions);
+
+            Vector statusRequestV2 = getMultiCertStatusRequest();
+            if (statusRequestV2 != null)
+            {
+                TlsExtensionsUtils.addStatusRequestV2Extension(clientExtensions, statusRequestV2);
+            }
+
+            Vector trustedCAKeys = getTrustedCAIndication();
+            if (trustedCAKeys != null)
+            {
+                TlsExtensionsUtils.addTrustedCAKeysExtensionClient(clientExtensions, trustedCAKeys);
+            }
+        }
+
         ProtocolVersion clientVersion = context.getClientVersion();
 
         /*
@@ -222,9 +296,21 @@ public abstract class AbstractTlsClient
          */
         if (TlsUtils.isSignatureAlgorithmsExtensionAllowed(clientVersion))
         {
-            this.supportedSignatureAlgorithms = getSupportedSignatureAlgorithms();
+            Vector supportedSigAlgs = getSupportedSignatureAlgorithms();
+            if (null != supportedSigAlgs && !supportedSigAlgs.isEmpty())
+            {
+                this.supportedSignatureAlgorithms = supportedSigAlgs;
 
-            TlsExtensionsUtils.addSignatureAlgorithmsExtension(clientExtensions, supportedSignatureAlgorithms);
+                TlsExtensionsUtils.addSignatureAlgorithmsExtension(clientExtensions, supportedSigAlgs);
+            }
+
+            Vector supportedSigAlgsCert = getSupportedSignatureAlgorithmsCert();
+            if (null != supportedSigAlgsCert && !supportedSigAlgsCert.isEmpty())
+            {
+                this.supportedSignatureAlgorithmsCert = supportedSigAlgsCert;
+
+                TlsExtensionsUtils.addSignatureAlgorithmsCertExtension(clientExtensions, supportedSigAlgsCert);
+            }
         }
 
         Vector namedGroupRoles = getNamedGroupRoles();
@@ -237,16 +323,42 @@ public abstract class AbstractTlsClient
             TlsExtensionsUtils.addSupportedGroupsExtension(clientExtensions, supportedGroups);
         }
 
-        if (namedGroupRoles.contains(Integers.valueOf(NamedGroupRole.ecdh))
-            || namedGroupRoles.contains(Integers.valueOf(NamedGroupRole.ecdsa)))
+        if (offeringPreTLSv13)
         {
-            TlsExtensionsUtils.addSupportedPointFormatsExtension(clientExtensions,
-                new short[]{ ECPointFormat.uncompressed });
+            if (namedGroupRoles.contains(Integers.valueOf(NamedGroupRole.ecdh))
+                || namedGroupRoles.contains(Integers.valueOf(NamedGroupRole.ecdsa)))
+            {
+                TlsExtensionsUtils.addSupportedPointFormatsExtension(clientExtensions, new short[]{ ECPointFormat.uncompressed });
+            }
         }
 
         return clientExtensions;
     }
 
+    public Vector getEarlyKeyShareGroups()
+    {
+        /*
+         * RFC 8446 4.2.8. Each KeyShareEntry value MUST correspond to a group offered in the
+         * "supported_groups" extension and MUST appear in the same order. However, the values MAY
+         * be a non-contiguous subset of the "supported_groups" extension and MAY omit the most
+         * preferred groups.
+         */
+
+        if (null == supportedGroups || supportedGroups.isEmpty())
+        {
+            return null;
+        }
+        if (supportedGroups.contains(Integers.valueOf(NamedGroup.x25519)))
+        {
+            return TlsUtils.vectorOfOne(Integers.valueOf(NamedGroup.x25519));
+        }
+        if (supportedGroups.contains(Integers.valueOf(NamedGroup.secp256r1)))
+        {
+            return TlsUtils.vectorOfOne(Integers.valueOf(NamedGroup.secp256r1));
+        }
+        return TlsUtils.vectorOfOne(supportedGroups.elementAt(0));
+    }
+
     public void notifyServerVersion(ProtocolVersion serverVersion)
         throws IOException
     {
@@ -263,17 +375,27 @@ public abstract class AbstractTlsClient
     public void processServerExtensions(Hashtable serverExtensions)
         throws IOException
     {
-        /*
-         * TlsProtocol implementation validates that any server extensions received correspond to
-         * client extensions sent. By default, we don't send any, and this method is not called.
-         */
-        if (serverExtensions != null)
+        if (null == serverExtensions)
+        {
+            return;
+        }
+
+        SecurityParameters securityParameters = context.getSecurityParametersHandshake();
+        boolean isTLSv13 = TlsUtils.isTLSv13(securityParameters.getNegotiatedVersion());
+
+        if (isTLSv13)
+        {
+            /*
+             * NOTE: From TLS 1.3 the protocol classes are strict about what extensions can appear.
+             */
+        }
+        else
         {
             /*
              * RFC 5246 7.4.1.4.1. Servers MUST NOT send this extension.
              */
-            checkForUnexpectedServerExtension(serverExtensions, TlsUtils.EXT_signature_algorithms);
-            checkForUnexpectedServerExtension(serverExtensions, TlsUtils.EXT_signature_algorithms_cert);
+            checkForUnexpectedServerExtension(serverExtensions, TlsExtensionsUtils.EXT_signature_algorithms);
+            checkForUnexpectedServerExtension(serverExtensions, TlsExtensionsUtils.EXT_signature_algorithms_cert);
 
             checkForUnexpectedServerExtension(serverExtensions, TlsExtensionsUtils.EXT_supported_groups);
 
