@@ -12,23 +12,65 @@ public abstract class AbstractTlsPeer
 {
     private final TlsCrypto crypto;
 
+    private volatile TlsCloseable closeHandle;
+
     protected AbstractTlsPeer(TlsCrypto crypto)
     {
         this.crypto = crypto;
     }
 
+    /**
+     * Get the {@link ProtocolVersion} values that are supported by this peer.
+     * <p/>
+     * WARNING: Mixing DTLS and TLS versions in the returned array is currently NOT supported. Use a
+     * separate (sub-)class for each case.
+     *
+     * @return an array of supported {@link ProtocolVersion} values.
+     */
+    protected ProtocolVersion[] getSupportedVersions()
+    {
+        // TODO[tls13] Enable TLSv13 by default in due course
+        return ProtocolVersion.TLSv12.downTo(ProtocolVersion.TLSv10);
+    }
+
+    protected abstract int[] getSupportedCipherSuites();
+
+    public void cancel() throws IOException
+    {
+        TlsCloseable closeHandle = this.closeHandle;
+        if (null != closeHandle)
+        {
+            closeHandle.close();
+        }
+    }
+
     public TlsCrypto getCrypto()
     {
         return crypto;
     }
 
+    public void notifyCloseHandle(TlsCloseable closeHandle)
+    {
+        this.closeHandle = closeHandle;
+    }
+
     public void notifyHandshakeBeginning() throws IOException
     {
     }
 
-    public ProtocolVersion[] getSupportedVersions()
+    public int getHandshakeTimeoutMillis()
     {
-        return ProtocolVersion.TLSv12.downTo(ProtocolVersion.TLSv10);
+        return 0;
+    }
+
+    public boolean allowLegacyResumption()
+    {
+        return false;
+    }
+
+    public boolean requiresCloseNotify()
+    {
+        return true;
     }
 
     public boolean requiresExtendedMasterSecret()
@@ -41,6 +83,11 @@ public abstract class AbstractTlsPeer
         return true;
     }
 
+    public boolean shouldUseExtendedMasterSecret()
+    {
+        return true;
+    }
+
     public boolean shouldUseExtendedPadding()
     {
         return false;
@@ -61,15 +108,11 @@ public abstract class AbstractTlsPeer
     {
         if (!secureRenegotiation)
         {
-            /*
-             * RFC 5746 3.4/3.6. In this case, some clients/servers may want to terminate the handshake instead
-             * of continuing; see Section 4.1/4.3 for discussion.
-             */
             throw new TlsFatalAlert(AlertDescription.handshake_failure);
         }
     }
 
-   public TlsKeyExchangeFactory getKeyExchangeFactory() throws IOException
+    public TlsKeyExchangeFactory getKeyExchangeFactory() throws IOException
     {
         return new DefaultTlsKeyExchangeFactory();
     }
@@ -86,6 +129,16 @@ public abstract class AbstractTlsPeer
     {
     }
 
+    public TlsHeartbeat getHeartbeat()
+    {
+        return null;
+    }
+
+    public short getHeartbeatPolicy()
+    {
+        return HeartbeatMode.peer_not_allowed_to_send;
+    }
+
     public int getRenegotiationPolicy()
     {
         return RenegotiationPolicy.DENY;
