@@ -9,13 +9,14 @@ import org.bouncycastle.tls.crypto.TlsDHConfig;
 import org.bouncycastle.tls.crypto.TlsECConfig;
 
 /**
- * Base class for a TLS client.
+ * Base class for a TLS server.
  */
 public abstract class AbstractTlsServer
     extends AbstractTlsPeer
     implements TlsServer
 {
     protected TlsServerContext context;
+    protected ProtocolVersion[] protocolVersions;
     protected int[] cipherSuites;
 
     protected int[] offeredCipherSuites;
@@ -26,11 +27,14 @@ public abstract class AbstractTlsServer
     protected boolean truncatedHMacOffered;
     protected boolean clientSentECPointFormats;
     protected CertificateStatusRequest certificateStatusRequest;
+    protected Vector statusRequestV2;
+    protected Vector trustedCAKeys;
 
-    protected ProtocolVersion serverVersion;
     protected int selectedCipherSuite;
+    protected Vector clientProtocolNames;
     protected ProtocolName selectedProtocolName;
-    protected Hashtable serverExtensions;
+
+    protected final Hashtable serverExtensions = new Hashtable();
 
     public AbstractTlsServer(TlsCrypto crypto)
     {
@@ -47,19 +51,25 @@ public abstract class AbstractTlsServer
         return true;
     }
 
+    protected boolean allowMultiCertStatus()
+    {
+        return false;
+    }
+
     protected boolean allowTruncatedHMac()
     {
         return false;
     }
 
-    protected Hashtable checkServerExtensions()
+    protected boolean allowTrustedCAIndication()
     {
-        return this.serverExtensions = TlsExtensionsUtils.ensureExtensionsInitialised(this.serverExtensions);
+        return false;
     }
 
-    protected int[] getCipherSuites()
+    /** @deprecated Use 'serverExtensions' directly, it is now never null */
+    protected Hashtable checkServerExtensions()
     {
-        return cipherSuites;
+        return serverExtensions;
     }
 
     protected int getMaximumNegotiableCurveBits()
@@ -104,11 +114,10 @@ public abstract class AbstractTlsServer
         return null;
     }
 
-    protected abstract int[] getSupportedCipherSuites();
-
     protected boolean isSelectableCipherSuite(int cipherSuite, int availCurveBits, int availFiniteFieldBits, Vector sigAlgs)
     {
-        return TlsUtils.isValidCipherSuiteForVersion(cipherSuite, context.getServerVersion())
+        // TODO[tls13] The version check should be separated out (eventually select ciphersuite before version)
+        return TlsUtils.isValidVersionForCipherSuite(cipherSuite, context.getServerVersion())
             && availCurveBits >= TlsECCUtils.getMinimumCurveBits(cipherSuite)
             && availFiniteFieldBits >= TlsDHUtils.getMinimumFiniteFieldBits(cipherSuite)
             && TlsUtils.isValidCipherSuiteForSignatureAlgorithms(cipherSuite, sigAlgs);
@@ -185,6 +194,23 @@ public abstract class AbstractTlsServer
             :  -1;
     }
 
+    protected ProtocolName selectProtocolName() throws IOException
+    {
+        Vector serverProtocolNames = getProtocolNames();
+        if (null == serverProtocolNames || serverProtocolNames.isEmpty())
+        {
+            return null;
+        }
+
+        ProtocolName result = selectProtocolName(clientProtocolNames, serverProtocolNames);
+        if (null == result)
+        {
+            throw new TlsFatalAlert(AlertDescription.no_application_protocol);
+        }
+
+        return result;
+    }
+
     protected ProtocolName selectProtocolName(Vector clientProtocolNames, Vector serverProtocolNames)
     {
         for (int i = 0; i < serverProtocolNames.size(); ++i)
@@ -198,13 +224,29 @@ public abstract class AbstractTlsServer
         return null;
     }
 
+    protected boolean shouldSelectProtocolNameEarly()
+    {
+        return true;
+    }
+
     public void init(TlsServerContext context)
     {
         this.context = context;
 
+        this.protocolVersions = getSupportedVersions();
         this.cipherSuites = getSupportedCipherSuites();
     }
 
+    public ProtocolVersion[] getProtocolVersions()
+    {
+        return protocolVersions;
+    }
+
+    public int[] getCipherSuites()
+    {
+        return cipherSuites;
+    }
+
     public void notifyHandshakeBeginning() throws IOException
     {
         super.notifyHandshakeBeginning();
@@ -218,7 +260,7 @@ public abstract class AbstractTlsServer
         this.certificateStatusRequest = null;
         this.selectedCipherSuite = -1;
         this.selectedProtocolName = null;
-        this.serverExtensions = null;
+        this.serverExtensions.clear();
     }
 
     public TlsSession getSessionToResume(byte[] sessionID)
@@ -226,6 +268,15 @@ public abstract class AbstractTlsServer
         return null;
     }
 
+    public byte[] getNewSessionID()
+    {
+        return null;
+    }
+
+    public void notifySession(TlsSession session)
+    {
+    }
+
     public void notifyClientVersion(ProtocolVersion clientVersion)
         throws IOException
     {
@@ -241,7 +292,7 @@ public abstract class AbstractTlsServer
          */
         if (isFallback)
         {
-            ProtocolVersion[] serverVersions = getSupportedVersions();
+            ProtocolVersion[] serverVersions = getProtocolVersions();
             ProtocolVersion clientVersion = context.getClientVersion();
 
             ProtocolVersion latestServerVersion;
@@ -276,43 +327,39 @@ public abstract class AbstractTlsServer
     {
         this.clientExtensions = clientExtensions;
 
-        if (clientExtensions != null)
+        if (null != clientExtensions)
         {
-            Vector clientProtocolNames = TlsExtensionsUtils.getALPNExtensionClient(clientExtensions);
-            if (clientProtocolNames != null && !clientProtocolNames.isEmpty())
+            this.clientProtocolNames = TlsExtensionsUtils.getALPNExtensionClient(clientExtensions);
+            
+            if (shouldSelectProtocolNameEarly())
             {
-                Vector serverProtocolNames = getProtocolNames();
-                if (serverProtocolNames != null && !serverProtocolNames.isEmpty())
+                if (null != clientProtocolNames && !clientProtocolNames.isEmpty())
                 {
-                    this.selectedProtocolName = selectProtocolName(clientProtocolNames, serverProtocolNames);
-                    if (selectedProtocolName == null)
-                    {
-                        throw new TlsFatalAlert(AlertDescription.no_application_protocol);
-                    }
+                    this.selectedProtocolName = selectProtocolName();
                 }
             }
 
             this.encryptThenMACOffered = TlsExtensionsUtils.hasEncryptThenMACExtension(clientExtensions);
+            this.truncatedHMacOffered = TlsExtensionsUtils.hasTruncatedHMacExtension(clientExtensions);
+            this.certificateStatusRequest = TlsExtensionsUtils.getStatusRequestExtension(clientExtensions);
+            this.statusRequestV2 = TlsExtensionsUtils.getStatusRequestV2Extension(clientExtensions);
+            this.trustedCAKeys = TlsExtensionsUtils.getTrustedCAKeysExtensionClient(clientExtensions);
+
+            // We only support uncompressed format, this is just to validate the extension, and note its presence.
+            this.clientSentECPointFormats = (null != TlsExtensionsUtils.getSupportedPointFormatsExtension(clientExtensions));
 
             this.maxFragmentLengthOffered = TlsExtensionsUtils.getMaxFragmentLengthExtension(clientExtensions);
             if (maxFragmentLengthOffered >= 0 && !MaxFragmentLength.isValid(maxFragmentLengthOffered))
             {
                 throw new TlsFatalAlert(AlertDescription.illegal_parameter);
             }
-
-            this.truncatedHMacOffered = TlsExtensionsUtils.hasTruncatedHMacExtension(clientExtensions);
-
-            // We only support uncompressed format, this is just to validate the extension, and note its presence.
-            this.clientSentECPointFormats = (null != TlsExtensionsUtils.getSupportedPointFormatsExtension(clientExtensions));
-
-            this.certificateStatusRequest = TlsExtensionsUtils.getStatusRequestExtension(clientExtensions);
         }
     }
 
     public ProtocolVersion getServerVersion()
         throws IOException
     {
-        ProtocolVersion[] serverVersions = getSupportedVersions();
+        ProtocolVersion[] serverVersions = getProtocolVersions();
         ProtocolVersion[] clientVersions = context.getClientSupportedVersions();
 
         for (int i = 0; i < clientVersions.length; ++i)
@@ -327,6 +374,13 @@ public abstract class AbstractTlsServer
         throw new TlsFatalAlert(AlertDescription.protocol_version);
     }
 
+    public int[] getSupportedGroups() throws IOException
+    {
+        // TODO[tls13] The rest of this class assumes all named groups are supported
+        return new int[]{ NamedGroup.x25519, NamedGroup.x448, NamedGroup.secp256r1, NamedGroup.secp384r1,
+            NamedGroup.ffdhe2048, NamedGroup.ffdhe3072, NamedGroup.ffdhe4096 };
+    }
+
     public int getSelectedCipherSuite()
         throws IOException
     {
@@ -368,59 +422,110 @@ public abstract class AbstractTlsServer
     public Hashtable getServerExtensions()
         throws IOException
     {
-        if (this.selectedProtocolName != null)
-        {
-            TlsExtensionsUtils.addALPNExtensionServer(checkServerExtensions(), selectedProtocolName);
-        }
+        final boolean isTLSv13 = TlsUtils.isTLSv13(context);
 
-        if (this.encryptThenMACOffered && allowEncryptThenMAC())
+        if (isTLSv13)
         {
-            /*
-             * RFC 7366 3. If a server receives an encrypt-then-MAC request extension from a client
-             * and then selects a stream or Authenticated Encryption with Associated Data (AEAD)
-             * ciphersuite, it MUST NOT send an encrypt-then-MAC response extension back to the
-             * client.
-             */
-            if (TlsUtils.isBlockCipherSuite(this.selectedCipherSuite))
+            if (null != this.certificateStatusRequest && allowCertificateStatus())
             {
-                TlsExtensionsUtils.addEncryptThenMACExtension(checkServerExtensions());
+                /*
+                 * TODO[tls13] RFC 8446 4.4.2.1. OCSP Status and SCT Extensions.
+                 * 
+                 * OCSP information is carried in an extension for a CertificateEntry.
+                 */
+            }
+        }
+        else
+        {
+            if (this.encryptThenMACOffered && allowEncryptThenMAC())
+            {
+                /*
+                 * RFC 7366 3. If a server receives an encrypt-then-MAC request extension from a client
+                 * and then selects a stream or Authenticated Encryption with Associated Data (AEAD)
+                 * ciphersuite, it MUST NOT send an encrypt-then-MAC response extension back to the
+                 * client.
+                 */
+                if (TlsUtils.isBlockCipherSuite(this.selectedCipherSuite))
+                {
+                    TlsExtensionsUtils.addEncryptThenMACExtension(serverExtensions);
+                }
+            }
+
+            if (this.truncatedHMacOffered && allowTruncatedHMac())
+            {
+                TlsExtensionsUtils.addTruncatedHMacExtension(serverExtensions);
+            }
+
+            if (this.clientSentECPointFormats && TlsECCUtils.isECCCipherSuite(this.selectedCipherSuite))
+            {
+                /*
+                 * RFC 4492 5.2. A server that selects an ECC cipher suite in response to a ClientHello
+                 * message including a Supported Point Formats Extension appends this extension (along
+                 * with others) to its ServerHello message, enumerating the point formats it can parse.
+                 */
+                TlsExtensionsUtils.addSupportedPointFormatsExtension(serverExtensions,
+                    new short[]{ ECPointFormat.uncompressed });
+            }
+
+            // TODO[tls13] See RFC 8446 4.4.2.1
+            if (null != this.statusRequestV2 && allowMultiCertStatus())
+            {
+                /*
+                 * RFC 6961 2.2. If a server returns a "CertificateStatus" message in response to a
+                 * "status_request_v2" request, then the server MUST have included an extension of type
+                 * "status_request_v2" with empty "extension_data" in the extended server hello..
+                 */
+                TlsExtensionsUtils.addEmptyExtensionData(serverExtensions, TlsExtensionsUtils.EXT_status_request_v2);
+            }
+            else if (null != this.certificateStatusRequest && allowCertificateStatus())
+            {
+                /*
+                 * RFC 6066 8. If a server returns a "CertificateStatus" message, then the server MUST
+                 * have included an extension of type "status_request" with empty "extension_data" in
+                 * the extended server hello.
+                 */
+                TlsExtensionsUtils.addEmptyExtensionData(serverExtensions, TlsExtensionsUtils.EXT_status_request);
+            }
+
+            if (null != this.trustedCAKeys && allowTrustedCAIndication())
+            {
+                TlsExtensionsUtils.addTrustedCAKeysExtensionServer(serverExtensions);
             }
         }
 
         if (this.maxFragmentLengthOffered >= 0 && MaxFragmentLength.isValid(maxFragmentLengthOffered))
         {
-            TlsExtensionsUtils.addMaxFragmentLengthExtension(checkServerExtensions(), this.maxFragmentLengthOffered);
-        }
-
-        if (this.truncatedHMacOffered && allowTruncatedHMac())
-        {
-            TlsExtensionsUtils.addTruncatedHMacExtension(checkServerExtensions());
-        }
-
-        if (this.clientSentECPointFormats && TlsECCUtils.isECCCipherSuite(this.selectedCipherSuite))
-        {
-            /*
-             * RFC 4492 5.2. A server that selects an ECC cipher suite in response to a ClientHello
-             * message including a Supported Point Formats Extension appends this extension (along
-             * with others) to its ServerHello message, enumerating the point formats it can parse.
-             */
-            TlsExtensionsUtils.addSupportedPointFormatsExtension(checkServerExtensions(),
-                new short[]{ ECPointFormat.uncompressed });
-        }
-
-        if (null != this.certificateStatusRequest && allowCertificateStatus())
-        {
-            /*
-             * RFC 6066 8. If a server returns a "CertificateStatus" message, then the server MUST
-             * have included an extension of type "status_request" with empty "extension_data" in
-             * the extended server hello.
-             */
-            checkServerExtensions().put(TlsExtensionsUtils.EXT_status_request, TlsExtensionsUtils.createEmptyExtensionData());
+            TlsExtensionsUtils.addMaxFragmentLengthExtension(serverExtensions, this.maxFragmentLengthOffered);
         }
 
         return serverExtensions;
     }
 
+    public void getServerExtensionsForConnection(Hashtable serverExtensions) throws IOException
+    {
+        if (!shouldSelectProtocolNameEarly())
+        {
+            if (null != clientProtocolNames && !clientProtocolNames.isEmpty())
+            {
+                this.selectedProtocolName = selectProtocolName();
+            }
+        }
+
+        /*
+         * RFC 7301 3.1. When session resumption or session tickets [...] are used, the previous
+         * contents of this extension are irrelevant, and only the values in the new handshake
+         * messages are considered.
+         */
+        if (null == selectedProtocolName)
+        {
+            serverExtensions.remove(TlsExtensionsUtils.EXT_application_layer_protocol_negotiation);
+        }
+        else
+        {
+            TlsExtensionsUtils.addALPNExtensionServer(serverExtensions, selectedProtocolName);
+        }
+    }
+
     public Vector getServerSupplementalData()
         throws IOException
     {
@@ -452,27 +557,15 @@ public abstract class AbstractTlsServer
     public TlsDHConfig getDHConfig() throws IOException
     {
         int minimumFiniteFieldBits = TlsDHUtils.getMinimumFiniteFieldBits(selectedCipherSuite);
-
         int namedGroup = selectDH(minimumFiniteFieldBits);
-        if (namedGroup < 0)
-        {
-            throw new TlsFatalAlert(AlertDescription.internal_error);
-        }
-
-        return new TlsDHConfig(namedGroup);
+        return TlsDHUtils.createNamedDHConfig(context, namedGroup);
     }
 
     public TlsECConfig getECDHConfig() throws IOException
     {
         int minimumCurveBits = TlsECCUtils.getMinimumCurveBits(selectedCipherSuite);
-
         int namedGroup = selectECDH(minimumCurveBits);
-        if (namedGroup < 0)
-        {
-            throw new TlsFatalAlert(AlertDescription.internal_error);
-        }
-
-        return new TlsECConfig(namedGroup);
+        return TlsECCUtils.createNamedECConfig(context, namedGroup);
     }
 
     public void processClientSupplementalData(Vector clientSupplementalData)
