@@ -3,7 +3,12 @@ package org.bouncycastle.tls;
 public class CertificateStatusType
 {
     /*
-     *  RFC 3546 3.6
+     *  RFC 6066
      */
     public static final short ocsp = 1;
+
+    /*
+     *  RFC 6961
+     */
+    public static final short ocsp_multi = 2;
 }
