@@ -9,6 +9,17 @@ public abstract class DefaultTlsServer
 {
     private static final int[] DEFAULT_CIPHER_SUITES = new int[]
     {
+        // TODO[tls13]
+//        /*
+//         * TLS 1.3
+//         */
+//        CipherSuite.TLS_CHACHA20_POLY1305_SHA256,
+//        CipherSuite.TLS_AES_256_GCM_SHA384,
+//        CipherSuite.TLS_AES_128_GCM_SHA256,
+
+        /*
+         * pre-TLS 1.3
+         */
         CipherSuite.TLS_ECDHE_RSA_WITH_CHACHA20_POLY1305_SHA256,
         CipherSuite.TLS_ECDHE_RSA_WITH_AES_256_GCM_SHA384,
         CipherSuite.TLS_ECDHE_RSA_WITH_AES_128_GCM_SHA256,
@@ -62,13 +73,13 @@ public abstract class DefaultTlsServer
 
     protected int[] getSupportedCipherSuites()
     {
-        return TlsUtils.getSupportedCipherSuites(context.getCrypto(), DEFAULT_CIPHER_SUITES);
+        return TlsUtils.getSupportedCipherSuites(getCrypto(), DEFAULT_CIPHER_SUITES);
     }
 
     public TlsCredentials getCredentials()
         throws IOException
     {
-        int keyExchangeAlgorithm = TlsUtils.getKeyExchangeAlgorithm(selectedCipherSuite);
+        int keyExchangeAlgorithm = context.getSecurityParametersHandshake().getKeyExchangeAlgorithm();
 
         switch (keyExchangeAlgorithm)
         {
