@@ -37,14 +37,25 @@ public class PSKTlsServer
         throw new TlsFatalAlert(AlertDescription.internal_error);
     }
 
+    protected ProtocolVersion[] getSupportedVersions()
+    {
+        return ProtocolVersion.TLSv12.downTo(ProtocolVersion.TLSv10);
+    }
+
     protected int[] getSupportedCipherSuites()
     {
-        return TlsUtils.getSupportedCipherSuites(context.getCrypto(), DEFAULT_CIPHER_SUITES);
+        return TlsUtils.getSupportedCipherSuites(getCrypto(), DEFAULT_CIPHER_SUITES);
+    }
+
+    /** @deprecated Unused; will be removed */
+    public ProtocolVersion getMaximumVersion()
+    {
+        return ProtocolVersion.TLSv12;
     }
 
     public TlsCredentials getCredentials() throws IOException
     {
-        int keyExchangeAlgorithm = TlsUtils.getKeyExchangeAlgorithm(selectedCipherSuite);
+        int keyExchangeAlgorithm = context.getSecurityParametersHandshake().getKeyExchangeAlgorithm();
 
         switch (keyExchangeAlgorithm)
         {
