@@ -7,6 +7,8 @@ import java.io.InterruptedIOException;
 import java.io.OutputStream;
 
 import org.bouncycastle.tls.crypto.TlsCipher;
+import org.bouncycastle.tls.crypto.TlsDecodeResult;
+import org.bouncycastle.tls.crypto.TlsEncodeResult;
 import org.bouncycastle.tls.crypto.TlsNullNullCipher;
 
 /**
@@ -17,26 +19,21 @@ class RecordStream
     private static int DEFAULT_PLAINTEXT_LIMIT = (1 << 14);
 
     private final Record inputRecord = new Record();
+    private final SequenceNumber readSeqNo = new SequenceNumber(), writeSeqNo = new SequenceNumber();
 
     private TlsProtocol handler;
     private InputStream input;
     private OutputStream output;
-    private TlsContext context = null;
-    private TlsCipher pendingCipher = null, readCipher = null, writeCipher = null;
-    private SequenceNumber readSeqNo = new SequenceNumber(), writeSeqNo = new SequenceNumber();
-
-    private TlsHandshakeHash handshakeHash = null;
-    private SimpleOutputStream handshakeHashUpdater = new SimpleOutputStream()
-    {
-        public void write(byte[] buf, int off, int len) throws IOException
-        {
-            handshakeHash.update(buf, off, len);
-        }
-    };
+    private TlsCipher pendingCipher = null;
+    private TlsCipher readCipher = TlsNullNullCipher.INSTANCE;
+    private TlsCipher readCipherDeferred = null;
+    private TlsCipher writeCipher = TlsNullNullCipher.INSTANCE;
 
     private ProtocolVersion writeVersion = null;
 
-    private int plaintextLimit, ciphertextLimit;
+    private int plaintextLimit = DEFAULT_PLAINTEXT_LIMIT;
+    private int ciphertextLimit = DEFAULT_PLAINTEXT_LIMIT;
+    private boolean ignoreChangeCipherSpec = false;
 
     RecordStream(TlsProtocol handler, InputStream input, OutputStream output)
     {
@@ -45,16 +42,6 @@ class RecordStream
         this.output = output;
     }
 
-    void init(TlsContext context)
-    {
-        this.context = context;
-        this.readCipher = new TlsNullNullCipher();
-        this.writeCipher = this.readCipher;
-        this.handshakeHash = new DeferredHash(context);
-
-        setPlaintextLimit(DEFAULT_PLAINTEXT_LIMIT);
-    }
-
     int getPlaintextLimit()
     {
         return plaintextLimit;
@@ -63,7 +50,7 @@ class RecordStream
     void setPlaintextLimit(int plaintextLimit)
     {
         this.plaintextLimit = plaintextLimit;
-        this.ciphertextLimit = this.plaintextLimit + 1024;
+        this.ciphertextLimit = readCipher.getCiphertextDecodeLimit(plaintextLimit);
     }
 
     void setWriteVersion(ProtocolVersion writeVersion)
@@ -71,31 +58,59 @@ class RecordStream
         this.writeVersion = writeVersion;
     }
 
-    void setPendingConnectionState(TlsCipher tlsCipher)
+    void setIgnoreChangeCipherSpec(boolean ignoreChangeCipherSpec)
+    {
+        this.ignoreChangeCipherSpec = ignoreChangeCipherSpec;
+    }
+
+    void setPendingCipher(TlsCipher tlsCipher)
     {
         this.pendingCipher = tlsCipher;
     }
 
-    void sentWriteCipherSpec()
+    void notifyChangeCipherSpecReceived()
         throws IOException
     {
         if (pendingCipher == null)
         {
-            throw new TlsFatalAlert(AlertDescription.handshake_failure);
+            throw new TlsFatalAlert(AlertDescription.unexpected_message);
         }
-        this.writeCipher = this.pendingCipher;
-        this.writeSeqNo = new SequenceNumber();
+
+        enablePendingCipherRead(false);
     }
 
-    void receivedReadCipherSpec()
+    void enablePendingCipherRead(boolean deferred)
         throws IOException
     {
         if (pendingCipher == null)
         {
-            throw new TlsFatalAlert(AlertDescription.handshake_failure);
+            throw new TlsFatalAlert(AlertDescription.internal_error);
         }
-        this.readCipher = this.pendingCipher;
-        this.readSeqNo = new SequenceNumber();
+        if (readCipherDeferred != null)
+        {
+            throw new TlsFatalAlert(AlertDescription.internal_error);
+        }
+        if (deferred)
+        {
+            this.readCipherDeferred = pendingCipher;
+        }
+        else
+        {
+            this.readCipher = pendingCipher;
+            this.ciphertextLimit = readCipher.getCiphertextDecodeLimit(plaintextLimit);
+            readSeqNo.reset();
+        }
+    }
+
+    void enablePendingCipherWrite()
+        throws IOException
+    {
+        if (pendingCipher == null)
+        {
+            throw new TlsFatalAlert(AlertDescription.internal_error);
+        }
+        this.writeCipher = this.pendingCipher;
+        writeSeqNo.reset();
     }
 
     void finaliseHandshake()
@@ -106,27 +121,30 @@ class RecordStream
             throw new TlsFatalAlert(AlertDescription.handshake_failure);
         }
         this.pendingCipher = null;
-        this.handshakeHash = new DeferredHash(context);
     }
 
-    RecordPreview previewRecordHeader(byte[] recordHeader, boolean appDataReady) throws IOException
+    boolean needsKeyUpdate()
     {
-        short type = TlsUtils.readUint8(recordHeader, RecordFormat.TYPE_OFFSET);
+        return writeSeqNo.currentValue() >= (1L << 20);
+    }
 
-        if (!appDataReady && type == ContentType.application_data)
-        {
-            throw new TlsFatalAlert(AlertDescription.unexpected_message);
-        }
+    void notifyKeyUpdateReceived() throws IOException
+    {
+        readCipher.rekeyDecoder();
+        readSeqNo.reset();
+    }
 
-        /*
-         * RFC 5246 6. If a TLS implementation receives an unexpected record type, it MUST send an
-         * unexpected_message alert.
-         */
-        checkType(type, AlertDescription.unexpected_message);
+    void notifyKeyUpdateSent() throws IOException
+    {
+        writeCipher.rekeyEncoder();
+        writeSeqNo.reset();
+    }
 
-        /*
-         * legacy_record_version (2 octets at RecordFormat.VERSION_OFFSET) is ignored.
-         */
+    RecordPreview previewRecordHeader(byte[] recordHeader) throws IOException
+    {
+        short recordType = checkRecordType(recordHeader, RecordFormat.TYPE_OFFSET);
+
+//        ProtocolVersion recordVersion = TlsUtils.readVersion(recordHeader, RecordFormat.VERSION_OFFSET);
 
         int length = TlsUtils.readUint16(recordHeader, RecordFormat.LENGTH_OFFSET);
 
@@ -135,21 +153,26 @@ class RecordStream
         int recordSize = RecordFormat.FRAGMENT_OFFSET + length;
         int applicationDataLimit = 0;
 
-        if (type == ContentType.application_data)
+        // NOTE: For TLS 1.3, this only MIGHT be application data
+        if (ContentType.application_data == recordType && handler.isApplicationDataReady())
         {
-            applicationDataLimit = Math.min(getPlaintextLimit(), readCipher.getPlaintextLimit(length));
+            applicationDataLimit = Math.max(0, Math.min(plaintextLimit, readCipher.getPlaintextLimit(length)));
         }
 
         return new RecordPreview(recordSize, applicationDataLimit);
     }
 
-    RecordPreview previewOutputRecord(int applicationDataSize)
+    RecordPreview previewOutputRecord(int contentLength)
     {
-        int applicationDataLimit = Math.max(0, Math.min(getPlaintextLimit(), applicationDataSize));
+        int contentLimit = Math.max(0, Math.min(plaintextLimit, contentLength));
+        int recordSize = previewOutputRecordSize(contentLimit);
+        return new RecordPreview(recordSize, contentLimit);
+    }
 
-        int recordSize = writeCipher.getCiphertextLimit(applicationDataLimit) + RecordFormat.FRAGMENT_OFFSET;
-
-        return new RecordPreview(recordSize, applicationDataLimit);
+    int previewOutputRecordSize(int contentLength)
+    {
+//        assert contentLength <= plaintextLimit
+        return RecordFormat.FRAGMENT_OFFSET + writeCipher.getCiphertextEncodeLimit(contentLength, plaintextLimit);
     }
 
     boolean readFullRecord(byte[] input, int inputOff, int inputLen)
@@ -166,22 +189,22 @@ class RecordStream
             return false;
         }
 
-        short type = TlsUtils.readUint8(input, inputOff + RecordFormat.TYPE_OFFSET);
+        short recordType = checkRecordType(input, inputOff + RecordFormat.TYPE_OFFSET);
 
-        /*
-         * RFC 5246 6. If a TLS implementation receives an unexpected record type, it MUST send an
-         * unexpected_message alert.
-         */
-        checkType(type, AlertDescription.unexpected_message);
-
-        /*
-         * legacy_record_version (2 octets at RecordFormat.VERSION_OFFSET) is ignored.
-         */
+        ProtocolVersion recordVersion = TlsUtils.readVersion(input, inputOff + RecordFormat.VERSION_OFFSET);
 
         checkLength(length, ciphertextLimit, AlertDescription.record_overflow);
 
-        byte[] plaintext = decodeAndVerify(type, input, inputOff + RecordFormat.FRAGMENT_OFFSET, length);
-        handler.processRecord(type, plaintext, 0, plaintext.length);
+        if (ignoreChangeCipherSpec && ContentType.change_cipher_spec == recordType)
+        {
+            checkChangeCipherSpec(input, inputOff + RecordFormat.FRAGMENT_OFFSET, length);
+            return true;
+        }
+
+        TlsDecodeResult decoded = decodeAndVerify(recordType, recordVersion, input,
+            inputOff + RecordFormat.FRAGMENT_OFFSET, length);
+
+        handler.processRecord(decoded.contentType, decoded.buf, decoded.off, decoded.len);
         return true;
     }
 
@@ -193,17 +216,9 @@ class RecordStream
             return false;
         }
 
-        short type = TlsUtils.readUint8(inputRecord.buf, RecordFormat.TYPE_OFFSET);
+        short recordType = checkRecordType(inputRecord.buf, RecordFormat.TYPE_OFFSET);
 
-        /*
-         * RFC 5246 6. If a TLS implementation receives an unexpected record type, it MUST send an
-         * unexpected_message alert.
-         */
-        checkType(type, AlertDescription.unexpected_message);
-
-        /*
-         * legacy_record_version (2 octets at RecordFormat.VERSION_OFFSET) is ignored.
-         */
+        ProtocolVersion recordVersion = TlsUtils.readVersion(inputRecord.buf, RecordFormat.VERSION_OFFSET);
 
         int length = TlsUtils.readUint16(inputRecord.buf, RecordFormat.LENGTH_OFFSET);
 
@@ -211,33 +226,39 @@ class RecordStream
 
         inputRecord.readFragment(input, length);
 
-        byte[] plaintext;
+        TlsDecodeResult decoded;
         try
         {
-            plaintext = decodeAndVerify(type, inputRecord.buf, RecordFormat.FRAGMENT_OFFSET, length);
+            if (ignoreChangeCipherSpec && ContentType.change_cipher_spec == recordType)
+            {
+                checkChangeCipherSpec(inputRecord.buf, RecordFormat.FRAGMENT_OFFSET, length);
+                return true;
+            }
+
+            decoded = decodeAndVerify(recordType, recordVersion, inputRecord.buf, RecordFormat.FRAGMENT_OFFSET, length);
         }
         finally
         {
             inputRecord.reset();
         }
 
-        handler.processRecord(type, plaintext, 0, plaintext.length);
+        handler.processRecord(decoded.contentType, decoded.buf, decoded.off, decoded.len);
         return true;
     }
 
-    byte[] decodeAndVerify(short type, byte[] ciphertext, int off, int len)
+    TlsDecodeResult decodeAndVerify(short recordType, ProtocolVersion recordVersion, byte[] ciphertext, int off, int len)
         throws IOException
     {
         long seqNo = readSeqNo.nextValue(AlertDescription.unexpected_message);
-        byte[] decoded = readCipher.decodeCiphertext(seqNo, type, ciphertext, off, len);
+        TlsDecodeResult decoded = readCipher.decodeCiphertext(seqNo, recordType, recordVersion, ciphertext, off, len);
 
-        checkLength(decoded.length, plaintextLimit, AlertDescription.record_overflow);
+        checkLength(decoded.len, plaintextLimit, AlertDescription.record_overflow);
 
         /*
          * RFC 5246 6.2.1 Implementations MUST NOT send zero-length fragments of Handshake, Alert,
          * or ChangeCipherSpec content types.
          */
-        if (decoded.length < 1 && type != ContentType.application_data)
+        if (decoded.len < 1 && decoded.contentType != ContentType.application_data)
         {
             throw new TlsFatalAlert(AlertDescription.illegal_parameter);
         }
@@ -245,7 +266,7 @@ class RecordStream
         return decoded;
     }
 
-    void writeRecord(short type, byte[] plaintext, int plaintextOffset, int plaintextLength)
+    void writeRecord(short contentType, byte[] plaintext, int plaintextOffset, int plaintextLength)
         throws IOException
     {
         // Never send anything until a valid ClientHello has been received
@@ -254,12 +275,6 @@ class RecordStream
             return;
         }
 
-        /*
-         * RFC 5246 6. Implementations MUST NOT send record types not defined in this document
-         * unless negotiated by some extension.
-         */
-        checkType(type, AlertDescription.internal_error);
-
         /*
          * RFC 5246 6.2.1 The length should not exceed 2^14.
          */
@@ -269,29 +284,27 @@ class RecordStream
          * RFC 5246 6.2.1 Implementations MUST NOT send zero-length fragments of Handshake, Alert,
          * or ChangeCipherSpec content types.
          */
-        if (plaintextLength < 1 && type != ContentType.application_data)
+        if (plaintextLength < 1 && contentType != ContentType.application_data)
         {
             throw new TlsFatalAlert(AlertDescription.internal_error);
         }
 
         long seqNo = writeSeqNo.nextValue(AlertDescription.internal_error);
+        ProtocolVersion recordVersion = writeVersion;
 
-        byte[] ciphertext = writeCipher.encodePlaintext(seqNo, type, plaintext, plaintextOffset, plaintextLength);
+        TlsEncodeResult encoded = writeCipher.encodePlaintext(seqNo, contentType, recordVersion,
+            RecordFormat.FRAGMENT_OFFSET, plaintext, plaintextOffset, plaintextLength);
 
-        /*
-         * RFC 5246 6.2.3. The length may not exceed 2^14 + 2048.
-         */
-        checkLength(ciphertext.length, ciphertextLimit, AlertDescription.internal_error);
+        int ciphertextLength = encoded.len - RecordFormat.FRAGMENT_OFFSET;
+        TlsUtils.checkUint16(ciphertextLength);
 
-        byte[] record = new byte[RecordFormat.FRAGMENT_OFFSET + ciphertext.length];
-        TlsUtils.writeUint8(type, record, RecordFormat.TYPE_OFFSET);
-        TlsUtils.writeVersion(writeVersion, record, RecordFormat.VERSION_OFFSET);
-        TlsUtils.writeUint16(ciphertext.length, record, RecordFormat.LENGTH_OFFSET);
-        System.arraycopy(ciphertext, 0, record, RecordFormat.FRAGMENT_OFFSET, ciphertext.length);
+        TlsUtils.writeUint8(encoded.recordType, encoded.buf, encoded.off + RecordFormat.TYPE_OFFSET);
+        TlsUtils.writeVersion(recordVersion, encoded.buf, encoded.off + RecordFormat.VERSION_OFFSET);
+        TlsUtils.writeUint16(ciphertextLength, encoded.buf, encoded.off + RecordFormat.LENGTH_OFFSET);
 
         try
         {
-            output.write(record);
+            output.write(encoded.buf, encoded.off, encoded.len);
         }
         catch (InterruptedIOException e)
         {
@@ -301,28 +314,6 @@ class RecordStream
         output.flush();
     }
 
-    void notifyHelloComplete()
-    {
-        this.handshakeHash = handshakeHash.notifyPRFDetermined();
-    }
-
-    TlsHandshakeHash getHandshakeHash()
-    {
-        return handshakeHash;
-    }
-
-    OutputStream getHandshakeHashUpdater()
-    {
-        return handshakeHashUpdater;
-    }
-
-    TlsHandshakeHash prepareToFinish()
-    {
-        TlsHandshakeHash result = handshakeHash;
-        this.handshakeHash = handshakeHash.stopTracking();
-        return result;
-    }
-
     void close() throws IOException
     {
         inputRecord.reset();
@@ -360,26 +351,64 @@ class RecordStream
         }
     }
 
-    void flush()
+    private void checkChangeCipherSpec(byte[] buf, int off, int len)
         throws IOException
     {
-        output.flush();
+        if (1 != len || (byte)ChangeCipherSpec.change_cipher_spec != buf[off])
+        {
+            throw new TlsFatalAlert(AlertDescription.unexpected_message);
+        }
     }
 
-    private static void checkType(short type, short alertDescription)
+    private short checkRecordType(byte[] buf, int off)
         throws IOException
     {
-        switch (type)
+        short recordType = TlsUtils.readUint8(buf, off);
+
+        if (null != readCipherDeferred && recordType == ContentType.application_data)
         {
-        case ContentType.application_data:
-        case ContentType.alert:
-        case ContentType.change_cipher_spec:
-        case ContentType.handshake:
-//        case ContentType.heartbeat:
-            break;
-        default:
-            throw new TlsFatalAlert(alertDescription);
+            this.readCipher = readCipherDeferred;
+            this.readCipherDeferred = null;
+            this.ciphertextLimit = readCipher.getCiphertextDecodeLimit(plaintextLimit);
+            readSeqNo.reset();
         }
+        else if (readCipher.usesOpaqueRecordType())
+        {
+            if (ContentType.application_data != recordType)
+            {
+                if (ignoreChangeCipherSpec && ContentType.change_cipher_spec == recordType)
+                {
+                    // See RFC 8446 D.4.
+                }
+                else
+                {
+                    throw new TlsFatalAlert(AlertDescription.unexpected_message, ContentType.getText(recordType));
+                }
+            }
+        }
+        else
+        {
+            switch (recordType)
+            {
+            case ContentType.application_data:
+            {
+                if (!handler.isApplicationDataReady())
+                {
+                    throw new TlsFatalAlert(AlertDescription.unexpected_message);
+                }
+                break;
+            }
+            case ContentType.alert:
+            case ContentType.change_cipher_spec:
+            case ContentType.handshake:
+    //        case ContentType.heartbeat:
+                break;
+            default:
+                throw new TlsFatalAlert(AlertDescription.unexpected_message);
+            }
+        }
+
+        return recordType;
     }
 
     private static void checkLength(int length, int limit, short alertDescription)
@@ -477,6 +506,11 @@ class RecordStream
         private long value = 0L;
         private boolean exhausted = false;
 
+        synchronized long currentValue()
+        {
+            return value;
+        }
+
         synchronized long nextValue(short alertDescription) throws TlsFatalAlert
         {
             if (exhausted)
@@ -490,5 +524,11 @@ class RecordStream
             }
             return result;
         }
+
+        synchronized void reset()
+        {
+            this.value = 0L;
+            this.exhausted = false;
+        }
     }
 }
