@@ -29,7 +29,12 @@ public class SRPTlsClient
 
     protected int[] getSupportedCipherSuites()
     {
-        return TlsUtils.getSupportedCipherSuites(context.getCrypto(), DEFAULT_CIPHER_SUITES);
+        return TlsUtils.getSupportedCipherSuites(getCrypto(), DEFAULT_CIPHER_SUITES);
+    }
+
+    protected ProtocolVersion[] getSupportedVersions()
+    {
+        return ProtocolVersion.TLSv12.downTo(ProtocolVersion.TLSv10);
     }
 
     protected boolean requireSRPServerExtension()
@@ -38,6 +43,12 @@ public class SRPTlsClient
         return false;
     }
 
+    /** @deprecated Unused; will be removed */
+    public ProtocolVersion getClientVersion()
+    {
+        return ProtocolVersion.TLSv12;
+    }
+
     public Hashtable getClientExtensions()
         throws IOException
     {
