@@ -21,6 +21,11 @@ public class TlsCryptoParameters
         this.context = context;
     }
 
+    public SecurityParameters getSecurityParametersConnection()
+    {
+        return context.getSecurityParametersConnection();
+    }
+
     public SecurityParameters getSecurityParametersHandshake()
     {
         return context.getSecurityParametersHandshake();
@@ -31,6 +36,11 @@ public class TlsCryptoParameters
         return context.getClientVersion();
     }
 
+    public ProtocolVersion getRSAPreMasterSecretVersion()
+    {
+        return context.getRSAPreMasterSecretVersion();
+    }
+
     public ProtocolVersion getServerVersion()
     {
         return context.getServerVersion();
