@@ -2,8 +2,7 @@ package org.bouncycastle.tls.crypto;
 
 import java.io.IOException;
 
-import org.bouncycastle.tls.EncryptionAlgorithm;
-import org.bouncycastle.tls.MACAlgorithm;
+import org.bouncycastle.tls.HashAlgorithm;
 
 /**
  * Interface supporting the generation of key material and other SSL/TLS secret values from PRFs.
@@ -21,19 +20,6 @@ public interface TlsSecret
      */
     TlsSecret deriveUsingPRF(int prfAlgorithm, String label, byte[] seed, int length);
 
-    /**
-     * Create a cipher suite that matches the passed in encryption algorithm and mac algorithm.
-     * <p>
-     * See enumeration classes {@link EncryptionAlgorithm}, {@link MACAlgorithm} for appropriate argument values.
-     * </p>
-     * @param cryptoParams context specific parameters.
-     * @param encryptionAlgorithm the encryption algorithm to be employed by the cipher suite.
-     * @param macAlgorithm  the MAC algorithm to be employed by the cipher suite.
-     * @return a TlsCipherSuite supporting the encryption and mac algorithm.
-     * @throws IOException
-     */
-    TlsCipher createCipher(TlsCryptoParameters cryptoParams, int encryptionAlgorithm, int macAlgorithm) throws IOException;
-
     /**
      * Destroy the internal state of the secret. After this call, any attempt to use the
      * {@link TlsSecret} will result in an {@link IllegalStateException} being thrown.
@@ -56,4 +42,32 @@ public interface TlsSecret
      * @return the secret's internal data.
      */
     byte[] extract();
+
+    /**
+     * RFC 5869 HKDF-Expand function, with this secret's data as the pseudo-random key ('prk').
+     * 
+     * @param hashAlgorithm
+     *            the hash algorithm to instantiate HMAC with. See {@link HashAlgorithm} for values.
+     * @param info
+     *            optional context and application specific information (can be zero-length).
+     * @param length
+     *            length of output keying material in octets.
+     * @return output keying material (of 'length' octets).
+     */
+    TlsSecret hkdfExpand(short hashAlgorithm, byte[] info, int length);
+
+    /**
+     * RFC 5869 HKDF-Extract function, with this secret's data as the 'salt'. The {@link TlsSecret}
+     * does not keep a copy of the data. After this call, any attempt to use the {@link TlsSecret}
+     * will result in an {@link IllegalStateException} being thrown.
+     * 
+     * @param hashAlgorithm
+     *            the hash algorithm to instantiate HMAC with. See {@link HashAlgorithm} for values.
+     * @param ikm
+     *            input keying material.
+     * @return a pseudo-random key (of HashLen octets).
+     */
+    TlsSecret hkdfExtract(short hashAlgorithm, byte[] ikm);
+    
+    boolean isAlive();
 }
