@@ -3,8 +3,6 @@ package org.bouncycastle.tls.crypto.impl;
 import java.io.IOException;
 
 import org.bouncycastle.tls.crypto.TlsCertificate;
-import org.bouncycastle.tls.crypto.TlsCipher;
-import org.bouncycastle.tls.crypto.TlsCryptoParameters;
 import org.bouncycastle.tls.crypto.TlsSecret;
 import org.bouncycastle.util.Arrays;
 
@@ -36,12 +34,6 @@ public abstract class AbstractTlsSecret
 
     protected abstract AbstractTlsCrypto getCrypto();
 
-    public TlsCipher createCipher(TlsCryptoParameters cryptoParams, int encryptionAlgorithm, int macAlgorithm)
-        throws IOException
-    {
-        return getCrypto().createCipher(cryptoParams, encryptionAlgorithm, macAlgorithm);
-    }
-
     public synchronized void destroy()
     {
         if (data != null)
@@ -68,6 +60,11 @@ public abstract class AbstractTlsSecret
         return result;
     }
 
+    public synchronized boolean isAlive()
+    {
+        return null != data;
+    }
+
     synchronized byte[] copyData()
     {
         return Arrays.clone(data);
