@@ -1,6 +1,8 @@
 package org.bouncycastle.tls.crypto.impl.jcajce;
 
+import java.io.IOException;
 import java.security.PrivateKey;
+import java.security.PublicKey;
 import java.security.interfaces.DSAPrivateKey;
 import java.security.interfaces.RSAPrivateKey;
 
@@ -17,13 +19,27 @@ import org.bouncycastle.tls.crypto.TlsSigner;
 public class JcaDefaultTlsCredentialedSigner
     extends DefaultTlsCredentialedSigner
 {
-    private static TlsSigner makeSigner(JcaTlsCrypto crypto, PrivateKey privateKey,
+    private static JcaTlsCertificate getEndEntity(JcaTlsCrypto crypto, Certificate certificate) throws IOException
+    {
+        if (certificate == null || certificate.isEmpty())
+        {
+            throw new IllegalArgumentException("No certificate");
+        }
+
+        return JcaTlsCertificate.convert(crypto, certificate.getCertificateAt(0));
+    }
+
+    private static TlsSigner makeSigner(JcaTlsCrypto crypto, PrivateKey privateKey, Certificate certificate,
         SignatureAndHashAlgorithm signatureAndHashAlgorithm)
     {
         String algorithm = privateKey.getAlgorithm();
 
         TlsSigner signer;
-        if (privateKey instanceof RSAPrivateKey || "RSA".equals(algorithm))
+
+        // TODO We probably want better distinction b/w the rsa_pss_pss and rsa_pss_rsae cases here
+        if (privateKey instanceof RSAPrivateKey
+            || "RSA".equalsIgnoreCase(algorithm)
+            || "RSASSA-PSS".equalsIgnoreCase(algorithm))
         {
             if (signatureAndHashAlgorithm != null)
             {
@@ -40,9 +56,20 @@ public class JcaDefaultTlsCredentialedSigner
                 }
             }
 
-            signer = new JcaTlsRSASigner(crypto, privateKey);
+            PublicKey publicKey;
+            try
+            {
+                publicKey = getEndEntity(crypto, certificate).getPubKeyRSA();
+            }
+            catch (Exception e)
+            {
+                throw new RuntimeException(e);
+            }
+
+            signer = new JcaTlsRSASigner(crypto, privateKey, publicKey);
         }
-        else if (privateKey instanceof DSAPrivateKey || "DSA".equals(algorithm))
+        else if (privateKey instanceof DSAPrivateKey
+            || "DSA".equalsIgnoreCase(algorithm))
         {
             signer = new JcaTlsDSASigner(crypto, privateKey);
         }
@@ -50,14 +77,12 @@ public class JcaDefaultTlsCredentialedSigner
         {
             signer = new JcaTlsECDSASigner(crypto, privateKey);
         }
-        else if ("Ed25519".equals(algorithm))
+        else if ("Ed25519".equalsIgnoreCase(algorithm))
         {
-            // TODO[RFC 8422] Extract public key from certificate?
             signer = new JcaTlsEd25519Signer(crypto, privateKey);
         }
-        else if ("Ed448".equals(algorithm))
+        else if ("Ed448".equalsIgnoreCase(algorithm))
         {
-            // TODO[RFC 8422] Extract public key from certificate?
             signer = new JcaTlsEd448Signer(crypto, privateKey);
         }
         else
@@ -71,7 +96,7 @@ public class JcaDefaultTlsCredentialedSigner
     public JcaDefaultTlsCredentialedSigner(TlsCryptoParameters cryptoParams, JcaTlsCrypto crypto, PrivateKey privateKey,
         Certificate certificate, SignatureAndHashAlgorithm signatureAndHashAlgorithm)
     {
-        super(cryptoParams, makeSigner(crypto, privateKey, signatureAndHashAlgorithm), certificate,
+        super(cryptoParams, makeSigner(crypto, privateKey, certificate, signatureAndHashAlgorithm), certificate,
             signatureAndHashAlgorithm);
     }
 }
