@@ -2,21 +2,23 @@ package org.bouncycastle.tls.crypto.impl.jcajce;
 
 import java.io.IOException;
 import java.math.BigInteger;
+import java.security.AlgorithmParameters;
 import java.security.GeneralSecurityException;
 import java.security.NoSuchAlgorithmException;
 import java.security.PrivateKey;
 import java.security.PublicKey;
 import java.security.SecureRandom;
-import java.security.interfaces.RSAPublicKey;
+import java.security.Signature;
+import java.security.spec.AlgorithmParameterSpec;
 import java.util.Hashtable;
 
 import javax.crypto.Cipher;
 import javax.crypto.KeyAgreement;
 import javax.crypto.spec.SecretKeySpec;
 
-import org.bouncycastle.asn1.x509.KeyUsage;
 import org.bouncycastle.jcajce.util.JcaJceHelper;
 import org.bouncycastle.tls.AlertDescription;
+import org.bouncycastle.tls.DigitallySigned;
 import org.bouncycastle.tls.EncryptionAlgorithm;
 import org.bouncycastle.tls.HashAlgorithm;
 import org.bouncycastle.tls.MACAlgorithm;
@@ -24,6 +26,8 @@ import org.bouncycastle.tls.NamedGroup;
 import org.bouncycastle.tls.ProtocolVersion;
 import org.bouncycastle.tls.SignatureAlgorithm;
 import org.bouncycastle.tls.SignatureAndHashAlgorithm;
+import org.bouncycastle.tls.SignatureScheme;
+import org.bouncycastle.tls.TlsDHUtils;
 import org.bouncycastle.tls.TlsFatalAlert;
 import org.bouncycastle.tls.TlsUtils;
 import org.bouncycastle.tls.crypto.SRP6Group;
@@ -43,6 +47,8 @@ import org.bouncycastle.tls.crypto.TlsSRP6Server;
 import org.bouncycastle.tls.crypto.TlsSRP6VerifierGenerator;
 import org.bouncycastle.tls.crypto.TlsSRPConfig;
 import org.bouncycastle.tls.crypto.TlsSecret;
+import org.bouncycastle.tls.crypto.TlsStreamSigner;
+import org.bouncycastle.tls.crypto.TlsStreamVerifier;
 import org.bouncycastle.tls.crypto.impl.AbstractTlsCrypto;
 import org.bouncycastle.tls.crypto.impl.TlsAEADCipher;
 import org.bouncycastle.tls.crypto.impl.TlsAEADCipherImpl;
@@ -71,7 +77,9 @@ public class JcaTlsCrypto
     private final SecureRandom entropySource;
     private final SecureRandom nonceEntropySource;
 
-    private final Hashtable supportedGroups = new Hashtable();
+    private final Hashtable supportedEncryptionAlgorithms = new Hashtable();
+    private final Hashtable supportedNamedGroups = new Hashtable();
+    private final Hashtable supportedOther = new Hashtable();
 
     /**
      * Base constructor.
@@ -142,7 +150,7 @@ public class JcaTlsCrypto
         return new JcaTlsCertificate(this, encoding);
     }
 
-    protected TlsCipher createCipher(TlsCryptoParameters cryptoParams, int encryptionAlgorithm, int macAlgorithm)
+    public TlsCipher createCipher(TlsCryptoParameters cryptoParams, int encryptionAlgorithm, int macAlgorithm)
         throws IOException
     {
         try
@@ -162,9 +170,6 @@ public class JcaTlsCrypto
             case EncryptionAlgorithm.AES_128_GCM:
                 // NOTE: Ignores macAlgorithm
                 return createCipher_AES_GCM(cryptoParams, 16, 16);
-            case EncryptionAlgorithm.AES_128_OCB_TAGLEN96:
-                // NOTE: Ignores macAlgorithm
-                return createCipher_AES_OCB(cryptoParams, 16, 12);
             case EncryptionAlgorithm.AES_256_CBC:
                 return createAESCipher(cryptoParams, 32, macAlgorithm);
             case EncryptionAlgorithm.AES_256_CCM:
@@ -176,9 +181,6 @@ public class JcaTlsCrypto
             case EncryptionAlgorithm.AES_256_GCM:
                 // NOTE: Ignores macAlgorithm
                 return createCipher_AES_GCM(cryptoParams, 32, 16);
-            case EncryptionAlgorithm.AES_256_OCB_TAGLEN96:
-                // NOTE: Ignores macAlgorithm
-                return createCipher_AES_OCB(cryptoParams, 32, 12);
             case EncryptionAlgorithm.ARIA_128_CBC:
                 return createARIACipher(cryptoParams, 16, macAlgorithm);
             case EncryptionAlgorithm.ARIA_128_GCM:
@@ -206,6 +208,13 @@ public class JcaTlsCrypto
                 return createNullCipher(cryptoParams, macAlgorithm);
             case EncryptionAlgorithm.SEED_CBC:
                 return createSEEDCipher(cryptoParams, macAlgorithm);
+
+            case EncryptionAlgorithm.DES40_CBC:
+            case EncryptionAlgorithm.DES_CBC:
+            case EncryptionAlgorithm.IDEA_CBC:
+            case EncryptionAlgorithm.RC2_CBC_40:
+            case EncryptionAlgorithm.RC4_128:
+            case EncryptionAlgorithm.RC4_40:
             default:
                 throw new TlsFatalAlert(AlertDescription.internal_error);
             }
@@ -216,24 +225,46 @@ public class JcaTlsCrypto
         }
     }
 
+    public TlsHMAC createHMAC(short hashAlgorithm)
+    {
+        return createHMAC(getHMACAlgorithmName(hashAlgorithm));
+    }
+
     public TlsHMAC createHMAC(int macAlgorithm)
+    {
+        return createHMAC(TlsUtils.getHashAlgorithmForHMACAlgorithm(macAlgorithm));
+    }
+
+    protected TlsHMAC createHMAC_SSL(int macAlgorithm)
+        throws GeneralSecurityException, IOException
     {
         switch (macAlgorithm)
         {
-        case MACAlgorithm._null:
-            return null;
         case MACAlgorithm.hmac_md5:
-            return createHMAC("HmacMD5");
+            return new JcaSSL3HMAC(createHash(getDigestName(HashAlgorithm.md5)), 16, 64);
         case MACAlgorithm.hmac_sha1:
-            return createHMAC("HmacSHA1");
+            return new JcaSSL3HMAC(createHash(getDigestName(HashAlgorithm.sha1)), 20, 64);
         case MACAlgorithm.hmac_sha256:
-            return createHMAC("HmacSHA256");
+            return new JcaSSL3HMAC(createHash(getDigestName(HashAlgorithm.sha256)), 32, 64);
         case MACAlgorithm.hmac_sha384:
-            return createHMAC("HmacSHA384");
+            return new JcaSSL3HMAC(createHash(getDigestName(HashAlgorithm.sha384)), 48, 128);
         case MACAlgorithm.hmac_sha512:
-            return createHMAC("HmacSHA512");
+            return new JcaSSL3HMAC(createHash(getDigestName(HashAlgorithm.sha512)), 64, 128);
         default:
-            throw new IllegalArgumentException("unknown MACAlgorithm: " + MACAlgorithm.getText(macAlgorithm));
+            throw new TlsFatalAlert(AlertDescription.internal_error);
+        }
+    }
+
+    protected TlsHMAC createMAC(TlsCryptoParameters cryptoParams, int macAlgorithm)
+        throws GeneralSecurityException, IOException
+    {
+        if (TlsImplUtils.isSSL(cryptoParams))
+        {
+            return createHMAC_SSL(macAlgorithm);
+        }
+        else
+        {
+            return createHMAC(macAlgorithm);
         }
     }
 
@@ -311,6 +342,87 @@ public class JcaTlsCrypto
         };
     }
 
+    public String getHMACAlgorithmName(short hashAlgorithm)
+    {
+        switch (hashAlgorithm)
+        {
+        case HashAlgorithm.md5:
+            return "HmacMD5";
+        case HashAlgorithm.sha1:
+            return "HmacSHA1";
+        case HashAlgorithm.sha224:
+            return "HmacSHA224";
+        case HashAlgorithm.sha256:
+            return "HmacSHA256";
+        case HashAlgorithm.sha384:
+            return "HmacSHA384";
+        case HashAlgorithm.sha512:
+            return "HmacSHA512";
+        default:
+            throw new IllegalArgumentException("invalid HashAlgorithm: " + HashAlgorithm.getText(hashAlgorithm));
+        }
+    }
+
+    public AlgorithmParameters getNamedGroupAlgorithmParameters(int namedGroup) throws GeneralSecurityException
+    {
+        if (NamedGroup.refersToAnXDHCurve(namedGroup))
+        {
+            switch (namedGroup)
+            {
+            /*
+             * TODO Return AlgorithmParameters to check against disabled algorithms
+             * 
+             * NOTE: The JDK doesn't even support AlgorithmParameters for XDH, so SunJSSE also winds
+             * up using null AlgorithmParameters when checking algorithm constraints.
+             */
+            case NamedGroup.x25519:
+            case NamedGroup.x448:
+                return null;
+            }
+        }
+        else if (NamedGroup.refersToAnECDSACurve(namedGroup))
+        {
+            return ECUtil.getAlgorithmParameters(this, NamedGroup.getName(namedGroup));
+        }
+        else if (NamedGroup.refersToASpecificFiniteField(namedGroup))
+        {
+            return DHUtil.getAlgorithmParameters(this, TlsDHUtils.getNamedDHGroup(namedGroup));
+        }
+
+        throw new IllegalArgumentException("NamedGroup not supported: " + NamedGroup.getText(namedGroup));
+    }
+
+    public AlgorithmParameters getSignatureSchemeAlgorithmParameters(int signatureScheme)
+        throws GeneralSecurityException
+    {
+        switch (signatureScheme)
+        {
+        case SignatureScheme.rsa_pss_pss_sha256:
+        case SignatureScheme.rsa_pss_rsae_sha256:
+        case SignatureScheme.rsa_pss_pss_sha384:
+        case SignatureScheme.rsa_pss_rsae_sha384:
+        case SignatureScheme.rsa_pss_pss_sha512:
+        case SignatureScheme.rsa_pss_rsae_sha512:
+        {
+            short hash = SignatureScheme.getRSAPSSHashAlgorithm(signatureScheme);
+            String digestName = getDigestName(hash);
+            String sigName = RSAUtil.getDigestSigAlgName(digestName) + "WITHRSAANDMGF1";
+
+            AlgorithmParameterSpec pssSpec = RSAUtil.getPSSParameterSpec(hash, digestName, getHelper());
+
+            Signature signer = getHelper().createSignature(sigName);
+
+            // NOTE: We explicitly set them even though they should be the defaults, because providers vary
+            signer.setParameter(pssSpec);
+
+            return signer.getParameters();
+        }
+
+        default:
+            return null;
+        }
+    }
+
     public boolean hasAllRawSignatureAlgorithms()
     {
         // TODO[RFC 8422] Revisit the need to buffer the handshake for "Intrinsic" hash signatures
@@ -331,57 +443,35 @@ public class JcaTlsCrypto
 
     public boolean hasEncryptionAlgorithm(int encryptionAlgorithm)
     {
-        try
+        final Integer key = Integers.valueOf(encryptionAlgorithm);
+        synchronized (supportedEncryptionAlgorithms)
         {
-            switch (encryptionAlgorithm)
+            Boolean cached = (Boolean)supportedEncryptionAlgorithms.get(key);
+            if (cached != null)
             {
-            case EncryptionAlgorithm.CHACHA20_POLY1305:
-            {
-                helper.createCipher("ChaCha7539");
-                helper.createMac("Poly1305");
-                break;
-            }
-            case EncryptionAlgorithm.AES_128_CCM:
-            case EncryptionAlgorithm.AES_128_CCM_8:
-            case EncryptionAlgorithm.AES_256_CCM:
-            case EncryptionAlgorithm.AES_256_CCM_8:
-            {
-                helper.createCipher("AES/CCM/NoPadding");
-                break;
-            }
-            case EncryptionAlgorithm.AES_128_GCM:
-            case EncryptionAlgorithm.AES_256_GCM:
-            {
-                helper.createCipher("AES/GCM/NoPadding");
-                break;
-            }
-            case EncryptionAlgorithm.ARIA_128_GCM:
-            case EncryptionAlgorithm.ARIA_256_GCM:
-            {
-                helper.createCipher("ARIA/GCM/NoPadding");
-                break;
-            }
-            case EncryptionAlgorithm.CAMELLIA_128_GCM:
-            case EncryptionAlgorithm.CAMELLIA_256_GCM:
-            {
-                helper.createCipher("CAMELLIA/GCM/NoPadding");
-                break;
-            }
-
-            case EncryptionAlgorithm.DES40_CBC:
-            case EncryptionAlgorithm.DES_CBC:
-            case EncryptionAlgorithm.IDEA_CBC:
-            case EncryptionAlgorithm.RC2_CBC_40:
-            case EncryptionAlgorithm.RC4_128:
-            case EncryptionAlgorithm.RC4_40:
-                return false;
+                return cached.booleanValue();
             }
         }
-        catch (GeneralSecurityException e)
+
+        Boolean supported = isSupportedEncryptionAlgorithm(encryptionAlgorithm);
+        if (null == supported)
         {
             return false;
         }
-        return true;
+
+        synchronized (supportedEncryptionAlgorithms)
+        {
+            Boolean cached = (Boolean)supportedEncryptionAlgorithms.put(key, supported);
+
+            // Unlikely, but we want a consistent result
+            if (null != cached && supported != cached)
+            {
+                supportedEncryptionAlgorithms.put(key, cached);
+                supported = cached;
+            }
+        }
+
+        return supported.booleanValue();
     }
 
     public boolean hasHashAlgorithm(short hashAlgorithm)
@@ -398,84 +488,73 @@ public class JcaTlsCrypto
 
     public boolean hasNamedGroup(int namedGroup)
     {
-        // TODO[tls] Actually check for DH support for the individual groups
-        if (NamedGroup.refersToASpecificFiniteField(namedGroup))
+        final Integer key = Integers.valueOf(namedGroup);
+        synchronized (supportedNamedGroups)
         {
-            return true;
+            Boolean cached = (Boolean)supportedNamedGroups.get(key);
+            if (null != cached)
+            {
+                return cached.booleanValue();
+            }
         }
 
-        if (!NamedGroup.refersToASpecificCurve(namedGroup))
+        Boolean supported = isSupportedNamedGroup(namedGroup);
+        if (null == supported)
         {
             return false;
         }
 
-        String groupName = NamedGroup.getName(namedGroup);
-        if (groupName == null)
+        synchronized (supportedNamedGroups)
         {
-            return false;
+            Boolean cached = (Boolean)supportedNamedGroups.put(key, supported);
+
+            // Unlikely, but we want a consistent result
+            if (null != cached && supported != cached)
+            {
+                supportedNamedGroups.put(key, cached);
+                supported = cached;
+            }
         }
 
-        Integer key = Integers.valueOf(namedGroup);
+        return supported.booleanValue();
+    }
 
-        synchronized (supportedGroups)
+    public boolean hasRSAEncryption()
+    {
+        final String key = "KE_RSA";
+        synchronized (supportedOther)
         {
-            Boolean cached = (Boolean)supportedGroups.get(key);
+            Boolean cached = (Boolean)supportedOther.get(key);
             if (cached != null)
             {
                 return cached.booleanValue();
             }
         }
 
-        boolean result = true;
-        try
-        {
-            switch (namedGroup)
-            {
-            case NamedGroup.x25519:
-            {
-                helper.createKeyFactory("X25519");
-                helper.createKeyPairGenerator("X25519");
-                helper.createKeyAgreement("X25519");
-                break;
-            }
-            case NamedGroup.x448:
-            {
-                helper.createKeyFactory("X448");
-                helper.createKeyPairGenerator("X448");
-                helper.createKeyAgreement("X448");
-                break;
-            }
-            default:
-            {
-                result &= isCurveSupported(groupName);
-                break;
-            }
-            }
-        }
-        catch (GeneralSecurityException e)
-        {
-            result = false;
-        }
-
-        synchronized (supportedGroups)
-        {
-            supportedGroups.put(key, Boolean.valueOf(result));
-        }
-
-        return result;
-    }
-
-    public boolean hasRSAEncryption()
-    {
+        Boolean supported;
         try
         {
             createRSAEncryptionCipher();
-            return true;
+            supported = Boolean.TRUE;
         }
         catch (GeneralSecurityException e)
         {
-            return false;
+            supported = Boolean.FALSE;
         }
+
+        synchronized (supportedOther)
+        {
+            Boolean cached = (Boolean)supportedOther.put(key, supported);
+
+            // Unlikely, but we want a consistent result
+            if (null != cached && supported != cached)
+            {
+                supportedOther.put(key, cached);
+                supported = cached;
+            }
+        }
+
+        return supported.booleanValue();
     }
 
     public boolean hasSignatureAlgorithm(short signatureAlgorithm)
@@ -512,6 +591,19 @@ public class JcaTlsCrypto
         return hasSignatureAlgorithm(sigAndHashAlgorithm.getSignature());
     }
 
+    public boolean hasSignatureScheme(int signatureScheme)
+    {
+        /*
+         * This is somewhat overkill, but much simpler for now. It's also consistent with SunJSSE behaviour.
+         */
+        if ((signatureScheme >>> 8) == HashAlgorithm.sha224 && JcaUtils.isSunMSCAPIProviderActive())
+        {
+            return false;
+        }
+
+        return hasSignatureAlgorithm((short)(signatureScheme & 0xFF));
+    }
+
     public boolean hasSRPAuthentication()
     {
         return true;
@@ -575,9 +667,9 @@ public class JcaTlsCrypto
         throws IOException
     {
         JcaTlsCertificate jcaCert = JcaTlsCertificate.convert(this, certificate);
-        jcaCert.validateKeyUsage(KeyUsage.keyEncipherment);
+        jcaCert.validateKeyUsageBit(JcaTlsCertificate.KU_KEY_ENCIPHERMENT);
 
-        final RSAPublicKey pubKeyRSA = jcaCert.getPubKeyRSA();
+        final PublicKey pubKeyRSA = jcaCert.getPubKeyRSA();
 
         return new TlsEncryptor()
         {
@@ -619,6 +711,11 @@ public class JcaTlsCrypto
         };
     }
 
+    public TlsSecret hkdfInit(short hashAlgorithm)
+    {
+        return adoptLocalSecret(new byte[HashAlgorithm.getOutputSize(hashAlgorithm)]);
+    }
+
     /**
      * If you want to create your own versions of the AEAD ciphers required, override this method.
      *
@@ -709,12 +806,234 @@ public class JcaTlsCrypto
     protected TlsNullCipher createNullCipher(TlsCryptoParameters cryptoParams, int macAlgorithm)
         throws IOException, GeneralSecurityException
     {
-        return new TlsNullCipher(cryptoParams, createMAC(macAlgorithm), createMAC(macAlgorithm));
+        return new TlsNullCipher(cryptoParams, createMAC(cryptoParams, macAlgorithm),
+            createMAC(cryptoParams, macAlgorithm));
     }
 
-    protected boolean isCurveSupported(String curveName)
+    protected TlsStreamSigner createStreamSigner(SignatureAndHashAlgorithm algorithm, PrivateKey privateKey,
+        boolean needsRandom) throws IOException
     {
-        return ECUtil.isCurveSupported(curveName, this.getHelper());
+        String algorithmName = JcaUtils.getJcaAlgorithmName(algorithm);
+
+        return createStreamSigner(algorithmName, null, privateKey, needsRandom);
+    }
+
+    protected TlsStreamSigner createStreamSigner(String algorithmName, AlgorithmParameterSpec parameter,
+        PrivateKey privateKey, boolean needsRandom) throws IOException
+    {
+        try
+        {
+            Signature signer = getHelper().createSignature(algorithmName);
+            if (null != parameter)
+            {
+                signer.setParameter(parameter);
+            }
+            signer.initSign(privateKey, needsRandom ? getSecureRandom() : null);
+            return new JcaTlsStreamSigner(signer);
+        }
+        catch (GeneralSecurityException e)
+        {
+            throw new TlsFatalAlert(AlertDescription.internal_error, e);
+        }
+    }
+
+    protected TlsStreamVerifier createStreamVerifier(DigitallySigned signature, PublicKey publicKey) throws IOException
+    {
+        String algorithmName = JcaUtils.getJcaAlgorithmName(signature.getAlgorithm());
+
+        return createStreamVerifier(algorithmName, null, signature.getSignature(), publicKey);
+    }
+
+    protected TlsStreamVerifier createStreamVerifier(String algorithmName, AlgorithmParameterSpec parameter,
+        byte[] signature, PublicKey publicKey) throws IOException
+    {
+        try
+        {
+            Signature verifier = getHelper().createSignature(algorithmName);
+            if (null != parameter)
+            {
+                verifier.setParameter(parameter);
+            }
+            verifier.initVerify(publicKey);
+            return new JcaTlsStreamVerifier(verifier, signature);
+        }
+        catch (GeneralSecurityException e)
+        {
+            throw new TlsFatalAlert(AlertDescription.internal_error, e);
+        }
+    }
+
+    protected TlsStreamSigner createVerifyingStreamSigner(SignatureAndHashAlgorithm algorithm, PrivateKey privateKey,
+        boolean needsRandom, PublicKey publicKey) throws IOException
+    {
+        String algorithmName = JcaUtils.getJcaAlgorithmName(algorithm);
+
+        return createVerifyingStreamSigner(algorithmName, null, privateKey, needsRandom, publicKey);
+    }
+
+    protected TlsStreamSigner createVerifyingStreamSigner(String algorithmName, AlgorithmParameterSpec parameter,
+        PrivateKey privateKey, boolean needsRandom, PublicKey publicKey) throws IOException
+    {
+        try
+        {
+            Signature signer = getHelper().createSignature(algorithmName);
+            Signature verifier = getHelper().createSignature(algorithmName);
+
+            if (null != parameter)
+            {
+                signer.setParameter(parameter);
+                verifier.setParameter(parameter);
+            }
+
+            signer.initSign(privateKey, needsRandom ? getSecureRandom() : null);
+            verifier.initVerify(publicKey);
+
+            return new JcaVerifyingStreamSigner(signer, verifier);
+        }
+        catch (GeneralSecurityException e)
+        {
+            throw new TlsFatalAlert(AlertDescription.internal_error, e);
+        }
+    }
+
+    protected Boolean isSupportedEncryptionAlgorithm(int encryptionAlgorithm)
+    {
+        try
+        {
+            switch (encryptionAlgorithm)
+            {
+            case EncryptionAlgorithm.CHACHA20_POLY1305:
+            {
+                helper.createCipher("ChaCha7539");
+                helper.createMac("Poly1305");
+                return Boolean.TRUE;
+            }
+            case EncryptionAlgorithm._3DES_EDE_CBC:
+            {
+                helper.createCipher("DESede/CBC/NoPadding");
+                return Boolean.TRUE;
+            }
+            case EncryptionAlgorithm.AES_128_CBC:
+            case EncryptionAlgorithm.AES_256_CBC:
+            {
+                helper.createCipher("AES/CBC/NoPadding");
+                return Boolean.TRUE;
+            }
+            case EncryptionAlgorithm.AES_128_CCM:
+            case EncryptionAlgorithm.AES_128_CCM_8:
+            case EncryptionAlgorithm.AES_256_CCM:
+            case EncryptionAlgorithm.AES_256_CCM_8:
+            {
+                helper.createCipher("AES/CCM/NoPadding");
+                return Boolean.TRUE;
+            }
+            case EncryptionAlgorithm.AES_128_GCM:
+            case EncryptionAlgorithm.AES_256_GCM:
+            {
+                helper.createCipher("AES/GCM/NoPadding");
+                return Boolean.TRUE;
+            }
+            case EncryptionAlgorithm.ARIA_128_CBC:
+            case EncryptionAlgorithm.ARIA_256_CBC:
+            {
+                helper.createCipher("ARIA/CBC/NoPadding");
+                return Boolean.TRUE;
+            }
+            case EncryptionAlgorithm.ARIA_128_GCM:
+            case EncryptionAlgorithm.ARIA_256_GCM:
+            {
+                helper.createCipher("ARIA/GCM/NoPadding");
+                return Boolean.TRUE;
+            }
+            case EncryptionAlgorithm.CAMELLIA_128_CBC:
+            case EncryptionAlgorithm.CAMELLIA_256_CBC:
+            {
+                helper.createCipher("Camellia/CBC/NoPadding");
+                return Boolean.TRUE;
+            }
+            case EncryptionAlgorithm.CAMELLIA_128_GCM:
+            case EncryptionAlgorithm.CAMELLIA_256_GCM:
+            {
+                helper.createCipher("Camellia/GCM/NoPadding");
+                return Boolean.TRUE;
+            }
+            case EncryptionAlgorithm.SEED_CBC:
+            {
+                helper.createCipher("SEED/CBC/NoPadding");
+                return Boolean.TRUE;
+            }
+            case EncryptionAlgorithm.NULL:
+            {
+                return Boolean.TRUE;
+            }
+
+            case EncryptionAlgorithm.DES40_CBC:
+            case EncryptionAlgorithm.DES_CBC:
+            case EncryptionAlgorithm.IDEA_CBC:
+            case EncryptionAlgorithm.RC2_CBC_40:
+            case EncryptionAlgorithm.RC4_128:
+            case EncryptionAlgorithm.RC4_40:
+            {
+                return Boolean.FALSE;
+            }
+            }
+        }
+        catch (GeneralSecurityException e)
+        {
+            return Boolean.FALSE;
+        }
+
+        return null;
+    }
+
+    protected Boolean isSupportedNamedGroup(int namedGroup)
+    {
+        try
+        {
+            if (NamedGroup.refersToAnXDHCurve(namedGroup))
+            {
+                /*
+                 * NOTE: We don't check for AlgorithmParameters support because even the SunEC
+                 * provider doesn't support them. We skip checking KeyFactory and KeyPairGenerator
+                 * for performance reasons (and this is consistent with SunJSSE behaviour).
+                 */
+                switch (namedGroup)
+                {
+                case NamedGroup.x25519:
+                {
+//                    helper.createAlgorithmParameters("X25519");
+                    helper.createKeyAgreement("X25519");
+//                    helper.createKeyFactory("X25519");
+//                    helper.createKeyPairGenerator("X25519");
+                    return Boolean.TRUE;
+                }
+                case NamedGroup.x448:
+                {
+//                    helper.createAlgorithmParameters("X448");
+                    helper.createKeyAgreement("X448");
+//                    helper.createKeyFactory("X448");
+//                    helper.createKeyPairGenerator("X448");
+                    return Boolean.TRUE;
+                }
+                }
+            }
+            else if (NamedGroup.refersToAnECDSACurve(namedGroup))
+            {
+                return Boolean.valueOf(ECUtil.isCurveSupported(this, NamedGroup.getName(namedGroup)));
+            }
+            else if (NamedGroup.refersToASpecificFiniteField(namedGroup))
+            {
+                // TODO[tls] Actually check for DH support for the individual groups
+                return Boolean.TRUE;
+            }
+        }
+        catch (GeneralSecurityException e)
+        {
+            return Boolean.FALSE;
+        }
+
+        // 'null' means we don't even recognize the NamedGroup
+        return null;
     }
 
     public JcaJceHelper getHelper()
@@ -725,36 +1044,42 @@ public class JcaTlsCrypto
     private TlsBlockCipher createAESCipher(TlsCryptoParameters cryptoParams, int cipherKeySize, int macAlgorithm)
         throws IOException, GeneralSecurityException
     {
-        return new TlsBlockCipher(this, cryptoParams, createCBCBlockOperator(cryptoParams, "AES", true, cipherKeySize), createCBCBlockOperator(cryptoParams, "AES", false, cipherKeySize),
-            createMAC(macAlgorithm), createMAC(macAlgorithm), cipherKeySize);
+        return new TlsBlockCipher(this, cryptoParams, createCBCBlockOperator(cryptoParams, "AES", true, cipherKeySize),
+            createCBCBlockOperator(cryptoParams, "AES", false, cipherKeySize), createMAC(cryptoParams, macAlgorithm),
+            createMAC(cryptoParams, macAlgorithm), cipherKeySize);
     }
 
     private TlsBlockCipher createARIACipher(TlsCryptoParameters cryptoParams, int cipherKeySize, int macAlgorithm)
         throws IOException, GeneralSecurityException
     {
-        return new TlsBlockCipher(this, cryptoParams, createCBCBlockOperator(cryptoParams, "ARIA", true, cipherKeySize), createCBCBlockOperator(cryptoParams, "ARIA", false, cipherKeySize),
-            createMAC(macAlgorithm), createMAC(macAlgorithm), cipherKeySize);
+        return new TlsBlockCipher(this, cryptoParams, createCBCBlockOperator(cryptoParams, "ARIA", true, cipherKeySize),
+            createCBCBlockOperator(cryptoParams, "ARIA", false, cipherKeySize), createMAC(cryptoParams, macAlgorithm),
+            createMAC(cryptoParams, macAlgorithm), cipherKeySize);
     }
 
     private TlsBlockCipher createCamelliaCipher(TlsCryptoParameters cryptoParams, int cipherKeySize, int macAlgorithm)
         throws IOException, GeneralSecurityException
     {
-        return new TlsBlockCipher(this, cryptoParams, createCBCBlockOperator(cryptoParams, "Camellia", true, cipherKeySize), createCBCBlockOperator(cryptoParams, "Camellia", false, cipherKeySize),
-            createMAC(macAlgorithm), createMAC(macAlgorithm), cipherKeySize);
+        return new TlsBlockCipher(this, cryptoParams,
+            createCBCBlockOperator(cryptoParams, "Camellia", true, cipherKeySize),
+            createCBCBlockOperator(cryptoParams, "Camellia", false, cipherKeySize),
+            createMAC(cryptoParams, macAlgorithm), createMAC(cryptoParams, macAlgorithm), cipherKeySize);
     }
 
     private TlsBlockCipher createDESedeCipher(TlsCryptoParameters cryptoParams, int macAlgorithm)
         throws IOException, GeneralSecurityException
     {
-        return new TlsBlockCipher(this, cryptoParams, createCBCBlockOperator(cryptoParams, "DESede", true, 24), createCBCBlockOperator(cryptoParams, "DESede", false, 24),
-            createMAC(macAlgorithm), createMAC(macAlgorithm), 24);
+        return new TlsBlockCipher(this, cryptoParams, createCBCBlockOperator(cryptoParams, "DESede", true, 24),
+            createCBCBlockOperator(cryptoParams, "DESede", false, 24), createMAC(cryptoParams, macAlgorithm),
+            createMAC(cryptoParams, macAlgorithm), 24);
     }
 
     private TlsBlockCipher createSEEDCipher(TlsCryptoParameters cryptoParams, int macAlgorithm)
         throws IOException, GeneralSecurityException
     {
-        return new TlsBlockCipher(this, cryptoParams, createCBCBlockOperator(cryptoParams, "SEED", true, 16), createCBCBlockOperator(cryptoParams, "SEED", false, 16),
-            createMAC(macAlgorithm), createMAC(macAlgorithm), 16);
+        return new TlsBlockCipher(this, cryptoParams, createCBCBlockOperator(cryptoParams, "SEED", true, 16),
+            createCBCBlockOperator(cryptoParams, "SEED", false, 16), createMAC(cryptoParams, macAlgorithm),
+            createMAC(cryptoParams, macAlgorithm), 16);
     }
 
     private TlsBlockCipherImpl createCBCBlockOperator(TlsCryptoParameters cryptoParams, String algorithm, boolean forEncryption, int keySize)
@@ -772,51 +1097,44 @@ public class JcaTlsCrypto
         }
     }
 
-    private TlsHMAC createMAC(int macAlgorithm) throws IOException
-    {
-        return createHMAC(macAlgorithm);
-    }
-
     private TlsCipher createChaCha20Poly1305(TlsCryptoParameters cryptoParams)
         throws IOException, GeneralSecurityException
     {
         return new TlsAEADCipher(cryptoParams, new JceChaCha20Poly1305(helper, true),
-            new JceChaCha20Poly1305(helper, false), 32, 16, TlsAEADCipher.NONCE_RFC7905);
+            new JceChaCha20Poly1305(helper, false), 32, 16, TlsAEADCipher.AEAD_CHACHA20_POLY1305);
     }
 
     private TlsAEADCipher createCipher_AES_CCM(TlsCryptoParameters cryptoParams, int cipherKeySize, int macSize)
         throws IOException, GeneralSecurityException
     {
-        return new TlsAEADCipher(cryptoParams, createAEADCipher("AES/CCM/NoPadding", "AES", cipherKeySize, true), createAEADCipher("AES/CCM/NoPadding", "AES", cipherKeySize, false),
-            cipherKeySize, macSize);
+        return new TlsAEADCipher(cryptoParams, createAEADCipher("AES/CCM/NoPadding", "AES", cipherKeySize, true),
+            createAEADCipher("AES/CCM/NoPadding", "AES", cipherKeySize, false), cipherKeySize, macSize,
+            TlsAEADCipher.AEAD_CCM);
     }
 
     private TlsAEADCipher createCipher_AES_GCM(TlsCryptoParameters cryptoParams, int cipherKeySize, int macSize)
         throws IOException, GeneralSecurityException
     {
-        return new TlsAEADCipher(cryptoParams, createAEADCipher("AES/GCM/NoPadding", "AES", cipherKeySize, true), createAEADCipher("AES/GCM/NoPadding", "AES", cipherKeySize, false),
-            cipherKeySize, macSize);
-    }
-
-    private TlsAEADCipher createCipher_AES_OCB(TlsCryptoParameters cryptoParams, int cipherKeySize, int macSize)
-        throws IOException, GeneralSecurityException
-    {
-        return new TlsAEADCipher(cryptoParams, createAEADCipher("AES/OCB/NoPadding", "AES", cipherKeySize, true), createAEADCipher("AES/OCB/NoPadding", "AES", cipherKeySize, false),
-            cipherKeySize, macSize, TlsAEADCipher.NONCE_RFC7905);
+        return new TlsAEADCipher(cryptoParams, createAEADCipher("AES/GCM/NoPadding", "AES", cipherKeySize, true),
+            createAEADCipher("AES/GCM/NoPadding", "AES", cipherKeySize, false), cipherKeySize, macSize,
+            TlsAEADCipher.AEAD_GCM);
     }
 
     private TlsAEADCipher createCipher_ARIA_GCM(TlsCryptoParameters cryptoParams, int cipherKeySize, int macSize)
         throws IOException, GeneralSecurityException
     {
-        return new TlsAEADCipher(cryptoParams, createAEADCipher("ARIA/GCM/NoPadding", "ARIA", cipherKeySize, true), createAEADCipher("ARIA/GCM/NoPadding", "ARIA", cipherKeySize, false),
-            cipherKeySize, macSize);
+        return new TlsAEADCipher(cryptoParams, createAEADCipher("ARIA/GCM/NoPadding", "ARIA", cipherKeySize, true),
+            createAEADCipher("ARIA/GCM/NoPadding", "ARIA", cipherKeySize, false), cipherKeySize, macSize,
+            TlsAEADCipher.AEAD_GCM);
     }
 
     private TlsAEADCipher createCipher_Camellia_GCM(TlsCryptoParameters cryptoParams, int cipherKeySize, int macSize)
         throws IOException, GeneralSecurityException
     {
-        return new TlsAEADCipher(cryptoParams, createAEADCipher("Camellia/GCM/NoPadding", "Camellia", cipherKeySize, true), createAEADCipher("Camellia/GCM/NoPadding", "Camellia", cipherKeySize, false),
-            cipherKeySize, macSize);
+        return new TlsAEADCipher(cryptoParams,
+            createAEADCipher("Camellia/GCM/NoPadding", "Camellia", cipherKeySize, true),
+            createAEADCipher("Camellia/GCM/NoPadding", "Camellia", cipherKeySize, false), cipherKeySize, macSize,
+            TlsAEADCipher.AEAD_GCM);
     }
 
     String getDigestName(short hashAlgorithm)
