@@ -51,7 +51,7 @@ public class JcaTlsRSAPSSSigner
             || algorithm.getSignature() != signatureAlgorithm
             || algorithm.getHash() != HashAlgorithm.Intrinsic)
         {
-            throw new IllegalStateException();
+            throw new IllegalStateException("Invalid algorithm: " + algorithm);
         }
 
         short hash = SignatureAlgorithm.getRSAPSSHashAlgorithm(signatureAlgorithm);
