@@ -1,5 +1,6 @@
 package org.bouncycastle.tls.crypto.impl.jcajce;
 
+import java.io.IOException;
 import java.security.AccessController;
 import java.security.AlgorithmParameters;
 import java.security.GeneralSecurityException;
@@ -12,6 +13,8 @@ import javax.crypto.spec.SecretKeySpec;
 import org.bouncycastle.asn1.cms.GCMParameters;
 import org.bouncycastle.jcajce.spec.AEADParameterSpec;
 import org.bouncycastle.jcajce.util.JcaJceHelper;
+import org.bouncycastle.tls.AlertDescription;
+import org.bouncycastle.tls.TlsFatalAlert;
 import org.bouncycastle.tls.crypto.impl.TlsAEADCipherImpl;
 
 /**
@@ -20,6 +23,9 @@ import org.bouncycastle.tls.crypto.impl.TlsAEADCipherImpl;
 public class JceAEADCipherImpl
     implements TlsAEADCipherImpl
 {
+    private static final int BUF_SIZE = 32 * 1024;
+
+    @SuppressWarnings({ "rawtypes", "unchecked" })
     private static boolean checkForAEAD()
     {
         return (Boolean)AccessController.doPrivileged(new PrivilegedAction()
@@ -120,15 +126,45 @@ public class JceAEADCipherImpl
         return cipher.getOutputSize(inputLength);
     }
 
-    public int doFinal(byte[] input, int inputOffset, int inputLength, byte[] output, int outputOffset)
+    public int doFinal(byte[] input, int inputOffset, int inputLength, byte[] extraInput, byte[] output,
+        int outputOffset) throws IOException
     {
+        int extraInputLength = extraInput.length;
+        if (extraInputLength > 0 && Cipher.ENCRYPT_MODE != cipherMode)
+        {
+            throw new TlsFatalAlert(AlertDescription.internal_error);
+        }
+
         try
         {
-            return cipher.doFinal(input, inputOffset, inputLength, output, outputOffset);
+            int len = updateCipher(input, inputOffset, inputLength, output, outputOffset);
+
+            if (extraInputLength > 0)
+            {
+                len += updateCipher(extraInput, 0, extraInputLength, output, outputOffset + len);
+            }
+
+            len += cipher.doFinal(output, outputOffset + len);
+
+            return len;
         }
         catch (GeneralSecurityException e)
         {
             throw Exceptions.illegalStateException("", e);
         }
     }
+
+    protected int updateCipher(byte[] input, int inOff, int inLen, byte[] output, int outOff)
+        throws GeneralSecurityException
+    {
+        // to avoid performance issue in FIPS jar 1.0.0-1.0.2, process in chunks
+        int readOff = 0, writeOff = 0;
+        while (readOff < inLen)
+        {
+            int updateSize = Math.min(BUF_SIZE, inLen - readOff);
+            writeOff += cipher.update(input, inOff + readOff, updateSize, output, outOff + writeOff);
+            readOff += updateSize;
+        }
+        return writeOff;
+    }
 }
