@@ -6,11 +6,12 @@ import java.security.GeneralSecurityException;
 import java.security.KeyFactory;
 import java.security.KeyPair;
 import java.security.KeyPairGenerator;
+import java.security.spec.KeySpec;
+import java.util.Arrays;
 
 import javax.crypto.interfaces.DHPrivateKey;
 import javax.crypto.interfaces.DHPublicKey;
 import javax.crypto.spec.DHParameterSpec;
-import javax.crypto.spec.DHPublicKeySpec;
 
 import org.bouncycastle.tls.AlertDescription;
 import org.bouncycastle.tls.TlsDHUtils;
@@ -40,40 +41,26 @@ public class JceTlsDHDomain
         return (dh.getP().bitLength() + 7) / 8;
     }
 
-    public static DHParameterSpec getParameters(TlsDHConfig dhConfig)
-    {
-        DHGroup dhGroup = TlsDHUtils.getDHGroup(dhConfig);
-        if (dhGroup == null)
-        {
-            throw new IllegalArgumentException("No DH configuration provided");
-        }
-
-        // NOTE: dhGroup.getQ() is ignored here
-        return new DHParameterSpec(dhGroup.getP(), dhGroup.getG(), dhGroup.getL());
-    }
-
-    protected final JcaTlsCrypto crypto;
-    protected final TlsDHConfig dhConfig;
-    protected final DHParameterSpec dhParameterSpec;
-
-    public JceTlsDHDomain(JcaTlsCrypto crypto, TlsDHConfig dhConfig)
-    {
-        this.crypto = crypto;
-        this.dhConfig = dhConfig;
-        this.dhParameterSpec = getParameters(dhConfig);
-    }
-
-    public JceTlsSecret calculateDHAgreement(DHPrivateKey privateKey, DHPublicKey publicKey)
-        throws IOException
+    public static JceTlsSecret calculateDHAgreement(JcaTlsCrypto crypto, DHPrivateKey privateKey, DHPublicKey publicKey,
+        boolean padded) throws IOException
     {
         try
         {
             /*
-             * RFC 5246 8.1.2. Leading bytes of Z that contain all zero bits are stripped before it is
-             * used as the pre_master_secret. We use the convention established by the JSSE to signal this
-             * by asking for "TlsPremasterSecret".
+             * RFC 5246 8.1.2. Leading bytes of Z that contain all zero bits are stripped before it
+             * is used as the pre_master_secret. We use the convention established by the JSSE to
+             * signal this by asking for "TlsPremasterSecret".
              */
-            byte[] secret = crypto.calculateKeyAgreement("DH", privateKey, publicKey, "TlsPremasterSecret");
+            byte[] secret = crypto.calculateKeyAgreement("DiffieHellman", privateKey, publicKey, "TlsPremasterSecret");
+
+            if (padded)
+            {
+                int length = getValueLength(privateKey.getParams());
+                byte[] tmp = new byte[length];
+                System.arraycopy(secret, 0, tmp, length - secret.length, secret.length);
+                Arrays.fill(secret, (byte)0);
+                secret = tmp;
+            }
 
             return crypto.adoptLocalSecret(secret);
         }
@@ -83,13 +70,46 @@ public class JceTlsDHDomain
         }
     }
 
+    protected final JcaTlsCrypto crypto;
+    protected final TlsDHConfig dhConfig;
+    protected final DHParameterSpec dhSpec;
+
+    public JceTlsDHDomain(JcaTlsCrypto crypto, TlsDHConfig dhConfig)
+    {
+        DHGroup dhGroup = TlsDHUtils.getDHGroup(dhConfig);
+        if (null != dhGroup)
+        {
+            DHParameterSpec spec = DHUtil.getDHParameterSpec(crypto, dhGroup);
+            if (null != spec)
+            {
+                this.crypto = crypto;
+                this.dhConfig = dhConfig;
+                this.dhSpec = spec;
+                return;
+            }
+        }
+
+        throw new IllegalArgumentException("No DH configuration provided");
+    }
+
+    public JceTlsSecret calculateDHAgreement(DHPrivateKey privateKey, DHPublicKey publicKey)
+        throws IOException
+    {
+        return calculateDHAgreement(crypto, privateKey, publicKey, dhConfig.isPadded());
+    }
+
     public TlsAgreement createDH()
     {
         return new JceTlsDH(this);
     }
 
-    public static BigInteger decodeParameter(byte[] encoding) throws IOException
+    public BigInteger decodeParameter(byte[] encoding) throws IOException
     {
+        if (dhConfig.isPadded() && getValueLength(dhSpec) != encoding.length)
+        {
+            throw new TlsFatalAlert(AlertDescription.illegal_parameter);
+        }
+
         return new BigInteger(1, encoding);
     }
 
@@ -103,10 +123,14 @@ public class JceTlsDHDomain
         try
         {
             BigInteger y = decodeParameter(encoding);
+            KeySpec publicKeySpec = DHUtil.createPublicKeySpec(y, dhSpec);
 
-            KeyFactory keyFactory = crypto.getHelper().createKeyFactory("DH");
-
-            return (DHPublicKey)keyFactory.generatePublic(new DHPublicKeySpec(y, dhParameterSpec.getP(), dhParameterSpec.getG()));
+            KeyFactory keyFactory = crypto.getHelper().createKeyFactory("DiffieHellman");
+            return (DHPublicKey)keyFactory.generatePublic(publicKeySpec);
+        }
+        catch (IOException e)
+        {
+            throw e;
         }
         catch (Exception e)
         {
@@ -116,20 +140,20 @@ public class JceTlsDHDomain
 
     public byte[] encodeParameter(BigInteger x) throws IOException
     {
-        return encodeValue(dhParameterSpec, false, x);
+        return encodeValue(dhSpec, dhConfig.isPadded(), x);
     }
 
     public byte[] encodePublicKey(DHPublicKey publicKey) throws IOException
     {
-        return encodeValue(dhParameterSpec, true, publicKey.getY());
+        return encodeValue(dhSpec, true, publicKey.getY());
     }
 
     public KeyPair generateKeyPair() throws IOException
     {
         try
         {
-            KeyPairGenerator keyPairGenerator = crypto.getHelper().createKeyPairGenerator("DH");
-            keyPairGenerator.initialize(dhParameterSpec, crypto.getSecureRandom());
+            KeyPairGenerator keyPairGenerator = crypto.getHelper().createKeyPairGenerator("DiffieHellman");
+            keyPairGenerator.initialize(dhSpec, crypto.getSecureRandom());
             return keyPairGenerator.generateKeyPair();
         }
         catch (GeneralSecurityException e)
