@@ -13,6 +13,7 @@ import java.security.interfaces.RSAPublicKey;
 
 import javax.crypto.interfaces.DHPublicKey;
 
+import org.bouncycastle.asn1.ASN1Encodable;
 import org.bouncycastle.asn1.ASN1Encoding;
 import org.bouncycastle.asn1.ASN1ObjectIdentifier;
 import org.bouncycastle.asn1.ASN1OctetString;
@@ -26,6 +27,7 @@ import org.bouncycastle.tls.ConnectionEnd;
 import org.bouncycastle.tls.KeyExchangeAlgorithm;
 import org.bouncycastle.tls.SignatureAlgorithm;
 import org.bouncycastle.tls.TlsFatalAlert;
+import org.bouncycastle.tls.TlsUtils;
 import org.bouncycastle.tls.crypto.TlsCertificate;
 import org.bouncycastle.tls.crypto.TlsCryptoException;
 import org.bouncycastle.tls.crypto.TlsVerifier;
@@ -42,7 +44,7 @@ public class JcaTlsCertificate
     protected static final int KU_KEY_ENCIPHERMENT = 2;
     protected static final int KU_DATA_ENCIPHERMENT = 3;
     protected static final int KU_KEY_AGREEMENT = 4;
-    protected static final int KU_CERT_SIGN = 5;
+    protected static final int KU_KEY_CERT_SIGN = 5;
     protected static final int KU_CRL_SIGN = 6;
     protected static final int KU_ENCIPHER_ONLY = 7;
     protected static final int KU_DECIPHER_ONLY = 8;
@@ -72,7 +74,8 @@ public class JcaTlsCertificate
             byte[] derEncoding = Certificate.getInstance(encoding).getEncoded(ASN1Encoding.DER);
 
             ByteArrayInputStream input = new ByteArrayInputStream(derEncoding);
-            X509Certificate certificate = (X509Certificate)helper.createCertificateFactory("X.509").generateCertificate(input);
+            X509Certificate certificate = (X509Certificate)helper.createCertificateFactory("X.509")
+                .generateCertificate(input);
             if (input.available() != 0)
             {
                 throw new IOException("Extra data detected in stream");
@@ -90,7 +93,7 @@ public class JcaTlsCertificate
 
     protected DHPublicKey pubKeyDH = null;
     protected ECPublicKey pubKeyEC = null;
-    protected RSAPublicKey pubKeyRSA = null;
+    protected PublicKey pubKeyRSA = null;
 
     public JcaTlsCertificate(JcaTlsCrypto crypto, byte[] encoding)
         throws IOException
@@ -172,6 +175,13 @@ public class JcaTlsCertificate
         return certificate.getSigAlgOID();
     }
 
+    public ASN1Encodable getSigAlgParams() throws IOException
+    {
+        byte[] derEncoding = certificate.getSigAlgParams();
+
+        return null == derEncoding ? null : TlsUtils.readDERObject(derEncoding);
+    }
+
     DHPublicKey getPubKeyDH() throws IOException
     {
         try
@@ -228,107 +238,70 @@ public class JcaTlsCertificate
         return publicKey;
     }
 
-    RSAPublicKey getPubKeyRSA() throws IOException
+    PublicKey getPubKeyRSA() throws IOException
     {
-        try
-        {
-            return (RSAPublicKey)getPublicKey();
-        }
-        catch (ClassCastException e)
-        {
-            throw new TlsFatalAlert(AlertDescription.certificate_unknown, e);
-        }
+        return getPublicKey();
     }
 
     public short getLegacySignatureAlgorithm() throws IOException
     {
-         PublicKey publicKey = getPublicKey();
+        PublicKey publicKey = getPublicKey();
 
-         if (!supportsKeyUsageBit(KU_DIGITAL_SIGNATURE))
-         {
-             return -1;
-         }
+        if (!supportsKeyUsageBit(KU_DIGITAL_SIGNATURE))
+        {
+            return -1;
+        }
 
-         /*
-          * RFC 5246 7.4.6. Client Certificate
-          */
+        /*
+         * RFC 5246 7.4.6. Client Certificate
+         */
 
-         /*
-          * RSA public key; the certificate MUST allow the key to be used for signing with the
-          * signature scheme and hash algorithm that will be employed in the certificate verify
-          * message.
-          */
-         if (publicKey instanceof RSAPublicKey)
-         {
-             return SignatureAlgorithm.rsa;
-         }
+        /*
+         * RSA public key; the certificate MUST allow the key to be used for signing with the
+         * signature scheme and hash algorithm that will be employed in the certificate verify
+         * message.
+         */
+        if (publicKey instanceof RSAPublicKey)
+        {
+            return SignatureAlgorithm.rsa;
+        }
 
-         /*
-          * DSA public key; the certificate MUST allow the key to be used for signing with the
-          * hash algorithm that will be employed in the certificate verify message.
-          */
-         if (publicKey instanceof DSAPublicKey)
-         {
-             return SignatureAlgorithm.dsa;
-         }
+        /*
+         * DSA public key; the certificate MUST allow the key to be used for signing with the hash
+         * algorithm that will be employed in the certificate verify message.
+         */
+        if (publicKey instanceof DSAPublicKey)
+        {
+            return SignatureAlgorithm.dsa;
+        }
 
-         /*
-          * ECDSA-capable public key; the certificate MUST allow the key to be used for signing
-          * with the hash algorithm that will be employed in the certificate verify message; the
-          * public key MUST use a curve and point format supported by the server.
-          */
-         if (publicKey instanceof ECPublicKey)
-         {
-             // TODO Check the curve and point format
-             return SignatureAlgorithm.ecdsa;
-         }
+        /*
+         * ECDSA-capable public key; the certificate MUST allow the key to be used for signing with
+         * the hash algorithm that will be employed in the certificate verify message; the public
+         * key MUST use a curve and point format supported by the server.
+         */
+        if (publicKey instanceof ECPublicKey)
+        {
+            // TODO Check the curve and point format
+            return SignatureAlgorithm.ecdsa;
+        }
 
-         return -1;
+        return -1;
     }
 
-    public boolean supportsSignatureAlgorithm(short signatureAlgorithm)
-        throws IOException
+    public boolean supportsSignatureAlgorithm(short signatureAlgorithm) throws IOException
     {
         if (!supportsKeyUsageBit(KU_DIGITAL_SIGNATURE))
         {
             return false;
         }
 
-        PublicKey publicKey = getPublicKey();
+        return implSupportsSignatureAlgorithm(signatureAlgorithm);
+    }
 
-        switch (signatureAlgorithm)
-        {
-        case SignatureAlgorithm.rsa:
-            return supportsRSA_PKCS1()
-                && publicKey instanceof RSAPublicKey;
-
-        case SignatureAlgorithm.dsa:
-            return publicKey instanceof DSAPublicKey;
-
-        case SignatureAlgorithm.ecdsa:
-            return publicKey instanceof ECPublicKey;
-
-        case SignatureAlgorithm.ed25519:
-            return "Ed25519".equals(publicKey.getAlgorithm());
-
-        case SignatureAlgorithm.ed448:
-            return "Ed448".equals(publicKey.getAlgorithm());
-
-        case SignatureAlgorithm.rsa_pss_rsae_sha256:
-        case SignatureAlgorithm.rsa_pss_rsae_sha384:
-        case SignatureAlgorithm.rsa_pss_rsae_sha512:
-            return supportsRSA_PSS_RSAE()
-                && publicKey instanceof RSAPublicKey;
-
-        case SignatureAlgorithm.rsa_pss_pss_sha256:
-        case SignatureAlgorithm.rsa_pss_pss_sha384:
-        case SignatureAlgorithm.rsa_pss_pss_sha512:
-            return supportsRSA_PSS_PSS(signatureAlgorithm)
-                && publicKey instanceof RSAPublicKey;
-
-        default:
-            return false;
-        }
+    public boolean supportsSignatureAlgorithmCA(short signatureAlgorithm) throws IOException
+    {
+        return implSupportsSignatureAlgorithm(signatureAlgorithm);
     }
 
     public TlsCertificate useInRole(int connectionEnd, int keyExchangeAlgorithm) throws IOException
@@ -369,6 +342,45 @@ public class JcaTlsCertificate
         throw new TlsFatalAlert(AlertDescription.certificate_unknown);
     }
 
+    protected boolean implSupportsSignatureAlgorithm(short signatureAlgorithm) throws IOException
+    {
+        PublicKey publicKey = getPublicKey();
+
+        switch (signatureAlgorithm)
+        {
+        case SignatureAlgorithm.rsa:
+            return supportsRSA_PKCS1()
+                && publicKey instanceof RSAPublicKey;
+
+        case SignatureAlgorithm.dsa:
+            return publicKey instanceof DSAPublicKey;
+
+        case SignatureAlgorithm.ecdsa:
+            return publicKey instanceof ECPublicKey;
+
+        case SignatureAlgorithm.ed25519:
+            return "Ed25519".equals(publicKey.getAlgorithm());
+
+        case SignatureAlgorithm.ed448:
+            return "Ed448".equals(publicKey.getAlgorithm());
+
+        case SignatureAlgorithm.rsa_pss_rsae_sha256:
+        case SignatureAlgorithm.rsa_pss_rsae_sha384:
+        case SignatureAlgorithm.rsa_pss_rsae_sha512:
+            return supportsRSA_PSS_RSAE()
+                && publicKey instanceof RSAPublicKey;
+
+        case SignatureAlgorithm.rsa_pss_pss_sha256:
+        case SignatureAlgorithm.rsa_pss_pss_sha384:
+        case SignatureAlgorithm.rsa_pss_pss_sha512:
+            return supportsRSA_PSS_PSS(signatureAlgorithm)
+                && publicKey instanceof RSAPublicKey;
+
+        default:
+            return false;
+        }
+    }
+
     protected PublicKey getPublicKey() throws IOException
     {
         try
