@@ -2,12 +2,17 @@ package org.bouncycastle.tls.crypto.test;
 
 import java.io.IOException;
 
+import org.bouncycastle.tls.DefaultTlsDHGroupVerifier;
 import org.bouncycastle.tls.HashAlgorithm;
 import org.bouncycastle.tls.NamedGroup;
+import org.bouncycastle.tls.TlsDHUtils;
 import org.bouncycastle.tls.TlsUtils;
+import org.bouncycastle.tls.crypto.DHGroup;
 import org.bouncycastle.tls.crypto.TlsAgreement;
 import org.bouncycastle.tls.crypto.TlsCrypto;
 import org.bouncycastle.tls.crypto.TlsCryptoUtils;
+import org.bouncycastle.tls.crypto.TlsDHConfig;
+import org.bouncycastle.tls.crypto.TlsDHDomain;
 import org.bouncycastle.tls.crypto.TlsECConfig;
 import org.bouncycastle.tls.crypto.TlsECDomain;
 import org.bouncycastle.tls.crypto.TlsHMAC;
@@ -82,6 +87,40 @@ public abstract class TlsCryptoTest
         this.crypto = crypto;
     }
 
+    public void testDHDomain() throws Exception
+    {
+        if (!crypto.hasDHAgreement())
+        {
+            return;
+        }
+
+        for (int namedGroup = 256; namedGroup < 512; ++namedGroup)
+        {
+            if (!NamedGroup.refersToASpecificFiniteField(namedGroup) || !crypto.hasNamedGroup(namedGroup))
+            {
+                continue;
+            }
+
+            implTestDHDomain(new TlsDHConfig(namedGroup, false));
+            implTestDHDomain(new TlsDHConfig(namedGroup, true));
+        }
+
+        new DefaultTlsDHGroupVerifier() {{
+            for (int i = 0; i < DEFAULT_GROUPS.size(); ++i)
+            {
+                DHGroup dhGroup = (DHGroup)DEFAULT_GROUPS.elementAt(i);
+                int namedGroup = TlsDHUtils.getNamedGroupForDHParameters(dhGroup.getP(), dhGroup.getG());
+                if (NamedGroup.refersToASpecificFiniteField(namedGroup))
+                {
+                    // Already tested the named groups
+                    continue;
+                }
+
+                implTestDHDomain(new TlsDHConfig(dhGroup));
+            }
+        }};
+    }
+
     public void testECDomain() throws Exception
     {
         if (!crypto.hasECDHAgreement())
@@ -96,24 +135,7 @@ public abstract class TlsCryptoTest
                 continue;
             }
 
-            TlsECDomain d = crypto.createECDomain(new TlsECConfig(namedGroup));
-
-            for (int i = 0; i < 10; ++i)
-            {
-                TlsAgreement aA = d.createECDH();
-                TlsAgreement aB = d.createECDH();
-
-                byte[] pA = aA.generateEphemeral();
-                byte[] pB = aB.generateEphemeral();
-
-                aA.receivePeerValue(pB);
-                aB.receivePeerValue(pA);
-
-                TlsSecret sA = aA.calculateSecret();
-                TlsSecret sB = aB.calculateSecret();
-
-                assertArrayEquals(extract(sA), extract(sB));
-            }
+            implTestECDomain(new TlsECConfig(namedGroup));
         }
     }
 
@@ -297,6 +319,46 @@ public abstract class TlsCryptoTest
         }
     }
 
+    public void testHKDFExpandLimit()
+    {
+        short[] hashes = new short[]{ HashAlgorithm.md5, HashAlgorithm.sha1, HashAlgorithm.sha224,
+            HashAlgorithm.sha256, HashAlgorithm.sha384, HashAlgorithm.sha512 };
+
+        for (int i = 0; i < hashes.length; ++i)
+        {
+            short hash = HashAlgorithm.sha256;
+            int hashLen = HashAlgorithm.getOutputSize(hash);
+            byte[] zeroes = new byte[hashLen];
+
+            int limit = 255 * hashLen;
+
+            TlsSecret secret = crypto.hkdfInit(hash).hkdfExtract(hash, zeroes);
+
+            try
+            {
+                secret.hkdfExpand(hash, TlsUtils.EMPTY_BYTES, limit);
+            }
+            catch (Exception e)
+            {
+                fail("Unexpected exception: " + e.getMessage());
+            }
+
+            try
+            {
+                secret.hkdfExpand(hash, TlsUtils.EMPTY_BYTES, limit + 1);
+                fail("Expected an exception!");
+            }
+            catch (IllegalArgumentException e)
+            {
+                // Expected
+            }
+            catch (Exception e)
+            {
+                fail("Unexpected exception: " + e.getMessage());
+            }
+        }
+    }
+
     private byte[] calculateHMAC(short hash, TlsSecret hmacKey, byte[] hmacInput)
     {
         byte[] keyBytes = extract(hmacKey);
@@ -341,4 +403,54 @@ public abstract class TlsCryptoTest
     {
         return Hex.decode(s.replaceAll(" ", ""));
     }
+
+    private void implTestAgreement(TlsAgreement aA, TlsAgreement aB) throws IOException
+    {
+        byte[] pA = aA.generateEphemeral();
+        byte[] pB = aB.generateEphemeral();
+
+        aA.receivePeerValue(pB);
+        aB.receivePeerValue(pA);
+
+        TlsSecret sA = aA.calculateSecret();
+        TlsSecret sB = aB.calculateSecret();
+
+        assertArrayEquals(extract(sA), extract(sB));
+    }
+
+    private void implTestDHDomain(TlsDHConfig dhConfig) throws IOException
+    {
+        int namedGroup = dhConfig.getNamedGroup();
+        int bits = namedGroup >= 0
+            ?   NamedGroup.getFiniteFieldBits(namedGroup)
+            :   dhConfig.getExplicitGroup().getP().bitLength();
+
+        int rounds = Math.max(2, 11 - (bits >>> 10));
+
+        TlsDHDomain d = crypto.createDHDomain(dhConfig);
+
+        for (int i = 0; i < rounds; ++i)
+        {
+            TlsAgreement aA = d.createDH();
+            TlsAgreement aB = d.createDH();
+
+            implTestAgreement(aA, aB);
+        }
+    }
+
+    private void implTestECDomain(TlsECConfig ecConfig) throws IOException
+    {
+        int bits = NamedGroup.getCurveBits(ecConfig.getNamedGroup());
+        int rounds = Math.max(2, 12 - (bits >>> 6));
+
+        TlsECDomain d = crypto.createECDomain(ecConfig);
+
+        for (int i = 0; i < rounds; ++i)
+        {
+            TlsAgreement aA = d.createECDH();
+            TlsAgreement aB = d.createECDH();
+
+            implTestAgreement(aA, aB);
+        }
+    }
 }
