@@ -10,17 +10,20 @@ import org.bouncycastle.asn1.x509.Certificate;
 import org.bouncycastle.tls.AlertDescription;
 import org.bouncycastle.tls.AlertLevel;
 import org.bouncycastle.tls.CertificateRequest;
+import org.bouncycastle.tls.ChannelBinding;
 import org.bouncycastle.tls.ClientCertificateType;
 import org.bouncycastle.tls.DefaultTlsServer;
 import org.bouncycastle.tls.ProtocolVersion;
 import org.bouncycastle.tls.SignatureAlgorithm;
 import org.bouncycastle.tls.TlsCredentialedDecryptor;
 import org.bouncycastle.tls.TlsCredentialedSigner;
+import org.bouncycastle.tls.TlsFatalAlert;
 import org.bouncycastle.tls.TlsUtils;
 import org.bouncycastle.tls.crypto.TlsCertificate;
 import org.bouncycastle.tls.crypto.impl.bc.BcTlsCrypto;
+import org.bouncycastle.util.encoders.Hex;
 
-public class MockDTLSServer
+class MockDTLSServer
     extends DefaultTlsServer
 {
     MockDTLSServer()
@@ -35,7 +38,7 @@ public class MockDTLSServer
             + ", " + AlertDescription.getText(alertDescription));
         if (message != null)
         {
-            out.println(message);
+            out.println("> " + message);
         }
         if (cause != null)
         {
@@ -50,6 +53,15 @@ public class MockDTLSServer
             + ", " + AlertDescription.getText(alertDescription));
     }
 
+    public ProtocolVersion getServerVersion() throws IOException
+    {
+        ProtocolVersion serverVersion = super.getServerVersion();
+
+        System.out.println("DTLS server negotiated " + serverVersion);
+
+        return serverVersion;
+    }
+
     public CertificateRequest getCertificateRequest() throws IOException
     {
         short[] certificateTypes = new short[]{ ClientCertificateType.rsa_sign,
@@ -72,24 +84,55 @@ public class MockDTLSServer
         return new CertificateRequest(certificateTypes, serverSigAlgs, certificateAuthorities);
     }
 
-    public void notifyClientCertificate(org.bouncycastle.tls.Certificate clientCertificate)
-        throws IOException
+    public void notifyClientCertificate(org.bouncycastle.tls.Certificate clientCertificate) throws IOException
     {
         TlsCertificate[] chain = clientCertificate.getCertificateList();
+
         System.out.println("DTLS server received client certificate chain of length " + chain.length);
         for (int i = 0; i != chain.length; i++)
         {
             Certificate entry = Certificate.getInstance(chain[i].getEncoded());
             // TODO Create fingerprint based on certificate signature algorithm digest
-            System.out.println("    fingerprint:SHA-256 " + TlsTestUtils.fingerprint(entry) + " (" + entry.getSubject()
-                + ")");
+            System.out.println("    fingerprint:SHA-256 " + TlsTestUtils.fingerprint(entry) + " ("
+                + entry.getSubject() + ")");
         }
+
+        boolean isEmpty = (clientCertificate == null || clientCertificate.isEmpty());
+
+        if (isEmpty)
+        {
+            return;
+        }
+
+        String[] trustedCertResources = new String[]{ "x509-client-dsa.pem", "x509-client-ecdh.pem",
+            "x509-client-ecdsa.pem", "x509-client-ed25519.pem", "x509-client-ed448.pem", "x509-client-rsa_pss_256.pem",
+            "x509-client-rsa_pss_384.pem", "x509-client-rsa_pss_512.pem", "x509-client-rsa.pem" };
+
+        TlsCertificate[] certPath = TlsTestUtils.getTrustedCertPath(context.getCrypto(), chain[0],
+            trustedCertResources);
+
+        if (null == certPath)
+        {
+            throw new TlsFatalAlert(AlertDescription.bad_certificate);
+        }
+
+        TlsUtils.checkPeerSigAlgs(context, certPath);
     }
 
-    protected TlsCredentialedDecryptor getRSAEncryptionCredentials()
-        throws IOException
+    public void notifyHandshakeComplete() throws IOException
     {
-        return TlsTestUtils.loadEncryptionCredentials(context, new String[]{"x509-server-rsa-enc.pem", "x509-ca-rsa.pem"},
+        super.notifyHandshakeComplete();
+
+        byte[] tlsServerEndPoint = context.exportChannelBinding(ChannelBinding.tls_server_end_point);
+        System.out.println("Server 'tls-server-end-point': " + hex(tlsServerEndPoint));
+
+        byte[] tlsUnique = context.exportChannelBinding(ChannelBinding.tls_unique);
+        System.out.println("Server 'tls-unique': " + hex(tlsUnique));
+    }
+
+    protected TlsCredentialedDecryptor getRSAEncryptionCredentials() throws IOException
+    {
+        return TlsTestUtils.loadEncryptionCredentials(context, new String[]{ "x509-server-rsa-enc.pem", "x509-ca-rsa.pem" },
             "x509-server-key-rsa-enc.pem");
     }
 
@@ -99,6 +142,11 @@ public class MockDTLSServer
         return TlsTestUtils.loadSignerCredentialsServer(context, clientSigAlgs, SignatureAlgorithm.rsa);
     }
 
+    protected String hex(byte[] data)
+    {
+        return data == null ? "(null)" : Hex.toHexString(data);
+    }
+
     protected ProtocolVersion[] getSupportedVersions()
     {
         return ProtocolVersion.DTLSv12.downTo(ProtocolVersion.DTLSv10);
