@@ -93,8 +93,7 @@ class MockTlsServer
         return new CertificateRequest(certificateTypes, serverSigAlgs, certificateAuthorities);
     }
 
-    public void notifyClientCertificate(org.bouncycastle.tls.Certificate clientCertificate)
-        throws IOException
+    public void notifyClientCertificate(org.bouncycastle.tls.Certificate clientCertificate) throws IOException
     {
         TlsCertificate[] chain = clientCertificate.getCertificateList();
 
@@ -108,14 +107,25 @@ class MockTlsServer
         }
 
         boolean isEmpty = (clientCertificate == null || clientCertificate.isEmpty());
-        if (!isEmpty && !TlsTestUtils.isCertificateOneOf(context.getCrypto(), chain[0],
-            new String[]
-            { "x509-client-dsa.pem", "x509-client-ecdh.pem", "x509-client-ecdsa.pem", "x509-client-ed25519.pem",
-                "x509-client-rsa_pss_256.pem", "x509-client-rsa_pss_384.pem", "x509-client-rsa_pss_512.pem",
-                "x509-client-rsa.pem" }))
+
+        if (isEmpty)
+        {
+            return;
+        }
+
+        String[] trustedCertResources = new String[]{ "x509-client-dsa.pem", "x509-client-ecdh.pem",
+            "x509-client-ecdsa.pem", "x509-client-ed25519.pem", "x509-client-ed448.pem", "x509-client-rsa_pss_256.pem",
+            "x509-client-rsa_pss_384.pem", "x509-client-rsa_pss_512.pem", "x509-client-rsa.pem" };
+
+        TlsCertificate[] certPath = TlsTestUtils.getTrustedCertPath(context.getCrypto(), chain[0],
+            trustedCertResources);
+
+        if (null == certPath)
         {
             throw new TlsFatalAlert(AlertDescription.bad_certificate);
         }
+
+        TlsUtils.checkPeerSigAlgs(context, certPath);
     }
 
     public void notifyHandshakeComplete() throws IOException
@@ -135,10 +145,9 @@ class MockTlsServer
         System.out.println("Server 'tls-unique': " + hex(tlsUnique));
     }
 
-    protected TlsCredentialedDecryptor getRSAEncryptionCredentials()
-        throws IOException
+    protected TlsCredentialedDecryptor getRSAEncryptionCredentials() throws IOException
     {
-        return TlsTestUtils.loadEncryptionCredentials(context, new String[]{"x509-server-rsa-enc.pem", "x509-ca-rsa.pem"},
+        return TlsTestUtils.loadEncryptionCredentials(context, new String[]{ "x509-server-rsa-enc.pem", "x509-ca-rsa.pem" },
             "x509-server-key-rsa-enc.pem");
     }
 
@@ -152,4 +161,4 @@ class MockTlsServer
     {
         return data == null ? "(null)" : Hex.toHexString(data);
     }
-}
\ No newline at end of file
+}
