@@ -13,8 +13,10 @@ import org.bouncycastle.asn1.DERSequence;
 import org.bouncycastle.asn1.x509.Certificate;
 import org.bouncycastle.tls.AlertDescription;
 import org.bouncycastle.tls.AlertLevel;
+import org.bouncycastle.tls.CertificateEntry;
 import org.bouncycastle.tls.CertificateRequest;
 import org.bouncycastle.tls.ChannelBinding;
+import org.bouncycastle.tls.CipherSuite;
 import org.bouncycastle.tls.ClientCertificateType;
 import org.bouncycastle.tls.ConnectionEnd;
 import org.bouncycastle.tls.DefaultTlsClient;
@@ -27,6 +29,7 @@ import org.bouncycastle.tls.TlsCredentials;
 import org.bouncycastle.tls.TlsExtensionsUtils;
 import org.bouncycastle.tls.TlsFatalAlert;
 import org.bouncycastle.tls.TlsServerCertificate;
+import org.bouncycastle.tls.TlsUtils;
 import org.bouncycastle.tls.crypto.TlsCertificate;
 import org.bouncycastle.tls.crypto.TlsCrypto;
 import org.bouncycastle.tls.crypto.TlsStreamSigner;
@@ -37,11 +40,40 @@ import org.bouncycastle.util.encoders.Hex;
 class TlsTestClientImpl
     extends DefaultTlsClient
 {
+    private static final int[] TEST_CIPHER_SUITES = new int[]
+    {
+        /*
+         * TLS 1.3
+         */
+        CipherSuite.TLS_CHACHA20_POLY1305_SHA256,
+        CipherSuite.TLS_AES_128_GCM_SHA256,
+
+        /*
+         * pre-TLS 1.3
+         */
+        CipherSuite.TLS_ECDHE_ECDSA_WITH_CHACHA20_POLY1305_SHA256,
+        CipherSuite.TLS_ECDHE_ECDSA_WITH_AES_128_GCM_SHA256,
+        CipherSuite.TLS_ECDHE_ECDSA_WITH_AES_128_CBC_SHA256,
+        CipherSuite.TLS_ECDHE_ECDSA_WITH_AES_128_CBC_SHA,
+        CipherSuite.TLS_ECDHE_RSA_WITH_CHACHA20_POLY1305_SHA256,
+        CipherSuite.TLS_ECDHE_RSA_WITH_AES_128_GCM_SHA256,
+        CipherSuite.TLS_ECDHE_RSA_WITH_AES_128_CBC_SHA256,
+        CipherSuite.TLS_ECDHE_RSA_WITH_AES_128_CBC_SHA,
+        CipherSuite.TLS_DHE_RSA_WITH_CHACHA20_POLY1305_SHA256,
+        CipherSuite.TLS_DHE_RSA_WITH_AES_128_GCM_SHA256,
+        CipherSuite.TLS_DHE_RSA_WITH_AES_128_CBC_SHA256,
+        CipherSuite.TLS_DHE_RSA_WITH_AES_128_CBC_SHA,
+        CipherSuite.TLS_RSA_WITH_AES_128_GCM_SHA256,
+        CipherSuite.TLS_RSA_WITH_AES_128_CBC_SHA256,
+        CipherSuite.TLS_RSA_WITH_AES_128_CBC_SHA,
+    };
+
     protected final TlsTestConfig config;
 
     protected int firstFatalAlertConnectionEnd = -1;
     protected short firstFatalAlertDescription = -1;
 
+    ProtocolVersion negotiatedVersion = null;
     byte[] tlsServerEndPoint = null;
     byte[] tlsUnique = null;
 
@@ -62,11 +94,6 @@ class TlsTestClientImpl
         return firstFatalAlertDescription;
     }
 
-    public boolean shouldCheckSigAlgOfPeerCerts()
-    {
-        return config.clientCheckSigAlgOfServerCerts;
-    }
-
     public TlsCrypto getCrypto()
     {
         switch (config.clientCrypto)
@@ -97,6 +124,16 @@ class TlsTestClientImpl
         return clientExtensions;
     }
 
+    public Vector getEarlyKeyShareGroups()
+    {
+        if (config.clientEmptyKeyShare)
+        {
+            return null;
+        }
+
+        return super.getEarlyKeyShareGroups();
+    }
+
     public boolean isFallback()
     {
         return config.clientFallback;
@@ -160,6 +197,8 @@ class TlsTestClientImpl
     {
         super.notifyServerVersion(serverVersion);
 
+        this.negotiatedVersion = serverVersion;
+
         if (TlsTestConfig.DEBUG)
         {
             System.out.println("TLS client negotiated " + serverVersion);
@@ -190,14 +229,29 @@ class TlsTestClientImpl
 
                 boolean isEmpty = serverCertificate == null || serverCertificate.getCertificate() == null
                     || serverCertificate.getCertificate().isEmpty();
-                if (isEmpty || !TlsTestUtils.isCertificateOneOf(context.getCrypto(), chain[0],
-                    new String[]
-                    { "x509-server-dsa.pem", "x509-server-ecdh.pem", "x509-server-ecdsa.pem", "x509-server-ed25519.pem",
-                        "x509-server-rsa_pss_256.pem", "x509-server-rsa_pss_384.pem", "x509-server-rsa_pss_512.pem",
-                        "x509-server-rsa-enc.pem", "x509-server-rsa-sign.pem" }))
+
+                if (isEmpty)
                 {
                     throw new TlsFatalAlert(AlertDescription.bad_certificate);
                 }
+
+                String[] trustedCertResources = new String[]{ "x509-server-dsa.pem", "x509-server-ecdh.pem",
+                    "x509-server-ecdsa.pem", "x509-server-ed25519.pem", "x509-server-ed448.pem",
+                    "x509-server-rsa_pss_256.pem", "x509-server-rsa_pss_384.pem", "x509-server-rsa_pss_512.pem",
+                    "x509-server-rsa-enc.pem", "x509-server-rsa-sign.pem" };
+
+                TlsCertificate[] certPath = TlsTestUtils.getTrustedCertPath(context.getCrypto(), chain[0],
+                    trustedCertResources);
+
+                if (null == certPath)
+                {
+                    throw new TlsFatalAlert(AlertDescription.bad_certificate);
+                }
+
+                if (config.clientCheckSigAlgOfServerCerts)
+                {
+                    TlsUtils.checkPeerSigAlgs(context, certPath);
+                }
             }
 
             public TlsCredentials getClientCredentials(CertificateRequest certificateRequest)
@@ -212,10 +266,15 @@ class TlsTestClientImpl
                     return null;
                 }
 
-                short[] certificateTypes = certificateRequest.getCertificateTypes();
-                if (certificateTypes == null || !Arrays.contains(certificateTypes, ClientCertificateType.rsa_sign))
+                boolean isTLSv13 = TlsUtils.isTLSv13(context);
+
+                if (!isTLSv13)
                 {
-                    return null;
+                    short[] certificateTypes = certificateRequest.getCertificateTypes();
+                    if (certificateTypes == null || !Arrays.contains(certificateTypes, ClientCertificateType.rsa_sign))
+                    {
+                        return null;
+                    }
                 }
 
                 Vector supportedSigAlgs = certificateRequest.getSupportedSignatureAlgorithms();
@@ -225,6 +284,8 @@ class TlsTestClientImpl
                     supportedSigAlgs.addElement(config.clientAuthSigAlg);
                 }
 
+                // TODO[tls13] Check also supportedSigAlgsCert against the chain signature(s)
+
                 final TlsCredentialedSigner signerCredentials = TlsTestUtils.loadSignerCredentials(context,
                     supportedSigAlgs, SignatureAlgorithm.rsa, "x509-client-rsa.pem", "x509-client-key-rsa.pem");
 
@@ -275,16 +336,18 @@ class TlsTestClientImpl
 
     protected org.bouncycastle.tls.Certificate corruptCertificate(TlsCrypto crypto, org.bouncycastle.tls.Certificate cert)
     {
-        TlsCertificate[] certList = cert.getCertificateList();
+        CertificateEntry[] certEntryList = cert.getCertificateEntryList();
         try
         {
-            certList[0] = corruptCertificateSignature(crypto, certList[0]);
+            CertificateEntry ee = certEntryList[0];
+            TlsCertificate corruptCert = corruptCertificateSignature(crypto, ee.getCertificate());
+            certEntryList[0] = new CertificateEntry(corruptCert, ee.getExtensions());
         }
         catch (IOException e)
         {
             throw new RuntimeException(e);
         }
-        return new org.bouncycastle.tls.Certificate(certList);
+        return new org.bouncycastle.tls.Certificate(cert.getCertificateRequestContext(), certEntryList);
     }
 
     protected TlsCertificate corruptCertificateSignature(TlsCrypto crypto, TlsCertificate tlsCertificate) throws IOException
@@ -315,6 +378,11 @@ class TlsTestClientImpl
         return bs;
     }
 
+    protected int[] getSupportedCipherSuites()
+    {
+        return TlsUtils.getSupportedCipherSuites(getCrypto(), TEST_CIPHER_SUITES);
+    }
+
     protected ProtocolVersion[] getSupportedVersions()
     {
         if (null != config.clientSupportedVersions)
