@@ -52,13 +52,12 @@ public class TlsTestSuite extends TestSuite
         addVersionTests(testSuite, ProtocolVersion.TLSv10, clientCrypto, serverCrypto);
         addVersionTests(testSuite, ProtocolVersion.TLSv11, clientCrypto, serverCrypto);
         addVersionTests(testSuite, ProtocolVersion.TLSv12, clientCrypto, serverCrypto);
-        // TODO[tls13]
-//        addVersionTests(testSuite, ProtocolVersion.TLSv13, clientCrypto, serverCrypto);
+        addVersionTests(testSuite, ProtocolVersion.TLSv13, clientCrypto, serverCrypto);
     }
 
     private static void addFallbackTests(TestSuite testSuite, int clientCrypto, int serverCrypto)
     {
-        String prefix =  getCryptoName(clientCrypto) + "_" + getCryptoName(serverCrypto) + "_";
+        String prefix = getCryptoName(clientCrypto) + "_" + getCryptoName(serverCrypto) + "_";
 
         {
             TlsTestConfig c = createTlsTestConfig(ProtocolVersion.TLSv12, clientCrypto, serverCrypto);
@@ -86,20 +85,36 @@ public class TlsTestSuite extends TestSuite
 
     private static void addVersionTests(TestSuite testSuite, ProtocolVersion version, int clientCrypto, int serverCrypto)
     {
-        String prefix =  getCryptoName(clientCrypto) + "_" + getCryptoName(serverCrypto) + "_"
+        String prefix = getCryptoName(clientCrypto) + "_" + getCryptoName(serverCrypto) + "_"
             + version.toString().replaceAll("[ \\.]", "") + "_";
 
+        final boolean isTLSv12 = TlsUtils.isTLSv12(version);
+        final boolean isTLSv13 = TlsUtils.isTLSv13(version);
+        final boolean isTLSv12Exactly = isTLSv12 && !isTLSv13;
+
+        final short certReqDeclinedAlert = TlsUtils.isTLSv13(version)
+            ?   AlertDescription.certificate_required
+            :   AlertDescription.handshake_failure;
+
         {
             TlsTestConfig c = createTlsTestConfig(version, clientCrypto, serverCrypto);
 
             addTestCase(testSuite, c, prefix + "GoodDefault");
         }
 
+        if (isTLSv13)
+        {
+            TlsTestConfig c = createTlsTestConfig(version, clientCrypto, serverCrypto);
+            c.clientEmptyKeyShare = true;
+
+            addTestCase(testSuite, c, prefix + "GoodEmptyKeyShare");
+        }
+
         /*
          * Server only declares support for SHA1/RSA, client selects MD5/RSA. Since the client is
          * NOT actually tracking MD5 over the handshake, we expect fatal alert from the client.
          */
-        if (TlsUtils.isTLSv12(version))
+        if (isTLSv12Exactly)
         {
             TlsTestConfig c = createTlsTestConfig(version, clientCrypto, serverCrypto);
             c.clientAuth = C.CLIENT_AUTH_VALID;
@@ -117,7 +132,7 @@ public class TlsTestSuite extends TestSuite
          * when it verifies the selected algorithm against the CertificateRequest supported
          * algorithms.
          */
-        if (TlsUtils.isTLSv12(version))
+        if (isTLSv12)
         {
             TlsTestConfig c = createTlsTestConfig(version, clientCrypto, serverCrypto);
             c.clientAuth = C.CLIENT_AUTH_VALID;
@@ -136,7 +151,7 @@ public class TlsTestSuite extends TestSuite
          * we expect fatal alert to come from the server when it finds the claimed algorithm
          * doesn't match the client certificate.
          */
-        if (TlsUtils.isTLSv12(version))
+        if (isTLSv12)
         {
             TlsTestConfig c = createTlsTestConfig(version, clientCrypto, serverCrypto);
             c.clientAuth = C.CLIENT_AUTH_VALID;
@@ -164,11 +179,25 @@ public class TlsTestSuite extends TestSuite
             addTestCase(testSuite, c, prefix + "BadClientCertificate");
         }
 
+        if (isTLSv13)
+        {
+            /*
+             * For TLS 1.3 the supported_algorithms extension is required in ClientHello when the
+             * server authenticates via a certificate.
+             */
+            TlsTestConfig c = createTlsTestConfig(version, clientCrypto, serverCrypto);
+            c.clientSendSignatureAlgorithms = false;
+            c.clientSendSignatureAlgorithmsCert = false;
+            c.expectServerFatalAlert(AlertDescription.missing_extension);
+
+            addTestCase(testSuite, c, prefix + "BadClientSigAlgs");
+        }
+
         {
             TlsTestConfig c = createTlsTestConfig(version, clientCrypto, serverCrypto);
             c.clientAuth = C.CLIENT_AUTH_NONE;
             c.serverCertReq = C.SERVER_CERT_REQ_MANDATORY;
-            c.expectServerFatalAlert(AlertDescription.handshake_failure);
+            c.expectServerFatalAlert(certReqDeclinedAlert);
 
             addTestCase(testSuite, c, prefix + "BadMandatoryCertReqDeclined");
         }
@@ -178,7 +207,7 @@ public class TlsTestSuite extends TestSuite
          * absent signature_algorithms extension. We expect fatal alert from the client when it
          * verifies the certificate's 'signatureAlgorithm' against the implicit default signature_algorithms.
          */
-        if (TlsUtils.isTLSv12(version))
+        if (isTLSv12Exactly)
         {
             TlsTestConfig c = createTlsTestConfig(version, clientCrypto, serverCrypto);
             c.clientSendSignatureAlgorithms = false;
@@ -208,7 +237,7 @@ public class TlsTestSuite extends TestSuite
          * implied by the absent signature_algorithms extension. We expect fatal alert from the
          * client when it verifies the selected algorithm against the implicit default.
          */
-        if (TlsUtils.isTLSv12(version))
+        if (isTLSv12Exactly)
         {
             TlsTestConfig c = createTlsTestConfig(version, clientCrypto, serverCrypto);
             c.clientCheckSigAlgOfServerCerts = false;
@@ -238,16 +267,15 @@ public class TlsTestSuite extends TestSuite
          * Server generates downgraded (RFC 8446) 1.1 ServerHello. We expect fatal alert
          * (illegal_parameter) from the client.
          */
-        // TODO[tls13]
-//        if (!TlsUtils.isTLSv13(version))
-//        {
-//            TlsTestConfig c = createTlsTestConfig(version, clientCrypto, serverCrypto);
-//            c.serverNegotiateVersion = version;
-//            c.serverSupportedVersions = ProtocolVersion.TLSv13.downTo(version);
-//            c.expectClientFatalAlert(AlertDescription.illegal_parameter);
-//
-//            addTestCase(testSuite, c, prefix + "BadDowngrade");
-//        }
+        if (!isTLSv13)
+        {
+            TlsTestConfig c = createTlsTestConfig(version, clientCrypto, serverCrypto);
+            c.serverNegotiateVersion = version;
+            c.serverSupportedVersions = ProtocolVersion.TLSv13.downTo(version);
+            c.expectClientFatalAlert(AlertDescription.illegal_parameter);
+
+            addTestCase(testSuite, c, prefix + "BadDowngrade");
+        }
     }
 
     private static void addTestCase(TestSuite testSuite, TlsTestConfig config, String name)
@@ -259,9 +287,7 @@ public class TlsTestSuite extends TestSuite
     {
         TlsTestConfig c = new TlsTestConfig();
         c.clientCrypto = clientCrypto;
-        // TODO[tls13]
-//        c.clientSupportedVersions = ProtocolVersion.TLSv13.downTo(ProtocolVersion.SSLv3);
-        c.clientSupportedVersions = ProtocolVersion.TLSv12.downTo(ProtocolVersion.SSLv3);
+        c.clientSupportedVersions = ProtocolVersion.TLSv13.downTo(ProtocolVersion.SSLv3);
         c.serverCrypto = serverCrypto;
         c.serverSupportedVersions = serverMaxVersion.downTo(ProtocolVersion.SSLv3);
         return c;
