@@ -11,6 +11,7 @@ import java.security.PrivateKey;
 import java.security.Security;
 import java.security.spec.PKCS8EncodedKeySpec;
 import java.security.spec.RSAPrivateCrtKeySpec;
+import java.util.Hashtable;
 import java.util.Vector;
 
 import javax.net.ssl.KeyManagerFactory;
@@ -30,6 +31,7 @@ import org.bouncycastle.crypto.params.RSAPrivateCrtKeyParameters;
 import org.bouncycastle.crypto.util.PrivateKeyFactory;
 import org.bouncycastle.tls.AlertDescription;
 import org.bouncycastle.tls.Certificate;
+import org.bouncycastle.tls.CertificateEntry;
 import org.bouncycastle.tls.SignatureAlgorithm;
 import org.bouncycastle.tls.SignatureAndHashAlgorithm;
 import org.bouncycastle.tls.TlsContext;
@@ -112,6 +114,65 @@ public class TlsTestUtils
         return "x509-ca-" + getResourceName(signatureAlgorithm) + ".pem";
     }
 
+    static String getCACertResource(String eeCertResource) throws IOException
+    {
+        if (eeCertResource.startsWith("x509-client-"))
+        {
+            eeCertResource = eeCertResource.substring("x509-client-".length());
+        }
+        if (eeCertResource.startsWith("x509-server-"))
+        {
+            eeCertResource = eeCertResource.substring("x509-server-".length());
+        }
+        if (eeCertResource.endsWith(".pem"))
+        {
+            eeCertResource = eeCertResource.substring(0, eeCertResource.length() - ".pem".length());
+        }
+
+        if ("dsa".equalsIgnoreCase(eeCertResource))
+        {
+            return getCACertResource(SignatureAlgorithm.dsa);
+        }
+
+        if ("ecdh".equalsIgnoreCase(eeCertResource)
+            || "ecdsa".equalsIgnoreCase(eeCertResource))
+        {
+            return getCACertResource(SignatureAlgorithm.ecdsa);
+        }
+
+        if ("ed25519".equalsIgnoreCase(eeCertResource))
+        {
+            return getCACertResource(SignatureAlgorithm.ed25519);
+        }
+
+        if ("ed448".equalsIgnoreCase(eeCertResource))
+        {
+            return getCACertResource(SignatureAlgorithm.ed448);
+        }
+
+        if ("rsa".equalsIgnoreCase(eeCertResource)
+            || "rsa-enc".equalsIgnoreCase(eeCertResource)
+            || "rsa-sign".equalsIgnoreCase(eeCertResource))
+        {
+            return getCACertResource(SignatureAlgorithm.rsa);
+        }
+
+        if ("rsa_pss_256".equalsIgnoreCase(eeCertResource))
+        {
+            return getCACertResource(SignatureAlgorithm.rsa_pss_pss_sha256);
+        }
+        if ("rsa_pss_384".equalsIgnoreCase(eeCertResource))
+        {
+            return getCACertResource(SignatureAlgorithm.rsa_pss_pss_sha384);
+        }
+        if ("rsa_pss_512".equalsIgnoreCase(eeCertResource))
+        {
+            return getCACertResource(SignatureAlgorithm.rsa_pss_pss_sha512);
+        }
+
+        throw new TlsFatalAlert(AlertDescription.internal_error);
+    }
+
     static String getResourceName(short signatureAlgorithm) throws IOException
     {
         switch (signatureAlgorithm)
@@ -144,7 +205,7 @@ public class TlsTestUtils
         String keyResource) throws IOException
     {
         TlsCrypto crypto = context.getCrypto();
-        Certificate certificate = loadCertificateChain(crypto, certResources);
+        Certificate certificate = loadCertificateChain(context, certResources);
 
         // TODO[tls-ops] Need to have TlsCrypto construct the credentials from the certs/key (as raw data)
         if (crypto instanceof BcTlsCrypto)
@@ -166,7 +227,7 @@ public class TlsTestUtils
         String keyResource) throws IOException
     {
         TlsCrypto crypto = context.getCrypto();
-        Certificate certificate = loadCertificateChain(crypto, certResources);
+        Certificate certificate = loadCertificateChain(context, certResources);
 
         // TODO[tls-ops] Need to have TlsCrypto construct the credentials from the certs/key (as raw data)
         if (crypto instanceof BcTlsCrypto)
@@ -188,7 +249,7 @@ public class TlsTestUtils
         SignatureAndHashAlgorithm signatureAndHashAlgorithm) throws IOException
     {
         TlsCrypto crypto = context.getCrypto();
-        Certificate certificate = loadCertificateChain(crypto, certResources);
+        Certificate certificate = loadCertificateChain(context, certResources);
         TlsCryptoParameters cryptoParams = new TlsCryptoParameters(context);
 
         // TODO[tls-ops] Need to have TlsCrypto construct the credentials from the certs/key (as raw data)
@@ -233,8 +294,7 @@ public class TlsTestUtils
             return null;
         }
 
-        return loadSignerCredentials(context, new String[]{ certResource, getCACertResource(signatureAlgorithm) },
-            keyResource, signatureAndHashAlgorithm);
+        return loadSignerCredentials(context, new String[]{ certResource }, keyResource, signatureAndHashAlgorithm);
     }
 
     static TlsCredentialedSigner loadSignerCredentialsServer(TlsContext context, Vector supportedSignatureAlgorithms,
@@ -258,15 +318,38 @@ public class TlsTestUtils
         return loadSignerCredentials(context, supportedSignatureAlgorithms, signatureAlgorithm, certResource, keyResource);
     }
 
-    static Certificate loadCertificateChain(TlsCrypto crypto, String[] resources)
+    static Certificate loadCertificateChain(TlsContext context, String[] resources)
         throws IOException
     {
-        TlsCertificate[] chain = new TlsCertificate[resources.length];
-        for (int i = 0; i < resources.length; ++i)
+        TlsCrypto crypto = context.getCrypto();
+
+        if (TlsUtils.isTLSv13(context))
         {
-            chain[i] = loadCertificateResource(crypto, resources[i]);
+            CertificateEntry[] certificateEntryList = new CertificateEntry[resources.length];
+            for (int i = 0; i < resources.length; ++i)
+            {
+                TlsCertificate certificate = loadCertificateResource(crypto, resources[i]);
+
+                // TODO[tls13] Add possibility of specifying e.g. CertificateStatus 
+                Hashtable extensions = null;
+
+                certificateEntryList[i] = new CertificateEntry(certificate, extensions);
+            }
+
+            // TODO[tls13] Support for non-empty request context
+            byte[] certificateRequestContext = TlsUtils.EMPTY_BYTES;
+
+            return new Certificate(certificateRequestContext, certificateEntryList);
+        }
+        else
+        {
+            TlsCertificate[] chain = new TlsCertificate[resources.length];
+            for (int i = 0; i < resources.length; ++i)
+            {
+                chain[i] = loadCertificateResource(crypto, resources[i]);
+            }
+            return new Certificate(chain);
         }
-        return new Certificate(chain);
     }
 
     static org.bouncycastle.asn1.x509.Certificate loadBcCertificateResource(String resource)
@@ -411,16 +494,24 @@ public class TlsTestUtils
         return Arrays.areEqual(a.getEncoded(), b.getEncoded());
     }
 
-    static boolean isCertificateOneOf(TlsCrypto crypto, TlsCertificate cert, String[] resources) throws IOException
+    static TlsCertificate[] getTrustedCertPath(TlsCrypto crypto, TlsCertificate cert, String[] resources)
+        throws IOException
     {
         for (int i = 0; i < resources.length; ++i)
         {
-            if (areSameCertificate(crypto, cert, resources[i]))
+            String eeCertResource = resources[i];
+            TlsCertificate eeCert = loadCertificateResource(crypto, eeCertResource);
+            if (areSameCertificate(cert, eeCert))
             {
-                return true;
+                String caCertResource = getCACertResource(eeCertResource);
+                TlsCertificate caCert = loadCertificateResource(crypto, caCertResource);
+                if (null != caCert)
+                {
+                    return new TlsCertificate[]{ eeCert, caCert };
+                }
             }
         }
-        return false;
+        return null;
     }
 
     static TrustManagerFactory getSunX509TrustManagerFactory()
